package org.gvsig.fmap.dal;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.tools.util.UnmodifiableBasicSet;
import org.gvsig.tools.util.UnmodifiableBasicSetAdapter;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class BaseStoresRepository 
        extends AbstractStoresRepository
        implements StoresRepository 
    {

    private final Map<String,DataStoreParameters> repository;

    public BaseStoresRepository(String name) {
        super(name);
        this.repository = new HashMap<>();
    }
    
    
    @Override
    protected DataStoreParameters getMyParameters(String name) {
        return this.repository.get(name);
    }

    @Override
    protected boolean isEmptyMyRepository() {
        return this.repository.isEmpty();
    }
    
    @Override
    protected int getMySize() {
        return this.repository.size();
    }

    @Override
    protected UnmodifiableBasicSet<String> getMyKeySet() {
        Set<String> keyset = this.repository.keySet();
        if( keyset == null || keyset.isEmpty() ) {
            return UnmodifiableBasicSet.EMPTY_UNMODIFIABLEBASICSET;
        }
        return new UnmodifiableBasicSetAdapter<>(keyset);
    }

    @Override
    public void add(String name, DataStoreParameters parameters) {
        if( parameters == null ) {
            throw new IllegalArgumentException("parameters can't be null.");
        }
        if( StringUtils.isBlank(name) ) {
            throw new IllegalArgumentException("name can't be blank.");
        }
        if( this.notifyObservers(NOTIFICATION_ADD, name, parameters).isCanceled() ) {
            return;
        }
        this.repository.put(name, parameters);
    }

    @Override
    public void remove(String name) {
        if( StringUtils.isBlank(name) ) {
            LOGGER.warn("parameter name can't be blank");
            return;
        }
        if( this.notifyObservers(NOTIFICATION_REMOVE, name).isCanceled() ) {
            return;
        }
        this.repository.remove(name);
    }
    
}
