package org.gvsig.fmap.dal.feature;

import java.util.List;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.fmap.dal.StoresRepository;
import org.gvsig.fmap.dal.expressionevaluator.FeatureSymbolTable;
import org.gvsig.tools.dispose.Disposable;
import org.gvsig.tools.dynobject.DynObjectValueItem;

/**
 *
 * @author jjdelcerro
 */
public interface ForeingKey {

    public static final int MAX_AVAILABLE_VALUES = 1000;
    
    public interface ContextForeingKey extends Disposable {
        public StoresRepository getStoresRepository();
        public FeatureStore getFeatureStore();
        public Expression getLabelExpression();
        public FeatureSymbolTable getFeatureSymbolTable();
        public SymbolTable getSymbolTable();
    }
    
    public ContextForeingKey createContext();
    
    public boolean isForeingKey();

    /**
     * Indicates whether the foreign key reference to a closed list of values.
     * 
     * @return 
     */
    public boolean isClosedList();
    
    public String getTableName();

    public String getCodeName();

    public String getLabelFormula();

    
    public boolean isEmpty();

    public Object getCode(ContextForeingKey context, Feature feature);

    public Feature getFeature(ContextForeingKey context, Object codeValue);

    public FeatureStore getFeatureStore(ContextForeingKey context);

    public FeatureType getFeatureType(ContextForeingKey context);

    public List<Feature> getFeatures(ContextForeingKey context);

    public String getLabel(ContextForeingKey context, Object codeValue);

    public String getLabel(ContextForeingKey context, Feature feature);

    public Expression getLabelExpression(ContextForeingKey context);

    public FeatureQuery getQuery(ContextForeingKey context, Object codeValue);

    public StoresRepository getStoresRepository(ContextForeingKey context);

    public DynObjectValueItem[] getAvailableValues(ContextForeingKey context);
    
    public String getLabelForValue(Object value);

    public boolean getEnsureReferentialIntegrity();
    
    /**
     * If the data store supports it, it ensures that referential integrity is respected.
     * 
     * @param ensureReferentialIntegrity 
     */
    public void setEnsureReferentialIntegrity(boolean ensureReferentialIntegrity);
}
