package org.gvsig.fmap.dal.feature;

import java.util.List;
import org.cresques.cts.IProjection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.dataTypes.DataType;

public interface SQLBuilder_save {

    public enum Privilege {
        SELECT,
        INSERT,
        UPDATE,
        DELETE,
        TRUNCATE,
        REFERENCE,
        TRIGGER,
        ALL
    };

    public enum GeometrySupportType {
        WKT,
        WKB,
        EWKB
    }
    
    public interface Parameter {
        public int getType();
        public String getName();
        public IProjection getCRS();
        public boolean isConstant();
        public Object getValue();
    }

    public interface ExpBuilder {
        public boolean isEmpty();
        public ExpBuilder set(String value);
        public ExpBuilder set(OpBuilder op);
        public ExpBuilder and(String value);
        public ExpBuilder and(OpBuilder op);
        public ExpBuilder or(String value);
        public ExpBuilder or(OpBuilder op);
    }
    
    public interface ColumnBuilder {
        public ColumnBuilder name(String name);
        public ColumnBuilder geometry(String name);
        public ColumnBuilder expression(String value);
        public ColumnBuilder as(String alias);
        public String getName();
        public String getAlias();
        public String getExpression();
    }
    
    public interface OpBuilder {
        public OpBuilder setop1(String op);
        public OpBuilder setop2(String op);
    }
    
    public interface FromBuilder {
        public FromBuilder table(String tableName);
        public FromBuilder table(String dbname, String schemaName, String tableName);
        public FromBuilder subquery(String subquery);
        public FromBuilder passThrough(String passthrough);
    }
    
    public interface SelectBuilder {
        public ColumnBuilder column();
        public boolean has_column(String name);
        public FromBuilder from();
        public FromBuilder from(String s);
        public ExpBuilder where();
        public ExpBuilder where(String s);
        public ExpBuilder where(OpBuilder s);
        public SelectBuilder limit(long limit);
        public SelectBuilder offset(long offset);
        public SelectBuilder add_order_by(String order);
        public SelectBuilder add_order_by(String column, boolean ascending);
        public boolean has_order_by();
    }
    
    public interface UpdateBuilder {
        public UpdateBuilder table(String tableName);
        public UpdateBuilder table(String dbName, String schemaName, String tableName);
        public UpdateBuilder set(String fieldname, String value);
        public UpdateBuilder setGeometry(String fieldname, String value, IProjection crs);
        public ExpBuilder where();
        public boolean hasWhere();
    }
    
    public interface InsertBuilder {
        public InsertBuilder table(String tableName);
        public InsertBuilder table(String dbName, String schemaName, String tableName);
        public InsertBuilder set(String fieldname, String value);
        public InsertBuilder setGeometry(String fieldname, String value, IProjection crs);
    }
    
    public interface DeleteBuilder {
        public DeleteBuilder table(String tableid);
        public DeleteBuilder table(String dbName, String schemaName, String tableName);
        public ExpBuilder where();
        public boolean hasWhere();
    }
    
    public interface AlterTableBuilder {
        public AlterTableBuilder table(String tableName);
        public AlterTableBuilder table(String dbName, String schemaName, String tableName);
        public AlterTableBuilder drop_column(String columnName);
        public AlterTableBuilder add_column(String columnName, int type, int type_p, int type_s, boolean isPk, boolean allowNulls, boolean isAutomatic, Object defaultValue);
        public AlterTableBuilder alter_column(String columnName, int type, int type_p, int type_s, boolean isPk, boolean allowNulls, boolean isAutomatic, Object defaultValue);
        public AlterTableBuilder rename_column(String source, String target);
        public List<String> toStrings();
    }
    
    public interface CreateTableBuilder {
        public CreateTableBuilder table(String tableName);
        public CreateTableBuilder table(String dbname, String schemaName, String tableName);
        public CreateTableBuilder add_column(String columnName, int type, int type_p, int type_s, boolean isPk, boolean allowNulls, boolean isAutomatic, Object defaultValue);
        public List<String> toStrings();
    }
    
    public interface UpdateTableStatisticsBuilder {
        public UpdateTableStatisticsBuilder table(String tableName);
        public UpdateTableStatisticsBuilder table(String dbName, String schemaName, String tableName);
        public List<String> toStrings();
    }
        
    public interface GrantBuilder {
        public GrantBuilder table(String tableName);
        public GrantBuilder table(String dbName, String schemaName, String tableName);
        public GrantBuilder add(String roleid, Privilege privilege);
        public GrantBuilder select(String roleid);
        public GrantBuilder insert(String roleid);
        public GrantBuilder delete(String roleid);
        public GrantBuilder truncate(String roleid);
        public GrantBuilder reference(String roleid);
        public GrantBuilder trigger(String roleid);
        public GrantBuilder all(String roleid);
        public GrantBuilder select();
        public GrantBuilder insert();
        public GrantBuilder delete();
        public GrantBuilder truncate();
        public GrantBuilder reference();
        public GrantBuilder trigger();
        public GrantBuilder all();
        public List<String> toStrings();
    }
        
    public interface DropTableBuilder {
        public DropTableBuilder table(String tableName);
        public DropTableBuilder table(String dbName, String schemaName, String tableName);
        public List<String> toStrings();
    }
    
    boolean has_spatial_functions();
    
    String ST_AsText(String geom);

    String ST_AsBinary(String geom);

    String ST_AsEWKB(String geom);

    String ST_Envelope(String geom);

    String ST_ExtentAggregate(String geom);

    String ST_UnionAggregate(String geom);
    
    String ST_GeomFromText(String geom, String crs);
    
    String ST_GeomFromWKB(String geom, String crs);
            
    String ST_GeomFromEWKB(String geom, String crs);

    String ST_GeomFromText(String geom, int crs);

    String ST_GeomFromText(Geometry geom, IProjection crs);

    String ST_GeomFromWKB(Geometry geom, IProjection crs);
    
    String ST_GeomFromEWKB(Geometry geom, IProjection crs);

    String ST_GeomFromText(Envelope env, IProjection crs);

    String ST_Intersects(String geom1, String geom2);

    String ST_Contains(String geom1, String geom2);

    String ST_Crosses(String geom1, String geom2);

    String ST_IsClosed(String geom1, String geom2);

    String ST_Overlaps(String geom1, String geom2);

    String ST_Touches(String geom1, String geom2);

    String ST_Within(String geom1, String geom2);

    String isNull(String s);

    String default_schema();
    
    String count(String column);
    
    String lcase(String s);
    
    String ucase(String s);
    
    String getWKT(Geometry geom);
    byte[] getWKB(Geometry geom);
    byte[] getEWKB(Geometry geom);
        
    String getWKT(Envelope env);
    byte[] getWKB(Envelope env);
    byte[] getEWKB(Envelope env);

    String geometry(Geometry geom, IProjection crs);
    
    String geometry(Envelope env, IProjection crs);
    
    String string(String s);
    
    String identifier(String id);
    
    String bytearray(byte[] data);
    
    String crs(String crs);
    
    String crs(IProjection crs);

    String sqltype(int dataType, int p, int s);

    GeometrySupportType geometry_support_type();
    
    OpBuilder and(String op1, String op2);

    OpBuilder or(String op1, String op2);

    OpBuilder eq(String op1, String op2);
    
    OpBuilder ne(String op1, String op2);
    
    OpBuilder gt(String op1, String op2);
    
    OpBuilder ge(String op1, String op2);

    OpBuilder lt(String op1, String op2);
    
    OpBuilder le(String op1, String op2);

    OpBuilder like(String op1, String op2);

    OpBuilder ilike(String op1, String op2);

    ExpBuilder createExpresionBuilder(String x);
    
    ExpBuilder createExpresionBuilder(OpBuilder x);
    
    ExpBuilder createExpresionBuilder();
    
    void clearParameters();
    
    void addParameter(DataType type, String name);
    
    void addParameter(int type, String name);
    
    void addParameter(int type, String name, IProjection crs);
    
    void addConstantParameter(Object value);

    List<Parameter> getParameters();
    
    SelectBuilder select();
    
    UpdateBuilder update();

    InsertBuilder insert();
    
    DeleteBuilder delete();
    
    AlterTableBuilder alter_table();

    CreateTableBuilder create_table();

    GrantBuilder grant();

    DropTableBuilder drop_table();

    UpdateTableStatisticsBuilder update_table_statistics();
}
