package org.gvsig.fmap.dal.store.h2.expressionbuilderformatter;

import java.text.MessageFormat;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.expressionevaluator.ExpressionBuilder.Value;
import org.gvsig.expressionevaluator.Formatter;
import org.gvsig.fmap.dal.SQLBuilder;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryUtils;

/**
 *
 * @author jjdelcerro
 */
public class Constant implements Formatter<Value> {
    
    private static final String FORMAT_ST_GEOMFROMWKB = "ST_GeomFromWKB(({0}), ({1}))";
    private static final String FORMAT_ST_GEOMFROMEWKB = "ST_GeomFromEWKB(({0}), ({1}))";

    private final SQLBuilder sqlbuilder;
    private final Formatter<Value> formatter;
    
    public Constant(SQLBuilder sqlbuilder, Formatter<Value> formatter) {
        this.sqlbuilder = sqlbuilder;
        this.formatter = formatter;
    }
    
    @Override
    public boolean canApply(ExpressionBuilder.Value value) {
        if (value instanceof ExpressionBuilder.Constant) {
            Object x = ((ExpressionBuilder.Constant)value).value();
            if( x instanceof byte[] ) {
                return true;
            }
            if( x instanceof Geometry ) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String format(Value constant) {
        Object x = ((ExpressionBuilder.Constant)constant).value();
        ExpressionBuilder builder = this.sqlbuilder.expression();
        if( x instanceof byte[] ) {
            return builder.bytearray_x((byte[]) x);
        }
        if( x instanceof Geometry ) {
            Geometry geometry = (Geometry) x;
            switch (builder.geometry_support_type()) {
                case EWKB:
                    return MessageFormat.format(
                            FORMAT_ST_GEOMFROMEWKB,
                            builder.bytearray_x(GeometryUtils.toEWKB(geometry)),
                            String.valueOf(builder.srs_id(geometry.getProjection()))
                    );
                case WKB:
                    return MessageFormat.format(
                            FORMAT_ST_GEOMFROMWKB,
                            builder.bytearray_x(GeometryUtils.toWKB(geometry)),
                            String.valueOf(builder.srs_id(geometry.getProjection()))
                    );
                case WKT:
                default:
                    return constant.toString(null);
            }
            
        }
        return null;
    }
    
}
