package org.gvsig.fmap.dal.store.h2.expressionbuilderformatter;

import java.util.List;
import java.util.Objects;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_DECODE;
import org.gvsig.expressionevaluator.ExpressionBuilder.Function;
import org.gvsig.expressionevaluator.ExpressionBuilder.Value;
import org.gvsig.expressionevaluator.Formatter;
import org.gvsig.fmap.dal.SQLBuilder;

/**
 *
 * @author jjdelcerro
 */
public class Decode implements Formatter<Value> {
    
    private final SQLBuilder sqlbuilder;
    private final Formatter<Value> formatter;
    
    public Decode(SQLBuilder sqlbuilder, Formatter<Value> formatter) {
        this.sqlbuilder = sqlbuilder;
        this.formatter = formatter;
    }
    
    @Override
    public boolean canApply(ExpressionBuilder.Value value) {
        if (value instanceof ExpressionBuilder.Function) {
            if( StringUtils.equalsIgnoreCase(FUNCTION_DECODE, ((Function) value).name()) ) {
                List<Value> parameters = ((Function) value).parameters();
                if( parameters.size()==2) {
                    Value p1 = parameters.get(0);
                    Value p2 = parameters.get(1);
                    if( p1 instanceof ExpressionBuilder.Constant && 
                        p2 instanceof ExpressionBuilder.Constant &&
                        ((ExpressionBuilder.Constant)p1).value() instanceof String &&
                        ((ExpressionBuilder.Constant)p2).value() instanceof String 
                        ) {
                        String p2value = ((ExpressionBuilder.Constant)p2).value().toString();
                        if( StringUtils.equalsIgnoreCase(p2value, "hex") ) {
                            return true;
                        }
                    }
                    
                }
            }
        }
        return false;
    }

    @Override
    public String format(Value function) {
        List<Value> parameters = ((Function) function).parameters();
        Value p1 = parameters.get(0);
        String p1value = Objects.toString(((ExpressionBuilder.Constant)p1).value(), "");
        String r = "X'"+p1value+"'";
        return r;
    }
    
}
