package org.gvsig.fmap.dal.store.h2.expressionbuilderformatter;

import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.expressionevaluator.ExpressionBuilder.Value;
import org.gvsig.expressionevaluator.Formatter;
import org.gvsig.fmap.dal.SQLBuilder;

/**
 *
 * @author jjdelcerro
 */
public class H2SpatialFormatter implements Formatter<Value> {

    private final SQLBuilder builder;
    private final Formatter<ExpressionBuilder.Value>[] formatters;

    public H2SpatialFormatter(SQLBuilder builder) {

        this.builder = builder;
        this.formatters = new Formatter[]{
            new IfNull(this.builder, this),
            new IsNotNull(this.builder, this),
            new IsNull(this.builder, this),
            new ST_AsEWKB(this.builder, this),
            new ST_GeomFromEWKB(this.builder, this),
            new ST_Intersects(this.builder, this),
            new ST_ExtentAggregate(this.builder, this),
            new Decode(this.builder, this),
            new ILike(this.builder, this),
            new Constant(this.builder, this),
            new ST_AsBinary(this.builder, this)
        };
    }

    @Override
    public boolean canApply(Value value) {
        for (Formatter<Value> formatter : formatters) {
            if (formatter.canApply(value)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public String format(Value value) {
        for (Formatter<Value> formatter : formatters) {
            if (formatter.canApply(value)) {
                return formatter.format(value);
            }
        }
        return value.toString(this);
    }
    
}
