/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.store.jdbc2;

import org.gvsig.fmap.dal.DALLibrary;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.resource.db.DBResourceLibrary;
import org.gvsig.fmap.dal.spi.DataManagerProviderServices;
import org.gvsig.fmap.dal.store.db.DBHelper;
import org.gvsig.fmap.dal.store.db.DBStoreLibrary;
import org.gvsig.fmap.dal.store.jdbc.JDBCNewStoreParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCResourceParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCStoreParameters;
import org.gvsig.metadata.exceptions.MetadataException;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.LibraryException;

/**
 * Initialize JDBC Provider Library
 *
 */
public class JDBCLibrary extends AbstractLibrary {

    public static final String NAME = "JDBC2";

    @Override
    public void doRegistration() {
        registerAsServiceOf(DALLibrary.class);
        require(DBStoreLibrary.class);
        require(DBResourceLibrary.class);
    }

    @Override
    protected void doInitialize() throws LibraryException {
    }

    @Override
    protected void doPostInitialize() throws LibraryException {
        
        LibraryException ex = null;

        DataManagerProviderServices dataman = (DataManagerProviderServices) DALLocator
                .getDataManager();

        DBHelper.registerParametersDefinition(
                NAME + "StoreParameters",
                JDBCStoreParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );
        DBHelper.registerParametersDefinition(
                NAME + "NewStoreParameters",
                JDBCNewStoreParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );
        DBHelper.registerParametersDefinition(
                NAME + "ServerExplorerParameters",
                JDBCServerExplorerParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );
        DBHelper.registerParametersDefinition(
                NAME + "ResourceParameters",
                JDBCResourceParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );

        try {
            DBHelper.registerMetadataDefinition(
                    NAME,
                    JDBCStoreProvider.class,
                    dataman.getResourceAsStream(this, NAME + "Metadata.xml")
            );
        } catch (MetadataException e) {
            ex = new LibraryException(this.getClass(), e);
        }

//        ResourceManagerProviderServices resman = (ResourceManagerProviderServices) DALLocator
//                .getResourceManager();
//
//        if (!resman.getResourceProviders().contains(NAME)) {
//            resman.register(
//                    JDBCResource.NAME, 
//                    JDBCResource.DESCRIPTION,
//                    JDBCResource.class,
//                    JDBCResourceParameters.class
//            );
//        }

//        if (!dataman.getStoreProviderRegister().exits(NAME)) {
//            dataman.registerStoreProviderFactory(new JDBCStoreProviderFactory());
//        }
//
//        if (!dataman.getServerExplorerRegister().exits(NAME)) {
//            dataman.registerServerExplorerFactory(new JDBCServerExplorerFactory());
//        }

        if (ex != null) {
            throw ex;
        }
    }
}
