package org.gvsig.fmap.dal.store.jdbc2;

import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang3.tuple.Pair;
import org.cresques.cts.IProjection;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.EditableFeatureType;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.SQLBuilder;
import org.gvsig.fmap.dal.feature.spi.FeatureReferenceProviderServices;
import org.gvsig.fmap.dal.store.jdbc.JDBCNewStoreParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCStoreParameters;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.AppendOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.CalculateEnvelopeOfColumnOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.CanCreateTablesOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.CanModifyTableOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.CreateTableOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.FetchFeatureProviderByReferenceOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.FetchFeatureTypeOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.TableIsEmptyOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.ListTablesOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.PerformChangesOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.DropTableOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.ResultSetForSetProviderOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.CountOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.ExecuteOperation;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.UpdateTableStatisticsOperation;
import org.gvsig.fmap.geom.primitive.Envelope;

public interface OperationsFactory {

    public interface TableReference {
        public String getDatabase();
        public String getSchema();
        public String getTable();
        public String getSubquery();
        public boolean hasDatabase();
        public boolean hasSchema();
        public boolean hasTable();
        public boolean hasSubquery();
    }
    
    public TableReference createTableReference(
            String database,
            String schema,
            String table,
            String subquery            
    );
    
    public TableReference createTableReference(JDBCStoreParameters params);
    public TableReference createTableReference(JDBCNewStoreParameters params);
    
    public FetchFeatureTypeOperation createFetchFeatureType(
            EditableFeatureType type,
            TableReference table,
            List<String> primaryKeys,
            String defaultGeometryField,
            IProjection crs
    );
    
    public FetchFeatureProviderByReferenceOperation createFetchFeatureProviderByReference(
            FeatureReferenceProviderServices reference,
            FeatureType featureType,
            TableReference table
    );

    public CalculateEnvelopeOfColumnOperation createCalculateEnvelopeOfColumn(
            FeatureType featureType,
            TableReference table,
            String columnName,
            String baseFilter,
            Envelope workingArea,
            IProjection crs
    );

    public PerformChangesOperation createPerformChanges(
            TableReference table,
            FeatureType type,
            Iterator deleteds,
            Iterator inserteds,
            Iterator updateds,
            Iterator featureTypesChanged
    );

    public AppendOperation createAppend(
            TableReference table,
            FeatureType type
    );

    public CountOperation createCount(
            FeatureType featureType,
            TableReference table,
            String baseFilter,
            String filter
    );

    public TableIsEmptyOperation createTableIsEmpty(
            FeatureType featureType,
            TableReference table,
            String baseFilter,
            String filtersql
    );

    public ResultSetForSetProviderOperation createResultSetForSetProvider(
            TableReference table,
            String baseFilter,
            String baseOrder,
            FeatureQuery query,
            FeatureType storeType,
            FeatureType setType,
            long limit,
            long offset,
            int fetchSize
    );

    public ListTablesOperation createListTables(
            int mode,
            JDBCStoreParameters baseParameters,
            boolean informationTables
    );

    public DropTableOperation createDropTable(
            TableReference table
    );

    public CreateTableOperation createTable(
            TableReference table,
            FeatureType type,
            List<Pair<String, SQLBuilder.Privilege>> userAndPrivileges,
            List<String> additionalSQLs
    ) throws DataException;

    public CanCreateTablesOperation createCanCreateTables();
    
    public UpdateTableStatisticsOperation createUpdateTableStatistics(
            TableReference table
    );

    public CanModifyTableOperation createCanModifyTableOperation(
            TableReference table
    );
    
    public ExecuteOperation createExecute(String sql);
    
}
