/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.store.jdbc2.impl;

import java.sql.SQLException;
import java.util.NoSuchElementException;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.spi.AbstractFeatureProviderIterator;
import org.gvsig.fmap.dal.feature.spi.FeatureProvider;
import org.gvsig.fmap.dal.store.jdbc2.JDBCStoreProvider;
import org.gvsig.fmap.dal.store.jdbc2.ResulSetControler.ResultSetEntry;
import org.gvsig.tools.exception.BaseException;

public class JDBCIterator extends AbstractFeatureProviderIterator {

    protected ResultSetEntry resulsetEntry;
    protected Boolean hasNext = null;
    protected FeatureType featureType;
    protected JDBCSetProvider set;

    protected JDBCIterator(
            JDBCStoreProvider store, 
            JDBCSetProvider set,
            FeatureType featureType,
            ResultSetEntry resulsetEntry
        ) throws DataException {
        super(store);
        this.resulsetEntry = resulsetEntry;
        this.featureType = featureType;
        this.set = set;
        this.hasNext = null;
    }

    @Override
    protected JDBCStoreProvider getFeatureStoreProvider() {
        return (JDBCStoreProvider) super.getFeatureStoreProvider();
    }

    @Override
    public final Object next() {
        return internalNext();
    }

    @Override
    public final boolean hasNext() {
        return internalHasNext();
    }

    @Override
    protected boolean internalHasNext() {
        if (hasNext == null) {
            try {
                if ( resulsetEntry.next() ) {
                    hasNext = Boolean.TRUE;
                } else {
                    hasNext = Boolean.FALSE;
                    this.close();
                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        }
        return hasNext;
    }

    @Override
    protected Object internalNext() {
        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        FeatureProvider data;
        try {
            data = getFeatureProvider();
        } catch (DataException e) {
            throw new RuntimeException(e);
        }
        hasNext = null;
        return data;
    }

    @Override
    public void remove() {
        throw new UnsupportedOperationException();
    }

    protected FeatureProvider createFeatureProvider() throws DataException {
        return getFeatureStoreProvider().createFeatureProvider(featureType);
    }

    protected FeatureProvider getFeatureProvider() throws DataException {
        FeatureProvider data = createFeatureProvider();
        this.getFeatureStoreProvider().getHelper().fetchFeature(data, this.resulsetEntry);
        return data;
    }

    @Override
    protected void doDispose() throws BaseException {
        if (this.resulsetEntry!=null) {
            this.close();
        }
        this.featureType = null;
    }

    protected void close() {
        try {
            this.resulsetEntry.close();
            this.resulsetEntry = null;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
