/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.store.jdbc2.impl;

import org.gvsig.fmap.dal.store.jdbc2.JDBCLibrary;

import org.gvsig.fmap.dal.DataParameters;
import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.feature.spi.AbstractFeatureStoreProviderFactory;
import org.gvsig.fmap.dal.spi.DataStoreProviderServices;
import org.gvsig.fmap.dal.store.jdbc.JDBCConnectionParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCStoreParameters;
import org.gvsig.fmap.dal.store.jdbc2.JDBCHelper;
import org.gvsig.fmap.dal.store.jdbc2.JDBCStoreProvider;
import org.gvsig.fmap.dal.store.jdbc2.spi.JDBCHelperBase;

public class JDBCStoreProviderFactory 
    extends AbstractFeatureStoreProviderFactory 
    {

    private static final String NAME = JDBCLibrary.NAME;
    
    public JDBCStoreProviderFactory() {
        this(
                NAME, 
                "JDBC generic store (2)"
        );
    }
    
    @Override
    public int hasTabularSupport() {
        return YES;
    }    

    @Override
    public int hasSQLSupport() {
        return YES;
    }
    
    protected JDBCStoreProviderFactory(String name, String description) {
        super(name, description);
    }
    
    protected JDBCStoreProviderFactory(String name, String description, boolean hidden) {
        super(name, description, hidden);
    }

    @Override
    public JDBCStoreProvider createProvider(
            DataParameters parameters,
            DataStoreProviderServices providerServices
    ) throws InitializeException {
        JDBCHelper helper = new JDBCHelperBase((JDBCConnectionParameters) parameters);
        JDBCStoreProvider provider = helper.createProvider(
                (JDBCStoreParameters) parameters, 
                providerServices
        );
        return provider;
    }
    
    @Override
    public JDBCStoreParameters createParameters() {
        JDBCStoreParameters params = new JDBCStoreParameters(
                NAME + "StoreParameters",
                NAME 
        );
        return params;
    }

    @Override
    public int isOptimalRecoverFeaturesByReference() {
        return NO;
    }

    public int useLocalIndexesCanImprovePerformance() {
        return NO;
    }
    
}
