package org.gvsig.fmap.dal.store.jdbc2.spi.operations;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.store.jdbc2.JDBCHelper;
import org.gvsig.fmap.dal.store.jdbc2.spi.JDBCSQLBuilderBase;
import org.gvsig.fmap.dal.store.jdbc.exception.JDBCSQLException;
import org.gvsig.fmap.dal.store.jdbc2.JDBCUtils;
import org.gvsig.fmap.dal.store.jdbc2.OperationsFactory.TableReference;

public class DropTableOperation extends AbstractConnectionWritableOperation {

    private final TableReference table;

    public DropTableOperation(
            JDBCHelper helper
    ) {
        this(helper, null);
    }

    public DropTableOperation(
            JDBCHelper helper,
            TableReference table
    ) {
        super(helper);
        this.table = table;
    }

    @Override
    public final Object perform(Connection conn) throws DataException {
        this.dropTable(conn, table);
        return true;
    }

    public void dropTable(Connection conn,
            TableReference table) throws DataException {

        JDBCSQLBuilderBase sqlbuilder = this.createSQLBuilder();
        sqlbuilder.drop_table().table()
                .database(this.table.getDatabase())
                .schema(this.table.getSchema())
                .name(this.table.getTable());

        Statement st = null;
        try {
            st = conn.createStatement();
            for (String sql : sqlbuilder.drop_table().toStrings()) {
                JDBCUtils.execute(st, sql);
            }
        } catch (SQLException ex) {
            throw new JDBCSQLException(ex);
        } finally {
            JDBCUtils.closeQuietly(st);
        }
    }
}
