package org.gvsig.fmap.dal.store.jdbc2.spi.operations;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.store.jdbc.JDBCStoreParameters;
import org.gvsig.fmap.dal.store.jdbc2.JDBCHelper;
import org.gvsig.fmap.dal.store.jdbc2.JDBCUtils;

public class ListTablesOperation extends AbstractConnectionOperation {
    
    private static final String METADATA_COLUMN_TABLE_CATALOG = "TABLE_CAT";
    private static final String METADATA_COLUMN_TABLE_SCHEMA = "TABLE_SCHEM";
    private static final String METADATA_COLUMN_TABLE_NAME = "TABLE_NAME";
    
    private boolean informationTables = false;
    private final JDBCStoreParameters baseParameters;
    private final int mode;
    
    public ListTablesOperation(
            JDBCHelper helper,
            int mode,
            JDBCStoreParameters baseParameters,
            boolean informationTables
        ) {
        super(helper);
        this.mode = mode;
        this.informationTables = informationTables;
        this.baseParameters = baseParameters;
    }

    @Override
    public final Object perform(Connection conn) throws DataException {
        List<JDBCStoreParameters> tables = this.listTables(
                conn,
                mode,
                baseParameters,
                informationTables
        );
        return tables;
    }

    public List<JDBCStoreParameters> listTables(
            Connection conn,
            int mode,
            JDBCStoreParameters baseParameters,
            boolean informationTables
    ) {
        ResultSet rs = null;
        List<JDBCStoreParameters> tables = new ArrayList<>();

        try {
            String[] tableTypes = null;
            if (!informationTables) {
                tableTypes = new String[]{"TABLE", "VIEW"};
            }
            DatabaseMetaData metadata = conn.getMetaData();
            rs = metadata.getTables(null, "%", "%", tableTypes);
            while (rs.next()) {
                JDBCStoreParameters params = baseParameters.getCopy();
                params.setCatalog(rs.getString(METADATA_COLUMN_TABLE_CATALOG));
                params.setSchema(rs.getString(METADATA_COLUMN_TABLE_SCHEMA));
                params.setTable(rs.getString(METADATA_COLUMN_TABLE_NAME));
                tables.add(params);
            }
            return tables;
            
        } catch (SQLException ex) {
            throw new RuntimeException("Can't fetch tables information",ex);
            
        } finally {
            JDBCUtils.closeQuietly(rs);
        }

    }
}
