package org.gvsig.fmap.dal.store.jdbc2.spi.operations;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.store.jdbc2.JDBCHelper;
import org.gvsig.fmap.dal.store.jdbc2.spi.JDBCSQLBuilderBase;
import org.gvsig.fmap.dal.store.jdbc.exception.JDBCSQLException;
import org.gvsig.fmap.dal.store.jdbc2.JDBCUtils;
import org.gvsig.fmap.dal.store.jdbc2.OperationsFactory.TableReference;

public class UpdateTableStatisticsOperation extends AbstractConnectionWritableOperation {

    private final TableReference table;

    public UpdateTableStatisticsOperation(
            JDBCHelper helper
        ) {
        this(helper, null);
    }

    public UpdateTableStatisticsOperation(
            JDBCHelper helper,
            TableReference table
        ) {
        super(helper);
        this.table = table;
    }

    @Override
    public final Object perform(Connection conn) throws DataException {
        this.updateTableStatistics(conn, table);
        return true;
    }

    public void updateTableStatistics(
            Connection conn,
            TableReference table
        ) throws DataException {

        JDBCSQLBuilderBase sqlbuilder = this.createSQLBuilder();
        sqlbuilder.update_table_statistics().table()
                .database(this.table.getDatabase())
                .schema(this.table.getSchema())
                .name(this.table.getTable());
        List<String> sqls;
        sqls = sqlbuilder.update_table_statistics().toStrings();

        Statement st = null;
        try {
            st = conn.createStatement();
            for (String sql : sqls) {
                JDBCUtils.execute(st, sql);
            }
        } catch (SQLException ex) {
            throw new JDBCSQLException(ex);
        } finally {
            JDBCUtils.closeQuietly(st);
        }
    }
}
