package org.gvsig.fmap.dal.store.mdb;

import java.io.File;
import java.util.Properties;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.fmap.dal.DataParameters;
import org.gvsig.fmap.dal.exception.ValidateDataParametersException;
import org.gvsig.fmap.dal.resource.db.DBParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCConnectionParameters;

public class MdbConnectionParametersHelper {

    private final JDBCConnectionParameters parameters;

    private static final String DATABASE_FILE = "database_file";

    public MdbConnectionParametersHelper(JDBCConnectionParameters parameters) {
        this.parameters = parameters;
    }

    public String getUrl() {
        String url = (String) this.getDynValue(JDBCConnectionParameters.URL_PARAMTER_NAME);
        if( StringUtils.isEmpty(url) ) {
            url = MdbHelper.getConnectionURL((MdbConnectionParameters) this.parameters);
            this.setDynValue(JDBCConnectionParameters.URL_PARAMTER_NAME, url);
        }
        return url;
    }

    public void validate() throws ValidateDataParametersException {
        if( this.getDynValue(JDBCConnectionParameters.JDBC_DRIVER_CLASS_PARAMTER_NAME) == null ) {
            this.setDynValue(
                JDBCConnectionParameters.JDBC_DRIVER_CLASS_PARAMTER_NAME,
                MdbHelper.MdbJDBCDriver
            );
        }
        if( getFile() == null ) {
            if( this.getDynValue(JDBCConnectionParameters.DBNAME_PARAMTER_NAME) == null ) {
                throw new ValidateDataParametersException();
            } else {
                setFile(new File((String) this.getDynValue(JDBCConnectionParameters.DBNAME_PARAMTER_NAME)));
            }
        } else {
            String dbName = (String) this.getDynValue(JDBCConnectionParameters.DBNAME_PARAMTER_NAME);
            if( StringUtils.isEmpty(dbName) ) {
                dbName = getDbNameFromFile(getFile());
                this.setDynValue(
                    JDBCConnectionParameters.DBNAME_PARAMTER_NAME,
                    dbName.toUpperCase()
                );
            }
        }
        if( this.getDynValue(JDBCConnectionParameters.URL_PARAMTER_NAME) == null ) {
            String url = MdbHelper.getConnectionURL((MdbConnectionParameters) this.parameters);
            this.setDynValue(JDBCConnectionParameters.URL_PARAMTER_NAME, url);
        }
    }

    private Object getDynValue(String name) {
        return ((DataParameters) this.parameters).getDynValue(name);
    }

    private void setDynValue(String name, Object value) {
        ((DataParameters) this.parameters).setDynValue(name, value);
    }

    public File getFile() {
        File f = (File) this.getDynValue(DATABASE_FILE);
        if( this.getDynValue(DBParameters.DBNAME_PARAMTER_NAME) == null
            && f != null ) {
            String dbname = getDbNameFromFile(f);
            this.setDynValue(DBParameters.DBNAME_PARAMTER_NAME, dbname);
        }
        return f;
    }

    public void setFile(File database) {
        this.setDynValue(DATABASE_FILE, database);
        if( this.getDynValue(DBParameters.DBNAME_PARAMTER_NAME) == null
            && database != null ) {
            String dbname = getDbNameFromFile(database);
            this.setDynValue(DBParameters.DBNAME_PARAMTER_NAME, dbname);
        }
    }

    public Properties getProperties() {
        Properties props = new Properties();
        return props;
    }
    
    private String getDbNameFromFile(File f) {
        String fname = f.getName();
        if( fname.toLowerCase().endsWith(".mv.db") ) {
            fname = fname.substring(0, fname.length()-6);
        } else {
            fname = FilenameUtils.getBaseName(fname);
        }
        return fname;
    }

}
