/* gvSIG. Geographic Information System of the Valencian Government
 *
 * Copyright (C) 2007-2008 Infrastructures and Transports Department
 * of the Valencian Government (CIT)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 */
package org.gvsig.fmap.dal.store.mdb;

import org.gvsig.fmap.dal.DALLibrary;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.spi.DataManagerProviderServices;
import org.gvsig.fmap.dal.store.db.DBHelper;
import org.gvsig.fmap.dal.store.jdbc2.JDBCLibrary;
import org.gvsig.fmap.dal.store.jdbc2.spi.JDBCStoreProviderBase;
import org.gvsig.metadata.exceptions.MetadataException;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.LibraryException;

public class MdbLibrary extends AbstractLibrary {

    public static final String NAME = "Mdb";

    @Override
    public void doRegistration() {
        registerAsServiceOf(DALLibrary.class);
        require(JDBCLibrary.class);
    }

    @Override
    protected void doInitialize() throws LibraryException {
    }

    @Override
    protected void doPostInitialize() throws LibraryException {
        LibraryException ex = null;

        DataManagerProviderServices dataman = 
                (DataManagerProviderServices) DALLocator.getDataManager();

        try {
            Class.forName(MdbHelper.MdbJDBCDriver);
        } catch(Throwable th) {
            MdbHelper.logger.warn("Can't load Mdb JDBC Driver.",th);
        }
        
        DBHelper.registerParametersDefinition(
                NAME + "StoreParameters",
                MdbStoreParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );
        DBHelper.registerParametersDefinition(
                NAME + "NewStoreParameters",
                MdbNewStoreParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );
        DBHelper.registerParametersDefinition(
                NAME + "ServerExplorerParameters",
                MdbExplorerParameters.class,
                dataman.getResourceAsStream(this, NAME + "Parameters.xml")
        );
        try {
            DBHelper.registerMetadataDefinition(
                NAME,
                JDBCStoreProviderBase.class,
                dataman.getResourceAsStream(this, NAME + "Metadata.xml")
            );
        } catch (MetadataException e) {
            ex = new LibraryException(this.getClass(), e);
        }

        if (!dataman.getStoreProviderRegister().exits(NAME)) {
            dataman.registerStoreProviderFactory(new MdbStoreProviderFactory());
        }

        if (!dataman.getServerExplorerRegister().exits(NAME)) {
            dataman.registerServerExplorerFactory(new MdbExplorerFactory());
        }
        if (ex != null) {
            throw ex;
        }
    }

}
