
package org.gvsig.fmap.dal.store.mdb;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.WKBReader;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.SQLException;
import net.ucanaccess.converters.TypesMap.AccessType;
import net.ucanaccess.ext.FunctionType; 
import net.ucanaccess.jdbc.UcanaccessConnection;
import org.gvsig.fmap.dal.ExpressionBuilder;


public class SpatialFunctions {

    
    public static void selfRegister(Connection conn) throws SQLException {
        UcanaccessConnection uc=(UcanaccessConnection)conn; 
        uc.addFunctions(SpatialFunctions.class);
    }
    
    private static Geometry toGeometry(Blob blob) {
        try {
            WKBReader reader = new WKBReader();
            Geometry geom = reader.read(blob.getBytes(1, (int) blob.length()));
            return geom;
        } catch (Exception ex) {
            return null;
        }
    }
    
    @FunctionType(functionName="ST_AsText", argumentTypes={AccessType.OLE},returnType=AccessType.TEXT) 
    public static String ST_AsText(Blob arg) {
        Geometry geom = toGeometry(arg);
        if( geom == null ) {
            return null;
        }
        return geom.toText();
    }

    @FunctionType(functionName="ST_AsBinary", argumentTypes={AccessType.OLE},returnType=AccessType.OLE) 
    public static Blob ST_AsBinary(Blob arg) {
        return arg;
    }
    
    @FunctionType(functionName="ST_Envelope", argumentTypes={AccessType.OLE},returnType=AccessType.OLE) 
    public static Blob ST_Envelope(Blob arg) {
        Geometry geom = toGeometry(arg);
        if( geom == null ) {
            return null;
        }
        Geometry env = geom.getEnvelope();
        
        return arg;
    }

    @FunctionType(functionName="ST_Touches", argumentTypes={AccessType.OLE,AccessType.OLE},returnType=AccessType.YESNO) 
    public static boolean ST_Touches(Blob arg1, Blob arg2) {
        Geometry geom1 = toGeometry(arg1);
        if( geom1 == null ) {
            return false;
        }
        Geometry geom2 = toGeometry(arg2);
        if( geom2 == null ) {
            return false;
        }
        return geom1.touches(geom2);
    }

  /*
    public ExpressionBuilder.Function ST_SRID(ExpressionBuilder.Value geom);
    
    public ExpressionBuilder.Function ST_AsEWKB(ExpressionBuilder.Value geom);

    public ExpressionBuilder.Function ST_Envelope(ExpressionBuilder.Value geom);

    public ExpressionBuilder.Function ST_GeomFromText(ExpressionBuilder.Value geom, ExpressionBuilder.Value crs);
    
    public ExpressionBuilder.Function ST_GeomFromWKB(ExpressionBuilder.Value geom, ExpressionBuilder.Value crs);
            
    public ExpressionBuilder.Function ST_GeomFromEWKB(ExpressionBuilder.Value geom, ExpressionBuilder.Value crs);

    public ExpressionBuilder.Function ST_Simplify(ExpressionBuilder.Value geom, ExpressionBuilder.Value tolerance);

    public ExpressionBuilder.Function ST_Equals(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);

    public ExpressionBuilder.Function ST_Intersects(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);

    public ExpressionBuilder.Function ST_Contains(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);

    public ExpressionBuilder.Function ST_Disjoint(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);

    public ExpressionBuilder.Function ST_Crosses(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);

    public ExpressionBuilder.Function ST_IsClosed(ExpressionBuilder.Value geom1);

    public ExpressionBuilder.Function ST_Overlaps(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);

    public ExpressionBuilder.Function ST_Within(ExpressionBuilder.Value geom1, ExpressionBuilder.Value geom2);
  */  
 }
