package org.gvsig.fmap.dal.store.mdb.operations;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.gvsig.fmap.dal.store.jdbc.JDBCStoreParameters;
import org.gvsig.fmap.dal.store.jdbc2.JDBCHelper;
import org.gvsig.fmap.dal.store.jdbc2.JDBCUtils;
import org.gvsig.fmap.dal.store.jdbc2.spi.operations.ListTablesOperation;

public class MdbListTablesOperation extends ListTablesOperation {

    public MdbListTablesOperation(JDBCHelper helper, int mode, JDBCStoreParameters baseParameters, boolean informationTables) {
        super(helper, mode, baseParameters, informationTables);
    }

    @Override
    public List<JDBCStoreParameters> listTables(
            Connection conn,
            int mode,
            JDBCStoreParameters baseParameters,
            boolean informationTables
    ) {
        ResultSet rs = null;
        List<JDBCStoreParameters> tables = new ArrayList<>();

        try {
            String sql = "SELECT TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME FROM INFORMATION_SCHEMA.TABLES where table_type in ('TABLE','VIEW','EXTERNAL')";
            rs = conn.createStatement().executeQuery(sql);
            while (rs.next()) {
                JDBCStoreParameters params = baseParameters.getCopy();
                params.setCatalog(rs.getString("TABLE_CATALOG"));
                params.setSchema(rs.getString("TABLE_SCHEMA"));
                params.setTable(rs.getString("TABLE_NAME"));
                tables.add(params);
            }
            return tables;
            
        } catch (SQLException ex) {
            throw new RuntimeException("Can't fetch tables information",ex);
            
        } finally {
            JDBCUtils.closeQuietly(rs);
        }

    }
}
