/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.feature.spi.simpleprovider;

import org.gvsig.fmap.dal.DataParameters;
import org.gvsig.fmap.dal.DataStoreProvider;
import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.feature.spi.AbstractFeatureStoreProviderFactory;
import org.gvsig.fmap.dal.spi.DataStoreProviderServices;
import org.gvsig.tools.dynobject.DynObject;

public class SimpleSequentialReaderStoreProviderFactory 
        extends AbstractFeatureStoreProviderFactory 
    {

    private final SimpleSequentialReaderFactory readerFactory;
    
    protected SimpleSequentialReaderStoreProviderFactory(SimpleSequentialReaderFactory readerFactory) {
        super(readerFactory.getName(), readerFactory.getDescription());
        this.readerFactory = readerFactory;
    }
    
    public SimpleSequentialReaderFactory getReaderFactory() {
        return this.readerFactory;
    }

    @Override
    public DynObject createParameters() {
        return new SimpleSequentialReaderStoreParameters(this.readerFactory);
    }

    @Override
    public DataStoreProvider createProvider(
            DataParameters parameters, 
            DataStoreProviderServices providerServices) throws InitializeException {
        return new SimpleSequentialReaderStoreProvider(
                this.readerFactory,
                (SimpleSequentialReaderStoreParameters) parameters,
                providerServices
        );
    }

    @Override
    public int allowCreate() {
        return NO;
    }

    @Override
    public int allowWrite() {
        return NO;
    }

    @Override
    public int allowRead() {
        return YES;
    }

    @Override
    public int hasRasterSupport() {
        return NO;
    }

    @Override
    public int hasTabularSupport() {
        return YES;
    }

    @Override
    public int hasVectorialSupport() {
        return YES;
    }

    @Override
    public int allowMultipleGeometryTypes() {
        return YES;
    }

    @Override
    public int allowEditableFeatureType() {
        return NO;
    }

}
