package org.gvsig.fmap.dal.serverexplorer.filesystem.impl;

import org.gvsig.fmap.dal.serverexplorer.filesystem.spi.FilesystemProviderServices;

/**
 * Abstract implementation of FilesystemServerExplorerProviderFactoryParameters
 * @author dmartinezizquierdo
 *
 */
public abstract class AbstractFilesystemProviderServices implements FilesystemProviderServices{

    String name=null;
    String description=null;
    int allowCreate=FilesystemProviderServices.UNKNOWN;
    int mode=-1;

    /**
     * Returns the name of the provider
     *
     * @return name of the provider
     */
    public String getName(){
        return name;
    }

    /**
     * Return a short descripion about the provider
     *
     * @return description about the provider
     */
    public String getDescription(){
        return description;
    }

    /**
     * The provider can create new stores.
     *
     * @return YES if has creation support
     */
    public int allowCreate(){
        return allowCreate;
    }

    /**
     * Returns the mode supported
     * @return DataServerExplorer.MODE
     */
    public int getMode(){
        return mode;
    }

    /**
     * Return true if any mode in the parameter is supported.
     *
     * @param mode
     * @return boolean
     */
    public boolean isMode(int mode){
        return this.mode==mode;
    }

}
