package org.gvsig.fmap.dal.serverexplorer.filesystem.impl;

import java.io.File;

import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.serverexplorer.filesystem.spi.FilesystemProviderServices;
import org.gvsig.fmap.dal.serverexplorer.filesystem.spi.FilesystemServerExplorerProvider;
import org.gvsig.fmap.dal.serverexplorer.filesystem.spi.FilesystemServerExplorerProviderFactory;

/**
 * Class to wrap a FilesystemServerExplorerProvider into a Factory
 * @author dmartinezizquierdo
 *
 */
public class FilesystemServerExplorerProviderFactoryWrapper
    implements FilesystemServerExplorerProviderFactory{

    String name;
    String description;
    FilesystemServerExplorerProvider wrappedFilesystemServerExplorerProvider;

    FilesystemServerExplorerProviderFactoryWrapper(String name, String description,
        Class filesystemServerExplorerProviderClass){
        this.name=name;
        this.description=description;
        try {
            this.wrappedFilesystemServerExplorerProvider=(FilesystemServerExplorerProvider) filesystemServerExplorerProviderClass.newInstance();
        } catch (Exception e) {
            throw new RuntimeException(e);// FIXME !!!
        }
    }

    @Override
    public FilesystemServerExplorerProvider createProvider()
        throws InitializeException {
        return wrappedFilesystemServerExplorerProvider;
    }



    @Override
    public boolean accept(File pathname) {
        return wrappedFilesystemServerExplorerProvider.accept(pathname);
    }



    @Override
    public String getName() {
        return name;
    }



    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public int allowCreate() {
        if (wrappedFilesystemServerExplorerProvider.canCreate()){
            return FilesystemProviderServices.YES;
        }else{
            return FilesystemProviderServices.NO;
        }
    }

    @Override
    public int getMode() {
        return wrappedFilesystemServerExplorerProvider.getMode();
    }

    @Override
    public boolean isMode(int mode) {
        return wrappedFilesystemServerExplorerProvider.isMode(mode);
    }


}
