
package org.gvsig.fmap.dal.feature.impl;

import java.util.Iterator;
import java.util.List;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureReference;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.spi.FeatureProvider;
import org.gvsig.fmap.dal.feature.spi.FeatureStoreProvider.FeatureTypeChanged;


public interface EditingManager {
    /*
    Este interface, con el tiempo deberia ir al SPI.
    
    Tendria que sustituir a:
    - FeatureManager
    - FeatureTypeManager
    - SpatialManager
    como mecanismos para guardar los datos mientras se esta en edicion.
    
    Habria que eliminar estos managers del DefaultFeatureStore y sustituirlos
    por este.
    
    En una primera implementacion, se mantendrian estos managers como parte interna
    de la implementacion del MemoryEditingManager.
    
    */
 
    public void dispose();
    
    public void delete(Feature feature);

    public void delete(FeatureReference featureReference);
    
    public void add(Feature newFeature);
    
    public int update(Feature newFeature, Feature oldFeature);
        
    public int update(FeatureType type, FeatureType oldType);
    
    public void restore(FeatureReference featureReference);

    public void restore(FeatureReference featureReference, int code);
    
    public void restore(String featureTypeId);

    public void restore(String featureTypeId, int code);
    
    public boolean isDeleted(Feature feature);

    public boolean isDeleted(FeatureReference featureReference);    
    
    public boolean isDeleted(FeatureType type);
    
    public boolean isDeleted(String featureTypeId);
    
    public boolean isSelectionCompromised();
    
    public Iterator<FeatureProvider> getDeletedFeatures();
    public Iterator<FeatureProvider> getInsertedFeatures();
    public Iterator<FeatureProvider> getUpdatedFeatures();
    
    public List<FeatureTypeChanged> getFeatureTypesChanged() throws DataException;
    
    public FeatureType getFeatureType(String id) throws DataException ;
    
    public Feature getFeature(FeatureReference reference, FeatureStore store, FeatureType type) throws DataException;


    /*
it = featureTypeManager.newsIterator();

position =  expansionManager.update(type, oldType);

expansionManager.restore(oldType.getId(),position);

    spatialManager.cancelModifies();
*/
}
