package org.gvsig.fmap.dal.feature.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.collections4.queue.CircularFifoQueue;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;

/**
 *
 * @author jjdelcerro
 */
public class RecentUsedsAttributesImpl {

    private final Map<String,CircularFifoQueue<String>> recentsForStore;
    
    public RecentUsedsAttributesImpl() {
        this.recentsForStore = new HashMap<>();
    }
    
    public Collection<String> getNames(FeatureType featureType) {
        FeatureStore store = featureType.getStore();
        if( store == null ) {
            return Collections.EMPTY_LIST;
        }
        CircularFifoQueue<String> recents = this.recentsForStore.get(store.getFullName());
        if( recents==null ) {
            return Collections.EMPTY_LIST;
        }
        return recents;
    }
    
    public List<FeatureAttributeDescriptor> getAttributes(FeatureType featureType) {
        List<FeatureAttributeDescriptor> r = new ArrayList<>();
        for (String name : this.getNames(featureType)) {
            r.add(featureType.getAttributeDescriptor(name));
        }
        return r;
    }

    public void add(FeatureAttributeDescriptor attribute) {
        this.add(attribute.getFeatureType(), attribute.getName());
    }
    
    public void add(FeatureType featureType, FeatureAttributeDescriptor attribute) {
        this.add(featureType, attribute.getName());
    }
    
    public void add(FeatureType featureType, String name) {
        String storeFullName = featureType.getStore().getFullName();
        CircularFifoQueue<String> recents = this.recentsForStore.get(storeFullName);
        if( recents==null ) {
            recents = new CircularFifoQueue(this.getMaxSize());
            this.recentsForStore.put(storeFullName, recents);
        }
        recents.remove(name);
        recents.add(name);
    }

    public int getMaxSize() {
        return 6;
    }
}
