/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.feature.impl.indexes.geometry;

import java.util.List;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.exception.FeatureIndexException;
import org.gvsig.fmap.dal.feature.spi.FeatureReferenceProviderServices;
import org.gvsig.fmap.dal.feature.spi.index.AbstractFeatureIndexProvider;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import java.util.Iterator;
import org.apache.commons.collections4.IteratorUtils;
import org.gvsig.fmap.geom.SpatialIndex;

public class GeometryIndexProvider extends AbstractFeatureIndexProvider {
	
	private SpatialIndex index = null;
		
	public GeometryIndexProvider() {
		
	}
	
    @Override
    public void initialize() {
    }
    
    protected Envelope getEnvelope(Object value) {
        Envelope env = null;

        if (value instanceof Envelope) {
            env = (Envelope) value;
        } else
            if (value instanceof Geometry) {
                env = ((Geometry) value).getEnvelope();
            }
        return env;
    }
        
    @Override
    public void insert(Object value, FeatureReferenceProviderServices fref) {
        this.index.insert(getEnvelope(value), fref);
    }

    @Override
    public void delete(Object value, FeatureReferenceProviderServices fref) {
        this.index.remove(getEnvelope(value), fref);
    }

    @Override
    public List match(Object value) throws FeatureIndexException {
        return this.index.queryAsList(getEnvelope(value));
    }

    @Override
    public List nearest(int count, Object value) {
        if( !this.index.getFactory().isNearestQuerySupported() ) {
            throw new UnsupportedOperationException();
        }
        if (value == null) {
            throw new IllegalArgumentException("value is null");
        }

        Iterator x = this.index.queryNearest(getEnvelope(value), count);
        return IteratorUtils.toList(x);
    }

    @Override
    public boolean isMatchSupported() {
        return true;
    }

    @Override
    public boolean isNearestSupported() {
        return this.index.getFactory().isNearestQuerySupported();
    }

    @Override
    public boolean isNearestToleranceSupported() {
        return false;
    }

    @Override
    public boolean isRangeSupported() {
        return false;
    }

    @Override
    public List nearest(int count, Object value, Object tolerance)
        throws FeatureIndexException {
        throw new UnsupportedOperationException();
    }

    @Override
    public List range(Object value1, Object value2)
        throws FeatureIndexException {
        throw new UnsupportedOperationException();
    }

    @Override
    public void clear() throws DataException {
        this.index.removeAll();
    }
 }
