/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.feature.impl.indexes.memorybasictypes;

import java.util.List;
import java.util.TreeMap;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.exception.FeatureIndexException;
import org.gvsig.fmap.dal.feature.spi.FeatureReferenceProviderServices;
import org.gvsig.fmap.dal.feature.spi.index.AbstractFeatureIndexProvider;

public class MemoryBasicTypesIndexProvider<T> extends AbstractFeatureIndexProvider {

    private TreeMap<T,ListOfLong> index = null;

    public MemoryBasicTypesIndexProvider() {

    }

    public void initialize() {
        try {
            this.index = new TreeMap<>();
        } catch (Exception e) {
            throw new RuntimeException();
        }
    }

    public void delete(Object key_o, FeatureReferenceProviderServices fref) {
        T key_s = (T) key_o;
        long oid = (Long) fref.getOID();
        ListOfLong items = this.index.get(key_s);
        if( items == null ) {
            return;
        }
        items.remove(oid);
        if( items.isEmpty() ) {
            this.index.remove(key_s);
        }
    }

    public void insert(Object key_o, FeatureReferenceProviderServices fref) {
        if (key_o == null) {
            return;
        }
        T key_s = (T) key_o;
        long oid = (Long) fref.getOID();
        ListOfLong items = this.index.get(key_s);
        if( items == null ) {
            items = new ArrayListOfLong();
            this.index.put(key_s, items);
        }
        items.add(oid);
    }

    public List match(Object key_o) throws FeatureIndexException {
        T key_s = (T) key_o;
        if( !this.index.containsKey(key_s) ) {
            return null;
        }
        ListOfLong items = this.index.get(key_s);
        if( items == null ) {
            return null;
        }
        return items;
    }

    public List match(Object min, Object max) {
        throw new UnsupportedOperationException(
                "Can't perform this kind of search.");
    }

    public List nearest(int count, Object value) throws FeatureIndexException {
        throw new UnsupportedOperationException(
                "Can't perform this kind of search.");
    }

    public boolean isMatchSupported() {
        return true;
    }

    public boolean isNearestSupported() {
        return false;
    }

    public boolean isNearestToleranceSupported() {
        return false;
    }

    public boolean isRangeSupported() {
        return false;
    }

    public List nearest(int count, Object value, Object tolerance)
            throws FeatureIndexException {
        throw new UnsupportedOperationException();
    }

    public List range(Object value1, Object value2)
            throws FeatureIndexException {
        throw new UnsupportedOperationException();
    }

    public void clear() throws DataException {
        this.index.clear();
    }
}
