/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.gvsig.fmap.dal.DALLibrary;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.fmap.dal.complements.Search;
import org.gvsig.fmap.dal.feature.AbstractFeatureStoreTransform;
import org.gvsig.fmap.dal.feature.impl.DALFile;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureQuery;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureReference;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureReferenceSelection;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureSelection;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureStore;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureStoreTransforms;
import org.gvsig.fmap.dal.feature.impl.DefaultFeatureType;
import org.gvsig.fmap.dal.feature.impl.DefaultForeingKey;
import org.gvsig.fmap.dal.feature.impl.indexes.jsir.JSIRSpatialIndexProviderFactory;
import org.gvsig.fmap.dal.feature.impl.indexes.memoryspatial.MemorySpatialIndexProviderFactory;
import org.gvsig.fmap.dal.feature.impl.dynobjectutils.ComputeAvailableValuesFromTable;
import org.gvsig.fmap.dal.feature.impl.dynobjectutils.DumbFeatureTypeDefinitionsManager;
import org.gvsig.fmap.dal.feature.impl.dynobjectutils.relations.DALLink_getRelatedFeature;
import org.gvsig.fmap.dal.feature.impl.dynobjectutils.relations.DALLink_getRelatedFeatures;
import org.gvsig.fmap.dal.feature.impl.indexes.memorybasictypes.MemoryDoubleIndexProviderFactory;
import org.gvsig.fmap.dal.feature.impl.indexes.memorybasictypes.MemoryFloatIndexProviderFactory;
import org.gvsig.fmap.dal.feature.impl.indexes.memorybasictypes.MemoryIntIndexProviderFactory;
import org.gvsig.fmap.dal.feature.impl.indexes.memorybasictypes.MemoryLongIndexProviderFactory;
import org.gvsig.fmap.dal.feature.impl.indexes.memorybasictypes.MemoryStringIndexProviderFactory;
import org.gvsig.fmap.dal.feature.spi.memory.MemoryResource;
import org.gvsig.fmap.dal.feature.spi.memory.MemoryResourceParameters;
import org.gvsig.fmap.dal.impl.expressionevaluator.DALSymbolTable;
import org.gvsig.fmap.dal.impl.expressionevaluator.DefaultFeatureAttributeEmulatorExpression;
import org.gvsig.fmap.dal.impl.expressionevaluator.DefaultFeatureSymbolTable;
import org.gvsig.fmap.dal.resource.impl.DefaultResourceManager;
import org.gvsig.fmap.dal.resource.spi.MultiResource;
import org.gvsig.fmap.dal.resource.spi.MultiResourceParameters;
import org.gvsig.fmap.dal.resource.spi.ResourceManagerProviderServices;
import org.gvsig.fmap.dal.spi.DataStoreProviderServices;
import org.gvsig.i18n.Messages;
import org.gvsig.metadata.exceptions.MetadataException;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynObjectManager;
import org.gvsig.tools.dynobject.DynObjectValueItem;
import org.gvsig.tools.identitymanagement.SimpleIdentityManager;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.LibraryException;
import org.gvsig.tools.locator.ReferenceNotRegisteredException;
import org.xmlpull.v1.XmlPullParserException;

/**
 * Initialize the implementation of DAL.
 * 
 * @author gvSIG Team
 * @version $Id$
 * 
 */
public class DALDefaultImplLibrary extends AbstractLibrary {

    @Override
    public void doRegistration() {
        registerAsImplementationOf(DALLibrary.class);
    }

    @Override
    protected void doInitialize() throws LibraryException {
        DALLocator.registerDefaultDataManager(DefaultDataManager.class);
        DALLocator.registerResourceManager(DefaultResourceManager.class);
        DALLocator.registerDefaultFeatureTypeDefinitionsManager(DumbFeatureTypeDefinitionsManager.class);
    }

    @Override
    protected void doPostInitialize() throws LibraryException {
        List exs = new ArrayList();

        Messages.addResourceFamily("org.gvsig.fmap.dal.i18n.text",
            DALDefaultImplLibrary.class.getClassLoader(),
            DALDefaultImplLibrary.class.getClass().getName());

        try {
            registerBaseNewProvidersParametersDefinition();
        } catch (XmlPullParserException e) {
            exs.add(e);
        } catch (IOException e) {
            exs.add(e);
        }

        SimpleIdentityManager identityManager = ToolsLocator.getIdentityManager();
        identityManager.registerAction(DataManager.CREATE_STORE_AUTHORIZATION);
        identityManager.registerAction(DataManager.READ_STORE_AUTHORIZATION);
        identityManager.registerAction(DataManager.WRITE_STORE_AUTHORIZATION);
        
        
        DefaultFeatureQuery.registerPersistent();
        DefaultFeatureReference.registerPersistent();
        DefaultFeatureStoreTransforms.registerPersistent();
        AbstractFeatureStoreTransform.registerPersistent();
        DefaultFeatureReferenceSelection.registerPersistent();
        DefaultFeatureSelection.registerPersistent();
        DefaultForeingKey.registerPersistenceDefinition();

        DefaultFeatureStore.registerPersistenceDefinition();
        DefaultDataServerExplorerPool.registerPersistenceDefinition();
        DefaultDataServerExplorerPoolEntry.registerPersistenceDefinition();
        
        DALFile.registerPersistenceDefinition();
        DefaultFeatureAttributeEmulatorExpression.registerPersistenceDefinition();

        org.gvsig.fmap.dal.impl.dataprofile.DataProfileText.selfRegister();
        
        org.gvsig.fmap.dal.complements.relatedfeatures.RelatedFeaturesFactory.selfRegister();
        org.gvsig.fmap.dal.complements.search.SearchFactory.selfRegister();
        
        try {
            DefaultFeatureStore.registerMetadataDefinition();
        } catch (MetadataException e) {
            exs.add(e);
        }

        // DefaultFeatureAttributeDescriptor.registerPersistent();

        //Register the FeatureAttributeTransform
        DefaultDataManager dataManager = (DefaultDataManager) DALLocator.getDataManager();
        if (dataManager == null) {
            throw new ReferenceNotRegisteredException(
                    DALLocator.DATA_MANAGER_NAME, DALLocator.getInstance());
        }
        
        MemorySpatialIndexProviderFactory.selfRegister();
        JSIRSpatialIndexProviderFactory.selfRegister();
        MemoryStringIndexProviderFactory.selfRegister();
        MemoryIntIndexProviderFactory.selfRegister();
        MemoryLongIndexProviderFactory.selfRegister();
        MemoryFloatIndexProviderFactory.selfRegister();
        MemoryDoubleIndexProviderFactory.selfRegister();

        ResourceManagerProviderServices resourceManager =
                (ResourceManagerProviderServices) DALLocator.getResourceManager();
        if (resourceManager == null) {
            throw new ReferenceNotRegisteredException(
                DALLocator.RESOURCE_MANAGER_NAME, DALLocator.getInstance());
        }

        if (!resourceManager.getResourceProviders().contains(
            MultiResource.TYPE_NAME)) {
            resourceManager.register(MultiResource.TYPE_NAME,
                MultiResource.DESCRIPTION, MultiResource.class,
                MultiResourceParameters.class);
        }

        if (!resourceManager.getResourceProviders().contains(
            MemoryResource.NAME)) {
            resourceManager.register(MemoryResource.NAME,
                MemoryResource.DESCRIPTION, MemoryResource.class,
                MemoryResourceParameters.class);
        }

        DynObjectManager dynObjectmanager = ToolsLocator.getDynObjectManager();
        dynObjectmanager.registerDynMethod(new DALLink_getRelatedFeatures());
        dynObjectmanager.registerDynMethod(new DALLink_getRelatedFeature());
        dynObjectmanager.registerDynMethod(new ComputeAvailableValuesFromTable("getAvailableValuesFromTable", "Retrieve available values from the elements of a table."));

        registerTags();
        DALSymbolTable.selfRegister();
        DefaultFeatureSymbolTable.selfRegister();
        DefaultFeatureAttributeDescriptor.registerPersistenceDefinition();
        DefaultFeatureType.registerPersistenceDefinition();
        
        if (exs.size() > 0) {
            throw new LibraryException(this.getClass(), exs);
        }
    }

    private void registerBaseNewProvidersParametersDefinition()
        throws XmlPullParserException, IOException {
        DynObjectManager manager = ToolsLocator.getDynObjectManager();
        if (manager
            .get(
                "DAL",
                DataStoreProviderServices.BASE_NEWPARAMETERS_PROVIDER_DEFINITION_NAME) == null) {
            Map definitions =
                manager
                    .importDynClassDefinitions(
                        DataStoreProviderServices.class
                            .getResourceAsStream("ProviderParametersDefinition.xml"),
                        DataStoreProviderServices.class.getClassLoader());
            manager
                .add((DynClass) definitions
                    .get(DataStoreProviderServices.BASE_NEWPARAMETERS_PROVIDER_DEFINITION_FULLNAME));
            manager
                .add((DynClass) definitions
                    .get(DataStoreProviderServices.BASE_PARAMETERS_PROVIDER_DEFINITION_FULLNAME));
        }
    }

    private void registerTags() {
        DynObjectManager dynObjectManager = ToolsLocator.getDynObjectManager();
        dynObjectManager.registerTag(
                DataManager.DAL_USE_LABELS,
                "Indicates the preference when using labels over names."
        ).setAvailableValues(
                new DynObjectValueItem[] {
                    new DynObjectValueItem(0, "Use names"),
                    new DynObjectValueItem(1, "Use labels"),
                    new DynObjectValueItem(2, "Use labels and names"),
                }
        ).setType(DataTypes.INT);
        
    }
}       
