
package org.gvsig.fmap.dal.impl;

import org.apache.commons.lang3.reflect.ConstructorUtils;
import org.gvsig.fmap.dal.DataServerExplorer;
import org.gvsig.fmap.dal.DataServerExplorerParameters;
import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.spi.AbstractDataServerExplorerFactory;
import org.gvsig.fmap.dal.spi.DataServerExplorerProviderServices;
import org.gvsig.tools.dynobject.DynObject;


public class DataServerExplorerToDataExplorerFactoryWrapper extends AbstractDataServerExplorerFactory {

    private final Class explorerClass;
    private final Class parametersClass;

    public DataServerExplorerToDataExplorerFactoryWrapper(
            String name, 
            String description, 
            Class explorerClass, 
            Class parametersClass
        ) {
        super(name, description);
        this.explorerClass = explorerClass;
        this.parametersClass = parametersClass;
        
    }

    @Override
    public DataServerExplorer create(DataServerExplorerParameters parameters, DataServerExplorerProviderServices providerServices) throws InitializeException {
        try {
            DataServerExplorer explorer = (DataServerExplorer) ConstructorUtils.invokeConstructor(
                    explorerClass, parameters, providerServices
            );
            return explorer;
        } catch (Exception ex) {
            throw new InitializeException(ex);
        }
    }

    @Override
    public DynObject createParameters() {
        try {
                return (DynObject) parametersClass.newInstance();
        } catch (Exception e) {
                throw new RuntimeException(e);
        }        
    }

}
