package org.gvsig.fmap.dal.impl.expressionevaluator;

import java.util.List;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.expressionevaluator.spi.AbstractSymbolTable;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureStore;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class DALSymbolTable extends AbstractSymbolTable {

    public static final String FUNCTION_FEATURES = "FEATURES";

    public static class FeaturesFunction extends AbstractFunction {

        public FeaturesFunction() {
            super(
                    "Data access",
                    FUNCTION_FEATURES,
                    Range.between(1, 3),
                    "Return a list of the features selecteds with filter in the store.",
                    FUNCTION_FEATURES + "({{store_name}}, filter, order)",
                    new String[]{
                        "store - data store name to be used",
                        "where - Optional. String value with a filter expression",
                        "order - Optional. String value with the order. must be a string with the names of separate fields with commas"
                    },
                    "LIST"
            );
        }

        @Override
        public Object call(Interpreter interpreter, Object[] args) throws Exception {
            String storeName = getStr(args, 0);
            String where = null;
            String order = null;
            switch (args.length) {
                case 2:
                    where = getStr(args, 1);
                    break;
                case 3:
                    where = getStr(args, 1);
                    order = getStr(args, 2);
                    break;
            }
            try {
                DataStore store = this.getStore(storeName);
                if (store == null || !(store instanceof FeatureStore)) {
                    throw new ExpressionRuntimeException("Cant locate the store '" + storeName + "' in function '" + FUNCTION_FEATURES + "'.");
                }
                if (!(store instanceof FeatureStore)) {
                    throw new ExpressionRuntimeException("The store'" + storeName + "' is not valid for function '" + FUNCTION_FEATURES + "', a FeatureStore is required.");
                }
                FeatureStore featureStore = (FeatureStore) store;
                List<Feature> features;
                if (where == null && order == null) {
                    features = featureStore.getFeatures();
                } else {
                    FeatureQuery query = featureStore.createFeatureQuery();
                    if (where != null) {
                        query.addFilter(where);
                    }
                    if (order != null) {
                        query.getOrder().add(order);
                    }
                    query.retrievesAllAttributes();
                    features = featureStore.getFeatures(query);
                }
                return features;
            } catch (ExpressionRuntimeException ex) {
                throw ex;
            } catch (Exception ex) {
                throw new ExpressionRuntimeException("Problems calling '" + FUNCTION_FEATURES + "' function", ex);
            }
        }

        protected DataStore getStore(String storeName) {
            DataManager dataManager = DALLocator.getDataManager();
            DataStore store = dataManager.getStoresRepository().getStore(storeName);
            return store;
        }
    }

    @SuppressWarnings("OverridableMethodCallInConstructor")
    public DALSymbolTable() {
        super(DataManager.DAL_SYMBOL_TABLE);

        this.addFunction(new FeaturesFunction());
    }

    public static void selfRegister() {
        ExpressionEvaluatorManager manager = ExpressionEvaluatorLocator.getManager();
        manager.registerSymbolTable(new DALSymbolTableFactory());
    }

}
