package org.gvsig.fmap.dal.raster.impl;

import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataParameters;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.DataStoreFactory;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.DataStoreProvider;
import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.impl.AbstractDataStoreFactory;
import org.gvsig.fmap.dal.spi.DataStoreInitializer;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.service.spi.Services;

public class RasterStoreOldFactory extends AbstractDataStoreFactory {
    private final DataStoreFactory oldfactory;

    public static final String NAME = "RasterStore";
    
    public RasterStoreOldFactory(String name, Class storeFactoryClass) {
        super(name,"");
        try {
            this.oldfactory = (DataStoreFactory) storeFactoryClass.newInstance();
        } catch (Exception ex) {
            throw new RuntimeException("Can't create factory (for old rasters)", ex);
        }
    }
    
    @Override
    public DynObject createParameters() {
        return null;
    }

    @Override
    public DataStore create(DynObject parameters, Services services) {
        try {
            DataStoreParameters params = (DataStoreParameters) parameters;
            DataManager dataManager = (DataManager) services;
            this.oldfactory.setParameters(params);
            DataStoreInitializer store = (DataStoreInitializer) oldfactory.createStore();
            store.intializePhase1(dataManager, params);
            return (DataStore) store;
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public boolean canUse(DataParameters parameters) {
        if( ! (parameters instanceof DataStoreParameters) ) {
            return false;
        }
        
        boolean use = ((DataStoreParameters)parameters).getDataStoreName().equalsIgnoreCase(this.getName());
        return use;
    }
    
    @Override
    public void setProvider(DataStore store, DataStoreProvider provider) {
        try {
            ((DataStoreInitializer)store).intializePhase2((org.gvsig.fmap.dal.spi.DataStoreProvider) provider);
        } catch (InitializeException ex) {
            throw new RuntimeException("Can't set the provider of the store.", ex);
        }
    }
               
}
