package org.gvsig.fmap.dal.spi;

import org.gvsig.fmap.dal.BaseStoresRepository;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.StoresRepository;
import org.gvsig.tools.dispose.impl.AbstractDisposable;
import org.gvsig.tools.resourcesstorage.ResourcesStorage;
import org.gvsig.tools.util.UnmodifiableBasicMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public abstract class AbstractDataStore 
        extends AbstractDisposable 
        implements DataStore 
    {
    
    protected static final Logger LOGGER = LoggerFactory.getLogger(AbstractDataStore.class);

    protected AbstractDataStore() {
        
    }

    protected abstract DataManager getDataManager();
    
    @Override
    public ResourcesStorage getResourcesStorage() {
        try {
            return this.getExplorer().getResourcesStorage(this);
        } catch (Exception ex) {
            LOGGER.warn("Can't create resources storage",ex);
            return null;
        }
    }

    @Override
    public StoresRepository getStoresRepository() {
        final StoresRepository mainRepository = this.getDataManager().getStoresRepository();
        StoresRepository repository = new BaseStoresRepository(this.getName()) {
            
            @Override
            protected DataStoreParameters getMyParameters(String name) {
                // Uff, esta implementacion no es nada buena
                DataStore store = getChildren().get(name);
                if( store==null ) {
                    return null;
                }
                return store.getParameters();
            }

            @Override
            protected boolean isEmptyMyRepository() {
                UnmodifiableBasicMap<String, DataStore> children = getChildren();
                return children==null || children.isEmpty();
            }

            @Override
            protected int getMySize() {
                UnmodifiableBasicMap<String, DataStore> children = getChildren();
                if( children==null ) {
                    return 0;
                }
                return children.size();
            }

            @Override
            public void add(String name, DataStoreParameters parameters) {
                mainRepository.add(name, parameters);
            }

            @Override
            public void remove(String name) {
                mainRepository.remove(name);
            }

        };
        return repository;
    }
    
    @Override
    public UnmodifiableBasicMap<String,DataStore> getChildren() {
            return UnmodifiableBasicMap.EMPTY_UNMODIFIABLEBASICMAP;
    }
    
    
}
