/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.spi;

import java.io.InputStream;
import java.net.URL;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataServerExplorerFactory;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.DataStoreFactory_v2_4;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.DataStoreProviderFactory;
import org.gvsig.fmap.dal.exception.InitializeException;
import org.gvsig.fmap.dal.exception.ProviderNotRegisteredException;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.spi.index.FeatureIndexProviderServices;
import org.gvsig.tools.dynobject.DynObjectValueItem;

/**
 * Inteface of DataManager of Services for data providers
 *
 */
public interface DataManagerProviderServices extends DataManager {

    /**
     *
     * Creates a new instance of the provider associated to the passed
     * parameters.
     *
     * @param providerServices
     * @param parameters
     * @return
     * @throws InitializeException
     * @throws ProviderNotRegisteredException
     */
    public DataStoreProvider createProvider(DataStoreProviderServices providerServices, DataStoreParameters parameters) throws InitializeException, ProviderNotRegisteredException;

    public void registerStoreProviderFactory(DataStoreProviderFactory factory);

    /**
     * Este metodo es temporal hasta que se integre el nuevo raster en el core
     * de gvSIG.
     *
     * @param rasterStoreClass
     */
    public void registerDefaultRasterStore(Class rasterStoreClass);

    /**
     * Returns a DataIndexProvider compatible with the attribute data type.
     *
     * @param name
     * @param store associated FeatureStore
     * @param type associated FeatureType
     * @param indexName
     * @param attr associated FeatureAttributeDescriptor
     * @return empty DataIndexProvider, initialized and ready to use
     * @throws org.gvsig.fmap.dal.exception.InitializeException
     * @throws org.gvsig.fmap.dal.exception.ProviderNotRegisteredException
     */
    public FeatureIndexProviderServices createFeatureIndexProvider(String name, FeatureStore store, FeatureType type,
            String indexName,
            FeatureAttributeDescriptor attr) throws InitializeException,
            ProviderNotRegisteredException;

    public void registerStoreFactory(DataStoreFactory_v2_4 factory);

    public void registerServerExplorerFactory(DataServerExplorerFactory factory);

    /**
     * @param name
     * @param dataSourceClass class of provider
     * @param parametersClass parameters class of provider
     * @deprecated use registerServerExplorerFactory
     */
    public void registerExplorerProvider(String name,
            Class dataSourceClass, Class parametersClass);

    /**
     * Registers a store factory.
     *
     * @param name
     * @param storeFactoryClass
     * @deprecated use registerStoreFactory
     */
    public void registerStoreFactory(String name,
            Class storeFactoryClass);

    /**
     * Registers a store .
     *
     * @param name
     * @param dataStoreProviderClass
     * @param parametersClass
     * @deprecated use registerStoreFactory
     */
    public void registerStoreProvider(String name,
            Class dataStoreProviderClass,
            Class parametersClass);

    /**
     * Return the URL associated to the indicated resource. If the data manager
     * has set a resource loader, use this to search the resource. If this don't
     * contain the indicated resource or the data manager has not set a resource
     * loader, search the resource in the class loader of the object
     * "resourceLoader" indicated as parameter. This is usefull to load the
     * resource files associateds to the xml of parameter and metadad
     * description.
     *
     * @param resourcesLoader
     * @param name
     * @return
     */
    public URL getResource(Object resourcesLoader, String name);

    /**
     * Return the InputStream associated to the indicated resource. If the data
     * manager has set a resource loader, use this to search the resource. If
     * this don't contain the indicated resource or the data manager has not set
     * a resource loader, search the resource in the class loader of the object
     * "resourceLoader" indicated as parameter. This is usefull to load the
     * resource files associateds to the xml of parameter and metadad
     * description.
     *
     * @param resourcesLoader
     * @param name
     * @return
     */
    public InputStream getResourceAsStream(Object resourcesLoader, String name);

    public DataStore openStore(DataStoreParameters parameters, DataStoreProvider provider);

    public DynObjectValueItem[] getAvailableValues(FeatureStore myStore, FeatureAttributeDescriptor descriptor);

}
