/* gvSIG. Desktop Geographic Information System.
 *
 * Copyright  2007-2014 gvSIG Association
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.swing;

import java.awt.Component;
import java.util.Collection;
import java.util.List;
import java.util.function.Predicate;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.table.TableModel;
import org.cresques.cts.IProjection;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.JExpressionBuilder;
import org.gvsig.featureform.swing.CreateJFeatureFormException;
import org.gvsig.featureform.swing.JFeatureForm;
import org.gvsig.featureform.swing.JFeaturesForm;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.paging.FeaturePagingHelper;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALActionFactory.DALActionContext;
import org.gvsig.fmap.dal.swing.expressionevaluator.FeatureStoreElement;
import org.gvsig.fmap.dal.swing.featuretype.AttributeDescriptorPickerController;
import org.gvsig.fmap.dal.swing.featuretype.FeatureTypeAttributePanel;
import org.gvsig.fmap.dal.swing.featuretype.FeatureTypePanel;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel;
import org.gvsig.fmap.dal.swing.searchpanel.FeatureStoreSearchPanel;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.swing.api.pickercontroller.PickerController;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionDialog;

/**
 * @author fdiaz
 *
 */
public interface DataSwingManager {
    public static final String  FEATURE_STORE_EXPRESSION_ELEMENT = "DAL.FeatureStoreExpressionElement";

    public JFeaturesForm createJFeaturesForm(FeatureStore store) throws CreateJFeatureFormException;

    public JFeatureForm createJFeatureForm(FeatureStore store) throws CreateJFeatureFormException;

    public JFeatureForm createJFeatureForm(Feature feature) throws CreateJFeatureFormException;

    public JDBCConnectionPanel createJDBCConnectionPanel();
    
    public JDBCConnectionDialog createJDBCConectionDialog();

    public JExpressionBuilder createQueryFilterExpresion(FeatureStore store);

    public FeatureTableModel createFeatureTableModel(FeatureStore featureStore, FeatureQuery featureQuery);

    public FeatureTableModel createFeatureTableModel(FeaturePagingHelper featurePager);

    public FeatureTableModel createEmptyFeatureTableModel(DynStruct struct);

    public JFeatureTable createJFeatureTable(FeatureTableModel model);

    public JFeatureTable createJFeatureTable(FeatureTableModel model, boolean isDoubleBuffered);

    public PickerController<IProjection> createProjectionPickerController(
            JTextField txtProjection,
            JButton btnSelectProjection
        );
 
    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JComboBox combo);

    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JList list);

    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JComboBox combo, Predicate<FeatureAttributeDescriptor> filter);
    
    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JList list, Predicate<FeatureAttributeDescriptor> filter);

    public FeatureTypePanel createFeatureTypePanel();
    
    public FeatureTypeAttributePanel createFeatureTypeAttributePanel();    

    public FeatureStoreElement createFeatureStoreElement();
    
    public FeatureStoreElement createFeatureStoreElement(FeatureStore store);
    
    public FeatureStoreSearchPanel createFeatureStoreSearchPanel(FeatureStore store);
    
    public void registerStoreAction(DALActionFactory action);
    
    public  Collection<DALActionFactory> getStoreActions();

    public  DALActionFactory getStoreAction(String name);
    
    public void setStoreAction(AbstractButton component, String name, boolean preserveLabel, DALActionContext context);

    public PickerController<JDBCServerExplorerParameters> createJDBCConnectionPickerController(
            JComboBox cboConnection, JButton btnConnection
    );
    public TableModel createSimpleFeaturesTableModel(FeatureStore store);
    
    public TableModel createSimpleFeaturesTableModel(FeatureType featureType, List<String> columnNames, List<Feature> features);
    
}
