/* gvSIG. Desktop Geographic Information System.
 *
 * Copyright  2007-2014 gvSIG Association
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import static javax.swing.Action.ACTION_COMMAND_KEY;
import static javax.swing.Action.NAME;
import static javax.swing.Action.SHORT_DESCRIPTION;
import static javax.swing.Action.SMALL_ICON;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.gvsig.featureform.swing.JFeaturesForm;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.StoresRepository;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.expressionevaluator.ExpressionEvaluator;
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FacadeOfAFeature;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureStoreNotification;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.paging.FeaturePagingHelper;
import org.gvsig.fmap.dal.swing.AbstractDALActionFactory.AbstractDALActionContext;
import org.gvsig.fmap.dal.swing.DALActionFactory;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.impl.DefaultDataSwingManager;
import org.gvsig.fmap.dal.swing.impl.actions.ShowFormAction.ShowFormActionFactory;
import org.gvsig.fmap.dal.swing.searchpanel.FeatureStoreSearchPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dispose.DisposeUtils;
import org.gvsig.tools.dynform.AbortActionException;
import org.gvsig.tools.dynform.DynFormDefinition;
import org.gvsig.tools.dynform.DynFormLocator;
import org.gvsig.tools.dynform.DynFormManager;
import org.gvsig.tools.dynform.JDynForm;
import org.gvsig.tools.dynform.JDynFormSet;
import static org.gvsig.tools.dynform.JDynFormSet.ACTION_CANCEL_NEW;
import static org.gvsig.tools.dynform.JDynFormSet.ACTION_DELETE;
import static org.gvsig.tools.dynform.JDynFormSet.ACTION_NEW;
import static org.gvsig.tools.dynform.JDynFormSet.ACTION_SAVE;
import static org.gvsig.tools.dynform.JDynFormSet.ACTION_SEARCH;
import org.gvsig.tools.dynform.JDynFormSet.JDynFormSetListener;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.evaluator.Evaluator;
import org.gvsig.tools.exception.BaseException;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.observer.Observable;
import org.gvsig.tools.observer.Observer;
import org.gvsig.tools.service.ServiceException;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;
import org.gvsig.tools.swing.api.windowmanager.WindowManager.MODE;
import org.gvsig.tools.swing.icontheme.IconTheme;
import org.gvsig.tools.resourcesstorage.ResourcesStorage;
import org.gvsig.tools.script.ScriptManager;

/**
 * @author fdiaz
 *
 */
@SuppressWarnings("UseSpecificCatch")
public class DefaultJFeaturesForm implements JFeaturesForm {

    private final String STARTEDITING_ACTION = "startEditing";
    private final String FINISHEDITING_ACTION = "finishEditing";
    private final String REFRESHFORM_ACTION = "refreshForm";


    public class DefaultFeaturesFormContext implements FeaturesFormContext {

        private DefaultFeaturesFormContext() {
            
        }

        @Override
        public FeatureStore getFeatureStore() {
            return store;
        }
        
        @Override
        public FeatureType getFeatureType() {
            try {
                return store.getDefaultFeatureType();
            } catch (DataException ex) {
                return null;
            }
        }
        
        @Override
        public ResourcesStorage getResourcesStorage() {
            if( store == null ) {
                return null;
            }
            return store.getResourcesStorage();
        }

        @Override
        public StoresRepository getStoresRepository() {
            if( store == null ) {
                return null;
            }
            return store.getStoresRepository();
        }

        @Override
        public ScriptManager getScriptManager() {
            return ExpressionEvaluatorLocator.getExpressionEvaluatorManager();
        }

        
    }
    
    private class FormActionContext extends AbstractDALActionContext {

        public FormActionContext() {
            super(ACTION_CONTEXT_NAME);
        }
        
        @Override
        public DataStore getStore() {
            return store;
        }

        @Override
        public Expression getFilter() {
            FeatureQuery theCurrentQuery = currentQuery;
            if( theCurrentQuery!=null ) {
                Evaluator filter = theCurrentQuery.getFilter();
                if( filter instanceof ExpressionEvaluator ) {
                    Expression expression = ((ExpressionEvaluator)filter).getExpression();
                    return expression;
                }
            }
            return null;
        }

        @Override
        public int getSelectedsCount() {
            return 0;
        }

        @Override
        public Expression getFilterForSelecteds() {
            return null;
        }

        @Override
        public JComponent getActionButton(String actionName) {
            if( formset == null ) {
                return null;
            }
            return formset.getActionButton(actionName);
        }
        
    }
    
    private static final Logger LOGGER = LoggerFactory.getLogger(DefaultJFeaturesForm.class);

    private static final int PAGE_SIZE = 500;
    private JPanel panel;
    private JDynFormSet formset;
    private FeatureStore store;
    private FeaturePagingHelper features;
    private DynFormDefinition definition = null;
    private FeatureQuery currentQuery;
    private List<Action> otherActions;

    public DefaultJFeaturesForm() {
        this.otherActions = new ArrayList<>();
        this.panel = new JPanel(new BorderLayout());
        this.panel.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentHidden(ComponentEvent e) {
                doDispose();
            }
        });
    }

public static void selfRegister() {
        String[][] iconNames = new String[][]{
            new String[]{"dalswing", "form-refresh-data"}
        };
        IconTheme theme = ToolsSwingLocator.getIconThemeManager().getCurrent();
        for (String[] icon : iconNames) {
            URL url = DefaultJFeaturesForm.class.getResource(icon[1] + ".png");
            theme.registerDefault("DALSwing", icon[0], icon[1], null, url);
        }
        
    }
    @Override
    public void setPreferredSize(Dimension dimension) {
//        panel.setPreferredSize(dimension);
    }

    private void updateForm() {
        if (this.formset == null) {
            this.panel.add(this.getFormset().asJComponent(), BorderLayout.CENTER);
        }
        try {
            if( this.currentQuery!=null ) {
                this.currentQuery.retrievesAllAttributes();
            }
            this.features = DALLocator.getDataManager().createFeaturePagingHelper(store, this.currentQuery, PAGE_SIZE);
            this.formset.setValues(features.asListOfDynObjects());
        } catch (Exception ex) {
            throw new RuntimeException("Can't update form", ex);
        }
        updateButtonEnabledStatus();
    }

    @Override
    public JComponent asJComponent() {
        if (this.features == null) {
            try {
                updateForm();
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        return this.panel;
    }

    @Override
    public void bind(FeatureStore store) {
        if (store == null) {
            throw new IllegalArgumentException("bind need a store as parameter, not a null.");
        }
        try {
            DefaultDataSwingManager manager = (DefaultDataSwingManager) DALSwingLocator.getSwingManager();
            DynClass theDefinition = manager.featureType2DynClass(store, store.getDefaultFeatureType());
            this.bind(store, theDefinition);
        } catch (Exception ex) {
            throw new RuntimeException("Can't bind store '" + store.getName() + "' to form", ex);
        }
    }

    public void bind(FeatureStore store, DynClass definition) throws ServiceException, DataException {
        if (this.store == store) {
            return;
        }
        DynFormManager formManager = DynFormLocator.getDynFormManager();
        this.definition = formManager.getDefinition(definition);
        if (formset != null) {
            this.panel.remove(formset.asJComponent());
            this.formset = null;
        }
        this.store = store;
        DisposeUtils.bind(store);
        this.features = null;
    }

    private void doDispose() {
        DisposeUtils.disposeQuietly(store);
        this.store = null;
        this.panel = null;
        this.formset = null;
        this.features = null;
        this.definition = null;
        this.currentQuery = null;
        this.otherActions = null;
    }
    
    @Override
    public JDynFormSet getFormset() {
        if (this.formset == null) {
            DynFormManager formManager = DynFormLocator.getDynFormManager();
            this.formset = formManager.createJDynFormSet(
                    new DefaultFeaturesFormContext(),
                    this.definition,
                    null
            );
            List<String> groups = this.definition.getGroups();
            if( groups.size()==1 && groups.get(0)==null ) {
                this.formset.setLayoutMode(JDynForm.USE_PLAIN);
            } else {
                this.formset.setLayoutMode(JDynForm.USE_TABS);
            }
            this.formset.setAllowNew(true);
            this.formset.setAllowDelete(true);
            this.formset.setAllowUpdate(true);
            this.formset.setAllowClose(true);
            this.formset.setAllowSearch(true);
            this.formset.setAutosave(true);

            this.formset.addAction(new StartEditingAction());
            this.formset.addAction(new FinishEditingAction());
            this.formset.addAction(new RefreshAction());
            
            FormActionContext actionContext = new FormActionContext();
            Collection<DALActionFactory> factories = DALSwingLocator.getSwingManager().getStoreActions();
            for (DALActionFactory factory : factories) {
                if( StringUtils.equalsIgnoreCase(factory.getName(), ShowFormActionFactory.ACTION_NAME) ) {
                    continue;
                }
                Action action = factory.createAction(actionContext);
                this.formset.addAction(action);
            }            
            for( Action action : this.otherActions ) {
                this.formset.addAction(action);
            }

            this.formset.addListener(new FormSetListener());
        }
        updateButtonEnabledStatus();
        return this.formset;
    }

    @Override
    public void addAction(Action action) {
        this.otherActions.add(action);
        if( this.formset!=null ) {
            this.formset.addAction(action);
        }
    }

    @Override
    public long getCurrentIndex() {
        if( this.formset==null ) {
            return -1;
        }
        return this.formset.getCurrentIndex();
    }

    @Override
    public Feature get(long index) {
        if( this.formset==null || this.features==null ) {
            return null;
        }
        try {
            return this.features.getFeatureAt(index);
        } catch (BaseException ex) {
            return null;
        }
    }
    
    private class RefreshAction extends AbstractAction {

        @SuppressWarnings({"OverridableMethodCallInConstructor", "LeakingThisInConstructor"})
        public RefreshAction() {
            I18nManager i18nManager = ToolsLocator.getI18nManager();
            IconTheme iconTheme = ToolsSwingLocator.getIconThemeManager().getDefault();

            this.putValue(NAME, null);
            this.putValue(SHORT_DESCRIPTION, i18nManager.getTranslation("_Refresh"));
            this.putValue(SMALL_ICON, iconTheme.get("form-refresh-data"));
            this.putValue(ACTION_COMMAND_KEY, REFRESHFORM_ACTION);
            this.setEnabled(!formset.isInNewState());
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            try {
                I18nManager i18n = ToolsLocator.getI18nManager();
                formset.message(i18n.getTranslation("_Form_refreshed"));
                updateForm();
            } catch (Exception ex) {
                LOGGER.warn("Can't refresh form", ex);
            }
        }
    }

       
    private class FinishEditingAction extends AbstractAction implements Observer {

        @SuppressWarnings({"OverridableMethodCallInConstructor", "LeakingThisInConstructor"})
        public FinishEditingAction() {
            I18nManager i18nManager = ToolsLocator.getI18nManager();
            IconTheme iconTheme = ToolsSwingLocator.getIconThemeManager().getDefault();

            this.putValue(NAME,null);
            this.putValue(SHORT_DESCRIPTION,i18nManager.getTranslation("_Stop_editing"));
            this.putValue(SMALL_ICON, iconTheme.get("table-stop-editing"));
            this.putValue(ACTION_COMMAND_KEY, FINISHEDITING_ACTION);

            this.setEnabled(store.isEditing());
            store.addObserver(this);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            if( store.isEditing() ) {
                try {
                    I18nManager i18nManager = ToolsLocator.getI18nManager();
                    ThreadSafeDialogsManager dialogManager = ToolsSwingLocator.getThreadSafeDialogsManager();
                    int x = dialogManager.confirmDialog(
                            " Desea terminar edicion y guardar los cambios ?\n\nPulse cancelar para cancelar la edicion y los cambios.",
                            i18nManager.getTranslation("_Stop_editing"),
                            JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE
                    );
                    switch(x) {
                        case JOptionPane.YES_OPTION:
                            store.finishEditing();
                            break;
                        case JOptionPane.NO_OPTION:
                            break;
                        case JOptionPane.CANCEL_OPTION:
                            store.cancelEditing();
                            break;
                    }
                } catch (DataException ex) {
                    LOGGER.warn("Can't finish editing in FeatureForm ("+store.getName()+").",ex);
                }
            }
            updateButtonEnabledStatus();
        }
    
        @Override
        public void update(Observable observable, Object notification) {
            if( store == null || formset==null ) {
                return;
            }
            if( notification instanceof FeatureStoreNotification ) {
                FeatureStoreNotification n =  (FeatureStoreNotification) notification;
                switch( n.getType() )  {
                    case FeatureStoreNotification.AFTER_STARTEDITING:
                    case FeatureStoreNotification.AFTER_FINISHEDITING:
                    case FeatureStoreNotification.AFTER_CANCELEDITING:
                        updateButtonEnabledStatus();
                        break;
                }
            }
        }

    }
    
    private class StartEditingAction extends AbstractAction implements Observer {

        @SuppressWarnings({"OverridableMethodCallInConstructor", "LeakingThisInConstructor"})
        public StartEditingAction() {
            I18nManager i18nManager = ToolsLocator.getI18nManager();
            IconTheme iconTheme = ToolsSwingLocator.getIconThemeManager().getDefault();

            this.putValue(NAME,null);
            this.putValue(SHORT_DESCRIPTION,i18nManager.getTranslation("_Start_editing"));
            this.putValue(SMALL_ICON, iconTheme.get("table-start-editing"));
            this.putValue(ACTION_COMMAND_KEY, STARTEDITING_ACTION);

            this.setEnabled(!store.isEditing());
            store.addObserver(this);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            if( !store.isEditing() ) {
                try {
                    store.edit();
                } catch (DataException ex) {
                    LOGGER.warn("Can't finish editing in FeatureForm ("+store.getName()+").",ex);
                }
            }
            updateButtonEnabledStatus();
        }

        @Override
        public void update(Observable observable, Object notification) {
            if( store == null || formset==null ) {
                return;
            }
            if( notification instanceof FeatureStoreNotification ) {
                FeatureStoreNotification n =  (FeatureStoreNotification) notification;
                switch( n.getType() )  {
                    case FeatureStoreNotification.AFTER_STARTEDITING:
                    case FeatureStoreNotification.AFTER_FINISHEDITING:
                    case FeatureStoreNotification.AFTER_CANCELEDITING:
                        updateButtonEnabledStatus();
                        break;
                }
            }
        }

    }

    @Override
    public void setQuery(FeatureQuery query) {
        if (this.features != null) {
            if (this.formset != null && !formset.isReadOnly() && this.formset.isAutosave() && this.formset.countValues() > 0) {
                if (!store.isEditing()) {
                    try {
                        store.edit();
                    } catch (DataException e1) {
                        throw new RuntimeException("Can't set query", e1);
                    }
                }
                saveChanges(this.formset);
            }
        }
        this.currentQuery = query;
        updateForm();
    }

    private FeatureQuery getCurrentQuery() {
        return this.currentQuery;
    }
    
    @Override
    public void showForm(MODE mode) {
        this.panel.add(this.getFormset().asJComponent(), BorderLayout.CENTER);
        WindowManager winmgr = ToolsSwingLocator.getWindowManager();
        String title = this.definition.getLabel();
        winmgr.showWindow(this.asJComponent(), title, mode);
    }

    private void saveChanges(JDynFormSet theFormSet) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        try {
            if( theFormSet.isInNewState() ) {
                Feature newFeature = store.createNewFeature(false);
                DynObject newElement = newFeature.getAsDynObject();
                theFormSet.getFormValues(newElement);
                features.insert(((FacadeOfAFeature)newElement).getEditableFeature());
                this.formset.message(i18n.getTranslation("_Record_saved"));
                try {
                    this.formset.setValues(features.asListOfDynObjects());
                    this.formset.setCurrentIndex((int)(features.getTotalSize())-1);
                } catch(Exception ex) {
                    LOGGER.warn("Can't reload form data after insert.",ex);
                }
            } else {
                int index = theFormSet.getCurrentIndex();
                DynObject currentElement = theFormSet.get(index);
                theFormSet.getFormValues(currentElement);
                features.update(((FacadeOfAFeature)currentElement).getEditableFeature());
                this.formset.message(i18n.getTranslation("_Record_saved"));
            }
        } catch(Exception ex) {
            theFormSet.message(i18n.getTranslation("error_saving_data_will_not_save"));
            throw new RuntimeException("Can't save values",ex);
            
        } finally {
            updateButtonEnabledStatus();
        }

    }
    private void deleteCurrentElement(JDynFormSet theFormSet) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        try {
            int index = theFormSet.getCurrentIndex();
            DynObject currentElement = theFormSet.get(index);
            theFormSet.getFormValues(currentElement);
            Feature feature = ((FacadeOfAFeature) currentElement).getFeature();
            if (feature instanceof EditableFeature) {
                feature = ((EditableFeature) feature).getNotEditableCopy();
            }
            features.delete(feature);
            
            this.formset.message(i18n.getTranslation("_Record_removed"));
            
            this.formset.setValues(features.asListOfDynObjects());
            if (features.getTotalSize()-1<index) {
                index = index-1;
            }
            this.formset.setCurrentIndex(index);

        } catch (Exception ex) {
            theFormSet.message(i18n.getTranslation("error_removing_data_will_not_remove"));
            throw new RuntimeException("Can't remove values", ex);

        } finally {
            updateButtonEnabledStatus();
        }

    }
    @Override
    public void saveChanges() {
        if (this.formset != null && this.formset.countValues() > 0) {
            if (store != null && !store.isEditing()) {
                try {
                    store.edit();
                } catch (DataException e1) {
                    LOGGER.warn("Can't edit the store " + store.getName());
                    throw new RuntimeException("Can't save changes.", e1);
                }
            }
            this.saveChanges(this.formset);
        }
    }

    private void updateButtonEnabledStatus() {
        if( this.formset == null ) {
            return;
        }
        if( this.store == null || store.isBroken() || store.isAppending() ) {
            this.formset.setReadOnly(true);
            formset.setActionVisible(STARTEDITING_ACTION, true);
            formset.setActionEnabled(STARTEDITING_ACTION, true);
            formset.setActionVisible(FINISHEDITING_ACTION, false);
            formset.setActionEnabled(FINISHEDITING_ACTION, false);
            formset.setActionEnabled(ACTION_DELETE, false);
            formset.setActionEnabled(ACTION_NEW,false);
            formset.setActionEnabled(ACTION_CANCEL_NEW,false);
            formset.setActionEnabled(ACTION_SAVE,false);
            formset.setActionEnabled(ACTION_SEARCH,false);
            formset.setActionEnabled(REFRESHFORM_ACTION, true);
            return;
        }
        if( store.isEditing() ) {
            this.formset.setReadOnly(false);
            formset.setActionVisible(STARTEDITING_ACTION, false);
            formset.setActionEnabled(STARTEDITING_ACTION, false);
            formset.setActionVisible(FINISHEDITING_ACTION, true);
            formset.setActionEnabled(FINISHEDITING_ACTION, true);
            if( formset.isInNewState() ) {
                formset.setActionEnabled(ACTION_DELETE, false);
                formset.setActionEnabled(ACTION_SEARCH,false);
                formset.setActionEnabled(ACTION_NEW,false);
                formset.setActionEnabled(ACTION_CANCEL_NEW,true);
                formset.setActionEnabled(ACTION_SAVE,true);
                formset.setActionEnabled(REFRESHFORM_ACTION, false);
                
            } else if( this.features!=null && this.features.isEmpty() ) {
                formset.setActionEnabled(ACTION_DELETE, false);
                formset.setActionEnabled(ACTION_SEARCH,false);
                formset.setActionEnabled(ACTION_NEW,true);
                formset.setActionEnabled(ACTION_CANCEL_NEW,true);
                formset.setActionEnabled(ACTION_SAVE,false);
                formset.setActionEnabled(REFRESHFORM_ACTION, true);

            } else if( formset.getForm().isModified() ) {
                formset.setActionEnabled(ACTION_DELETE, true);
                formset.setActionEnabled(ACTION_SEARCH,true);
                formset.setActionEnabled(ACTION_NEW,true);
                formset.setActionEnabled(ACTION_CANCEL_NEW,false);
                formset.setActionEnabled(ACTION_SAVE,true);
                formset.setActionEnabled(REFRESHFORM_ACTION, true);

            } else {
                formset.setActionEnabled(ACTION_DELETE, true);
                formset.setActionEnabled(ACTION_SEARCH,true);
                formset.setActionEnabled(ACTION_NEW,true);
                formset.setActionEnabled(ACTION_CANCEL_NEW,false);
                formset.setActionEnabled(ACTION_SAVE,true); //false);
                formset.setActionEnabled(REFRESHFORM_ACTION, true);
            }

            return;
        }
        this.formset.setReadOnly(true);
        formset.setActionVisible(STARTEDITING_ACTION, true);
        formset.setActionEnabled(STARTEDITING_ACTION, true);
        formset.setActionVisible(FINISHEDITING_ACTION, false);
        formset.setActionEnabled(FINISHEDITING_ACTION, false);
        formset.setActionEnabled(ACTION_DELETE, false);
        formset.setActionEnabled(ACTION_NEW,false);
        formset.setActionEnabled(ACTION_CANCEL_NEW,false);
        formset.setActionEnabled(ACTION_SAVE,false);
        formset.setActionEnabled(ACTION_SEARCH,true);
        formset.setActionEnabled(REFRESHFORM_ACTION,true);
    }
        
    @Override
    public long getDataSetSize() {
        if (this.features != null) {
            return features.getTotalSize();
        }
        return 0;
    }

    @Override
    public FeatureStore getFeatureStore() {
        return this.store;
    }

    private class FormSetListener implements JDynFormSetListener {

        @Override
        public void formMessage(String message) {
        }

        @Override
        public void formClose() {
            panel.setVisible(false);
        }

        @Override
        public void formMovedTo(int currentPosition) throws AbortActionException {
            LOGGER.trace("formMovedTo " + currentPosition);
            updateButtonEnabledStatus();
        }

        @Override
        public void formBeforeSave(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formBeforeSave");
            updateButtonEnabledStatus();
        }

        @Override
        public void formBeforeNew(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formBeforeNew");
            updateButtonEnabledStatus();
        }

        @Override
        public void formBeforeDelete(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formBeforeDelete");
            updateButtonEnabledStatus();
        }

        @Override
        public void formAfterSave(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formAfterSave");
            saveChanges(dynformSet);
            updateButtonEnabledStatus();
        }

        @Override
        public void formAfterNew(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formAfterNew");
            updateButtonEnabledStatus();
        }

        @Override
        public void formAfterDelete(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formAfterDelete");
            deleteCurrentElement(dynformSet);
            updateButtonEnabledStatus();
        }

//        @Override
//        public void formBeforeSearch(JDynFormSet dynformSet) throws AbortActionException {
//            LOGGER.trace("formBeforeSearch");
//            DataSwingManager dataSwingmanager = DALSwingLocator.getSwingManager();
//            WindowManager_v2 winmgr = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
//
//            JExpressionBuilder builder = dataSwingmanager.createQueryFilterExpresion(store);
//            FeatureQuery currentQuery = getCurrentQuery();
//            if( currentQuery!=null ) {
//                Evaluator filter = currentQuery.getFilter();
//                if( filter instanceof ExpressionEvaluator ) {
//                    Expression expression = ((ExpressionEvaluator)filter).getExpression();
//                    builder.setExpression(expression);
//                }
//            }
//            Dialog dialog = winmgr.createDialog(
//                    builder.asJComponent(),
//                    "Filtro",
//                    "Creacion de filtro sobre '"+store.getName()+"'",
//                    WindowManager_v2.BUTTONS_OK_CANCEL
//            );
//            dialog.show(WindowManager.MODE.DIALOG);
//            if( dialog.getAction() == WindowManager_v2.BUTTON_OK ) {
//                Expression expresion = builder.getExpression();
//                try {
//                    FeatureQuery query = store.createFeatureQuery();
//                    if( ExpressionUtils.isPhraseEmpty(expresion) ) {
//                        query.clearFilter();
//                    } else {
//                        query.setFilter(expresion);
//                    }
//                    setQuery(query);
//                } catch (Exception ex) {
//                    LOGGER.warn("Can't apply filter '" + expresion + "'.", ex);
//                }
//            }
//        }

        @Override
        public void formBeforeSearch(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formBeforeSearch");
            DataSwingManager dataSwingmanager = DALSwingLocator.getSwingManager();
            WindowManager_v2 winmgr = (WindowManager_v2) ToolsSwingLocator.getWindowManager();

            final FeatureStoreSearchPanel searchPanel = dataSwingmanager.createFeatureStoreSearchPanel(store);
            searchPanel.setShowActions(false);
            FeatureQuery currentQuery = getCurrentQuery();
            if( currentQuery!=null ) {
                Evaluator filter = currentQuery.getFilter();
                if( filter instanceof ExpressionEvaluator ) {
                    Expression expression = ((ExpressionEvaluator)filter).getExpression();
                    searchPanel.setFilter(expression);
                }
            }
            searchPanel.asJComponent().setPreferredSize(
                    new Dimension(FeatureStoreSearchPanel.DEFAULT_WIDTH, 320)
            );
            Dialog dialog = winmgr.createDialog(
                    searchPanel.asJComponent(),
                    "Filtro",
                    "Creacion de filtro sobre '"+store.getName()+"'",
                    WindowManager_v2.BUTTONS_OK_CANCEL
            );
            dialog.show(WindowManager.MODE.DIALOG);
            if( dialog.getAction() == WindowManager_v2.BUTTON_OK ) {
                Expression expresion = searchPanel.getFilter();
                try {
                    FeatureQuery query = store.createFeatureQuery();
                    if( ExpressionUtils.isPhraseEmpty(expresion) ) {
                        query.clearFilter();
                    } else {
                        query.setFilter(expresion);
                    }
                    setQuery(query);
                } catch (Exception ex) {
                    LOGGER.warn("Can't apply filter '" + expresion + "'.", ex);
                }
            }
        }

        @Override
        public void formAfterSearch(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formAfterSearch");
        }

        @Override
        public void formBeforeCancelNew(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formBeforeCancelNew");
            updateButtonEnabledStatus();
        }

        @Override
        public void formAfterCancelNew(JDynFormSet dynformSet) throws AbortActionException {
            LOGGER.trace("formAfterCancelNew");
            updateButtonEnabledStatus();
        }
    }

//    private static class StoreEditException extends AbortActionException {
//
//        /**
//         *
//         */
//        private static final long serialVersionUID = -7682017811778577130L;
//
//        public StoreEditException(Throwable cause, String storename) {
//            super("Can't edit the store '%(storename)'", cause, "cant_edit_the store_XstorenameX", serialVersionUID);
//            setValue("storename", storename);
//        }
//    }
}
