/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.ImageFile;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Objects;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.gvsig.imageviewer.ImageViewer;
import org.gvsig.tools.dataTypes.CoercionException;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.dynformfield.AbstractJDynFormField;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;
import org.gvsig.tools.dynobject.Tags;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.SimpleImage;
import org.gvsig.tools.util.ToolsUtilLocator;

public abstract class AbstractJDynFormFieldImage<T> extends AbstractJDynFormField implements JDynFormField, FocusListener {

    protected T assignedValue = null;
    protected T currentValue = null;
    protected boolean readonly = false;
    private ImageViewer imageViewer;
    private int fieldHeight;

    @SuppressWarnings("OverridableMethodCallInConstructor")
    public AbstractJDynFormFieldImage(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
        this.assignedValue = (T) value;
    }

    @Override
    public void loadDefaultValuesFromTags(Tags tags) {
        super.loadDefaultValuesFromTags(tags);
        this.fieldHeight = tags.getInt(DynFormSPIManager.TAG_DYNFORM_HEIGHT,200);
    }

    @Override
    public void setReadOnly(boolean readonly) {
        super.setReadOnly(readonly);
        if (this.contents != null) {
            this.contents.setEnabled(!readonly);
        }
    }

    @Override
    public Object getAssignedValue() {
        return this.assignedValue;
    }

    @Override
    public void initComponent() {
        this.initImageViewer();
        this.setValue(this.assignedValue);
    }

    protected void initImageViewer() {
        this.contents = new JPanel();
        this.contents.setLayout(new BorderLayout(2, 0));

        this.imageViewer = ToolsUtilLocator.getImageViewerManager().createImageViewer();
        JComponent jimageViewer = this.imageViewer.asJComponent();
        
        jimageViewer.setPreferredSize(new Dimension(300, this.fieldHeight));
        jimageViewer.setSize(300,this.fieldHeight);
        
        this.contents.add(jimageViewer, BorderLayout.CENTER);
        this.contents.setVisible(true);
    }

    @Override
    public boolean hasValidValue() {
        return true;
    }
    
    protected void fireFieldChangedEventIfChanged(T previous) {
        if (!Objects.equals(this.currentValue, previous)) {
            this.fireFieldChangedEvent();
        }
    }

    @Override
    public void focusGained(FocusEvent arg0) {
        fireFieldEnterEvent();
        this.problemIndicator().restore();
    }

    @Override
    public void focusLost(FocusEvent arg0) {
        fireFieldExitEvent();
    }

    @Override
    public void clear() {
        this.currentValue = null;
        this.updateImage(this.currentValue);
    }

    protected void updateImage(Object value) {
        if (value == null) {
            this.imageViewer.clean();
            return;
        }
        try {
            SimpleImage image = ToolsSwingLocator.getToolsSwingManager()
                    .createSimpleImage(value);
            this.imageViewer.setImage(image.getBufferedImage());
        } catch (Exception ex) {
            this.problemIndicator().set("Can't load image");
        }
    }


    @Override
    public double getResizeWeight() {
        Tags tags = this.getDefinition().getTags();
        if( !tags.has(DynFormSPIManager.TAG_DYNFORM_RESIZEWEIGHT) ) {
            return 1.0;
        }
        int resizeWeight = tags.getInt(DynFormSPIManager.TAG_DYNFORM_RESIZEWEIGHT, 0);
        if( resizeWeight<1 ) {
            return 0;
        }
        if( resizeWeight>100 ) {
            return 1;
        }
        return resizeWeight / 100.0;
    }

    @Override
    public abstract Object getValue();

    @Override
    public abstract void setValue(Object value);
}
