/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.ImageFile;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.dynformfield.JCustomTextField;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;
import org.gvsig.tools.dynobject.Tags;

public abstract class AbstractJDynFormFieldImageFile<T> extends AbstractJDynFormFieldImage<T> implements JDynFormField, FocusListener {

    protected JTextField jtext = null;
    protected JButton jbutton = null;

    public AbstractJDynFormFieldImageFile(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
    }

    @Override
    public void setReadOnly(boolean readonly) {
        super.setReadOnly(readonly);
        if (this.contents != null) {
            if (readonly) {
                this.jtext.setEditable(false);
                this.jbutton.setEnabled(false);
            } else {
                this.jtext.setEditable(true);
                this.jbutton.setEnabled(true);
            }
        }
    }

    @Override
    public void initComponent() {
        this.initImageViewer();
        
        this.jtext = new JCustomTextField(this.getLabel());
        this.jtext.addFocusListener(this);
        this.jbutton = new JButton("...");
        this.jbutton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                onClickBrowse();
            }
        });
        JPanel buttonsPanel = new JPanel();
        buttonsPanel.setLayout(new BorderLayout(0, 2));
        buttonsPanel.add(jtext, BorderLayout.CENTER);
        buttonsPanel.add(jbutton, BorderLayout.LINE_END);

        this.contents.add(buttonsPanel, BorderLayout.NORTH);

        this.jbutton.setEnabled(!this.isReadOnly());
        this.jtext.setEditable(!this.isReadOnly());
    }

    protected void onClickBrowse() {
        fireFieldEnterEvent();
        T previous = this.currentValue;

        this.problemIndicator().restore();

        File initialPath = null;
        FileFilter filter = null;
        Tags tags = this.getDefinition().getTags();
        if (tags.has("path")) {
            initialPath = new File((String) tags.get("path"));
        }
        if (tags.has("filter")) {
            try {
                String extensions = (String) tags.get("filter");
                String filterDescription = extensions;
                if (tags.has("filterDescription")) {
                    filterDescription = (String) tags.get("filterDescription");
                }
                filter = new FileNameExtensionFilter(filterDescription, extensions.split("[|]"));
            } catch (Exception ex) {
                // Ignore 
            }
        }

        if (this.getCurrentFile() != null) {
            initialPath = this.getCurrentFile().getParentFile();
        }
        File[] x = showOpenFileDialog(this.getLabel(), initialPath, filter);
        if (x == null) {
            return;
        }
        this.setValue(x[0]);
    }

    protected File[] showOpenFileDialog(String title, File initialPath, FileFilter filter) {
        return showChooserDialog(title, JFileChooser.OPEN_DIALOG, JFileChooser.FILES_ONLY, false, initialPath, filter, true);
    }

    protected File[] showChooserDialog(
            final String title,
            final int type, // SAVE_DIALOG / OPEN_DIALOG
            final int selectionMode, //    JFileChooser.FILES_ONLY, JFileChooser.DIRECTORIES_ONLY, JFileChooser.FILES_AND_DIRECTORIES
            final boolean multiselection,
            final File initialPath,
            final FileFilter filter,
            final boolean fileHidingEnabled
    ) {
        File[] returnValue;

        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle(title);
        fc.setDialogType(type);
        fc.setFileSelectionMode(selectionMode);
        fc.setMultiSelectionEnabled(multiselection);
        fc.setCurrentDirectory(initialPath);
        fc.setFileFilter(filter);
        fc.setFileHidingEnabled(fileHidingEnabled);
        int r;
        switch (type) {
            case JFileChooser.SAVE_DIALOG:
                r = fc.showSaveDialog(this.contents);
                break;
            case JFileChooser.OPEN_DIALOG:
            default:
                r = fc.showOpenDialog(this.contents);
                break;
        }
        if (r != JFileChooser.APPROVE_OPTION) {
            returnValue = null;
        } else {
            if (fc.isMultiSelectionEnabled()) {
                returnValue = fc.getSelectedFiles();
            } else {
                returnValue = new File[]{fc.getSelectedFile()};
            }
        }
        return returnValue;
    }

    @Override
    public void clear() {
        Object value = this.getDefinition().getDefaultValue();
        if (value != null) {
            value = value.toString();
        } else {
            value = "";
        }
        this.jtext.setText((String) value);
        this.currentValue = null;
        this.updateImage(this.currentValue);
    }

    @Override
    public abstract Object getValue();

    @Override
    public abstract void setValue(Object value);

    protected abstract File getCurrentFile();
}
