/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.ImageFile;

import java.awt.event.FocusListener;
import java.io.File;
import java.util.Objects;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;
import org.gvsig.tools.dynobject.Tags;

public class JDynFormFieldImageFile extends AbstractJDynFormFieldImageFile<File> implements JDynFormField, FocusListener {

    public JDynFormFieldImageFile(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
    }

    @Override
    protected File getCurrentFile() {
        return this.currentValue;
    }

    @Override
    public void setValue(Object value) {
        File previous = this.currentValue;
        if (value == null) {
            this.jtext.setText("");
        } else {
            if (!(value instanceof File)) {
                LOGGER.info("setValue invoked with non File value (" + value.toString() + ").");
                return;
            }
            this.jtext.setText(((File) value).getPath());
        }
        this.assignedValue = (File) value;
        this.currentValue = this.assignedValue;
        this.updateImage(this.currentValue);
        this.fireFieldChangedEventIfChanged(previous);
    }

    @Override
    public Object getValue() {
        File value;
        String s;
        s = this.jtext.getText();
        if (s.trim().length() == 0) {
            Object x = this.getDefinition().getDefaultValue();
            if ((x != null && x instanceof File)) {
                value = new File(x.toString());
            } else {
                value = null;
            }
        } else {
            value = new File(s);
        }
        return value;
    }

    @Override
    protected void updateImage(Object value) {
        if (value != null) {
            File f = (File) value;
            if (!f.isAbsolute()) {
                Tags tags = this.getDefinition().getTags();
                if (tags.has("path")) {
                    File folder;
                    if( tags.get("path") instanceof File ) {
                        folder = (File) tags.get("path");
                    } else {
                        folder = new File(Objects.toString(tags.get("path"), null));
                    }
                    File f2 = new File(folder, f.getPath());
                    if (f2.exists()) {
                        value = f2;
                    } else {
                        folder = ToolsLocator.getFoldersManager().get("Project");
                        if (folder != null) {
                            value = new File(folder, f.getPath());
                        }
                    }
                }
            }
        }
        super.updateImage(value);
    }

}
