/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.ImageURL;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.gvsig.featureform.swing.impl.dynformfield.ImageFile.AbstractJDynFormFieldImageFile;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;


public class JDynFormFieldImageURL extends AbstractJDynFormFieldImageFile<URL> {

    public JDynFormFieldImageURL(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
    }

    @Override
    protected File getCurrentFile() {
        if( this.currentValue==null ) {
            return null;
        }
        return new File(this.currentValue.getPath());
    }

    @Override
    public Object getValue() {
        URL value;
        String s;
        s = this.jtext.getText();
        if (s.trim().length() == 0) {
            Object x = this.getDefinition().getDefaultValue();
            if (x == null) {
                value = null;
            } else {
                try {
                    value = new URL(x.toString());
                } catch (MalformedURLException e) {
                    LOGGER.info("Error. URL Syntax: " + x.toString());
                    value = null;
                }
            }
        } else {
            try {
                value = new URL(s);
            } catch (MalformedURLException e) {
                LOGGER.info("Error. URL Syntax: " + s);
                value = null;
            }
        }
        return value;
    }

    @Override
    public void setValue(Object value) {
        if (value == null) {
            this.jtext.setText("");
            this.assignedValue = null;
        } else {
            if (!(value instanceof URL)) {
                LOGGER.info("setValue invoked with non URL value (" + value.toString() + ").");
                return;
            }
            this.jtext.setText(value.toString());
            try {
                this.assignedValue = new URL(((URL) value).toString());
            } catch (MalformedURLException e) {
                LOGGER.info("Error in URL creation.");
                this.assignedValue = null;
            }
        }
        this.currentValue = this.assignedValue;
        this.updateImage(this.currentValue);
    }

}
