/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.features;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import org.gvsig.featureform.swing.JFeaturesForm;
import org.gvsig.featureform.swing.JFeaturesForm.FeaturesFormContext;
import org.gvsig.fmap.dal.StoresRepository;
import org.gvsig.fmap.dal.complements.RelatedFeatures;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dispose.DisposeUtils;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynForm;

import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.DynFormSPIManager.ComponentsFactory;
import org.gvsig.tools.dynform.spi.DynFormSPIManager.ComponentsFactory.ScrolledComponent;
import org.gvsig.tools.dynform.spi.dynformfield.AbstractJDynFormField;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.dynobject.Tags;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;

@SuppressWarnings("UseSpecificCatch")
public class JDynFormFieldRelatedFeatures extends AbstractJDynFormField implements JDynFormField {

    private class FeaturesTableModel extends AbstractTableModel {

        private final List<Feature> features;
        private final List<String> columnNames;
        private final FeatureType featureType;

        public FeaturesTableModel(FeatureType featureType, List<String> columnNames, List<Feature> features) {
            this.features = features;
            this.columnNames = columnNames;
            this.featureType = featureType;
        }

        @Override
        public int getRowCount() {
            if (this.features == null) {
                return 0;
            }
            return this.features.size();
        }

        @Override
        public int getColumnCount() {
            if (this.features == null) {
                return 0;
            }
            return this.columnNames.size();
        }

        @Override
        public String getColumnName(int columnIndex) {
            if (this.features == null) {
                return "";
            }
            String attrName = this.columnNames.get(columnIndex);
            if (this.featureType == null) {
                return attrName;
            }
            FeatureAttributeDescriptor attrdesc = this.featureType.getAttributeDescriptor(attrName);
            if (attrdesc == null) {
                return "C" + columnIndex;
            }
            return attrdesc.getLocalizedShortLabel();
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            if (this.featureType == null) {
                return String.class;
            }
            String attrName = this.columnNames.get(columnIndex);
            FeatureAttributeDescriptor attrdesc = this.featureType.getAttributeDescriptor(attrName);
            if (attrdesc == null) {
                return String.class;
            }
            return attrdesc.getDataType().getDefaultClass();
        }

        @Override
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        @Override
        public Object getValueAt(int rowIndex, int columnIndex) {
            if (this.features == null) {
                return null;
            }
            Feature feature = this.features.get(rowIndex);
            String attrName = this.columnNames.get(columnIndex);
            try {
                return feature.get(attrName);
            } catch (Throwable th) {
                return null;
            }
        }

        @Override
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {

        }
    }

    private List<Feature> assignedValue = null;
    private List<Feature> value = null;

    private FeatureType featureType;
    private List<String> columnNames;

    private JTable tblFeatures = null;
    private JButton btnEdit = null;
//    private JButton btnNew = null;
    private Dimension preferredSize = null;

    public JDynFormFieldRelatedFeatures(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
        if (value != null) {
            this.assignedValue = (List<Feature>) value;
        }
    }

    @Override
    public void loadDefaultValuesFromTags(Tags tags) {
        super.loadDefaultValuesFromTags(tags);
        int width = tags.getInt(DynFormSPIManager.TAG_DYNFORM_WIDTH, 100);
        int height = tags.getInt(DynFormSPIManager.TAG_DYNFORM_HEIGHT, -1);
        if (height > 100) {
            this.preferredSize = new Dimension(width, height);
        }
        this.setReadOnly(true);
    }

    @Override
    public List<Feature> getAssignedValue() {
        return this.assignedValue;
    }

    public void initComponentIfNeed() {
        if (this.contents == null) {
            this.initComponent();
        }
    }

    private RelatedFeatures getRelatedFeatures() {
        RelatedFeatures relatedFeatures = (RelatedFeatures) ToolsLocator.
                getComplementsManager().get(
                        RelatedFeatures.COMPLEMENT_MANE,
                        this.getDefinition()
                );
        return relatedFeatures;
    }

    private StoresRepository getStoresRepository() {
        JDynForm.DynFormContext context = this.getForm().getContext();
        if (!(context instanceof FeaturesFormContext)) {
            return null;
        }
        StoresRepository repository = ((FeaturesFormContext) context).getStoresRepository();
        return repository;
    }

    @Override
    public void initComponent() {
        JPanel panel = new JPanel();
        try {
            ComponentsFactory components = this.getComponentsFactory();
            ScrolledComponent<JTable> comps = components.getJTable(this.getDefinition(), null);

            if (this.preferredSize != null) {
                comps.getScrollPane().setPreferredSize(this.preferredSize);
            }
            this.tblFeatures = comps.getComponent();
            this.btnEdit = components.getJButton(this.getDefinition(), "Edit");
            if (this.btnEdit == null) {
                this.btnEdit = new JButton();
            }
            this.initButton(this.btnEdit, "View selected item", "edit.png");
            this.btnEdit.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent ae) {
                    doEdit();
                }
            });
//            this.btnNew = components.getJButton(this.getDefinition(), "New");
//            if (this.btnNew == null) {
//                this.btnNew = new JButton();
//            }
//            this.initButton(this.btnNew, "Create new item", "new.png");
//            this.btnNew.addActionListener(new ActionListener() {
//                @Override
//                public void actionPerformed(ActionEvent ae) {
//                    doNew();
//                }
//            });
            if (!components.containsComponents(this.getDefinition())) {
                panel.setLayout(new BorderLayout());
                panel.add(comps.getScrollPane(), BorderLayout.CENTER);
                JPanel panelButtons = new JPanel();
                panelButtons.setLayout(new FlowLayout(FlowLayout.RIGHT, 4, 1));
//                panelButtons.add(this.btnNew);
                panelButtons.add(this.btnEdit);
                panel.add(panelButtons, BorderLayout.SOUTH);
            }
            this.contents = panel;
            final RelatedFeatures relatedFeatures = this.getRelatedFeatures();
            if (relatedFeatures == null) {
                this.problemIndicator().set("Unable to locate the related table.");
                return;
            }
            RelatedFeatures.ContextRelatedFeatures context = relatedFeatures.createContext();
            context.setStoresRepository(this.getStoresRepository());
            try {
                FeatureStore store = relatedFeatures.getFeatureStore(context);
                if (store == null) {
                    this.problemIndicator().set("Unable to locate the related table '" + relatedFeatures.getTableName() + "'.");
                    return;
                }
                this.columnNames = relatedFeatures.getColumns(context);
                this.featureType = store.getDefaultFeatureType();
            } finally {
                DisposeUtils.disposeQuietly(context);
            }
            this.tblFeatures.setModel(
                    new FeaturesTableModel(this.featureType, this.columnNames, this.assignedValue)
            );
        } catch (Throwable th) {
            LOGGER.warn("Can't initialize components of '" + this.getName() + "'.", th);
        }
        this.setReadOnly(true);
    }

    private void doEdit() {
        if (this.value == null) {
            return;
        }
        int selectedRow = this.tblFeatures.getSelectedRow();
        if (selectedRow < 0) {
            return;
        }
        try {
            this.btnEdit.setEnabled(false);
            DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();

            final RelatedFeatures relatedFeatures = this.getRelatedFeatures();
            if (relatedFeatures == null) {
                this.problemIndicator().set("Unable to locate the related table.");
                return;
            }
            RelatedFeatures.ContextRelatedFeatures context = relatedFeatures.createContext();
            context.setStoresRepository(this.getStoresRepository());
            FeatureStore store = relatedFeatures.getFeatureStore(context);
            if (store == null) {
                this.problemIndicator().set("Unable to locate the related table '" + relatedFeatures.getTableName() + "'.");
                return;
            }
            Feature f = this.value.get(selectedRow);
            FeatureQuery query = relatedFeatures.getUniqueKeyQuery(
                    context, 
                    relatedFeatures.getUniqueKey(context, f)
            );
            JFeaturesForm form = dataSwingManager.createJFeaturesForm(store);
            form.setQuery(query);
            form.showForm(WindowManager.MODE.WINDOW);

        } catch (Exception ex) {
            LOGGER.warn("Can't show linked form", ex);
        } finally {
            this.btnEdit.setEnabled(true);
        }
    }

//    private void doNew() {
//        if (this.value == null) {
//            return;
//        }
//        try {
//            this.btnNew.setEnabled(false);
//            DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();
//
//            final RelatedFeatures relatedFeatures = this.getRelatedFeatures();
//            if (relatedFeatures == null) {
//                this.problemIndicator().set("Unable to locate the related table.");
//                return;
//            }
//            RelatedFeatures.ContextRelatedFeatures context = relatedFeatures.createContext();
//            context.setStoresRepository(this.getStoresRepository());
//            FeatureStore store = relatedFeatures.getFeatureStore(context);
//            if (store == null) {
//                this.problemIndicator().set("Unable to locate the related table '" + relatedFeatures.getTableName() + "'.");
//                return;
//            }
//            String myPkName = ... 
//            String myPkValue = ...
//            FeatureQuery query = relatedFeatures.getForeingKeyQuery(
//                    context, 
//                    myPkName
//            );
//            JFeaturesForm form = dataSwingManager.createJFeaturesForm(store);
//            form.setQuery(query);
//            form.getFormset().fireEvent(ACTION_NEW,null);          
//            JDynForm jDynForm = form.getFormset().getForm();
//            jDynForm.setValue(myPkName, myPkValue);
//            
//            form.showForm(WindowManager.MODE.WINDOW);
//
//        } catch (Exception ex) {
//            LOGGER.warn("Can't show linked form", ex);
//        } finally {
//            this.btnEdit.setEnabled(true);
//        }
//    }

    private JButton initButton(JButton button, final String tip, final String image) {
        if (button.getIcon() == null) {
            URL url = this.getClass().getClassLoader().getResource("org/gvsig/featureform/swing/impl/" + image);
            Icon icon = new ImageIcon(url);
            button.setIcon(icon);
        }
        if (button.getText().trim().equals("...")) {
            button.setText("");
        }
        button.setBorder(BorderFactory.createEmptyBorder());
        button.setBorderPainted(false);
        button.setFocusPainted(false);
        button.setContentAreaFilled(false);
        button.setToolTipText(tip);
        button.setCursor(new Cursor(Cursor.HAND_CURSOR));
        return button;
    }

    @Override
    public void setValue(Object value) {
        initComponentIfNeed();
        if (value == null) {
            this.clear();
            return;
        }
        this.value = (List<Feature>) value;
        this.tblFeatures.setModel(
                new FeaturesTableModel(this.featureType, this.columnNames, this.value)
        );
    }

    @Override
    public Object getValue() {
        return this.value;
    }

    @Override
    public void fetch(DynObject container) {
    }

    @Override
    public boolean hasValidValue() {
        return true;
    }

    @Override
    public void clear() {
        initComponentIfNeed();
        this.value = null;
        this.tblFeatures.setModel(
                new FeaturesTableModel(this.featureType, this.columnNames, null)
        );
    }
}
