/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.linkforeingkey;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.List;
import java.util.Objects;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.event.ListDataListener;
import javax.swing.text.JTextComponent;
import org.gvsig.featureform.swing.JFeaturesForm;
import org.gvsig.featureform.swing.JFeaturesForm.FeaturesFormContext;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.ForeingKey;
import org.gvsig.fmap.dal.feature.ForeingKey.ContextForeingKey;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.tools.dispose.DisposeUtils;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynForm;

import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.DynFormSPIManager.ComponentsFactory;
import org.gvsig.tools.dynform.spi.dynformfield.AbstractJDynFormField;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;

@SuppressWarnings("UseSpecificCatch")
public class JDynFormFieldForeingKey extends AbstractJDynFormField implements JDynFormField {

    private Object assignedValue = null;
    private Object value = null;
    private JTextComponent txtDescription = null;
    private JButton btnLink = null;
    private JButton btnUnlink = null;
    private JButton btnEdit = null;

    public JDynFormFieldForeingKey(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
        if (value != null) {
            this.assignedValue = (DynObject) value;
        }
    }

    @Override
    public Object getAssignedValue() {
        return this.assignedValue;
    }

    public void initComponentIfNeed() {
        if (this.contents == null) {
            this.initComponent();
        }
    }

    @Override
    public void initComponent() {
        ComponentsFactory components = this.getComponentsFactory();
        this.txtDescription = components.getJTextField(this.getDefinition(), null);
        if( this.txtDescription==null ) {
            this.txtDescription = new JTextField();
        }
        this.btnLink = components.getJButton(this.getDefinition(), "Link");
        if( this.btnLink==null ) {
            this.btnLink = new JButton();
        }
        this.btnUnlink = components.getJButton(this.getDefinition(), "Unlink");
        if( this.btnUnlink==null ) {
            this.btnUnlink = new JButton();
        }
        this.btnEdit = components.getJButton(this.getDefinition(), "Edit");
        if( this.btnEdit==null ) {
            this.btnEdit = new JButton();
        }
        this.initButton(this.btnLink, "Select item to link", "link.png");
        this.initButton(this.btnUnlink, "Remove link", "unlink.png");
        this.initButton(this.btnEdit, "View linked item", "edit.png");

        this.btnLink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                doLink();
            }
        });
        this.btnUnlink.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                doUnlink();
            }
        });
        this.btnEdit.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                doEdit();
            }
        });

        this.txtDescription.setText("                        ");
        this.txtDescription.setEditable(false);

        boolean enabled = !this.isReadOnly();
        this.btnEdit.setEnabled(enabled);
        this.btnLink.setEnabled(enabled);
        this.btnUnlink.setEnabled(enabled);

        JPanel panel = new JPanel();
        if( !components.containsJTextField(this.getDefinition(), null) ) {
            panel.setLayout(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();
            c.fill = GridBagConstraints.HORIZONTAL;
            c.ipadx = 4;
            c.ipady = 1;
            c.gridx = 1;
            c.gridy = 0;
            c.weightx = 1;
            panel.add(this.txtDescription, c);
            c.fill = GridBagConstraints.NONE;
            c.ipadx = 4;
            c.ipady = 1;
            c.gridx = 2;
            c.gridy = 0;
            c.weightx = 0;
            panel.add(this.btnLink, c);
            c.fill = GridBagConstraints.NONE;
            c.ipadx = 4;
            c.ipady = 1;
            c.gridx = 3;
            c.gridy = 0;
            c.weightx = 0;
            panel.add(this.btnUnlink, c);
            c.fill = GridBagConstraints.NONE;
            c.ipadx = 4;
            c.ipady = 1;
            c.gridx = 4;
            c.gridy = 0;
            c.weightx = 0;
            panel.add(this.btnEdit, c);
        }
        this.contents = panel;
        this.setValue(this.assignedValue);

    }

    private JButton initButton(JButton button, final String tip, final String image) {
        if( button.getIcon()==null ) {
            URL url = this.getClass().getClassLoader().getResource("org/gvsig/featureform/swing/impl/" + image);
            Icon icon = new ImageIcon(url);
            button.setIcon(icon);
        }
        if( button.getText().trim().equals("...") ) {
            button.setText("");
        }
        button.setBorder(BorderFactory.createEmptyBorder());
        button.setBorderPainted(false);
        button.setFocusPainted(false);
        button.setContentAreaFilled(false);
        button.setToolTipText(tip);
        button.setCursor(new Cursor(Cursor.HAND_CURSOR));
        return button;
    }

    private ForeingKey getForeingKey() {
        JDynForm.DynFormContext context = this.getForm().getContext();
        if( !(context instanceof FeaturesFormContext) ) {
            return null;
        }
        FeatureType featureType = ((FeaturesFormContext)context).getFeatureType();
        if( featureType==null ) {
            return null;
        }
        FeatureAttributeDescriptor attribute = featureType.getAttributeDescriptor(this.getName());
        if( attribute == null ) {
            return null;
        }
        ForeingKey foreingKey = attribute.getForeingKey();
        return foreingKey;
    }
    
    private void doLink() {
        final ForeingKey foreingKey = this.getForeingKey();
        if( foreingKey==null ) {
            return;
        }
        final ContextForeingKey context = foreingKey.createContext();
        try {
            final List<Feature> features = foreingKey.getFeatures(context);
            
            ListModel<String> model = new ListModel<String>() {
                @Override
                public int getSize() {
                    return features.size();
                }

                @Override
                public String getElementAt(int index) {
                    Feature feature = features.get(index);
                    ForeingKey foreingKey = getForeingKey();
                    return foreingKey.getLabel(context, feature);
                }

                @Override
                public void addListDataListener(ListDataListener l) {
                }

                @Override
                public void removeListDataListener(ListDataListener l) {
                }
            };
            WindowManager_v2 winManager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();

            final JList<String> jlist = new JList<>();
            jlist.setModel(model);
            jlist.setPreferredSize(new Dimension(350, 200));
            final Dialog dialog = winManager.createDialog(
                    jlist,
                    "Select",
                    "Select the '"+this.getLabel()+"'.", 
                    WindowManager_v2.BUTTONS_OK_CANCEL
            );
            dialog.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if( dialog.getAction()==WindowManager_v2.BUTTON_OK ) {
                        int n = jlist.getSelectedIndex();
                        if( n<0 ) {
                            return;
                        }
                        Feature feature = features.get(n);
                        if( feature!=null ) {
                            setValue(foreingKey.getCode(context, feature));
                        }
                    }
                }
            });
            dialog.show(WindowManager.MODE.DIALOG);

        } catch (Exception ex) {
            LOGGER.warn("Can't show selector", ex);
        } finally {
            DisposeUtils.disposeQuietly(context);
        }
    }

    private void doUnlink() {
        this.clear();
    }

    private void doEdit() {
        if (this.value == null) {
            return;
        }
        final ForeingKey foreingKey = this.getForeingKey();
        if( foreingKey==null ) {
            return;
        }
        final ContextForeingKey context = foreingKey.createContext();
        try {
            this.btnEdit.setEnabled(false);
            DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();
            FeatureQuery query = foreingKey.getQuery(context, value);
            JFeaturesForm form = dataSwingManager.createJFeaturesForm(
                    foreingKey.getFeatureStore(context)
            );
            form.setQuery(query);
            form.showForm(WindowManager.MODE.WINDOW);

        } catch (Exception ex) {
            LOGGER.warn("Can't show linked form", ex);
        } finally {
            this.btnEdit.setEnabled(true);
            DisposeUtils.disposeQuietly(context);
        }
    }

    private String getDescription() {
        if (this.value == null) {
            return null;
        }
        try {
            final ForeingKey foreingKey = this.getForeingKey();
            if( foreingKey==null ) {
                return null;
            }
            String description = foreingKey.getLabel(null, value);
            return description;
        } catch (Exception ex) {
        }
        return null;

    }
    
    @Override
    public void setReadOnly(boolean readonly) {
        initComponentIfNeed();
        this.readOnly = readonly;
        boolean editable = !readonly;
        JComponent jlabel = this.getJLabel();
        if( jlabel !=null ) {
            jlabel.setEnabled(editable);
        }
        this.txtDescription.setEditable(false);
        this.btnEdit.setEnabled(true);
        this.btnLink.setEnabled(editable);
        this.btnUnlink.setEnabled(editable);
    }

    @Override
    public void setValue(Object value) {
        initComponentIfNeed();
        if (value == null) {
            this.clear();
            return;
        }
        this.value = value;
        this.txtDescription.setText(Objects.toString(this.getDescription(), ""));
    }

    @Override
    public Object getValue() {
        return this.value;
    }

    @Override
    public boolean hasValidValue() {
        return true;
    }

    @Override
    public void clear() {
        initComponentIfNeed();
        this.value = null;
        this.txtDescription.setText("");
    }
}
