/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.linkforeingkey;

import org.gvsig.featureform.swing.JFeaturesForm.FeaturesFormContext;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.ForeingKey;
import org.gvsig.tools.dataTypes.DataTypes;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynForm;
import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPILocator;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.dynformfield.AbstractJDynFormFieldFactory;

public class JDynFormFieldForeingKeyFactory extends AbstractJDynFormFieldFactory {

    public JDynFormFieldForeingKeyFactory() {
        super("FOREINGKEY", DataTypes.UNKNOWN, null);
    }

    @Override
    public JDynFormField create(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            DynFormFieldDefinition fieldDefinition,
            Object value
    ) {
        return new JDynFormFieldForeingKey(serviceManager, componentsFactory, this, fieldDefinition, value);
    }

    @Override
    public int getPriority() {
        return 500;
    }
    
    @Override
    public boolean isApplicableTo(JDynForm.DynFormContext context, DynFormFieldDefinition fieldDefinition) {
        if( !(context instanceof FeaturesFormContext) ) {
            return false;
        }
        FeatureType featureType = ((FeaturesFormContext)context).getFeatureType();
        FeatureAttributeDescriptor attribute = featureType.getAttributeDescriptor(fieldDefinition.getName());
        if( attribute == null ) {
            return false;
        }
        ForeingKey foreingKey = attribute.getForeingKey();
        if( foreingKey==null ) {
            return false;
        }
        return foreingKey.isForeingKey() && !foreingKey.isClosedList();
    }
    
    public static void selfRegister() {
        DynFormSPIManager manager = DynFormSPILocator.getDynFormSPIManager();
        manager.registerDynFieldFactory(new JDynFormFieldForeingKeyFactory());
    }
    
}
