/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl.dynformfield.selectableforeingkey;

import org.gvsig.featureform.swing.JFeaturesForm.FeaturesFormContext;
import org.gvsig.fmap.dal.StoresRepository;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.ForeingKey;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynform.DynFormFieldDefinition;
import org.gvsig.tools.dynform.JDynForm;
import org.gvsig.tools.dynform.JDynFormField;
import org.gvsig.tools.dynform.spi.DynFormSPIManager;
import org.gvsig.tools.dynform.spi.dynformfield.AbstractJDynFormFieldWithValueList;
import org.gvsig.tools.dynform.spi.dynformfield.JDynFormFieldFactory;
import org.gvsig.tools.dynobject.DynObjectValueItem;

@SuppressWarnings("UseSpecificCatch")
public class JDynFormFieldSelectableForeingKey extends AbstractJDynFormFieldWithValueList implements JDynFormField {

    public JDynFormFieldSelectableForeingKey(
            DynFormSPIManager serviceManager,
            DynFormSPIManager.ComponentsFactory componentsFactory,
            JDynFormFieldFactory factory,
            DynFormFieldDefinition definition,
            Object value
    ) {
        super(serviceManager, componentsFactory, factory, definition, value);
    }


    private ForeingKey getForeingKey() {
        JDynForm.DynFormContext context = this.getForm().getContext();
        if( !(context instanceof FeaturesFormContext) ) {
            return null;
        }
        FeatureType featureType = ((FeaturesFormContext)context).getFeatureType();
        if( featureType==null ) {
            return null;
        }
        FeatureAttributeDescriptor attribute = featureType.getAttributeDescriptor(this.getName());
        if( attribute == null ) {
            return null;
        }
        ForeingKey foreingKey = attribute.getForeingKey();
        return foreingKey;
    }
    
    @Override
    protected DynObjectValueItem[] getAvailableValues() {
        final ForeingKey foreingKey = this.getForeingKey();
        if( foreingKey==null || !foreingKey.isClosedList() ) {
            return null;
        }
        DynObjectValueItem[] values = foreingKey.getAvailableValues(null);
        return values;
    }

}
