/* gvSIG. Desktop Geographic Information System.
 *
 * Copyright  2007-2014 gvSIG Association
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.swing.impl;

import org.gvsig.featureform.swing.impl.DefaultJFeaturesForm;
import org.gvsig.featureform.swing.impl.dynformfield.ImageByteArray.JDynFormFieldImageByteArrayFactory;
import org.gvsig.featureform.swing.impl.dynformfield.ImageFile.JDynFormFieldImageFileFactory;
import org.gvsig.featureform.swing.impl.dynformfield.ImageURL.JDynFormFieldImageURLFactory;
import org.gvsig.featureform.swing.impl.dynformfield.features.JDynFormFieldRelatedFeaturesFactory;
import org.gvsig.featureform.swing.impl.dynformfield.linkedentity.JDynFormFieldFeatureLinkFactory;
import org.gvsig.featureform.swing.impl.dynformfield.linkedentities.JDynFormFieldFeaturesTableLinkFactory;
import org.gvsig.featureform.swing.impl.dynformfield.linkforeingkey.JDynFormFieldForeingKeyFactory;
import org.gvsig.featureform.swing.impl.dynformfield.selectableforeingkey.JDynFormFieldSelectableForeingKeyFactory;
import org.gvsig.fmap.dal.impl.DefaultEditingNotificationManager;
import org.gvsig.fmap.dal.swing.DALSwingLibrary;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.dataStoreParameters.DataStoreParametersPanelManager;
import org.gvsig.fmap.dal.swing.impl.dataStoreParameters.DataStoreDynObjectParametersPanelFactory;
import org.gvsig.fmap.dal.swing.impl.dataStoreParameters.DefaultDataStoreParametersPanelManager;
import org.gvsig.fmap.dal.swing.impl.dynobjectutils.CreateComboModelFromTable;
import org.gvsig.fmap.dal.swing.impl.expressionevaluator.FeatureStoreElementFactory;
import org.gvsig.fmap.dal.swing.impl.jdbc.JDBCConnectionPickerController;
import org.gvsig.fmap.dal.swing.impl.searchpanel.DefaultSearchPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynObjectManager;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.LibraryException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author fdiaz
 *
 */
public class DefaultDALSwingLibrary  extends AbstractLibrary{

    private static final Logger LOGGER = LoggerFactory.getLogger(DefaultDALSwingLibrary.class);

    @Override
    public void doRegistration() {
        registerAsImplementationOf(DALSwingLibrary.class);
    }


    @Override
    protected void doInitialize() throws LibraryException {
        DALSwingLocator.registerSwingManager(DefaultDataSwingManager.class);
        DALSwingLocator.registerEditingNotificationManager(DefaultEditingNotificationManager.class);

        DALSwingLocator.registerDataStoreParametersPanelManager(DefaultDataStoreParametersPanelManager.class);
    }


    @Override
    protected void doPostInitialize() throws LibraryException {
        try {
            JDynFormFieldFeatureLinkFactory.selfRegister();
            JDynFormFieldFeaturesTableLinkFactory.selfRegister();
            
            JDynFormFieldImageByteArrayFactory.selfRegister();
            JDynFormFieldImageFileFactory.selfRegister();
            JDynFormFieldImageURLFactory.selfRegister();
            JDynFormFieldForeingKeyFactory.selfRegister();
            JDynFormFieldSelectableForeingKeyFactory.selfRegister();
            JDynFormFieldRelatedFeaturesFactory.selfRegister();
            
            DefaultSearchPanel.selfRegister();
            JDBCConnectionPickerController.selfRegister();
            DefaultJFeaturesForm.selfRegister();

            DynObjectManager dynObjectmanager = ToolsLocator.getDynObjectManager();
            dynObjectmanager.registerDynMethod(new CreateComboModelFromTable("DAL.createComboModelFromTable", "XXXXXXXXXXXXXXXXXXXX."));

            DataStoreParametersPanelManager dataStoreParametersPanelManager = DALSwingLocator.getDataStoreParametersPanelManager();
            dataStoreParametersPanelManager.registerFactory(new DataStoreDynObjectParametersPanelFactory());
        } catch(Exception ex) {
            LOGGER.warn("Can't register DAL components in DynForms",ex);
        }
        try {
            FeatureStoreElementFactory.selfRegister();
        } catch(Exception ex) {
            LOGGER.warn("Can't register DAL components in ExpressionEvaluator",ex);
        }
    }

}
