/* gvSIG. Desktop Geographic Information System.
 *
 * Copyright  2007-2014 gvSIG Association
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.swing.impl;

import java.awt.Component;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.table.TableModel;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingLocator;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.expressionevaluator.swing.JExpressionBuilder;
import org.gvsig.featureform.swing.CreateJFeatureFormException;
import org.gvsig.featureform.swing.JFeatureForm;

import org.gvsig.featureform.swing.JFeaturesForm;
import org.gvsig.featureform.swing.impl.DefaultJFeatureForm;
import org.gvsig.featureform.swing.impl.DefaultJFeaturesForm;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataStoreProviderFactory;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.expressionevaluator.FeatureSymbolTable;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.FeatureTypeDefinitionsManager;
import org.gvsig.fmap.dal.feature.paging.FeaturePagingHelper;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.FeatureTableModel;
import org.gvsig.fmap.dal.swing.JFeatureTable;
import org.gvsig.fmap.dal.swing.expressionevaluator.FeatureStoreElement;
import org.gvsig.fmap.dal.swing.featuretype.AttributeDescriptorPickerController;
import org.gvsig.fmap.dal.swing.featuretype.FeatureTypeAttributePanel;
import org.gvsig.fmap.dal.swing.featuretype.FeatureTypePanel;
import org.gvsig.fmap.dal.swing.impl.expressionevaluator.DefaultFeatureStoreElement2;
import org.gvsig.fmap.dal.swing.impl.featuretable.FeatureTablePanel;
import org.gvsig.fmap.dal.swing.impl.featuretable.table.DefaultFeatureTableModel;
import org.gvsig.fmap.dal.swing.impl.featuretable.table.EmptyFeatureTableModel;
import org.gvsig.fmap.dal.swing.impl.featuretype.DefaultAttributeDescriptorPickerController;
import org.gvsig.fmap.dal.swing.impl.featuretype.DefaultFeatureTypeAttributePanel;
import org.gvsig.fmap.dal.swing.impl.featuretype.DefaultFeatureTypePanel;
import org.gvsig.fmap.dal.swing.impl.jdbc.DefaultJDBCConnectionPanel;
import org.gvsig.fmap.dal.swing.impl.searchpanel.DefaultSearchPanel;
import org.gvsig.fmap.dal.swing.impl.actions.SelectionAddAction.SelectionAddActionFactory;
import org.gvsig.fmap.dal.swing.impl.actions.SelectionFilterAction.SelectionFilterActionFactory;
import org.gvsig.fmap.dal.swing.impl.actions.SelectionSetAction.SelectionSetActionFactory;
import org.gvsig.fmap.dal.swing.impl.actions.ShowFormAction.ShowFormActionFactory;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel;
import org.gvsig.fmap.dal.swing.searchpanel.FeatureStoreSearchPanel;
import org.gvsig.tools.dynform.DynFormLocator;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.exception.BaseException;
import org.gvsig.fmap.dal.swing.DALActionFactory;
import org.gvsig.fmap.dal.swing.DALActionFactory.DALActionContext;
import org.gvsig.fmap.dal.swing.impl.featuretable.SimpleFeaturesTableModel;
import org.gvsig.fmap.dal.swing.impl.jdbc.DefaultJDBCConnectionDialog;
import org.gvsig.fmap.dal.swing.impl.jdbc.JDBCConnectionPickerController;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionDialog;
import org.gvsig.tools.swing.api.pickercontroller.PickerController;


/**
 * @author fdiaz
 *
 */
@SuppressWarnings("UseSpecificCatch")
public class DefaultDataSwingManager implements DataSwingManager {

    private final Map<String,DALActionFactory> featureStoreSearchActions;
    
    public DefaultDataSwingManager() {
        this.featureStoreSearchActions = new LinkedHashMap<>();
        this.registerStoreAction(new SelectionAddActionFactory());
        this.registerStoreAction(new SelectionFilterActionFactory());
        this.registerStoreAction(new SelectionSetActionFactory());
        this.registerStoreAction(new ShowFormActionFactory());
    }
    
    @Override
    public JFeaturesForm createJFeaturesForm(FeatureStore store) throws CreateJFeatureFormException {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            
            DefaultJFeaturesForm form = new DefaultJFeaturesForm();
            form.bind(store, featureType2DynClass(store, featureType));
            return form;
            
        } catch (Exception ex) {
            throw new CreateJFeatureFormException(ex);
        }
    }

    public DynClass featureType2DynClass(FeatureStore store, FeatureType type) {
        FeatureTypeDefinitionsManager featureTypeDefinitionsManager = DALLocator.getFeatureTypeDefinitionsManager();
        if( featureTypeDefinitionsManager.contains(store,type) ) {
            // Force reload next call to get.
            featureTypeDefinitionsManager.remove(store,type);
        }
        DynClass dynClass = featureTypeDefinitionsManager.get(store,type);
        // Force reload next call to get.
        DynFormLocator.getDynFormManager().removeDefinition(dynClass.getName());
        return dynClass;
    }
    
    @Override
    public JDBCConnectionPanel createJDBCConnectionPanel() {
        JDBCConnectionPanel x = new DefaultJDBCConnectionPanel();
        return x;
    }

    @Override
    public JDBCConnectionDialog createJDBCConectionDialog() {
        JDBCConnectionDialog x = new DefaultJDBCConnectionDialog();
        return x;
    }

    @Override
    public JExpressionBuilder createQueryFilterExpresion(FeatureStore store) {
        DataManager dataManager = DALLocator.getDataManager();
        ExpressionEvaluatorSwingManager swingManager = ExpressionEvaluatorSwingLocator.getManager();
        FeatureSymbolTable featureSymbolTable = dataManager.createFeatureSymbolTable();

        Feature sampleFeature = null;
        try {
            sampleFeature = store.getFeatureSelection().first();
            if( sampleFeature == null ) {
                sampleFeature = store.first();
            }
        } catch (DataException ex) {
        }      
        final JExpressionBuilder builder = swingManager.createJExpressionBuilder();
        if( sampleFeature!=null ) {
            featureSymbolTable.setFeature(sampleFeature);
            builder.setPreviewSymbolTable(featureSymbolTable.createParent());
        }
        builder.addSymbolTable(featureSymbolTable);
        FeatureStoreElement storeElement = new DefaultFeatureStoreElement2(store);
        builder.addElement(storeElement);
        if( store.getProviderFactory().hasSQLSupport()==DataStoreProviderFactory.YES ) {
            builder.getPreferences().setSQLCompatible(true);
        }
        builder.expandElement(storeElement);
        return builder;
    }

    @Override
    public JFeatureForm createJFeatureForm(FeatureStore store) throws CreateJFeatureFormException {
        JFeatureForm form = new DefaultJFeatureForm();
        form.setStore(store);
        return form;
    }

    @Override
    public JFeatureForm createJFeatureForm(Feature feature) throws CreateJFeatureFormException {
        JFeatureForm form = new DefaultJFeatureForm();
        form.setFeature(feature);
        return form;
    }

    @Override
    public JFeatureTable createJFeatureTable(FeatureTableModel model) {
        return this.createJFeatureTable(model, true);
    }
    
    @Override
    public JFeatureTable createJFeatureTable(FeatureTableModel model, boolean isDoubleBuffered) {
        return new FeatureTablePanel(model, isDoubleBuffered);
    }
    
    @Override
    public FeatureTableModel createEmptyFeatureTableModel(DynStruct struct) {
        return new EmptyFeatureTableModel(struct);
    }
    
    @Override
    public FeatureTableModel createFeatureTableModel(FeatureStore featureStore,
            FeatureQuery featureQuery) {
        try {
            FeaturePagingHelper pager = DALLocator.getDataManager().createFeaturePagingHelper(
                    featureStore, featureQuery,
                    FeaturePagingHelper.DEFAULT_PAGE_SIZE
            );
            return this.createFeatureTableModel(pager);
        } catch (BaseException ex) {
            throw new RuntimeException("Can't create a FeatureTableModel.", ex);
        }
    }
    
    @Override
    public FeatureTableModel createFeatureTableModel(FeaturePagingHelper featurePager) {
        return new DefaultFeatureTableModel(featurePager);
    }
    
    @Override
    public ProjectionPickerController createProjectionPickerController(
            JTextField txtProjection,
            JButton btnSelectProjection
        ) {
        ProjectionPickerController picker = new ProjectionPickerController(txtProjection, btnSelectProjection);
        return picker;
    }

    @Override
    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JComboBox combo) {
        AttributeDescriptorPickerController controller = new DefaultAttributeDescriptorPickerController(combo, null);
        return controller;
    }

    @Override
    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JList list) {
        AttributeDescriptorPickerController controller = new DefaultAttributeDescriptorPickerController(list, null);
        return controller;
    }

    @Override
    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JComboBox combo, Predicate<FeatureAttributeDescriptor> filter) {
        AttributeDescriptorPickerController controller = new DefaultAttributeDescriptorPickerController(combo, filter);
        return controller;
    }

    @Override
    public AttributeDescriptorPickerController createAttributeDescriptorPickerController(JList list, Predicate<FeatureAttributeDescriptor> filter) {
        AttributeDescriptorPickerController controller = new DefaultAttributeDescriptorPickerController(list, filter);
        return controller;
    }

    @Override
    public FeatureTypePanel createFeatureTypePanel() {
        FeatureTypePanel panel = new DefaultFeatureTypePanel();
        return panel;
    }
    
    @Override
    public FeatureTypeAttributePanel createFeatureTypeAttributePanel() {
        FeatureTypeAttributePanel panel = new DefaultFeatureTypeAttributePanel();
        return panel;
    }

    @Override
    public FeatureStoreElement createFeatureStoreElement() {
        FeatureStoreElement e = new DefaultFeatureStoreElement2(null);
        return e;
    }

    @Override
    public FeatureStoreElement createFeatureStoreElement(FeatureStore store) {
        FeatureStoreElement e = new DefaultFeatureStoreElement2(store);
        return e;
    }

    @Override
    public FeatureStoreSearchPanel createFeatureStoreSearchPanel(FeatureStore store) {
        DefaultSearchPanel e = new DefaultSearchPanel(store);
        return e;
    }

    @Override
    public final void registerStoreAction(DALActionFactory action) {
        this.featureStoreSearchActions.put(action.getName().toLowerCase(), action);
    }
    
    @Override
    public  Collection<DALActionFactory> getStoreActions() {
        return Collections.unmodifiableCollection(this.featureStoreSearchActions.values());
    }

    @Override
    public  DALActionFactory getStoreAction(String name) {
        DALActionFactory action = this.featureStoreSearchActions.get(name.toLowerCase());
        return action;
    }
    
    @Override
    public void setStoreAction(AbstractButton component, String name, boolean preserveLabel, DALActionContext context) {
        DALActionFactory factory = this.getStoreAction(name);
        Action action = factory.createAction(context);
        if( preserveLabel ) {
            String s = component.getText();
            component.setAction(action);
            component.setText(s);
        } else {
            component.setAction(action);
        }
    }


    @Override
    public PickerController<JDBCServerExplorerParameters> createJDBCConnectionPickerController(
            JComboBox cboConnection, JButton btnConnection
        ) {
        JDBCConnectionPickerController x = new JDBCConnectionPickerController(cboConnection, btnConnection);
        return x;
    }

    @Override
    public TableModel createSimpleFeaturesTableModel(FeatureStore store) {
        try {
            SimpleFeaturesTableModel m = new SimpleFeaturesTableModel(store);
            return m;
        } catch (DataException ex) {
            throw new RuntimeException("Can't creatre SimpleFeaturesTableModel.", ex);
        }
    }
    
    @Override
    public TableModel createSimpleFeaturesTableModel(FeatureType featureType, List<String> columnNames, List<Feature> features) {
        SimpleFeaturesTableModel m = new SimpleFeaturesTableModel(featureType, columnNames, features);
        return m;
    }
    
}
