package org.gvsig.fmap.dal.swing.impl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JTextField;
import org.cresques.DataTypes;
import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.app.gui.panels.CRSSelectPanelFactory;
import org.gvsig.app.gui.panels.crs.ISelectCrsPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.pickercontroller.AbstractPickerController;
import org.gvsig.tools.swing.api.pickercontroller.PickerController;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;

/**
 *
 * @author jjdelcerro
 */
public class ProjectionPickerController 
        extends AbstractPickerController<IProjection>
        implements PickerController<IProjection> 
    {

    private final JTextField txtProjection;
    private final JButton btnSelectProjection;
    private IProjection projection;

    public ProjectionPickerController(
            JTextField txtProjection,
            JButton btnSelectProjection
        ) {
        this.txtProjection = txtProjection;
        this.btnSelectProjection = btnSelectProjection;
        this.projection = null;
        this.initComponents();
    }
    
    private void initComponents() {
        this.txtProjection.setEditable(false);
        this.txtProjection.setText("");
        this.btnSelectProjection.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doSelectProjection();
            }
        });
        if( this.existsIcon("picker-crs") ) {
            this.btnSelectProjection.setText("");
            this.btnSelectProjection.setIcon(this.getIcon("picker-crs"));
        }
    }
    
    private void doSelectProjection() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        
        ISelectCrsPanel crsSelectPanel = CRSSelectPanelFactory.getUIFactory().getSelectCrsPanel(
                this.projection, 
                true
        );
        WindowManager winManager = ToolsSwingLocator.getWindowManager();
        winManager.showWindow(
                (JComponent) crsSelectPanel, 
                i18n.getTranslation("selecciona_sistema_de_referencia"), 
                WindowManager.MODE.DIALOG
        );
        if (crsSelectPanel.isOkPressed()) {
            this.set(crsSelectPanel.getProjection());
        }
    }
    
    public ICoordTrans getCoordinateTransformation() {
        return null;
    }
    
    @Override
    public IProjection get() {
        return this.projection;
    }

    @Override
    public void set(IProjection projection) {
        this.projection = projection;
        if( this.projection == null ) {
            this.txtProjection.setText("");
        } else {
            this.txtProjection.setText(this.projection.getAbrev());
        }
    }

    @Override
    public void coerceAndSet(Object o) {
        this.set((IProjection) this.coerce(DataTypes.CRS, o, null));
    }

    @Override
    public void setEnabled(boolean bln) {
        this.btnSelectProjection.setEnabled(bln);
    }

    @Override
    public boolean isEnabled() {
        return this.btnSelectProjection.isEnabled();
    }

    @Override
    public void setEditable(boolean editable) {
        super.setEditable(editable); 
        this.btnSelectProjection.setEnabled(editable);
    }

    @Override
    public boolean isEmpty() {
        return this.projection == null;
    }

}
