package org.gvsig.fmap.dal.swing.impl.actions;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.swing.AbstractDALActionFactory;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.fmap.dal.swing.DALActionFactory.DALActionContext;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class SelectionSetAction 
        extends AbstractAction 
    {

    public static class SelectionSetActionFactory extends AbstractDALActionFactory {

        public static final String ACTION_NAME = "SelectionSet";
    
        public SelectionSetActionFactory() {
            super(ACTION_NAME);
        }
        
        @Override
        public Action createAction(DALActionContext context) {
            return new SelectionSetAction(context);
        }
        
    }

    private static final Logger LOGGER = LoggerFactory.getLogger(SelectionSetAction.class);
    
    private final DALActionContext context;

    public SelectionSetAction(DALActionContext context) {
        this.context = context;
        I18nManager i18n = ToolsLocator.getI18nManager();
        this.putValue(
                Action.ACTION_COMMAND_KEY, 
                SelectionSetActionFactory.ACTION_NAME
        );
        this.putValue(
                Action.SHORT_DESCRIPTION, 
                i18n.getTranslation("_New_selection")
        );
        this.putValue(
                Action.SMALL_ICON, 
                ToolsSwingLocator.getIconThemeManager().getCurrent().get("search-action-select")
        );
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        try {
            DataStore store = this.context.getStore();
            if( !(store instanceof FeatureStore) ) {
                return ;
            }
            FeatureStore featureStore = (FeatureStore) store;
            FeatureSelection currentSelection = featureStore.getFeatureSelection();
            
            Expression filter = this.context.getFilterForSelecteds();
            if( filter == null ) {
                filter = this.context.getFilter();
            }
            FeatureQuery query = featureStore.createFeatureQuery();
            query.setFilter(filter);
            FeatureSet selection = featureStore.getFeatureSet(query);
            currentSelection.deselectAll();
            currentSelection.select(selection);
        } catch (Exception ex) {
            LOGGER.warn("Can't build selecction.", ex);
        }
    }

}
