package org.gvsig.fmap.dal.swing.impl.actions;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.featureform.swing.JFeaturesForm;
import org.gvsig.featureform.swing.impl.DefaultJFeaturesForm;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.swing.AbstractDALActionFactory;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.searchpanel.FeatureStoreSearchPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.fmap.dal.swing.DALActionFactory;
import org.gvsig.fmap.dal.swing.DALActionFactory.DALActionContext;
import org.gvsig.fmap.dal.swing.impl.searchpanel.DefaultSearchPanel;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class ShowFormAction 
        extends AbstractAction 
        implements ListSelectionListener
    {
    
    public static class ShowFormActionFactory extends AbstractDALActionFactory {

        public static final String ACTION_NAME = "ShowForm";
        
        public ShowFormActionFactory() {
            super(ACTION_NAME);
        }

        @Override
        public Action createAction(DALActionContext context) {
            return new ShowFormAction(context);
        }
        
//        public static void selfRegister() {
//            DataSwingManager dalSwingManager = DALSwingLocator.getSwingManager();
//            dalSwingManager.registerFeatureStoreSearchAction(new ShowFormActionFactory());
//            String[][] iconNames = new String[][]{
//                new String[]{"dalswing", "search-action-showform"}
//            };
//            IconTheme theme = ToolsSwingLocator.getIconThemeManager().getCurrent();
//            for (String[] icon : iconNames) {
//                URL url = ShowFormAction.class.getResource(icon[1] + ".png");
//                theme.registerDefault("DALSwing", icon[0], icon[1], null, url);
//            }
//        }
        
    }


    private static final Logger LOGGER = LoggerFactory.getLogger(ShowFormAction.class);
    
    private final DALActionContext context;
    private boolean showFormSingleElement;

    public ShowFormAction(DALActionContext context) {
        this.context = context;
        I18nManager i18n = ToolsLocator.getI18nManager();
        this.putValue(
                Action.ACTION_COMMAND_KEY, 
                ShowFormActionFactory.ACTION_NAME
        );
        this.putValue(
                Action.SHORT_DESCRIPTION, 
                i18n.getTranslation("_Show_form")
        );
        this.putValue(
                Action.SMALL_ICON, 
                ToolsSwingLocator.getIconThemeManager().getCurrent().get("search-action-showform")
        );
        try {
            this.showFormSingleElement = true;
            DataStore store = this.context.getStore();
            if( !(store instanceof FeatureStore) ) {
                return;
            }
            FeatureStore featureStore = (FeatureStore) store;
            FeatureAttributeDescriptor[] pks = featureStore.getDefaultFeatureType().getPrimaryKey();
            if (pks == null || pks.length < 1) {
                this.showFormSingleElement = false;
            }
        } catch (Exception ex) {
        }        
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        try {
            this.setEnabled(false);
            DataStore store = this.context.getStore();
            if( !(store instanceof FeatureStore) ) {
                return ;
            }
            FeatureStore featureStore = (FeatureStore) store;
            Expression filter = this.context.getFilterForSelecteds();
            if( filter==null ) {
                filter = this.context.getFilter();
            }
            DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();

            FeatureQuery query = featureStore.createFeatureQuery();
            query.setFilter(filter);
            query.retrievesAllAttributes();
            JFeaturesForm form = dataSwingManager.createJFeaturesForm(featureStore);
            form.setQuery(query);
            form.showForm(WindowManager.MODE.WINDOW);
        } catch (Exception ex) {
            LOGGER.warn("Can't show form", ex);
        } finally {
            this.setEnabled(true);
        }
    }

    @Override
    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting()) {
            return;
        }
        if (this.context.getSelectedsCount()== 0) {
            this.setEnabled(true);
            return;
        }
        this.setEnabled(showFormSingleElement);
    }
    
}
