package org.gvsig.fmap.dal.swing.impl.dataStoreParameters;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.fmap.dal.swing.dataStoreParameters.DataStoreParametersPanel;
import org.gvsig.fmap.dal.swing.dataStoreParameters.DataStoreParametersPanelFactory;
import org.gvsig.fmap.dal.swing.dataStoreParameters.DataStoreParametersPanelManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class DefaultDataStoreParametersPanelManager implements DataStoreParametersPanelManager {

    private static Logger logger = LoggerFactory.getLogger(DefaultDataStoreParametersPanelManager.class);
            
    private List<DataStoreParametersPanelFactory> factories = null;
    
    public DefaultDataStoreParametersPanelManager() {
        this.factories = new ArrayList<>();
    }
    
    @Override
    public void registerFactory(DataStoreParametersPanelFactory factory) {
        if( this.factories.contains(factory) ) {
            return;
        }
        this.factories.add(factory);
    }

    @Override
    public DataStoreParametersPanelFactory getFactory(DynObject parameters) {
        DataStoreParametersPanelFactory factory = null;
        for (DataStoreParametersPanelFactory currentFactory : factories) {
            if( currentFactory.canBeApplied(parameters) ) {
                if( factory == null ) {
                    factory = currentFactory;
                } else if( factory.getPriority() < currentFactory.getPriority() ) {
                    factory = currentFactory;
                }
            }
        }
        return factory;
    }

    @Override
    public DataStoreParametersPanel createDataStoreParametersPanel(DynObject parameters) {
        DataStoreParametersPanelFactory factory = this.getFactory(parameters);
        if( factory == null ) {
            return null;
        }
        return factory.create(parameters);
    }
    
    @Override
    public int showPropertiesDialog(final DynObject parameters) {
        
        final DataStoreParametersPanel panel = this.createDataStoreParametersPanel(parameters);
        return this.showPropertiesDialog(parameters, panel);
    }
    
    @Override
    public int showPropertiesDialog(final DynObject parameters, final DataStoreParametersPanel panel) {
        I18nManager i18nManager = ToolsLocator.getI18nManager();

        String title = parameters.getDynClass().getDescription();
        if( !StringUtils.isEmpty(title) ) {
        	title = parameters.getDynClass().getName();
        }
        
        WindowManager_v2 winmanager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
        Dialog dialog = winmanager.createDialog(
                panel.asJComponent(),
                title,
                null, 
                WindowManager_v2.BUTTONS_APPLY_OK_CANCEL
        );
        if( panel instanceof DataStoreDynObjectParametersPanel ) {
            dialog.setButtons(WindowManager_v2.BUTTONS_OK_CANCEL);
        } else {
            dialog.setButtonLabel(
                    WindowManager_v2.BUTTON_APPLY, 
                    i18nManager.getTranslation("Avanzado")
            );
        }
        dialog.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                switch(ae.getID() ) {
                    case WindowManager_v2.BUTTON_APPLY:
                        showAdvancedPropertiesDialog(parameters);
                        break;
                    case WindowManager_v2.BUTTON_OK:
                        panel.fetchParameters(parameters);
                        break;
                    case WindowManager_v2.BUTTON_CANCEL:
                        break;
                }
            }
        });
        dialog.show(WindowManager.MODE.DIALOG);
        return dialog.getAction();
    }
    
    private void showAdvancedPropertiesDialog(DynObject parameters) {
        final DataStoreParametersPanel panel =  new DataStoreDynObjectParametersPanel(parameters);
        this.showPropertiesDialog(parameters, panel);
   }    
}
