package org.gvsig.fmap.dal.swing.impl.expressionevaluator;

import org.gvsig.fmap.dal.swing.expressionevaluator.FeatureStoreElement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.SimpleElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.fmap.dal.complements.Search;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.tools.ToolsLocator;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class DefaultFeatureStoreElement
        extends AbstractElement
        implements FeatureStoreElement, SimpleElement {

    private FeatureStore store;
    private List<Element> elements;
    private String myName;

    public DefaultFeatureStoreElement(FeatureStore store) {
        super("store", "store", "featurestore-table");
        this.setFeatureStore(store);
    }

    @Override
    public Element get(int index) {
        return this.buildElements().get(index);
    }

    @Override
    public String getLabel() {
        return this.getName();
    }

    @Override
    public void setName(String name) {
        this.myName = name;
    }

    @Override
    public String getName() {
        if (!StringUtils.isBlank(this.myName)) {
            return this.myName;
        }
        if (this.store == null) {
            return super.getName();
        }
        return this.store.getName();
    }

    @Override
    public int size() {
        return this.buildElements().size();
    }

    @Override
    public Iterator<Element> iterator() {
        return this.buildElements().iterator();
    }

    @Override
    public void addElement(Element element) {
    }

    @Override
    public void addElement(Function function) {
    }

    @Override
    public List<Element> getElements() {
        return Collections.unmodifiableList(this.buildElements());
    }

    @Override
    public void setFeatureStore(FeatureStore store) {
        this.store = store;
        if( store == null ) {
            this.elements = null;
        }
        this.buildElements();
    }

    @Override
    public FeatureStore getFeatureStore() {
        return this.store;
    }

    @Override
    public Element setConfig(ExpressionBuilderConfig config) {
        super.setConfig(config);
        this.buildElements();
        return this;
    }

    private List<Element> buildElements() {
        if (this.elements == null) {
            if (this.store == null || this.getConfig() == null) {
                return Collections.EMPTY_LIST;
            }
            try {
                FeatureType type = this.store.getDefaultFeatureType();

                this.elements = new ArrayList<>();
                Search search = (Search) ToolsLocator.getComplementsManager().get(
                        Search.COMPLEMENT_MANE,
                        type
                );
                List<Search.OrderedAttribute> attributes = search.getOrderedAttributes(
                        FeatureType.ALL_FILTER,
                        Search.LABEL_ORDER,
                        -1
                );
                for (Search.OrderedAttribute attrdesc : attributes) {
                    Element element = new FeatureAttributeElement(this, this.store, attrdesc.getDescriptor());
                    element.setConfig(this.getConfig());
                    this.elements.add(element);
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
        return this.elements;
    }
}
