package org.gvsig.fmap.dal.swing.impl.expressionevaluator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.GroupElement;
import org.gvsig.expressionevaluator.swing.Element.SimpleElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.complements.Search;
import org.gvsig.fmap.dal.expressionevaluator.FeatureSymbolTable;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.ForeingKey;
import org.gvsig.fmap.dal.swing.expressionevaluator.FeatureStoreElement;
import org.gvsig.fmap.dal.swing.impl.searchpanel.AdvancedAttributeSelectionTreeModel;
import org.gvsig.fmap.dal.swing.impl.searchpanel.AdvancedAttributeSelectionTreeModel.Node;

/**
 *
 * @author jjdelcerro
 */
public class DefaultFeatureStoreElement2
        extends AbstractElement
        implements SimpleElement, GroupElement, FeatureStoreElement {

    private final static String DEFAULT_ELEMENT_NAME = "store";
    
    private class ForeingKeyElement
            extends FeatureAttributeElement
            implements SimpleElement, GroupElement {

        private final Node node;
        private List<Element> elements;

        public ForeingKeyElement(Element parent, Node node) {
            super(
                    parent,
                    node.getFeatureStore(),
                    node.getValue(), "featurestore-foreing-key"
            );
            this.node = node;
        }
        
        @Override
        public String getLabel() {
            FeatureAttributeDescriptor attribute = this.node.getValue();
            if( attribute!=null && attribute.isForeingKey() ) {
                ForeingKey foreingKey = attribute.getForeingKey();
                if (getConfig().getPreferences().getShowLabelsOfElements()) {
                    if (!StringUtils.equals(attrdesc.getName(), attrdesc.getLabel())) {
                        return String.format(
                            "<html>%s<br><font size=\"2\"><i>(%s[%s])</i></font></html>", 
                            attrdesc.getLocalizedLabel(), 
                            attrdesc.getName(),
                            foreingKey.getTableName()
                        );
                    }
                    return String.format(
                        "<html>%s<br><font size=\"2\"><i>([%s])</i></font></html>", 
                        attrdesc.getLocalizedLabel(), 
                        foreingKey.getTableName()
                    );
                }
                return String.format(
                    "<html>%s<br><font size=\"2\"><i>([%s])</i></font></html>", 
                    attrdesc.getName(), 
                    foreingKey.getTableName()
                );
            }
            return super.getLabel();
        }
        
        @Override
        public Element get(int index) {
            return this.getElements().get(index);
        }

        @Override
        public int size() {
            return this.getElements().size();
        }

        @Override
        public Iterator<Element> iterator() {
            return this.getElements().iterator();
        }

        @Override
        public void addElement(Element element) {

        }

        @Override
        public void addElement(Function function) {

        }

        @Override
        public List<Element> getElements() {
            if (this.elements == null) {
                this.elements = new ArrayList<>();
                List<Node> children = this.node.getChildren();
                for (Node child : children) {
                    Element e;
                    FeatureAttributeDescriptor attrdesc = child.getValue();
                    if (attrdesc.isForeingKey()) {
                        e = new ForeingKeyElement(this, child);
                    } else {
                        e = new FeatureAttributeElement(this, child.getFeatureStore(), attrdesc);
                    }
                    e.setConfig(this.getConfig());
                    this.elements.add(e);
                }
            }
            return this.elements;
        }

        @Override
        public Element setConfig(ExpressionBuilderConfig config) {
            super.setConfig(config);
            if (this.elements != null) {
                for (Element element : this.elements) {
                    element.setConfig(config);
                }
            }
            return this;
        }
    }

    private List<Element> elements;
    private FeatureStore store;
    private AdvancedAttributeSelectionTreeModel treemodel;

    public DefaultFeatureStoreElement2(FeatureStore store) {
        super(DEFAULT_ELEMENT_NAME, "store", "featurestore-table");
        this.setFeatureStore(store);
    }

    @Override
    public Element get(int index) {
        return this.getElements().get(index);
    }

    @Override
    public int size() {
        return this.getElements().size();
    }

    @Override
    public Iterator<Element> iterator() {
        return this.getElements().iterator();
    }

    @Override
    public void addElement(Element element) {
    }

    @Override
    public void addElement(Function function) {
    }

    @Override
    public Collection<Element> getValues() {
        return this.getElements();
    }

    @Override
    public List<Element> getElements() {
        if( this.treemodel==null ) {
            return null;
        }
        if (this.elements == null) {
            this.elements = new ArrayList<>();
            Node root = (Node) treemodel.getRoot();
            List<Node> children = root.getChildren();
            for (Node child : children) {
                Element e;
                FeatureAttributeDescriptor attrdesc = child.getValue();
                if (attrdesc.isForeingKey()) {
                    e = new ForeingKeyElement(this, child);
                } else {
                    e = new FeatureAttributeElement(this, child.getFeatureStore(), attrdesc);
                }
                e.setConfig(this.getConfig());
                this.elements.add(e);
            }
        }
        return this.elements;
    }

    @Override
    public String getLabel() {
        if (this.store == null) {
            return super.getLabel();
        }
        return this.store.getName();
    }
    
    @Override
    public String getRenderedValue() {
        if (this.store == null) {
            return super.getRenderedValue();
        }
        return this.store.getName();
    }
    
    @Override
    public void setName(String name) {
        super.setName(name);
    }

    @Override
    public FeatureStore getFeatureStore() {
        return this.store;
    }

    @Override
    public final void setFeatureStore(FeatureStore store) {
        this.store = store;
        this.init();
    }

    @Override
    public Element setConfig(ExpressionBuilderConfig config) {
        super.setConfig(config);
        this.init();
        return this;
    }

    private void init() {
        this.treemodel = null;
        this.elements = null;
        if (this.store == null || this.getConfig()==null ) {
            return;
        } 
        if( StringUtils.equalsIgnoreCase(DEFAULT_ELEMENT_NAME, this.store.getName()) ) {
            this.setName(this.store.getName());
        }
        Feature sampleFeature = this.store.getSampleFeature();
        if( sampleFeature!=null ) {
            FeatureSymbolTable featureSymbolTable = DALLocator.getDataManager().createFeatureSymbolTable();
            featureSymbolTable.setFeature(sampleFeature);
            this.getConfig().setPreviewSymbolTable(featureSymbolTable.createParent());
        }
        this.treemodel = new AdvancedAttributeSelectionTreeModel(this.store, Search.ALL_FILTER);
    }
}
