package org.gvsig.fmap.dal.swing.impl.expressionevaluator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import javax.swing.SwingUtilities;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.expressionevaluator.ExpressionBuilder.Constant;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.SimpleElement;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingLocator;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.exception.BaseException;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.icontheme.IconTheme;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.visitor.VisitCanceledException;
import org.gvsig.tools.visitor.Visitor;

/**
 *
 * @author jjdelcerro
 */
public class FeatureAttributeElement
        extends AbstractElement
        implements
        SimpleElement
    {

    final FeatureStore store;
    final FeatureAttributeDescriptor attrdesc;
    List<Element> values;
    boolean hasMoreValues;
    private final Element parent;

    FeatureAttributeElement(Element parent, FeatureStore store, FeatureAttributeDescriptor attrdesc) {
        this(parent, store, attrdesc, "featurestore-column");
    }
    
    FeatureAttributeElement(Element parent, FeatureStore store, FeatureAttributeDescriptor attrdesc, String iconName) {
        super(attrdesc.getName(), attrdesc.getName(), iconName);
        this.parent = parent;
        this.store = store;
        this.attrdesc = attrdesc;
        this.hasMoreValues = false;

        IconTheme iconTheme = ToolsSwingLocator.getIconThemeManager().getCurrent();
        String theIconName = attrdesc.getDataType().getIconName();
        IconTheme.Icon theIcon = iconTheme.getThemeIcon(theIconName);
        if( theIcon!=null && theIcon.getURL()!=null ) {
            this.icon = ToolsSwingLocator.getToolsSwingManager().createCompoundIcon(
                    SwingUtilities.HORIZONTAL, 
                    1, 
                    SwingUtilities.LEFT, 
                    SwingUtilities.TOP,
                    this.getIcon(),
                    theIcon.getImageIcon()
            );
        }
    }

    public Element getParent() {
        return this.parent;
    }

    @Override
    public String getRenderedValue() {
        if( this.parent instanceof DefaultFeatureStoreElement2 ) {
            return super.getRenderedValue();
        }
        List<String> fieldNames = new ArrayList<>();
        fieldNames.add(attrdesc.getName());
        Element theParent = this.parent;
        while( theParent!=null ) {
            if( theParent instanceof DefaultFeatureStoreElement2 ) {
                break;
            }
            if( !(theParent instanceof FeatureAttributeElement) ) {
                break; // Esto no deberia pasar
            }
            fieldNames.add(0, theParent.getName());
            theParent = ((FeatureAttributeElement)theParent).getParent();
        }
        ExpressionBuilder builder = ExpressionUtils.createExpressionBuilder();

        Constant[] params = new Constant[fieldNames.size()];
        for (int i = 0; i < fieldNames.size(); i++) {
            params[i] = builder.constant(fieldNames.get(i));
        }
        ExpressionBuilder.Function tuple = builder.tuple(params);

        String formula = builder.function(DataManager.FUNCTION_FOREING_VALUE, tuple).toString();
        return formula;
    }
    
    
    @Override
    public void reload() {
        this.values = null;
        hasMoreValues = false;
    }

    @Override
    public boolean isEnabled() {
        if (this.getConfig().getPreferences().isSQLCompatible()) {
            if (this.attrdesc.isComputed()) {
                return false;
            }
        }
        return true;
    }

    @Override
    public boolean hasMoreValues() {
        return this.hasMoreValues;
    }

    @Override
    public List<Element> getValues() {
        if (this.values == null) {
            hasMoreValues = false;
            ExpressionEvaluatorSwingManager manager = ExpressionEvaluatorSwingLocator.getManager();
            final List<Object> values = new ArrayList<>();
            final int limit = this.getConfig() == null ? 60 : this.getConfig().getPreferences().getSimpleElementsLimit();
            final long timeLimit = System.currentTimeMillis() + limit * 1000;
            try {
                FeatureSet set = this.store.getFeatureSet();
                set.accept(new Visitor() {
                    @Override
                    public void visit(Object o) throws VisitCanceledException, BaseException {
                        Object value = ((Feature) o).get(attrdesc.getName());
                        if (!values.contains(value)) {
                            values.add(value);
                        }
                        if (System.currentTimeMillis() > timeLimit) {
                            hasMoreValues = true;
                            throw new VisitCanceledException();
                        }
                    }
                });
            } catch (VisitCanceledException ex) {

            } catch (Exception ex) {
                // FIXME
            }
            this.values = new ArrayList<>();
            if (!values.isEmpty()) {
                LabeledValue[] availableValues = this.attrdesc.getAvailableValues();
                Map<String, String> availableValuesMap = new HashMap<>();
                if( availableValues!=null ) {
                    for (LabeledValue availableValue : availableValues) {
                        availableValuesMap.put(
                                Objects.toString(availableValue.getValue()),
                                availableValue.getLabel()
                        );
                    }
                }
                for (Object value : values) {
                    String key = Objects.toString(value);
                    String label = availableValuesMap.getOrDefault(key, null);
                    ConstantElement e = manager.createConstantElement(
                            value,
                            label
                    );
                    this.values.add(e);
                    e.setConfig(this.getConfig());
                }
            }
        }
        return this.values;
    }

    @Override
    public String getName() {
        return attrdesc.getName();
    }
    
    @Override
    public String getLabel() {
        if (getConfig().getPreferences().getShowLabelsOfElements()) {
            if (!StringUtils.equals(attrdesc.getName(), attrdesc.getLabel())) {
                return String.format(
                    "<html>%s<br><font size=\"2\"><i>(%s)</i></font></html>", 
                    attrdesc.getLocalizedLabel(), 
                    attrdesc.getName()
                );
            }
            return String.format(
                "<html>%s</html>", 
                attrdesc.getLocalizedLabel()
            );
        }
        return String.format(
            "<html>%s</html>", 
            attrdesc.getName()
        );
    }

    @Override
    public String getDescription() {
        I18nManager i18n = ToolsLocator.getI18nManager();

        StringBuilder html = new StringBuilder();
        html.append("<html>\n");

        html.append("<b>").append(this.attrdesc.getName()).append("</b><br>");
        if( !StringUtils.equalsIgnoreCase(attrdesc.getName(), attrdesc.getLabel()) ) {
            html.append(this.attrdesc.getLabel()).append("<br>\n");
        }
        html.append("<br>\n");
        html.append("<b>").append(i18n.getTranslation("_Source")).append("</b> ").append(this.store.getName()).append("<br>\n");
        html.append("<b>").append(i18n.getTranslation("_Name")).append("</b> ").append(this.getName()).append("<br>\n");
        html.append("<b>").append(i18n.getTranslation("_Label")).append("</b> ").append(this.attrdesc.getLocalizedLabel()).append("<br>\n");
        html.append("<b>").append(i18n.getTranslation("_Type")).append("</b> ").append(this.attrdesc.getDataTypeName()).append("<br>\n");
        if (!StringUtils.isBlank(this.attrdesc.getDescription())
                && StringUtils.equalsAnyIgnoreCase(this.getName(), this.attrdesc.getDescription())) {
            html.append("<b>").append(i18n.getTranslation("_Description")).append("</b><br>\n");
            html.append(this.attrdesc.getDescription().replace("\n", "<br>")).append("<br>\n");
        }
        html.append("</html>\n");
        return html.toString();
    }

    @Override
    @SuppressWarnings("empty-statement")
    public void used() {
        attrdesc.recentUsed();
    }
}
