package org.gvsig.fmap.dal.swing.impl.featuretype;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import org.gvsig.fmap.dal.swing.featuretype.AttributeDescriptorPickerController;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.function.Predicate;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.tools.swing.api.Component;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.pickercontroller.AbstractPickerController;

/**
 *
 * @author jjdelcerro
 */
public class DefaultAttributeDescriptorPickerController         
        extends AbstractPickerController<FeatureAttributeDescriptor> 
        implements AttributeDescriptorPickerController
    {

    private static class ComboOrList<T> implements Component {

        private JComboBox combo;
        private JList list;

        public ComboOrList(JComboBox combo) {
            this.combo = combo;
            this.combo.setEditable(false);
        }
        
        public ComboOrList(JList list) {
            this.list = list;
        }
        
        @Override
        public JComponent asJComponent() {
            if( this.combo == null ) {
                return this.list;
            }
            return this.combo;
        }

        private void addSelectionListener(final ActionListener listener) {
            if( this.combo == null ) {
                this.list.addListSelectionListener(new ListSelectionListener() {
                    @Override
                    public void valueChanged(ListSelectionEvent e) {
                        if( e.getValueIsAdjusting() ) {
                            return;
                        }
                        listener.actionPerformed(new ActionEvent(list, 0, "selected"));
                    }
                });
            } else {
                this.combo.addItemListener(new ItemListener() {
                    @Override
                    public void itemStateChanged(ItemEvent e) {
                        if( e.getStateChange()==ItemEvent.SELECTED ) {
                            listener.actionPerformed(new ActionEvent(combo, 0, "selected"));
                        }
                    }
                });
            }
        }

        public void clear() {
            if( this.combo == null ) {
                DefaultListModel model = new DefaultListModel();
                this.list.setModel(model);
            } else {
                DefaultComboBoxModel model = new DefaultComboBoxModel();
                this.combo.setModel(model);
            }
        }
        
        public void add(String label, T value) {
            ListElement<T> element = new ListElement<>(label,value);
            if( this.combo == null ) {
                DefaultListModel model = (DefaultListModel) this.list.getModel();
                model.addElement(element);
            } else {
                DefaultComboBoxModel model = (DefaultComboBoxModel) this.combo.getModel();
                model.addElement(element);
            }
        }

        public int getSize() {
            if( this.combo == null ) {
                return this.list.getModel().getSize();
            } else {
                return this.combo.getModel().getSize();
            }
        }

        public void setSelectedIndex(int i) {
            if( this.combo == null ) {
                this.list.setSelectedIndex(i);
            } else {
                this.combo.setSelectedIndex(i);
            }
        }

        public void setEnabled(boolean enabled) {
            if( this.combo == null ) {
                this.list.setEnabled(enabled);
            } else {
                this.combo.setEnabled(enabled);
            }
        }

        public boolean isEnabled() {
            if( this.combo == null ) {
                return this.list.isEnabled();
            } else {
                return this.combo.isEnabled();
            }
        }
        
        public void setSelectedValue(T value) {
            if( this.combo == null ) {
                ListElement.setSelected(this.list, value);
            } else {
                ListElement.setSelected(this.combo, value);
            }
        }
        
        public T getSelectedValue() {
            T value;
            if( this.combo == null ) {
                value = (T) ListElement.getSelected(this.list);
            } else {
                value = (T) ListElement.getSelected(this.combo);
            }
            return value;
        }
    }
    
    
    private final ComboOrList<FeatureAttributeDescriptor> combo;
    private FeatureType featureType;
    private final Predicate<FeatureAttributeDescriptor> filter;

    public DefaultAttributeDescriptorPickerController(JComboBox combo, Predicate<FeatureAttributeDescriptor> filter) {
        this(new ComboOrList(combo), filter);
    }
    
    public DefaultAttributeDescriptorPickerController(JList list, Predicate<FeatureAttributeDescriptor> filter) {
        this(new ComboOrList(list), filter);
    }
    
    private DefaultAttributeDescriptorPickerController(ComboOrList combo, Predicate<FeatureAttributeDescriptor> filter) {
        this.combo = combo; 
        this.combo.addSelectionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireChangeEvent();
            }
        });
        this.filter = filter;
    }
    
    @Override
    public void setFeatureType(FeatureType featureType) {
        this.featureType = featureType;
        this.combo.clear();
        if( this.featureType==null ) {
            return;
        }
        List<FeatureAttributeDescriptor> attributes = new ArrayList<>();
        for (FeatureAttributeDescriptor descriptor : featureType) {
            if( filter==null || filter.test(descriptor) ) {
                attributes.add(descriptor);
            }
        }
        attributes.sort(new Comparator<FeatureAttributeDescriptor>() {
            @Override
            public int compare(FeatureAttributeDescriptor o1, FeatureAttributeDescriptor o2) {
                return o1.getName().compareToIgnoreCase(o2.getName());
            }
        });
        for (FeatureAttributeDescriptor descriptor : attributes) {
            String nameDescription = descriptor.getName() + " [" + descriptor.getDataTypeName() + "]";
            this.combo.add(nameDescription, descriptor);
        }
        if( this.combo.getSize()>0 ) {
            this.combo.setSelectedIndex(0);
        } else {
            this.combo.setSelectedIndex(-1);
        }
    }
    
    @Override
    public FeatureAttributeDescriptor get() {
        FeatureAttributeDescriptor descriptor = this.combo.getSelectedValue();
        return descriptor;
    }

    @Override
    public void set(FeatureAttributeDescriptor descriptor) {
        this.combo.setSelectedValue(descriptor);
    }

    @Override
    public void set(String name) {
        FeatureAttributeDescriptor descriptor = this.featureType.getAttributeDescriptor(name);
        if( descriptor == null ) {
            // throw new IllegalArgumentException("The field '"+name+" don't exists.");
            this.combo.setSelectedIndex(-1);
            return;
        }
        this.set(descriptor);
    }

    @Override
    public void coerceAndSet(Object descriptor) {
        this.set((FeatureAttributeDescriptor) descriptor);
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.combo.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled() {
        return this.combo.isEnabled();
    }

    @Override
    public String getName() {
        FeatureAttributeDescriptor descriptor = this.get();
        if( descriptor==null ) {
            return null;
        }
        return descriptor.getName();
    }

    @Override
    public void clean() {
        this.combo.setSelectedIndex(-1);
    }

    
}
