package org.gvsig.fmap.dal.swing.impl.featuretype;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.text.JTextComponent;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynField;
import org.gvsig.tools.dynobject.DynObjectValueItem;
import org.gvsig.tools.dynobject.Tags;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.util.UnmodifiableBasicMap;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("Convert2Lambda")
class TagsController {
    
    private final JTable tblTags;
    private final JComboBox cboName;
    private final JComboBox cboValue;
    private final JButton btnAdd;
    private final JButton btnUpdate;
    private final JButton btnRemove;
    private final UnmodifiableBasicMap<String, DynField> tagDefinitions;
    private final JLabel lblDescription;

    public TagsController(
            JTable tblTags, 
            JComboBox cboName, 
            JComboBox cboValue, 
            JButton btnAdd, 
            JButton btnUpdate, 
            JButton btnRemove,
            JLabel lblDescription
        ) {
        this.tblTags = tblTags;
        this.cboName = cboName;
        this.cboValue = cboValue;
        this.btnAdd = btnAdd;
        this.btnUpdate = btnUpdate;
        this.btnRemove = btnRemove;
        this.lblDescription = lblDescription;
        this.tagDefinitions = ToolsLocator.getDynObjectManager().getTags();
        this.initComponents();
    }
    
    private void initComponents() {
        ToolsSwingManager swingManager = ToolsSwingLocator.getToolsSwingManager();
        swingManager.setDefaultPopupMenu(this.cboName);
        swingManager.setDefaultPopupMenu(this.cboValue);
        
        swingManager.addClearButton(this.cboValue);

        swingManager.translate(this.btnAdd);
        swingManager.translate(this.btnUpdate);
        swingManager.translate(this.btnRemove);
        
        this.tblTags.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.tblTags.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }
                doTableSelecctionChanged();
            }
        });
        this.btnAdd.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doAdd();
            }
        });
        this.btnUpdate.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doUpdate();
            }
        });
        this.btnRemove.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doRemove();
            }
        });
        List<String> tagsNames = new ArrayList<>();
        for (String tag : this.tagDefinitions.keySet()) {
            tagsNames.add(tag);
        }
        tagsNames.sort(null);
        DefaultComboBoxModel model = new DefaultComboBoxModel();
        model.addElement(" ");
        for (String tag : tagsNames) {
            model.addElement(tag);
        }
        this.cboName.setModel(model);
        this.cboName.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if( e.getStateChange()==ItemEvent.SELECTED ) {
                    doChangeTagName();
                }
            }
        });
    }

    public void clean() {
        this.tblTags.setModel(new DefaultTableModel());
        this.cboName.setSelectedIndex(-1);
        this.cboValue.setSelectedItem("");
        this.tblTags.getSelectionModel().clearSelection();
        this.lblDescription.setText("");
    }

    public Tags fetch(Tags tags) {
        TableModel model = this.tblTags.getModel();
        tags.removeAll();
        for (int row = 0; row < model.getRowCount(); row++) {
            String name = (String) model.getValueAt(row, 0);
            String value = (String) model.getValueAt(row, 1);
            if( StringUtils.isNotBlank(name) && StringUtils.isNotBlank(value) ) {
                tags.set(name,value);
            }
        }
        return tags;
    }

    public void set(Tags tags) {
        this.lblDescription.setText("");
        DefaultTableModel model = new DefaultTableModel();
        model.addColumn("_Name");
        model.addColumn("_Value");
        for (String tagName : tags) {
            model.addRow(new String[]{tagName, Objects.toString(tags.get(tagName), "")});
        }
        this.tblTags.setModel(model);
        if (model.getRowCount() > 0) {
            this.tblTags.getSelectionModel().setSelectionInterval(0, 0);
        } else {
            this.tblTags.getSelectionModel().clearSelection();
        }
    }

    public void setEditable(boolean editable) {
        this.tblTags.setEnabled(editable);
        this.cboName.setEnabled(editable);
        this.cboValue.setEditable(editable);
        this.btnAdd.setEnabled(editable);
        this.btnUpdate.setEnabled(editable);
        this.btnRemove.setEnabled(editable);
    }

    private void doTableSelecctionChanged() {
        int row = this.tblTags.getSelectedRow();
        if (row < 0) {
            this.cboName.setSelectedIndex(-1);
            this.cboValue.setSelectedItem("");
            this.btnAdd.setEnabled(true);
            this.btnUpdate.setEnabled(false);
            this.btnRemove.setEnabled(false);
            return;
        }
        DefaultTableModel model = (DefaultTableModel) this.tblTags.getModel();
        this.cboName.setSelectedItem(model.getValueAt(row, 0));
        this.cboValue.setSelectedItem((String) model.getValueAt(row, 1));
        this.btnAdd.setEnabled(true);
        this.btnUpdate.setEnabled(true);
        this.btnRemove.setEnabled(true);
        doChangeTagName();
    }

    private void doAdd() {
        I18nManager i18n = ToolsLocator.getI18nManager();        
        this.lblDescription.setText("");
        String[] value = new String[] { 
            i18n.getTranslation("_Name"), 
            i18n.getTranslation("_Value")
        };
        DefaultTableModel model = (DefaultTableModel) this.tblTags.getModel();
        model.addRow(value);
        this.tblTags.getSelectionModel().setSelectionInterval(0, model.getRowCount() - 1);
    }

    private void doUpdate() {
        this.lblDescription.setText("");
        int row = this.tblTags.getSelectedRow();
        if (row < 0) {
            return; // EEhh?? esto no deberia pasar
        }
        String name = StringUtils.defaultIfBlank((String) this.cboName.getSelectedItem(), "");
        Object value = this.cboValue.getSelectedItem();
        if( value instanceof LabeledValue ) {
            value = ((LabeledValue)value).getValue();
        }
        DefaultTableModel model = (DefaultTableModel) this.tblTags.getModel();
        model.setValueAt(name, row, 0);
        model.setValueAt(Objects.toString(value,""), row, 1);
    }

    private void doRemove() {
        this.lblDescription.setText("");
        int n = this.tblTags.getSelectedRow();
        if (n < 0) {
            return; // EEhh?? esto no deberia pasar
        }
        DefaultTableModel model = (DefaultTableModel) this.tblTags.getModel();
        model.removeRow(n);
        if (model.getRowCount() < 0) {
            this.tblTags.getSelectionModel().clearSelection();
        } else {
            if (--n < 0) {
                n = 0;
            }
            this.tblTags.getSelectionModel().setSelectionInterval(n, n);
        }
    }

    private void doChangeTagName() {
        String tagname = (String) this.cboName.getSelectedItem();
        if( StringUtils.isBlank(tagname) ) {
            this.lblDescription.setText("");
            return;
        }
        for (DynField tagDefinition : tagDefinitions) {
            if( tagname.equalsIgnoreCase(tagDefinition.getName()) ) {
                this.lblDescription.setText("<html>"+tagDefinition.getDescription()+"<html>");
                DefaultComboBoxModel model = new DefaultComboBoxModel();
                LabeledValue[] values = tagDefinition.getAvailableValues();
                if( values != null ) {
                    for (LabeledValue value : values) {
                        model.addElement(value);
                    }
                }
                this.cboValue.setModel(model);
                return;
            }
        }
        this.lblDescription.setText("");
    }
}
