/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.swing.impl.jdbc;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Objects;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.filedialogchooser.FileDialogChooser;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataServerExplorerPool;
import org.gvsig.fmap.dal.DataServerExplorerPoolEntry;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.dataStoreParameters.DataStoreParametersPanel;
import org.gvsig.fmap.dal.swing.dataStoreParameters.DataStoreParametersPanelManager;
import org.gvsig.fmap.dal.swing.impl.dataStoreParameters.DataStoreDynObjectParametersPanel;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionDialog;
import org.gvsig.tools.folders.FoldersManager;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.swing.api.DropDown;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.util.LabeledValueImpl;




@SuppressWarnings("UseSpecificCatch")
public class DefaultJDBCConnectionDialog extends DefaultJDBCConnectionDialogView implements JDBCConnectionDialog {

    private static final long serialVersionUID = -5493563028200403559L;

    private static final Logger LOG = LoggerFactory
        .getLogger(DefaultJDBCConnectionDialog.class);

    private boolean isCanceled = false;
    private JDBCConnectionPanel jdbcServerExplorer;
    private DropDown ddnMore;

    public DefaultJDBCConnectionDialog() {
        initComponents();
    }
    
    private void initComponents() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ToolsSwingManager toolsSwingManager = ToolsSwingLocator.getToolsSwingManager();
        this.btnAcept.setText(i18n.getTranslation("ok"));
        this.btnAcept.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                doAcept();
            }
        });
        this.btnCancel.setText(i18n.getTranslation("cancel"));
        this.btnCancel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent ae) {
                doCancel();
            }
        });
        DefaultComboBoxModel<LabeledValue<String>> model = new DefaultComboBoxModel<>();
        model.addElement(new LabeledValueImpl<>(i18n.getTranslation("advanced"), "advanced"));
        model.addElement(new LabeledValueImpl<>(i18n.getTranslation("delete"), "delete"));
        model.addElement(new LabeledValueImpl<>(i18n.getTranslation("load"), "load"));
        model.addElement(new LabeledValueImpl<>(i18n.getTranslation("save"), "save"));
        this.ddnMore = toolsSwingManager.createDropDown(this.btnMore);
        this.ddnMore.setModel(model);
        this.ddnMore.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                LabeledValue<String> selectedItem = (LabeledValue<String>) ddnMore.getSelectedItem();
                switch(selectedItem.getValue()) {
                    case "advanced":
                        doAdvanced();
                        break;
                    case "delete":
                        doDelete();
                        break;
                    case "load":
                        doLoad();
                        break;
                    case "save":
                        doSave();
                        break;
                }
            }
        });

        this.jdbcServerExplorer = DALSwingLocator.getSwingManager().createJDBCConnectionPanel();
        this.containerJDBCConnectionPanel.setLayout(new BorderLayout());
        this.containerJDBCConnectionPanel.add(
                this.jdbcServerExplorer.asJComponent(),
                BorderLayout.CENTER
        );
    }

    protected void doLoad() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
        FoldersManager folderManager = ToolsLocator.getFoldersManager();
        DataManager dataManager = DALLocator.getDataManager();

        File initialPath = folderManager.getLastPath("CONNECTIONS", null);
        File[] f = dialogs.showChooserDialog(
                i18n.getTranslation("_Load_connection"), 
                FileDialogChooser.OPEN_DIALOG, 
                FileDialogChooser.FILES_ONLY,
                false, 
                initialPath, 
                new FileFilter() {
                    @Override
                    public boolean accept(File f) {
                        return Objects.toString(f,"").toLowerCase().endsWith(".gvdbcon");
                    }

                    @Override
                    public String getDescription() {
                        return "Database connection (gvdbcon)";
                    }
                }, 
                true
        );
        if( ArrayUtils.isEmpty(f) || f[0]==null ) {
            return;
        }
        folderManager.setLastPath("CONNECTIONS", f[0].getParentFile());
        InputStream in = null;
        try {
            in = new FileInputStream(f[0]);
            PersistenceManager persistence = ToolsLocator.getPersistenceManager();
            DataServerExplorerPoolEntry entry = (DataServerExplorerPoolEntry) persistence.getObject(in);
            LabeledValueImpl<JDBCServerExplorerParameters> parameters = new LabeledValueImpl<>(
                    entry.getName(), 
                    (JDBCServerExplorerParameters)entry.getExplorerParameters()
            );
            this.jdbcServerExplorer.setLabeledServerExplorerParameters(parameters);
        } catch(Throwable th) {
            LOG.warn("Problems loading connection",th);
            dialogs.messageDialog(
                    i18n.getTranslation("_Problems_loading_the_connection"), 
                    i18n.getTranslation("_Load_connection"), 
                    JOptionPane.WARNING_MESSAGE
            );
//            return;
        } finally {
            IOUtils.closeQuietly(in);
        }
    }
    
    protected void doSave() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
        FoldersManager folderManager = ToolsLocator.getFoldersManager();
        DataManager dataManager = DALLocator.getDataManager();

        String problems = this.jdbcServerExplorer.getParametersProblems();
        if( !StringUtils.isBlank(problems) ) {
            dialogs.messageDialog(
                    i18n.getTranslation("_Some_values_are_invalids")+"\n\n"+problems,
                    i18n.getTranslation("_Save_connection"),
                    JOptionPane.WARNING_MESSAGE
            );
            return;
        }
        
        LabeledValue<JDBCServerExplorerParameters> parameters = this.getLabeledServerExplorerParameters();
        if( parameters == null || parameters.getValue()==null ) {
            dialogs.messageDialog(
                    i18n.getTranslation("_There_is_no_connection_to_save"), 
                    i18n.getTranslation("_Save_connection"), 
                    JOptionPane.INFORMATION_MESSAGE
            );
            return;
        }
        File initialPath = folderManager.getLastPath("CONNECTIONS", null);
        File[] f = dialogs.showChooserDialog(
                i18n.getTranslation("_Save_connection"), 
                FileDialogChooser.SAVE_DIALOG, 
                FileDialogChooser.FILES_ONLY,
                false, 
                initialPath, 
                new FileFilter() {
                    @Override
                    public boolean accept(File f) {
                        return Objects.toString(f,"").toLowerCase().endsWith(".gvdbcon");
                    }

                    @Override
                    public String getDescription() {
                        return "Database connection (gvdbcon)";
                    }
                }, 
                true
        );
        if( ArrayUtils.isEmpty(f) || f[0]==null ) {
            return;
        }
        folderManager.setLastPath("CONNECTIONS", f[0].getParentFile());

        File paramsFile = f[0];
        if( !f[0].toString().toLowerCase().endsWith(".gvdbcon") ) {
            paramsFile = new File(f[0].toString()+".gvdbcon");
        }

        DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();
        DataServerExplorerPoolEntry entry = pool.createEntry(
                parameters.getLabel(), 
                null, 
                parameters.getValue()
        );
        
        OutputStream out = null;
        try {
            out = new FileOutputStream(paramsFile);
            PersistenceManager persistence = ToolsLocator.getPersistenceManager();
            PersistentState state = persistence.getState(entry);
            persistence.saveState(state, out);
        } catch(Throwable th) {
            LOG.warn("Problems saving connection",th);
            dialogs.messageDialog(
                    i18n.getTranslation("_Problems_saving_the_connection"), 
                    i18n.getTranslation("_Save_connection"), 
                    JOptionPane.WARNING_MESSAGE
            );
//            return;
        } finally {
            IOUtils.closeQuietly(out);
        }
    }
    
    protected void doAdvanced() {
        JDBCServerExplorerParameters myParams = this.getServerExplorerParameters();
        try {
            myParams.validate();
        } catch (Exception e) {
            // ignore... only for fill default values
        }
        DataStoreParametersPanelManager paramsManager = DALSwingLocator.getDataStoreParametersPanelManager();
        DataStoreParametersPanel panel =  new DataStoreDynObjectParametersPanel(myParams);
        paramsManager.showPropertiesDialog(myParams, panel);
    }
    
    protected void doAcept() {
        this.isCanceled = false;
        String problems = this.jdbcServerExplorer.getParametersProblems();
        if( !StringUtils.isBlank(problems) ) {
            I18nManager i18n = ToolsLocator.getI18nManager();
            ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
            dialogs.messageDialog(
                    i18n.getTranslation("_Some_values_are_invalids")+"\n\n"+problems,
                    i18n.getTranslation("_Connection"),
                    JOptionPane.WARNING_MESSAGE
            );
            return;
        }
        this.setVisible(false);
    }
    
    protected void doCancel() {
        this.isCanceled = true;
        this.setVisible(false);
    }
    
    protected void doDelete() {
    	this.jdbcServerExplorer.delete();
    	this.jdbcServerExplorer.clear();
    }
    
    @Override
    public void showDialog() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        WindowManager winmgr = ToolsSwingLocator.getWindowManager();
        winmgr.showWindow(
                this, 
                i18nManager.getTranslation("_Connection_parameters"), 
                WindowManager.MODE.DIALOG
        );
    }
    
    @Override
    public boolean isCanceled() {
        return this.isCanceled;
    }
    
    @Override
    public JDBCServerExplorerParameters getServerExplorerParameters() {
        return this.jdbcServerExplorer.getServerExplorerParameters();
    }
    
    @Override
    public LabeledValue<JDBCServerExplorerParameters> getLabeledServerExplorerParameters() {
        return this.jdbcServerExplorer.getLabeledServerExplorerParameters();
    }
    
    @Override
    public String getConnectionName() {
        return this.jdbcServerExplorer.getConnectionName();
    }
   
}

