package org.gvsig.fmap.dal.swing.impl.jdbc;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Objects;
import javax.swing.ComboBoxModel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataFactory;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataServerExplorerParameters;
import org.gvsig.fmap.dal.DataServerExplorerPool;
import org.gvsig.fmap.dal.DataServerExplorerPoolEntry;
import org.gvsig.fmap.dal.exception.ValidateDataParametersException;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynField;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.util.LabeledValueImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

// org.gvsig.fmap.mapcontrol.dal.jdbc.JDBCConnectionPanel
@SuppressWarnings("UseSpecificCatch")
public class DefaultJDBCConnectionPanel extends DefaultJDBCConnectionPanelView implements JDBCConnectionPanel{

    private static final Logger LOGGER = LoggerFactory.getLogger(DefaultJDBCConnectionPanel.class);
    private static final long serialVersionUID = -6119489353793596382L;
    private JDBCServerExplorerParameters forcedParameters;
    private ItemListener onChangeConnectorItemListener;
    private ConnectorFilter connectorFilter=null;

    @Override
    public JComponent asJComponent() {
        return this;
    }
    
//    private static class ServerExplorerParametersComboItem {
//
//        private JDBCServerExplorerParameters params;
//        private String label;
//
//        public ServerExplorerParametersComboItem(String label, JDBCServerExplorerParameters params) {
//            this.params = params;
//            this.label = label;
//        }
//
//        public ServerExplorerParametersComboItem(JDBCServerExplorerParameters params) {
//            this(params.getExplorerName(), params);
//        }
//
//        @Override
//        public String toString() {
//            return this.label;
//        }
//
//        public JDBCServerExplorerParameters getParams() {
//            return this.params;
//        }
//
//        public String getLabel() {
//            return this.label;
//        }
//    }

    @SuppressWarnings("OverridableMethodCallInConstructor")
    public DefaultJDBCConnectionPanel() {
        initComponents();
    }

    protected void initComponents() {
        this.cboConnections.setEditable(true);
        this.cboConnections.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if(e.getStateChange() != ItemEvent.SELECTED) {
                    return;
                }                // Lo hago asi para evitar cuelgues durante la depuracion
                // al poner puntos de ruptura en un evento de un combo.
                SwingUtilities.invokeLater(new Runnable() {

                    @Override
                    public void run() {
                        onChangeConnection();
                    }
                });
            }
        });
        this.onChangeConnectorItemListener = new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if(e.getStateChange() != ItemEvent.SELECTED) {
                    return;
                }
                // Lo hago asi para evitar cuelgues durante la depuracion
                // al poner puntos de ruptura en un evento de un combo.
                SwingUtilities.invokeLater(new Runnable() {

                    @Override
                    public void run() {
                        onChangeConnector();
                    }
                });
            }
        };
        this.cboConnectors.addItemListener(onChangeConnectorItemListener);
        try {
            fillConnections();
            fillConnectors();
        } catch(Throwable th) {
            // Ignore it to allow use in GUI builders
            LOGGER.warn("Can't fill combos",th);
        }
        this.translate();
        this.btnDataBaseBrowser.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                onBrowseDatabase();
            }
        });
        adjustPreferedHeight(this.lblFoother,300);
    }

    private void adjustPreferedHeight(JLabel label, int prefWidth) {
        JLabel resizer = new JLabel();
        resizer.setText(label.getText());
        View view = (View) resizer.getClientProperty(BasicHTML.propertyKey);
        view.setSize(prefWidth,15);
        float w = view.getPreferredSpan(View.X_AXIS);
        float h = view.getPreferredSpan(View.Y_AXIS);
        Dimension dim = new Dimension(
                (int) Math.ceil(w),
                (int) Math.ceil(h)
        );
        label.setPreferredSize(dim);
    }
    
    private void translate() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        
        this.lblFile.setText(i18nManager.getTranslation("_File"));
        this.lblConnectionName.setText(i18nManager.getTranslation("_Connection_name"));
        this.lblConnector.setText(i18nManager.getTranslation("_Driver_type"));
        this.lblServer.setText(i18nManager.getTranslation("_Host"));
        this.lblPort.setText(i18nManager.getTranslation("_Port"));
        this.lblDataBase.setText(i18nManager.getTranslation("_Database"));
        this.lblUsername.setText(i18nManager.getTranslation("_User"));
        this.lblPassword.setText(i18nManager.getTranslation("_Password"));
        this.lblFoother.setText("<html>"+i18nManager.getTranslation("_JDBCConecctionPanel_foother")+"</html>");
    }

    @Override
    public void setLabeledServerExplorerParameters(LabeledValue<JDBCServerExplorerParameters> parameters) {
        if( parameters == null ) {
            return;
        }
        if( parameters.getValue()!=null ) {
            this.setServerExplorerParameters(parameters.getValue());
        }
        if( !StringUtils.isBlank(parameters.getLabel()) ) {
            this.cboConnections.setSelectedItem(parameters.getLabel());
        }
    }
    
    @Override
    public void setServerExplorerParameters(JDBCServerExplorerParameters parameters) {
        this.forcedParameters = (JDBCServerExplorerParameters) parameters.getCopy();
        
        int indexConnector = this.getIndexOfConnector(parameters);
        if ( indexConnector >= 0 && this.cboConnectors.getSelectedIndex()!=indexConnector ) {
            this.cboConnectors.removeItemListener(this.onChangeConnectorItemListener);
            this.cboConnectors.setSelectedIndex(indexConnector);
            this.cboConnectors.addItemListener(onChangeConnectorItemListener);
        }

        this.txtServer.setText(parameters.getHost());
        Integer port = parameters.getPort();
        if ( port == null ) {
            this.txtPort.setText("");
        } else {
            this.txtPort.setText(String.valueOf(port));
        }
        this.txtDataBase.setText(parameters.getDBName());
        this.txtUsername.setText(parameters.getUser());
        this.txtPassword.setText(parameters.getPassword());
        
        DynClass definition = parameters.getDynClass();
        
        DynField field = definition.getDynField(JDBCServerExplorerParameters.HOST_PARAMTER_NAME);
        this.txtServer.setEnabled( !( field.isReadOnly() || field.isHidden() ));
        field = definition.getDynField(JDBCServerExplorerParameters.PORT_PARAMTER_NAME);
        this.txtPort.setEnabled( !( field.isReadOnly() || field.isHidden() ));
        field = definition.getDynField(JDBCServerExplorerParameters.DBNAME_PARAMTER_NAME);
        this.txtDataBase.setEnabled( !( field.isReadOnly() || field.isHidden() ));
        field = definition.getDynField(JDBCServerExplorerParameters.USER_PARAMTER_NAME);
        this.txtUsername.setEnabled( !( field.isReadOnly() || field.isHidden() ));
        field = definition.getDynField(JDBCServerExplorerParameters.PASSWORD_PARAMTER_NAME);
        this.txtPassword.setEnabled( !( field.isReadOnly() || field.isHidden() ));
        if( parameters instanceof FilesystemStoreParameters ) {
            File f = ((FilesystemStoreParameters) parameters).getFile();
            if( f==null ) {
                this.txtFile.setText("");
            } else {
                this.txtFile.setText(f.getAbsolutePath());
            }
            this.btnDataBaseBrowser.setVisible(true);
            this.txtFile.setVisible(true);
            this.lblFile.setVisible(true);
        } else {
            this.btnDataBaseBrowser.setVisible(false);
            this.txtFile.setVisible(false);
            this.lblFile.setVisible(false);
        }
    }

    @Override
    public JDBCServerExplorerParameters getServerExplorerParameters() {
        LabeledValue<JDBCServerExplorerParameters> labeledConnection = this.getLabeledServerExplorerParameters();
        if( labeledConnection==null ) {
            return null;
        }
        return labeledConnection.getValue();
    }
    
    @Override
    public LabeledValue<JDBCServerExplorerParameters> getLabeledServerExplorerParameters() {
        JDBCServerExplorerParameters params;
        JDBCServerExplorerParameters connector = this.getConnector();
        if( connector==null ) {
            return null;
        }
        if( this.forcedParameters==null ) {
            params = (JDBCServerExplorerParameters) connector.getCopy();
        } else {
            params = (JDBCServerExplorerParameters) this.forcedParameters.getCopy();
        }
        String s = this.getServer();
        if( s!=null ) {
            params.setHost(s);
        }
        int n = this.getPort();
        if( n>0 ) {
            params.setPort(n);
        }
        s = this.getDataBaseName();
        if( s!=null ) {
            params.setDBName(s);
        }
        s = this.getUsername();
        if( s!=null ) {
            params.setUser(s);
        }
        s = this.getPassword();
        if( s!=null ) {
            params.setPassword(s);
        }
                
        String name = this.getConnectionName();
        if ( StringUtils.isBlank(name) ) {
            if( params instanceof FilesystemStoreParameters ) {
                File f = ((FilesystemStoreParameters) params).getFile();
                if( f!=null ) {
                    name = FilenameUtils.getBaseName(f.getPath());
                }
            }
            if( StringUtils.isBlank(name) ) {
                StringBuilder builder = new StringBuilder();
                builder.append(Objects.toString(params.getHost(),""));
                if( builder.length()>0 && !builder.toString().endsWith("-")) {
                    builder.append("-");
                }
                builder.append(Objects.toString(params.getPort(),""));
                if( builder.length()>0 && !builder.toString().endsWith("-")) {
                    builder.append("-");
                }
                builder.append(Objects.toString(params.getDBName(),""));
                name = builder.toString();
            }
        }
        
        if( StringUtils.isBlank(name) ) {
            name = getUniqueEntryName("Connection");
        } else if( this.isNewConnection() ) {
            name = getUniqueEntryName(name);
        }

        try {
            params.validate();
            DataManager dataManager = DALLocator.getDataManager();
            DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();
            pool.add(name, params);
        } catch (ValidateDataParametersException ex) {
            
        }
        
        return new LabeledValueImpl<>(name,params);
    }
    
    @Override
    public boolean areTheParametersValid() {
        try {
            LabeledValue<JDBCServerExplorerParameters> params = this.getLabeledServerExplorerParameters();
            params.getValue().validate();
            return true;
        } catch (ValidateDataParametersException ex) {
            return false;
        }
    }
    
    @Override
    public String getParametersProblems() {
        try {
            I18nManager i18n = ToolsLocator.getI18nManager();
            LabeledValue<JDBCServerExplorerParameters> params = this.getLabeledServerExplorerParameters();
            if( params == null ) {
                return i18n.getTranslation("You need to select a connector");
            }
            params.getValue().validate();
            return null;
        } catch (ValidateDataParametersException ex) {
            return ex.getLocalizedMessageStack();
        }
    }
    
    private String getUniqueEntryName(String name) {
        DataManager dataManager = DALLocator.getDataManager();
        DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();
        String baseName = name;
        for (int i = 0; i < 1000; i++) {
            if( !pool.contains(name) ) {
                return name;
            }
            name = baseName+"-"+i;
        }
        return name + System.currentTimeMillis();
    }

    protected void setConnectionName(String connectionName) {
        JTextField txtConnections = (JTextField) this.cboConnections.getEditor().getEditorComponent();
        txtConnections.setText(connectionName);
    }

    @Override
    public String getConnectionName() {
        Object value = this.cboConnections.getSelectedItem();
        return StringUtils.defaultIfBlank(Objects.toString(value, null), null);
    }

    @Override
    public boolean isNewConnection() {
        String name = this.getConnectionName();
        if( name == null ) {
            return true;
        }
        ComboBoxModel<LabeledValue<JDBCServerExplorerParameters>> model = this.cboConnections.getModel();
        for (int i = 0; i < model.getSize(); i++) {
            LabeledValue<JDBCServerExplorerParameters> conn = model.getElementAt(i);
            if( StringUtils.equalsIgnoreCase(name, conn.getLabel()) ) {
                return false;
            }
        }
        return true;
    }
    
    protected JDBCServerExplorerParameters getConnector() {
        LabeledValue<JDBCServerExplorerParameters> item = (LabeledValue<JDBCServerExplorerParameters>) this.cboConnectors.getSelectedItem();
        if( item == null ) {
            return null;
        }
        return item.getValue();
    }

    protected String getConnectorName() {
        JDBCServerExplorerParameters value = this.getConnector();
        if ( value == null ) {
            return null;
        }
        return (String) StringUtils.defaultIfBlank(value.getExplorerName(), null);
    }

    protected String getServer() {
        return (String) StringUtils.defaultIfBlank(this.txtServer.getText(), null);
    }

    protected int getPort() {
        String svalue = (String) StringUtils.defaultIfBlank(this.txtPort.getText(), null);
        int ivalue;
        try {
            ivalue = Integer.parseInt(svalue);
        } catch (Exception ex) {
            ivalue = -1;
        }
        return ivalue;
    }

    protected String getDataBaseName() {
        return (String) StringUtils.defaultIfBlank(this.txtDataBase.getText(), null);
    }

    protected String getUsername() {
        return (String) StringUtils.defaultIfBlank(this.txtUsername.getText(), null);
    }

    protected String getPassword() {
        return (String) StringUtils.defaultIfBlank(this.txtPassword.getText(), null);
    }

    private void onChangeConnector() {
        LabeledValue<JDBCServerExplorerParameters> item = (LabeledValue<JDBCServerExplorerParameters>) this.cboConnectors.getSelectedItem();
        if( item==null ) {
            return;
        }
        JDBCServerExplorerParameters connector = item.getValue();
        
        if ( connector == null ) {
            return;
        }
        this.setServerExplorerParameters(connector);
    }

    private void onChangeConnection() {
        Object value = this.cboConnections.getSelectedItem();
        if( value instanceof LabeledValue ) {
            LabeledValue<JDBCServerExplorerParameters> item = (LabeledValue<JDBCServerExplorerParameters>) value;
            JDBCServerExplorerParameters connection = item.getValue();
            if ( connection == null ) {
                return;
            }
            this.setServerExplorerParameters(connection);
        }
    }
    

    private int getIndexOfConnector(JDBCServerExplorerParameters explorerParameters) {
        String code = null;
        try {
            code = explorerParameters.toString();
            ComboBoxModel model = this.cboConnectors.getModel();
            for ( int i = 0; i < model.getSize(); i++ ) {
                LabeledValue<JDBCServerExplorerParameters> x = (LabeledValue<JDBCServerExplorerParameters>) model.getElementAt(i);
                if ( x!=null && x.getValue()!=null && x.getValue().getExplorerName().equalsIgnoreCase(explorerParameters.getExplorerName()) ) {
                    return i;
                }
            }
        } catch (Exception ex) {
            LOGGER.warn("Can't get index of exporer parameter '" + code + "'.", ex);
        }
        return -1;
    }
    
    @Override
    public void setConnectorFilter(ConnectorFilter connectorFilter) {
        this.connectorFilter = connectorFilter;
        fillConnections();
        fillConnectors();        
    }

    private void fillConnectors() {
        DataManager dataManager = DALLocator.getDataManager();

        LabeledValue<JDBCServerExplorerParameters> last = null;
        this.cboConnectors.removeAllItems();
        this.cboConnectors.addItem(new LabeledValueImpl("",(JDBCServerExplorerParameters)null));
        
        for (DataFactory factory :  dataManager.getServerExplorerRegister() ) {
            if( factory.isHidden() ) {
                continue;
            }
            DataServerExplorerParameters params = (DataServerExplorerParameters) factory.createParameters();
            if ( params instanceof JDBCServerExplorerParameters ) {
                if( this.connectorFilter==null || this.connectorFilter.accept((JDBCServerExplorerParameters) params) ) {
                    if( params.getClass() == JDBCServerExplorerParameters.class && 
                        last == null ) {
                        // Si es el proveedor generico de JDBC, lo guarda para 
                        // aadirlo al final del combo.
                        last = new LabeledValueImpl(
                                factory.getDescription(),
                                (JDBCServerExplorerParameters) params
                        );
                    } else {
                        this.cboConnectors.addItem(
                            new LabeledValueImpl(
                                    factory.getDescription(),
                                    (JDBCServerExplorerParameters) params
                            )
                        );
                    }
                }
            }
        }
        if( last!=null ) {
            this.cboConnectors.addItem(last);
        }
    }

    private void fillConnections() {
        DataManager dataManager = DALLocator.getDataManager();
        DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();

        this.cboConnections.removeAllItems();
        for (DataServerExplorerPoolEntry entry : pool) {
            if ( entry.getExplorerParameters() instanceof JDBCServerExplorerParameters ) {
                if( this.connectorFilter==null || this.connectorFilter.accept((JDBCServerExplorerParameters) entry.getExplorerParameters()) ) {
                    JDBCServerExplorerParameters dbParams = (JDBCServerExplorerParameters) entry.getExplorerParameters();
                    this.cboConnections.addItem(
                            new LabeledValueImpl<>(entry.getName(), dbParams)
                    );
                }
            }
        }
        this.cboConnections.setSelectedIndex(-1);
    }

    @Override
    public void delete() {
        String name = this.getConnectionName();
        DataManager dataManager = DALLocator.getDataManager();
        DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();
        
        pool.remove(name);
        fillConnections();
    }
    
    @Override
    public void clear() {
        this.cboConnections.setSelectedIndex(-1);
        this.cboConnectors.setSelectedIndex(-1);
        this.txtServer.setText("");
        this.txtPort.setText("");
        this.txtDataBase.setText("");
        this.txtUsername.setText("");
        this.txtPassword.setText("");
        this.txtFile.setText("");
    }
    
    private void onBrowseDatabase() {
        if( this.forcedParameters==null ) {
            return;
        }
        ThreadSafeDialogsManager dlgmanager = ToolsSwingLocator.getThreadSafeDialogsManager();
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        File[] files = dlgmanager.showOpenFileDialog(
                i18nManager.getTranslation("_Select_database"), 
                null
        );
        if( ArrayUtils.isEmpty(files) ) {
            return;
        }
        ((FilesystemStoreParameters)this.forcedParameters).setFile(files[0]);
        this.txtFile.setText(((FilesystemStoreParameters)(this.forcedParameters)).getFile().getAbsolutePath());
    }
}
