package org.gvsig.fmap.dal.swing.impl.jdbc;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.net.URL;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataServerExplorerParameters;
import org.gvsig.fmap.dal.DataServerExplorerPool;
import org.gvsig.fmap.dal.DataServerExplorerPoolEntry;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionDialog;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.pickercontroller.AbstractPickerController;
import org.gvsig.tools.swing.api.pickercontroller.PickerController;
import org.gvsig.tools.swing.icontheme.IconTheme;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.util.LabeledValueImpl;

/**
 *
 * @author jjdelcerro
 */
public class JDBCConnectionPickerController 
        extends AbstractPickerController<JDBCServerExplorerParameters>
        implements PickerController<JDBCServerExplorerParameters>
    {

    private final JComboBox cboConnection;
    private final JButton btnConnection;

    public JDBCConnectionPickerController(JComboBox cboConnection, JButton btnConnection) {
        this.btnConnection = btnConnection;
        this.cboConnection = cboConnection;
    
        this.fillConnections(this.cboConnection);
        this.btnConnection.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doAddConnection();
            }
        });
        if( "...".equals(this.btnConnection.getText()) ) {
            this.btnConnection.setText("");
        }
        IconTheme theme = ToolsSwingLocator.getIconThemeManager().getCurrent();
        this.btnConnection.setIcon(theme.get("database-connection-add"));
        
        this.cboConnection.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    fireChangeEvent();
                }
            }
        });
    }
    
    private void fillConnections(JComboBox combo) {
        DataManager dataManager = DALLocator.getDataManager();
        DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();

        DataServerExplorerParameters params;
        combo.removeAllItems();
        for (DataServerExplorerPoolEntry entry : pool) {
            if (entry.getExplorerParameters() instanceof JDBCServerExplorerParameters) {
                JDBCServerExplorerParameters dbParams = (JDBCServerExplorerParameters) entry.getExplorerParameters();
                combo.addItem(
                        new LabeledValueImpl<>(entry.getName(), dbParams)
                );
            }
        }
        combo.setSelectedIndex(-1);
    }

    private void doAddConnection() {
        JDBCConnectionDialog newco = DALSwingLocator.getSwingManager().createJDBCConectionDialog();
        newco.showDialog();
        if (newco.isCanceled()) {
            return;
        }
        DataManager dataManager = DALLocator.getDataManager();
        DataServerExplorerPool pool = dataManager.getDataServerExplorerPool();

        pool.add(newco.getConnectionName(), newco.getServerExplorerParameters());
        this.fillConnections(this.cboConnection);
    }
    
    @Override
    public JDBCServerExplorerParameters get() {
        LabeledValueImpl<JDBCServerExplorerParameters> item = (LabeledValueImpl<JDBCServerExplorerParameters>) this.cboConnection.getSelectedItem();
        if (item == null) {
            return null;
        }
        JDBCServerExplorerParameters conn = item.getValue();
        return conn;
    }

    @Override
    public void set(JDBCServerExplorerParameters value) {
        ComboBoxModel<LabeledValue<JDBCServerExplorerParameters>> model = this.cboConnection.getModel();
        for (int i = 0; i < model.getSize(); i++) {
            JDBCServerExplorerParameters params = model.getElementAt(i).getValue();
            if( params == value ) {
                this.cboConnection.setSelectedIndex(i);
                return;
            }
        }
    }

    @Override
    public void coerceAndSet(Object value) {
        JDBCServerExplorerParameters params;
        try {
            params = (JDBCServerExplorerParameters) value;
        } catch(Exception ex) {
            throw new IllegalArgumentException("Can't coerce value to JDBCServerExplorerParameters", ex);
        }
        this.set(params);
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.cboConnection.setEnabled(enabled);
        this.btnConnection.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled() {
        return this.cboConnection.isEnabled();
    }
    
    public static void selfRegister() {
        String[][] iconNames = new String[][]{
            new String[]{"dalswing", "database-connection-add"}
        };
        IconTheme theme = ToolsSwingLocator.getIconThemeManager().getCurrent();
        for (String[] icon : iconNames) {
            URL url = JDBCConnectionPickerController.class.getResource(icon[1] + ".png");
            theme.registerDefault("DALSwing", icon[0], icon[1], null, url);
        }
        
    }
    
    
}
