/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */

package org.gvsig.fmap.geom.jts;

import org.cresques.cts.ICRSFactory;
import org.cresques.cts.IProjection;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.Geometry.DIMENSIONS;
import org.gvsig.fmap.geom.InformationbuilderWithGeometrySupport;
import org.gvsig.fmap.geom.InformationbuilderWithGeometrySupport.EnvelopeElement;
import org.gvsig.fmap.geom.InformationbuilderWithGeometrySupport.PropertyElementWithGeometrySupport;
import org.gvsig.fmap.geom.InformationbuilderWithGeometrySupport.RawElementWithGeometrySupport;
import org.gvsig.fmap.geom.InformationbuilderWithGeometrySupport.TextElementWithGeometrySupport;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.util.impl.BaseInformationBuilder;


public class BaseInformationBuilderWithGeometrySupport extends BaseInformationBuilder implements InformationbuilderWithGeometrySupport {

    protected class BaseRawElementWithGeometrySupport extends BaseRawElement implements RawElementWithGeometrySupport {

        @Override
        public RawElementWithGeometrySupport value(Geometry geom) {
            return (RawElementWithGeometrySupport) asWKT(this, geom);
        }

        @Override
        public RawElementWithGeometrySupport value(IProjection proj) {
            return (RawElementWithGeometrySupport) asAbrev(this, proj);
        }

        @Override
        public RawElementWithGeometrySupport valueAsWKT(IProjection proj) {
            return (RawElementWithGeometrySupport) asWKT(this, proj);
        }

        @Override
        public RawElementWithGeometrySupport value(Envelope env) {
            return (RawElementWithGeometrySupport) asWKT(this, env);
        }
    }

    protected class BaseTextElementWithGeometrySupport extends BaseTextElement implements TextElementWithGeometrySupport {

        @Override
        public TextElementWithGeometrySupport value(Geometry geom) {
            return (TextElementWithGeometrySupport) asWKT(this, geom);
        }

        @Override
        public TextElementWithGeometrySupport value(IProjection proj) {
            return (TextElementWithGeometrySupport) asAbrev(this, proj);
        }

        @Override
        public TextElementWithGeometrySupport valueAsWKT(IProjection proj) {
            return (TextElementWithGeometrySupport) asWKT(this, proj);
        }

        @Override
        public TextElementWithGeometrySupport value(Envelope env) {
            return (TextElementWithGeometrySupport) asWKT(this, env);
        }

    }

    protected class BasePropertyElementWithGeometrySupport extends BasePropertyElement implements PropertyElementWithGeometrySupport {
        
        @Override
        public PropertyElementWithGeometrySupport value(Geometry geom) {
            return (PropertyElementWithGeometrySupport) asWKT(this, geom);
        }

        @Override
        public PropertyElementWithGeometrySupport value(IProjection proj) {
            return (PropertyElementWithGeometrySupport) asWKT(this, proj);
        }

        @Override
        public PropertyElementWithGeometrySupport valueAsWKT(IProjection proj) {
            return (PropertyElementWithGeometrySupport) asWKT(this, proj);
        }

        @Override
        public PropertyElementWithGeometrySupport value(Envelope env) {
            return (PropertyElementWithGeometrySupport) asWKT(this, env);
        }
    }

    private class DefaultEnvelopeElement extends BaseRawElement implements EnvelopeElement {

        private Envelope env;

        @Override
        public DefaultEnvelopeElement value(Envelope value) {
            this.env = value;
            asWKT(this,value);
            return this;
        }

        @Override
        public String build() {
            if( env == null ) {
                return new BaseTextElementWithGeometrySupport().value("None").build();
            } else {
                PropertyElement p1 = new BasePropertyElementWithGeometrySupport();
                p1.labelkey("_upper_left_corner");
                p1.value("%+f, %+f", env.getMinimum(DIMENSIONS.X), env.getMaximum(DIMENSIONS.Y));
                p1.monospace();

                PropertyElement p2 = new BasePropertyElementWithGeometrySupport();
                p2.labelkey("_upper_right_corner");
                p2.value("%+f, %+f", env.getMaximum(DIMENSIONS.X), env.getMaximum(DIMENSIONS.Y));
                p2.monospace();

                PropertyElement p3 = new BasePropertyElementWithGeometrySupport();
                p3.labelkey("_lower_right_corner");
                p3.value("%+f, %+f", env.getMaximum(DIMENSIONS.X), env.getMinimum(DIMENSIONS.Y));
                p3.monospace();

                PropertyElement p4 = new BasePropertyElementWithGeometrySupport();
                p4.labelkey("_lower_left_corner");
                p4.value("%+f, %+f", env.getMinimum(DIMENSIONS.X), env.getMinimum(DIMENSIONS.Y));
                p4.monospace();

                return p1.build() + p2.build() + p3.build() + p4.build();
            }
        }

    }

    public BaseInformationBuilderWithGeometrySupport() {
        super();
    }

    @Override
    public PropertyElementWithGeometrySupport property() {
        PropertyElementWithGeometrySupport element = new BasePropertyElementWithGeometrySupport();
        this.elements.add(element);
        return element;
    }

    @Override
    public TextElementWithGeometrySupport text() {
        TextElementWithGeometrySupport element = new BaseTextElementWithGeometrySupport();
        this.elements.add(element);
        return element;
    }

    @Override
    public EnvelopeElement envelope() {
        EnvelopeElement element = new DefaultEnvelopeElement();
        this.elements.add(element);
        return element;
    }

    @Override
    public RawElementWithGeometrySupport raw() {
        RawElementWithGeometrySupport element = new BaseRawElementWithGeometrySupport();
        this.elements.add(element);
        return element;
    }

    @Override
    public InformationbuilderWithGeometrySupport backgroundColor(String color) {
        return (InformationbuilderWithGeometrySupport) super.backgroundColor(color); 
    }

    @Override
    public InformationbuilderWithGeometrySupport backgroundColorTitle(String color) {
        return (InformationbuilderWithGeometrySupport) super.backgroundColorTitle(color); 
    }

    @Override
    public InformationbuilderWithGeometrySupport backgroundColorPropertyLabel(String color) {
        return (InformationbuilderWithGeometrySupport) super.backgroundColorPropertyLabel(color); 
    }

    @Override
    public InformationbuilderWithGeometrySupport backgroundColorPropertyValue1(String color) {
        return (InformationbuilderWithGeometrySupport) super.backgroundColorPropertyValue1(color); 
    }
    
    @Override
    public InformationbuilderWithGeometrySupport backgroundColorPropertyValue2(String color) {
        return (InformationbuilderWithGeometrySupport) super.backgroundColorPropertyValue2(color); 
    }
    
    protected Element asAbrev(Element e, IProjection proj) {
        if (proj == null) {
            return e.value("");
        }
        return e.value(proj.getAbrev());
    }

    protected Element asWKT(Element e, IProjection proj) {
        if (proj == null) {
            return e.value("");
        }
        String s = proj.export(ICRSFactory.FORMAT_WKT);
        if (s == null) {
            s = proj.export(ICRSFactory.FORMAT_WKT_ESRI);
        }
        if (s == null) {
            s = proj.export(ICRSFactory.FORMAT_PROJ4);
        }
        if (s == null) {
            s = proj.getAbrev();
        }
        if (s != null) {
            s = s.replaceAll("\\[", "[\n  ");
            s = s.replaceAll("]", "\n]");
        }
        return e.value(s);
    }

    protected Element asWKT(Element e, Envelope env) {
        if (env == null) {
            return e.value("");
        }
        return asWKT(e, env.getGeometry());
    }

    protected Element asWKT(Element e, Geometry geom) {
        String s = "";
        try {
            s = geom.convertToWKT();
        } catch (Exception ex) {
        }
        return e.value(s);
    }

}
