/* gvSIG. Desktop Geographic Information System.
 *
 * Copyright  2007-2015 gvSIG Association
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.geom.jts.primitive.surface.polygon;

import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.vividsolutions.jts.geom.Coordinate;
import java.awt.Shape;
import java.awt.geom.GeneralPath;

import org.cresques.cts.ICoordTrans;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryException;
import org.gvsig.fmap.geom.aggregate.MultiLine;
import org.gvsig.fmap.geom.aggregate.MultiPolygon;
import org.gvsig.fmap.geom.jts.GeometryJTS;
import org.gvsig.fmap.geom.jts.aggregate.MultiLine2D;
import org.gvsig.fmap.geom.jts.aggregate.MultiPolygon2D;
import org.gvsig.fmap.geom.jts.gputils.DefaultGeneralPathX;
import org.gvsig.fmap.geom.jts.primitive.curve.line.BaseLine2D;
import org.gvsig.fmap.geom.jts.primitive.curve.line.Line2D;
import org.gvsig.fmap.geom.jts.primitive.point.Point2D;
import org.gvsig.fmap.geom.jts.primitive.ring.Ring2D;
import org.gvsig.fmap.geom.jts.util.JTSUtils;
import org.gvsig.fmap.geom.operation.GeometryOperationException;
import org.gvsig.fmap.geom.operation.GeometryOperationNotSupportedException;
import org.gvsig.fmap.geom.primitive.Line;
import org.gvsig.fmap.geom.primitive.Point;
import org.gvsig.fmap.geom.primitive.Polygon;
import org.gvsig.fmap.geom.primitive.Ring;
import org.gvsig.fmap.geom.primitive.SurfaceAppearance;

/**
 * @author fdiaz
 *
 */
public class Polygon2D extends BaseLine2D implements Polygon {

    /**
     *
     */
    private static final long serialVersionUID = 4927309852756014929L;

    List<Ring> interiorRings = new ArrayList<Ring>();

    /**
     * @param subtype
     */
    public Polygon2D() {
        super(Geometry.TYPES.POLYGON);
    }

    /**
    *
    */
    public Polygon2D(Coordinate[] coordinates) {
        super(Geometry.TYPES.POLYGON, coordinates);
        closePrimitive();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.gvsig.fmap.geom.Geometry#cloneGeometry()
     */
    public Polygon cloneGeometry() {
        Polygon2D cloned = new Polygon2D(cloneCoordinates().toCoordinateArray());
        for (int i = 0; i < getNumInteriorRings(); i++){
            cloned.addInteriorRing((Ring)getInteriorRing(i).cloneGeometry());
        }
        return cloned;
    }

    @Override
    @SuppressWarnings("CloneDoesntCallSuperClone")
    public Polygon clone() throws CloneNotSupportedException {
        return this.cloneGeometry();
    }
    
    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.fmap.geom.primitive.Surface#setSurfaceAppearance(org.gvsig.
     * fmap.geom.primitive.SurfaceAppearance)
     */
    public void setSurfaceAppearance(SurfaceAppearance app) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     *
     * @see org.gvsig.fmap.geom.primitive.Surface#getSurfaceAppearance()
     */
    public SurfaceAppearance getSurfaceAppearance() {
        // TODO Auto-generated method stub
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.gvsig.fmap.geom.primitive.Surface#getNumInteriorRings()
     */
    public int getNumInteriorRings() {
        return interiorRings.size();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.gvsig.fmap.geom.primitive.Surface#getInteriorRing(int)
     */
    public Ring getInteriorRing(int index) {
        return interiorRings.get(index);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.fmap.geom.primitive.Surface#addInteriorRing(org.gvsig.fmap.
     * geom.primitive.Ring)
     */
    public void addInteriorRing(Ring ring) {
        interiorRings.add(ring);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.fmap.geom.primitive.Surface#addInteriorRing(org.gvsig.fmap.
     * geom.primitive.Line)
     */
    public void addInteriorRing(Line line) {
        Ring2D ring = new Ring2D();
        ring.ensureCapacity(line.getNumVertices());
        for(int i=0; i<line.getNumVertices(); i++){
            Point vertex = line.getVertex(i);
            ring.addVertex(new Point2D(vertex.getX(), vertex.getY()));
        }
        ring.closePrimitive();
        interiorRings.add(ring);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.fmap.geom.primitive.Surface#addInteriorRing(org.gvsig.fmap.
     * geom.primitive.Polygon)
     */
    public void addInteriorRing(Polygon polygon) {
        Ring2D ring = new Ring2D();
        ring.ensureCapacity(polygon.getNumVertices());
        for(int i=0; i<polygon.getNumVertices(); i++){
            Point vertex = polygon.getVertex(i);
            ring.addVertex(new Point2D(vertex.getX(), vertex.getY()));
        }
        ring.closePrimitive();
        interiorRings.add(ring);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.gvsig.fmap.geom.primitive.Surface#removeInteriorRing(int)
     */
    public void removeInteriorRing(int index) {
        interiorRings.remove(index);
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.primitive.Line#toLines()
     */
    public MultiLine toLines() throws GeometryException {
        MultiLine multiLine = new MultiLine2D();
        multiLine.addPrimitive(new Line2D(coordinates.toCoordinateArray()));
        return multiLine;
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.primitive.Line#toPolygons()
     */
    public MultiPolygon toPolygons() throws GeometryException {
        MultiPolygon multiPolygon = new MultiPolygon2D();
        multiPolygon.addPrimitive(this);
        return multiPolygon;
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.jts.GeometryJTS#getJTS()
     */
    public com.vividsolutions.jts.geom.Geometry getJTS() {
        return JTSUtils.createJTSPolygon(coordinates, interiorRings);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.fmap.geom.Geometry#getPathIterator(java.awt.geom.AffineTransform
     * )
     */
    public PathIterator getPathIterator(AffineTransform at) {
        PolygonIterator pi = new PolygonIterator(this, at);
        return pi;
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.jts.GeometryJTS#ensureOrientation(boolean)
     */
    public boolean ensureOrientation(boolean ccw) throws GeometryOperationNotSupportedException, GeometryOperationException {
        boolean result = super.ensureOrientation(ccw);
        for (Iterator<Ring> iterator = interiorRings.iterator(); iterator.hasNext();) {
            GeometryJTS ring = (GeometryJTS) iterator.next();
            ring.ensureOrientation(!ccw);
        }
        return result;
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.jts.primitive.curve.line.AbstractLine#flip()
     */
    @Override
    public void flip() throws GeometryOperationNotSupportedException, GeometryOperationException {
        super.flip();
        for (Iterator<Ring> iterator = interiorRings.iterator(); iterator.hasNext();) {
            GeometryJTS ring = (GeometryJTS) iterator.next();
            ring.flip();
        }
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.jts.primitive.curve.line.AbstractLine#transform(java.awt.geom.AffineTransform)
     */
    @Override
    public void transform(AffineTransform at) {
        super.transform(at);
        for (Iterator<Ring> iterator = interiorRings.iterator(); iterator.hasNext();) {
            GeometryJTS ring = (GeometryJTS) iterator.next();
            ring.transform(at);
        }
    }

  /* (non-Javadoc)
  * @see org.gvsig.fmap.geom.Geometry#offset(double)
  */
    public Geometry offset(double distance) throws GeometryOperationNotSupportedException, GeometryOperationException {
        return JTSUtils.createGeometry(this.getProjection(), getJTS().buffer(distance));
    }

    /* (non-Javadoc)
     * @see org.gvsig.fmap.geom.jts.primitive.curve.line.AbstractLine#reProject(org.cresques.cts.ICoordTrans)
     */
    @Override
    public void reProject(ICoordTrans ct) {
        super.reProject(ct);
        for (Iterator<Ring> iterator = interiorRings.iterator(); iterator.hasNext();) {
            GeometryJTS ring = (GeometryJTS) iterator.next();
            ring.reProject(ct);
        }
        if (coordinates.size()>=2 && !isClosed()) {
            closePrimitive();
        }
    }
    
    @Override
    public Shape getShape(AffineTransform affineTransform) {
        return PolygonHelper.getShape(this, affineTransform);
    }    
}
