/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.geom.jts.spatialindex;

import com.vividsolutions.jts.index.quadtree.Quadtree;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang3.SerializationUtils;

import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.SpatialIndex;
import org.gvsig.fmap.geom.SpatialIndexFactory;
import org.gvsig.fmap.geom.jts.GeometryJTS;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.visitor.Visitor;

public class SpatialIndexJTSQuadtree extends AbstractSpatialIndex implements SpatialIndex {

    private com.vividsolutions.jts.index.quadtree.Quadtree index = null;
    private boolean modified;

    public SpatialIndexJTSQuadtree(GeometryManager geometryManager, SpatialIndexFactory factory, DynObject parameters) {
        super(geometryManager, factory, parameters);
        this.index = null;
        this.modified = false;
        open();
    }
    
    @Override
    public void open() {
        this.index = new com.vividsolutions.jts.index.quadtree.Quadtree();
        File f = (File) this.getParameter("file");
        if( f!=null && f.exists() ) {
            try {
                this.load(f);
            } catch (FileNotFoundException ex) {
                throw new RuntimeException("Can't load spatial index from '"+f.getAbsolutePath()+"'.");
            }
        }
    }

    @Override
    public void close() {
        File f = (File) this.getParameter("file");
        if( f!=null && modified ) {
            try {
                this.save(f);
            } catch (FileNotFoundException ex) {
                throw new RuntimeException("Can't save spatial index to '"+f.getAbsolutePath()+"'.");
            }
        }
        this.index = null;
    }

    private void load(File fname) throws FileNotFoundException {
        FileInputStream is = new FileInputStream(fname);
        Object x = SerializationUtils.deserialize(is);
        this.index = (Quadtree) x;
        this.modified = false;
    }
    
    private void save(File fname) throws FileNotFoundException {
        FileOutputStream os = new FileOutputStream(fname);
        SerializationUtils.serialize(this.index, os);
        this.modified = false;
    }
    
    private com.vividsolutions.jts.geom.Geometry asJTS(org.gvsig.fmap.geom.Geometry geom) {
        return ((GeometryJTS)geom).getJTS();
    }

    @Override
    public long size() {
        return this.index.size();
    }

    @Override
    public void query(org.gvsig.fmap.geom.primitive.Envelope envelope,
            Visitor visitor) {
        com.vividsolutions.jts.index.ItemVisitor visitor_jts = new JTSVisitorWrapper(visitor);
        com.vividsolutions.jts.geom.Envelope env_jts = asJTS(envelope.getGeometry()).getEnvelopeInternal();
        this.index.query(env_jts, visitor_jts);

    }

    @Override
    public Iterator query(org.gvsig.fmap.geom.primitive.Envelope envelope,long limit) {
        if( limit!=0 ) {
            throw new UnsupportedOperationException("Not supported yet.");
        }
        com.vividsolutions.jts.geom.Envelope env_jts = asJTS(envelope.getGeometry()).getEnvelopeInternal();
        List result = this.index.query(env_jts);
        return result.iterator();
    }

    @Override
    public Iterator queryNearest(org.gvsig.fmap.geom.primitive.Envelope envelope, long limit) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public Iterator queryAll() {
        List result = this.index.queryAll();
        return result.iterator();
    }

    @Override
    public void insert(org.gvsig.fmap.geom.primitive.Envelope envelope, Object data) {
        com.vividsolutions.jts.geom.Envelope env_jts = asJTS(envelope.getGeometry()).getEnvelopeInternal();
        index.insert(env_jts, data);
        this.modified = true;
    }

    @Override
    public boolean remove(org.gvsig.fmap.geom.primitive.Envelope envelope, Object data) {
        com.vividsolutions.jts.geom.Envelope env_jts = asJTS(envelope.getGeometry()).getEnvelopeInternal();
        this.modified = true;
        return index.remove(env_jts, data);
    }

    @Override
    public void removeAll() {
        index = new com.vividsolutions.jts.index.quadtree.Quadtree();
        this.modified = true;
    }

    private class JTSVisitorWrapper implements com.vividsolutions.jts.index.ItemVisitor {

        private Visitor visitor = null;

        public JTSVisitorWrapper(Visitor visitor) {
            this.visitor = visitor;
        }

        @Override
        public void visitItem(Object arg0) {
            try {
                this.visitor.visit(arg0);
            } catch (Exception e) {
                throw new RuntimeException();
            }
        }

    }

}
