package org.gvsig.fmap.mapcontext.layers;

import org.cresques.cts.IProjection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;

public interface LayerInformationBuilder {

    public interface Element {

        public String build();
    }

    public interface TitleElement extends Element {

        public TitleElement label(String label);
        public TitleElement labelkey(String labelkey);
    }

    public interface RawElement extends Element {

        public RawElement value(String format, Object... args);

        public RawElement value(IProjection proj);

        public RawElement value(Envelope env);

        public RawElement value(Geometry geom);

        public RawElement asWKT(IProjection proj);

        public RawElement asWKT(Envelope env);

        public RawElement asWKT(Geometry geom);
    }

    public interface TextElement extends RawElement {

        @Override
        public TextElement value(String format, Object... args);

        @Override
        public TextElement value(IProjection proj);

        @Override
        public TextElement value(Envelope env);

        @Override
        public TextElement value(Geometry geom);

        @Override
        public TextElement asWKT(IProjection proj);

        @Override
        public TextElement asWKT(Envelope env);

        @Override
        public TextElement asWKT(Geometry geom);
    }

    public interface PropertyElement extends TextElement {

        public PropertyElement monospace();
        
        public PropertyElement label(String label);

        public PropertyElement labelkey(String labelkey);

        @Override
        public PropertyElement value(String format, Object... args);

        @Override
        public PropertyElement value(IProjection proj);

        @Override
        public PropertyElement value(Envelope env);

        @Override
        public PropertyElement value(Geometry geom);

        @Override
        public PropertyElement asWKT(IProjection proj);

        @Override
        public PropertyElement asWKT(Envelope env);

        @Override
        public PropertyElement asWKT(Geometry geom);
    }

    public interface EnvelopeElement extends Element {

        public EnvelopeElement value(Envelope value);
    }

    public LayerInformationBuilder backgroundColor(String color);

    public LayerInformationBuilder backgroundColorTitle(String color);

    public LayerInformationBuilder backgroundColorPropertyLabel(String color);

    public LayerInformationBuilder backgroundColorPropertyValue1(String color);

    public LayerInformationBuilder backgroundColorPropertyValue2(String color);

    public TitleElement title();

    public PropertyElement property();

    public TextElement text();

    public RawElement raw();

    public EnvelopeElement envelope();

    public String build();

}
