/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.mapcontext.layers.vectorial;

import org.cresques.cts.IProjection;
import org.gvsig.expressionevaluator.ExpressionBuilder;

import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.evaluator.AbstractEvaluator;
import org.gvsig.tools.evaluator.EvaluatorData;
import org.gvsig.tools.evaluator.EvaluatorException;

public class ContainsEnvelopeEvaluator extends AbstractEvaluator {

    private final Envelope envelope;
    private final String geomName;
    private final Envelope envelopeTrans;
    private final boolean isDefault;
    private final ExpressionBuilder builder;
    private final IProjection projection;
    private final FeatureAttributeDescriptor fad;

    ContainsEnvelopeEvaluator(
            Envelope envelope,
            IProjection envelopeProjection,
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        fad = (FeatureAttributeDescriptor) featureType.get(geomName);
        this.isDefault = featureType.getDefaultGeometryAttributeName().equals(geomName);
        this.projection = envelopeProjection;
        this.builder = builder;
        this.envelope = envelope;
        this.envelopeTrans = envelope;
        this.geomName = geomName;
        this.getFieldsInfo().addMatchFieldValue(geomName, envelopeTrans);
    }

    @Override
    public Object evaluate(EvaluatorData data) throws EvaluatorException {
        Envelope featureEnvelope;
        if (isDefault) {
            featureEnvelope = ((Feature) data.getContextValue("feature"))
                    .getDefaultEnvelope();

        } else {
            Geometry geom = (Geometry) data.getDataValue(geomName);
            if (geom == null) {
                return false;
            }
            featureEnvelope = geom.getEnvelope();
        }
        return envelopeTrans.contains(featureEnvelope);
    }

    @Override
    public String getName() {
        return "contains in envelope";
    }

    @Override
    public String getSQL() {
        return this.builder.set(
                builder.ST_Contains(
                        builder.envelope(envelope, projection),
                        builder.column(fad.getName())
                )
        ).toString();
    }

}
