/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.mapcontext.layers.vectorial;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.tools.evaluator.AbstractEvaluator;
import org.gvsig.tools.evaluator.EvaluatorData;
import org.gvsig.tools.evaluator.EvaluatorException;


public class DisjointGeometryEvaluator extends AbstractEvaluator {

    private final String geomName;
    private final Geometry geometry;
    private final Geometry geometryTrans;
    private final boolean isDefault;
    private final ExpressionBuilder builder;
    private final IProjection projection;
    private final FeatureAttributeDescriptor fad;
        
    DisjointGeometryEvaluator(
            Geometry geometry,
            IProjection projection,
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        fad = (FeatureAttributeDescriptor) featureType.get(geomName);
        this.isDefault = featureType.getDefaultGeometryAttributeName().equals(geomName);
        this.geometry = geometry;
        this.geometryTrans = geometry.cloneGeometry();
        this.projection = projection;
        this.builder = builder;
        
        IProjection fad_proj = fad.getSRS();
        ICoordTrans ct = null;

        if (fad_proj != null && !fad_proj.equals(projection)) {
            ct = projection.getCT(fad_proj);
        }

        if (ct != null) {
            geometryTrans.reProject(ct);
        }
        this.geomName = geomName;

        this.getFieldsInfo().addMatchFieldValue(geomName, geometryTrans);

    }

    @Override
    public Object evaluate(EvaluatorData data) throws EvaluatorException {
        try {
            Geometry geom;
            if (isDefault) {
                Feature feature = (Feature) data.getContextValue("feature");
                geom = feature.getDefaultGeometry();

            } else {
                geom = (Geometry) data.getDataValue(geomName);
            }
            if (geom == null) {
                return Boolean.FALSE;
            }
            return geometryTrans.disjoint(geom);

        } catch (Exception e) {
            throw new EvaluatorException(e);
        }
    }

    @Override
    public String getName() {
        return "disjoint with geometry";
    }

    @Override
    public String getSQL() {
        return this.builder.set(
                builder.ST_Disjoint(
                        builder.geometry(geometry, projection),
                        builder.column(fad.getName())
                )
        ).toString();
    }
}
