/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.mapcontext.layers.vectorial;

import org.cresques.cts.IProjection;
import org.gvsig.expressionevaluator.ExpressionBuilder;

import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.evaluator.AbstractEvaluator;
import org.gvsig.tools.evaluator.EvaluatorData;
import org.gvsig.tools.evaluator.EvaluatorException;

public class IntersectsEnvelopeEvaluator extends AbstractEvaluator {

    private final Envelope envelope;
    private final String geomName;
    private final boolean isDefault;
    private final String defaultGeometryAttributeName;
    private final IProjection projection;
    private final ExpressionBuilder builder;
    FeatureAttributeDescriptor fad;
        
    IntersectsEnvelopeEvaluator(
            Envelope envelope,
            IProjection envelopeProjection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        fad = (FeatureAttributeDescriptor) featureType.get(geomName);
        defaultGeometryAttributeName = featureType.getDefaultGeometryAttributeName();
        this.isDefault = defaultGeometryAttributeName.equals(geomName);
        this.envelope = envelope;
        this.projection = envelopeProjection;
        this.builder = builder;
        this.geomName = geomName;
        this.getFieldsInfo().addMatchFieldValue(geomName, envelope);
    }

    @Override
    public Object evaluate(EvaluatorData data) throws EvaluatorException {
        Envelope featureEnvelope;
        Geometry geom ;
        if (isDefault) {
            geom = (Geometry) data.getDataValue(defaultGeometryAttributeName);
            featureEnvelope = ((Feature) data.getContextValue("feature")).getDefaultEnvelope();
        } else {
            geom = (Geometry) data.getDataValue(geomName);
            if (geom == null) {
                return false;
            }
            featureEnvelope = geom.getEnvelope();
        }
        if (envelope.intersects(featureEnvelope)) {
            return envelope.intersects(geom);
        }
        return false;
    }

    @Override
    public String getName() {
        return "intersects envelope";
    }

    @Override
    public String getSQL() {
        ExpressionBuilder.Variable column = builder.column(fad.getName());
        IProjection theProjection = this.projection;
        if( this.fad.getSRS()==null ) {
            theProjection = null;
        }
        return builder.set(
                builder.and(
                    builder.not_is_null(column)    ,
                    builder.ST_Intersects(
                            column,
                            builder.geometry(envelope.getGeometry(), theProjection)
                    )
                )
        ).toString();
    }
}
