package org.gvsig.fmap.mapcontext.layers.vectorial;

import org.cresques.cts.IProjection;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.evaluator.Evaluator;

public class SpatialEvaluatorsFactory {
    
    private static final SpatialEvaluatorsFactory factory = new SpatialEvaluatorsFactory();
    
    public static SpatialEvaluatorsFactory getInstance() {
        return factory;
    }
    
    private SpatialEvaluatorsFactory() {
        
    }
    
//    public ExpressionBuilder createBuilder() {
//        DataManager manager = DALLocator.getDataManager();
//        ExpressionBuilder builder = manager.createSQLBuilder();
//        return builder;
//    }
    
    private Evaluator intersects(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new IntersectsGeometryEvaluator(geometry, projection, featureType, geomName, builder);
    }
    
    private Evaluator intersects(
            Envelope envelope,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new IntersectsEnvelopeEvaluator(envelope, projection, featureType, geomName, builder);
    }
    
    public Evaluator intersects(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = (ExpressionBuilder) store.createExpressionBuilder();
            return intersects(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create intersects evaluator.",ex);
        }
    }

    public Evaluator intersects(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store,
            String geomfield
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            ExpressionBuilder builder = (ExpressionBuilder) store.createExpressionBuilder();
            return intersects(geometry, projection, featureType, geomfield, builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create intersects evaluator.",ex);
        }
    }

    public Evaluator intersects(
            Envelope envelope,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return intersects(envelope, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create intersects evaluator.",ex);
        }
    }

    private Evaluator contains(
            Envelope envelope,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ){
        return new ContainsEnvelopeEvaluator(envelope, projection, featureType, geomName, builder);
    }

    private Evaluator contains(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ){
        return new ContainsGeometryEvaluator(geometry, projection, featureType, geomName, builder);
    }
    
    public Evaluator contains(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.contains(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create contains evaluator.",ex);
        }
    }

    public Evaluator contains(
            Envelope envelope,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.contains(envelope, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create contains evaluator.",ex);
        }
    }

    private Evaluator not_contains(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ){
        return new OutGeometryEvaluator(geometry, projection, featureType, geomName, builder);
    }
    
    public Evaluator not_contains(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.not_contains(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create not_contains evaluator.",ex);
        }
    }

    public Evaluator crosses(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new CrossesGeometryEvaluator(geometry, projection, featureType, geomName, builder);
    }

    private Evaluator crosses(
            Envelope envelope,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new CrossEnvelopeEvaluator(envelope, projection, featureType, geomName, builder);
    }

    public Evaluator crosses(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.crosses(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create crosses evaluator.",ex);
        }
    }

    public Evaluator crosses(
            Envelope envelope,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.crosses(envelope, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create crosses evaluator.",ex);
        }
    }

    private Evaluator disjoint(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new DisjointGeometryEvaluator(geometry, projection, featureType, geomName, builder);
    }    

    public Evaluator disjoint(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.disjoint(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create disjoint evaluator.",ex);
        }
    }
    
    private Evaluator overlaps(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new OverlapsGeometryEvaluator(geometry, projection, featureType, geomName, builder);
    }    

    public Evaluator overlaps(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.overlaps(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create overlaps evaluator.",ex);
        }
    }
    
    private Evaluator touches(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new TouchesGeometryEvaluator(geometry, projection, featureType, geomName,builder);
    }    

    public Evaluator touches(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.touches(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create touches evaluator.",ex);
        }
    }
    
    private Evaluator within(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new WithinGeometryEvaluator(geometry, projection, featureType, geomName,builder);
    }    

    public Evaluator within(
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.within(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create within evaluator.",ex);
        }
    }
    
    private Evaluator equals(
            Geometry geometry,
            IProjection projection, 
            FeatureType featureType,
            String geomName,
            ExpressionBuilder builder
        ) {
        return new EqualsGeometryEvaluator(geometry, projection, featureType, geomName,builder);
    }    
    
    public Evaluator equals(    
            Geometry geometry,
            IProjection projection, 
            FeatureStore store
        ) {
        try {
            FeatureType featureType = store.getDefaultFeatureType();
            String geomName = featureType.getDefaultGeometryAttributeName();
            ExpressionBuilder builder = store.createExpressionBuilder();
            return this.equals(geometry, projection, featureType, geomName,builder);
        } catch (DataException ex) {
            throw new RuntimeException("Can't create equals evaluator.",ex);
        }
    }
    
}
