package org.gvsig.andami.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.gvsig.andami.Arguments;

public class DefaultArguments implements Arguments {

    private final String[] args;
    protected Map<String, String> flags;
    protected List<String> parameters;

    public DefaultArguments(String args[]) {
        this.args = args;
        this.flags = new HashMap<>();
        this.parameters = new ArrayList<>();
        this.parse();
    }

    private void parse() {
        String name;
        String value;
        boolean acceptFlags = true;
        for( String arg : args ) {
            if( acceptFlags ) {
                if( arg.equals("--") ) {
                    acceptFlags = false;
                    continue;
                }
                if( arg.startsWith("--") ) {
                    int n = arg.indexOf("=");
                    if( n < 0 ) {
                        name = arg.substring(2);
                        value = "true";
                    } else {
                        name = arg.substring(2, n);
                        value = arg.substring(n + 1);
                    }
                    this.flags.put(name, value);
                } else {
                    this.parameters.add(arg);
                }
            } else {
                this.parameters.add(arg);
            }
        }
    }

    @Override
    public String get(String name) {
        return this.flags.get(name);
    }

    @Override
    public boolean contains(String name) {
        return this.flags.containsKey(name);
    }

    @Override
    public List<String> parameters() {
        return Collections.unmodifiableList(parameters);
    }

    @Override
    public Map<String, String> flags() {
        return Collections.unmodifiableMap(flags);
    }

    @Override
    public int size() {
        return this.parameters.size();
    }

    @Override
    public String get(int n) {
        return this.parameters.get(n);
    }

    @Override
    public Iterator<String> iterator() {
        return this.parameters.iterator();
    }
}
