/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export;

import org.gvsig.export.spi.AbstractExportParameters;
import org.gvsig.export.spi.AbstractExportParametersGeometry;
import org.gvsig.export.spi.CutAttributeNamesTranslator;
import org.gvsig.export.spi.DummyAttributeNamesTranslator;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.LibraryException;
import org.gvsig.tools.locator.ReferenceNotRegisteredException;

/**
 * Library for API initialization and configuration.
 * 
 * @author gvSIG team
 * @version $Id$
 */
public class ExportLibrary extends AbstractLibrary {

    @Override
    public void doRegistration() {
        registerAsAPI(ExportLibrary.class);
    }

    @Override
    protected void doInitialize() throws LibraryException {
        // Do nothing
    }

    @Override
    protected void doPostInitialize() throws LibraryException {
        // Validate there is any implementation registered.
        ExportManager manager = ExportLocator.getManager();
        if (manager == null) {
            throw new ReferenceNotRegisteredException(
                ExportLocator.MANAGER_NAME, ExportLocator.getInstance());
        }
        
        AbstractExportParameters.registerPersistence();
        AbstractExportParametersGeometry.registerPersistence();
        DummyAttributeNamesTranslator.registerPersistence();
        CutAttributeNamesTranslator.registerPersistence();
        
    }

}
