/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export;

import org.gvsig.export.spi.ExportServiceManager;
import org.gvsig.tools.locator.BaseLocator;
import org.gvsig.tools.locator.Locator;
import org.gvsig.tools.locator.LocatorException;

/**
 * This locator is the entry point for the Exportto library, providing
 * access to all Exportto services through the {@link ExportManager} .
 * 
 * @author gvSIG team
 * @version $Id$
 */
public class ExportLocator extends BaseLocator {

    public static final String MANAGER_NAME = "Export.manager";
    public static final String MANAGER_DESCRIPTION = "Export Manager";
    
    public static final String ENTRIES_MANAGER_NAME = "Export.entriesManager";
    public static final String ENTRIES_MANAGER_DESCRIPTION = "Export Entries Manager";
    
    public static final String SERVICE_MANAGER_NAME = "Export.serviceManager";
    public static final String SERVICE_MANAGER_DESCRIPTION = "Export Service Manager";

    private static final String LOCATOR_NAME = "Export.locator";

    /**
     * Unique instance.
     */
    private static final ExportLocator INSTANCE = new ExportLocator();

    /**
     * Return the singleton instance.
     * 
     * @return the singleton instance
     */
    public static ExportLocator getInstance() {
        return INSTANCE;
    }

    /**
     * Return the Locator's name.
     * 
     * @return a String with the Locator's name
     */
    @Override
    public final String getLocatorName() {
        return LOCATOR_NAME;
    }

    /**
     * Return a reference to the ExportManager.
     * 
     * @return a reference to the ExportManager
     * @throws LocatorException
     *             if there is no access to the class or the class cannot be
     *             instantiated
     * @see Locator#get(String)
     */
    public static ExportManager getManager() throws LocatorException {
        return (ExportManager) getInstance().get(MANAGER_NAME);
    }

    /**
     * Registers the Class implementing the ExportManager interface.
     * 
     * @param clazz
     *            implementing the ExportManager interface
     */
    public static void registerManager(Class<? extends ExportManager> clazz) {
        getInstance().register(MANAGER_NAME, MANAGER_DESCRIPTION, clazz);
    }

    /**
     * Return a reference to the ExportServiceMnager.
     * 
     * @return a reference to the ExportServiceMnager
     * @throws LocatorException
     *             if there is no access to the class or the class cannot be
     *             instantiated
     * @see Locator#get(String)
     */
    public static ExportServiceManager getServiceManager() throws LocatorException {
        return (ExportServiceManager) getInstance().get(SERVICE_MANAGER_NAME);
    }

    /**
     * Registers the Class implementing the ExportServiceManager interface.
     * 
     * @param clazz
     *            implementing the ExportServiceManager interface
     */
    public static void registerServiceManager(Class<? extends ExportServiceManager> clazz) {
        getInstance().register(SERVICE_MANAGER_NAME, SERVICE_MANAGER_DESCRIPTION, clazz);
    }
    
}
