package org.gvsig.export.spi;

import java.util.HashMap;
import java.util.Map;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;

/**
 *
 * @author jjdelcerro
 */
public abstract class AbstractExportServiceFactory implements ExportServiceFactory {

    private final String name;
    private final String description;

    private static final Map<ExportServiceFactory, Boolean>serviceEnabled = new HashMap<>();
    private final String label;

    public AbstractExportServiceFactory(
            String name,
            String label,
            String description
        ) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        this.name = name;
        this.label = i18n.getTranslation(label);
        this.description = i18n.getTranslation(description);
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getLabel() {
        return label;
    }
    
    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public boolean isEnabled() {
        Boolean enabled = serviceEnabled.getOrDefault(this, true);
        return enabled;
    }

    @Override
    public void setEnabled(boolean value) {
        serviceEnabled.put(this, value);
    }
    
}
