package org.gvsig.export.spi;

import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.export.ExportAttributes;
import org.gvsig.export.ExportAttributes.ExportAttribute;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;

public class CutAttributeNamesTranslator implements AttributeNamesTranslator {

    public int maxNameLen;

    public CutAttributeNamesTranslator(int maxNameLen) {
        this.maxNameLen = maxNameLen;
    }

    public CutAttributeNamesTranslator() {

    }

    private String buildTranslatedName(String source_name, Collection<String> current_names) {

        int len = source_name.length();
        if (source_name.isEmpty()) {
            source_name = "Field";
        }
        if (len <= this.maxNameLen && !current_names.contains(source_name)) {
            /*
			 * Should not happen
             */
            return source_name;
        }
        String target_name = source_name;
        /*
		 * GDAL field name truncation method (extended from 100 to 255)
		 * THISISAVERYLONGNAME => THISISAVER, THISISAV_1 ... THISISAV_9,
		 * THISISAV10 ... THISISAV99, THISISA100 ... THISISA255
		 * (255 = max number of fields in a SHP)
         */
        for (int i = 0; i < 255; i++) {
            if (len <= this.maxNameLen) {
                if (i <= 9) {
                    if (len == this.maxNameLen) {
                        target_name = source_name.substring(0, 8) + "_" + i;
                    } else {
                        target_name = source_name + "_" + i;
                    }
                } else if (i <= 99) {
                    if (len == this.maxNameLen) {
                        target_name = source_name.substring(0, 8) + i;;
                    } else {
                        target_name = source_name + i;
                    }
                } else {
                    if (len == this.maxNameLen - 1) {
                        target_name = source_name.substring(0, 7) + i;;
                    } else {
                        target_name = source_name + i;
                    }
                }

            } else {
                if (i == 0) {
                    target_name = source_name.substring(0, 10);
                } else if (i <= 9) {
                    target_name = source_name.substring(0, 8) + "_" + i;
                } else if (i <= 99) {
                    target_name = source_name.substring(0, 8) + i;
                } else {
                    target_name = source_name.substring(0, 7) + i;
                }
            }
            if (!current_names.contains(target_name)) {
                return target_name;
            }

        }
        /*
		 * Should not get here
         */
        return source_name.substring(0, 4) + "_" + (System.currentTimeMillis() % 1000000);
    }

    @Override
    public boolean isValidName(ExportAttributes attributes, int index, String name) {
        ExportAttribute selectedAttr = attributes.get(index);
        //
        String selfName = this.getNameSuggestion(attributes, index, name);
        if (!selfName.equals(name)) {
            return false;
        }

        // check with others values
        boolean valid = true;
        for (int i = 0; i < attributes.size(); i++) {
            if (i == index) {
                continue;
            }
            ExportAttribute attr = attributes.get(i);
            String newName = attr.getNewName();
            if (newName.isEmpty()) {
                return false;
            }
            if (newName.equals(name)) {
                return false;
            }
            if (newName.equals(this.getNameSuggestion(attributes, index, name))) {
                return false;
            }
        }
        return valid;
    }

    @Override
    public String getNameSuggestion(ExportAttributes attributes, int index, String name) {
        ArrayList names = new ArrayList();
        boolean oneTime = true;
        for (ExportAttribute attribute : attributes) {
            if (attribute.getNewName().equals(name) && oneTime) {
                oneTime = false;
                continue;
            }
            names.add(attribute.getNewName());
        }
        String target_name = buildTranslatedName(name, names);
        return target_name;
    }

    @Override
    public CutAttributeNamesTranslator clone() {
        CutAttributeNamesTranslator clone = null;
        try {
            clone = (CutAttributeNamesTranslator) super.clone();
        } catch (CloneNotSupportedException ex) {
            Logger.getLogger(CutAttributeNamesTranslator.class.getName()).log(Level.SEVERE, null, ex);
        }

        return clone;

    }

    @Override
    public String getNameSuggestion(String name) {
        String s = StringUtils.left(name, maxNameLen);
        return s;
    }

    public static void registerPersistence() {
        PersistenceManager manager = ToolsLocator.getPersistenceManager();
        if (manager.getDefinition("CutAttributeNamesTranslator") == null) {
            DynStruct definition = manager.addDefinition(CutAttributeNamesTranslator.class,
                    "CutAttributeNamesTranslator", "CutAttributeNamesTranslator persistence definition", null, null);
            definition.addDynFieldInt("maxNameLen");
        }
    }

    @Override
    public void saveToState(PersistentState state) throws PersistenceException {
        state.set("maxNameLen", this.maxNameLen);
    }

    @Override
    public void loadFromState(PersistentState state) throws PersistenceException {
        this.maxNameLen = state.getInt("maxNameLen");
    }
}
