/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.spi;

import java.util.List;
import org.gvsig.export.ExportException;
import org.gvsig.export.ExportParameters;
import org.gvsig.fmap.dal.OpenDataStoreParameters;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.tools.task.MonitorableTask;

/**
 * <p>
 * This service is used to export a source {@link FeatureStore} to a target
 * {@link FeatureStore}.
 * </p>
 * <p>
 * It inherits if {@link MonitorableTask}, and it means that the export process
 * can be monitorized by one or more observers that can listen all the export
 * events.
 * <p>
 * 
 * @author gvSIG team
 * @version $Id$
 */
public interface ExportService extends MonitorableTask {

    public interface ExportFinishListener {

        public void cancelled(ExportService exportService);
        
        public void finished(ExportService exportService);
    }

    public ExportServiceFactory getFactory();
    
    public ExportParameters getParameters() ;
    
    public void addFinishListener(ExportFinishListener listener);

//    public AttributeNamesTranslator getAttributeNamesTranslator();
    
    public void export(FeatureSet featureSet) throws ExportException;

    public List<OpenDataStoreParameters> getTargetOpenStoreParameters() throws ExportException;     


}
