package org.gvsig.export.spi;

import java.util.List;
import org.cresques.cts.ICoordTrans;
import org.gvsig.export.ExportAttributes;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.ExportParametersGeometry;
import org.gvsig.export.Filter;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.tools.bookmarksandhistory.Bookmarks;
import org.gvsig.tools.bookmarksandhistory.History;

/**
 *
 * @author jjdelcerro
 */
public interface ExportServiceManager {
    
    public interface FixGeometryStatus {
        public static final int STATE_OK = 0;
        public static final int STATE_SKIP = 1;
        public static final int STATE_ABORT = 2;

        public Geometry getGeometry();
        public int getState();
        public String getMessage();
    }

    public List<ExportServiceFactory> getAllServiceFactories();

    public List<ExportServiceFactory> getServiceFactories(Filter<ExportServiceFactory> filter);
    
    public ExportServiceFactory getServiceFactory(String name);
    
    public ExportService createService(ExportParameters paramaters);
    
    public ExportParameters createServiceParameters(String name);
    
    public void register(ExportServiceFactory factory);
    
    public FixGeometryStatus fixGeometry(
            ExportParametersGeometry parameters,
            ICoordTrans coord_trans,
            Geometry geometry
    );
    
    public ExportGeometryHelper createGeometryHelper(
            ExportParametersGeometry parameters,
            FeatureType theTargetFeatureType,
            FeatureType theSourceFeatureType
    );
    
    public ExportAttributes createExportAttributes();
}
