package org.gvsig.export.impl;

import org.cresques.cts.ICoordTrans;
import org.gvsig.export.ExportParametersGeometry;
import org.gvsig.export.spi.ExportServiceManager.FixGeometryStatus;
import org.gvsig.fmap.geom.Geometry;


/**
 *
 * @author jjdelcerro
 */
public class ExportGeometryUtils {

    public static class DefaultFixGeometryStatus
            implements FixGeometryStatus 
        {

        private Geometry geometry;
        private final int state;
        private String message;

        public DefaultFixGeometryStatus(Geometry geometry, int state) {
            this.geometry = geometry;
            this.state = state;
        }

        public DefaultFixGeometryStatus(Geometry geometry) {
            this.geometry = geometry;
            this.state = STATE_OK;
        }

        public DefaultFixGeometryStatus(int state) {
            this.geometry = null;
            this.state = state;
        }

        public DefaultFixGeometryStatus(int state, String message) {
            this.geometry = null;
            this.state = state;
            this.message = message;
        }

        private void setGeometry(Geometry geometry) {
            this.geometry = geometry;
        }

        @Override
        public Geometry getGeometry() {
            return this.geometry;
        }

        @Override
        public int getState() {
            return state;
        }

        @Override
        public String getMessage() {
            return this.message;
        }

    }

    public static FixGeometryStatus fixGeometry(
            ExportParametersGeometry options,
            ICoordTrans coord_trans,
            Geometry geometry
    ) {
        Geometry.ValidationStatus geometryCheck;
        DefaultFixGeometryStatus status;

        if (geometry == null) {
            return new DefaultFixGeometryStatus(geometry);
        }
        switch (options.getGeometryChecks()) {
            case ExportParametersGeometry.CHECKGEOMETRY_CHECK_IF_CORRUPT:
                geometryCheck = geometry.getValidationStatus();
                if (geometryCheck.isValid()) {
                    status = new DefaultFixGeometryStatus(geometry);
                    break;
                }
                switch (options.getGeometryChecksAction()) {
                    case ExportParametersGeometry.CHECKGEOMETRYACTION_SET_GEOMETRY_TO_NULL:
                        status = new DefaultFixGeometryStatus(null);
                        break;
                    case ExportParametersGeometry.CHECKGEOMETRYACTION_SKIP_FEATURE:
                        status = new DefaultFixGeometryStatus(FixGeometryStatus.STATE_SKIP, geometryCheck.getMessage());
                        break;
                    case ExportParametersGeometry.CHECKGEOMETRYACTION_ABORT:
                    default:
                        status = new DefaultFixGeometryStatus(FixGeometryStatus.STATE_ABORT, geometryCheck.getMessage());
                        break;
                }
                break;

            case ExportParametersGeometry.CHECKGEOMETRY_CHECK_IF_VALID:
                geometryCheck = geometry.getValidationStatus();
                if (geometryCheck.isValid()) {
                    status = new DefaultFixGeometryStatus(geometry);
                    break;
                }
                Geometry g;
                if (options.getTryToFixGeometry()) {
                    g = geometry.makeValid();
                    if (g != null) {
                        status = new DefaultFixGeometryStatus(g);
                        break;
                    }
                }
                switch (options.getGeometryChecksAction()) {
                    case ExportParametersGeometry.CHECKGEOMETRYACTION_SET_GEOMETRY_TO_NULL:
                        status = new DefaultFixGeometryStatus(null);
                        break;
                    case ExportParametersGeometry.CHECKGEOMETRYACTION_SKIP_FEATURE:
                        status = new DefaultFixGeometryStatus(FixGeometryStatus.STATE_SKIP, geometryCheck.getMessage());
                        break;
                    case ExportParametersGeometry.CHECKGEOMETRYACTION_ABORT:
                    default:
                        status = new DefaultFixGeometryStatus(FixGeometryStatus.STATE_ABORT, geometryCheck.getMessage());
                        break;
                }
                break;

            case ExportParametersGeometry.CHECKGEOMETRY_NONE:
            default:
                status = new DefaultFixGeometryStatus(geometry);
                break;
        }
        if (coord_trans != null) {
            if (status.getState() == FixGeometryStatus.STATE_OK) {
                if (status.getGeometry() != null) {
                    Geometry reproj_geom = status.getGeometry().cloneGeometry();
                    reproj_geom.reProject(coord_trans);
                    status.setGeometry(reproj_geom);
                }
            }
        }
        return status;
    }
}
