/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.gvsig.export.impl.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.cresques.cts.ICoordTrans;
import org.gvsig.export.ExportAttributes;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.ExportParametersGeometry;
import org.gvsig.export.Filter;
import org.gvsig.export.impl.ExportGeometryUtils;
import org.gvsig.export.spi.AttributeNamesTranslator;
import org.gvsig.export.spi.ExportGeometryHelper;
import org.gvsig.export.spi.ExportService;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.export.spi.ExportServiceManager;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.tools.bookmarksandhistory.Bookmark;
import org.gvsig.tools.bookmarksandhistory.Bookmarks;
import org.gvsig.tools.bookmarksandhistory.History;
import org.gvsig.tools.bookmarksandhistory.impl.BaseBookmarks;
import org.gvsig.tools.bookmarksandhistory.impl.BaseHistory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author jjdelcerro
 */
public class DefaultExportServiceManager implements ExportServiceManager {

    private static final Logger LOG = LoggerFactory.getLogger(DefaultExportServiceManager.class);

    private final Map<String, ExportServiceFactory> factories;
    
    public DefaultExportServiceManager() {
        super();
        this.factories = new HashMap<>();
    }

    @Override
    public ExportServiceFactory getServiceFactory(String name) {
        ExportServiceFactory factory = this.factories.get(name);
        if (factory == null) {
            throw new IllegalArgumentException("Don't exists export factory '" + name + "'.");
        }
        return factory;
    }

    @Override
    public ExportService createService(ExportParameters parameters) {
        return this.getServiceFactory(parameters.getServiceName()).createService(parameters);
    }

    @Override
    public ExportParameters createServiceParameters(String name) {
        return this.getServiceFactory(name).createParameters();
    }

    @Override
    public void register(ExportServiceFactory factory) {
        this.factories.put(factory.getName(), factory);
    }

    @Override
    public List<ExportServiceFactory> getAllServiceFactories() {
        List<ExportServiceFactory> result = new ArrayList<>();
        result.addAll(this.factories.values());
        return Collections.unmodifiableList(result);
    }

    @Override
    public List<ExportServiceFactory> getServiceFactories(Filter<ExportServiceFactory> filter) {
        List<ExportServiceFactory> result = new ArrayList<>();
        if (filter == null) {
            result.addAll(this.factories.values());
        } else {
            for (ExportServiceFactory factory : this.factories.values()) {
                if (factory.isEnabled() && filter.apply(factory)) {
                    result.add(factory);
                }
            }
        }
        return Collections.unmodifiableList(result);
    }

    @Override
    public FixGeometryStatus fixGeometry(
            ExportParametersGeometry parmeters,
            ICoordTrans coord_trans,
            Geometry geometry
    ) {
        return ExportGeometryUtils.fixGeometry(parmeters, coord_trans, geometry);
    }

    @Override
    public ExportGeometryHelper createGeometryHelper(ExportParametersGeometry parameters, FeatureType theTargetFeatureType, FeatureType theSourceFeatureType) {
        DefaultExportGeometryHelper helper = new DefaultExportGeometryHelper(
                parameters,
                theTargetFeatureType,
                theSourceFeatureType
        );
        return helper;

    }

    @Override
    public ExportAttributes createExportAttributes() {
        DefaultExportAttributes exp = new DefaultExportAttributes();
        return exp;
    }
}
