/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.swing;

import org.gvsig.export.swing.spi.ExportPanelsManager;
import org.gvsig.tools.locator.BaseLocator;

/**
 * This locator is the entry point for the Exportto swing library,
 * providing access to all Exportto swing services through the
 * {@link ExportSwingManager} .
 * 
 * @author gvSIG team
 * @version $Id$
 */
public class ExportSwingLocator extends BaseLocator {

    public static final String SWING_MANAGER_NAME = "Export.swing.manager";
    public static final String SWING_MANAGER_DESCRIPTION = "Export Swing Manager";

    public static final String PANELS_MANAGER_NAME = "Export.swing.panels.manager";
    public static final String PANELS_MANAGER_DESCRIPTION = "Export Panels Manager";

    private static final String LOCATOR_NAME = "Exportto.swing.locator";

    /**
     * Unique instance.
     */
    private static final ExportSwingLocator INSTANCE =
        new ExportSwingLocator();

    /**
     * Return the singleton instance.
     * 
     * @return the singleton instance
     */
    public static ExportSwingLocator getInstance() {
        return INSTANCE;
    }

    /**
     * Return the Locator's name
     * 
     * @return a String with the Locator's name
     */
    @Override
    public final String getLocatorName() {
        return LOCATOR_NAME;
    }

    /**
     * Registers the Class implementing the ExportSwingManager interface.
     * 
     * @param clazz
     *            implementing the ExportSwingManager interface
     */
    public static void registerSwingManager(
        Class<? extends ExportSwingManager> clazz) {
        getInstance().register(SWING_MANAGER_NAME, SWING_MANAGER_DESCRIPTION,
            clazz);
    }

    /**
     * Gets the instance of the {@link ExportSwingManager} registered.
     * 
     * @return {@link ExportSwingManager}
     */
    public static ExportSwingManager getSwingManager() {
        return (ExportSwingManager) getInstance().get(SWING_MANAGER_NAME);
    }

    /**
     * Registers the Class implementing the ExportPanelsManager interface.
     * 
     * @param clazz
     *            implementing the ExportPanelsManager interface
     */
    public static void registerExportPanelsManager(
        Class<? extends ExportPanelsManager> clazz) {
        getInstance().register(PANELS_MANAGER_NAME, PANELS_MANAGER_DESCRIPTION,
            clazz);
    }

    /**
     * Gets the instance of the {@link ScriptingUIManager} registered.
     * 
     * @return {@link ScriptingUIManager}
     */
    public static ExportPanelsManager getExportPanelsManager() {
        return (ExportPanelsManager) getInstance().get(PANELS_MANAGER_NAME);
    }
}
