package org.gvsig.export.swing.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.export.spi.ExportServiceManager;
import org.gvsig.export.swing.JExportConfigPanel;
import org.gvsig.tools.swing.api.ChangeListenerHelper;
import org.gvsig.tools.swing.api.JListWithCheckbox;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;

/**
 *
 * @author jjdelcerro
 */
public class DefaultJExportConfigPanel 
        extends DefaultJExportConfigPanelView
        implements JExportConfigPanel
    {

    private JListWithCheckbox lstFactories;
    private ChangeListenerHelper changeListenerHelper;

    public DefaultJExportConfigPanel() {
        this.initComponents();
    }
    
    private void initComponents() {
        ToolsSwingManager manager = ToolsSwingLocator.getToolsSwingManager();
        
        this.changeListenerHelper = manager.createChangeListenerHelper();
        
        this.lstFactories = manager.createJListWithCheckbox(lstServiceFactories);
        ListSelectionModel checkModel = this.lstFactories.getCheckedModel();
        checkModel.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if( e.getValueIsAdjusting() ) {
                    return;
                }
                changeListenerHelper.fireEvent(new ChangeEvent(DefaultJExportConfigPanel.this));
            }
        });
        manager.translate(this.lblHeader);
        manager.translate(this.lblServiceFactories);
        this.reset();
    }
    
    @Override
    public void reset() {
        ExportServiceManager manager = ExportLocator.getServiceManager();
        List<ExportServiceFactory> factories = manager.getAllServiceFactories();

        int index = 0;
        ListSelectionModel checkModel = this.lstFactories.getCheckedModel();
        checkModel.clearSelection();

        DefaultListModel model = new DefaultListModel();
        for (ExportServiceFactory factory : factories) {
            model.addElement(new ListElement(factory.getLabel(), factory));
            if( factory.isEnabled() ) {
                checkModel.addSelectionInterval(index, index);
            }
            index++;
        }
        this.lstFactories.setModel(model);
    }
    
    @Override
    public void apply() {
        ExportServiceManager manager = ExportLocator.getServiceManager();
        List<ExportServiceFactory> factories = manager.getAllServiceFactories();

        ListSelectionModel checkModel = this.lstFactories.getCheckedModel();
        int index = 0;
        for (ExportServiceFactory factory : factories) {
            factory.setEnabled(checkModel.isSelectedIndex(index));
            index++;
        }
    }

    @Override
    public List<ExportServiceFactory> getAllFactories() {
        ExportServiceManager manager = ExportLocator.getServiceManager();
        List<ExportServiceFactory> factories = manager.getAllServiceFactories();
        return Collections.unmodifiableList(factories);
    }
    
    
    @Override
    public List<ExportServiceFactory> getSelectedFactories() {
        List<ExportServiceFactory> result = new ArrayList<>();
        
        ExportServiceManager manager = ExportLocator.getServiceManager();
        List<ExportServiceFactory> factories = manager.getAllServiceFactories();

        ListSelectionModel checkModel = this.lstFactories.getCheckedModel();
        int index = 0;
        for (ExportServiceFactory factory : factories) {
            if( checkModel.isSelectedIndex(index) ) {
                result.add(factory);
            }
            index++;
        }        
        return result;
    } 
    
    public List<String> getSelectedNames() {
        List<String> result = new ArrayList<>();
        
        for (ExportServiceFactory selectedFactory : this.getSelectedFactories()) {
            result.add(selectedFactory.getName());
        }
        return result;
    } 

    @Override
    public JComponent asJComponent() {
        return this;
    }

    @Override
    public void addChangeListener(ChangeListener cl) {
        this.changeListenerHelper.addChangeListener(cl);
    }

    @Override
    public ChangeListener[] getChangeListeners() {
        return this.changeListenerHelper.getChangeListeners();
    }

    @Override
    public void removeChangeListener(ChangeListener cl) {
        this.changeListenerHelper.removeChangeListener(cl);
    }

    @Override
    public void removeAllChangeListener() {
        this.changeListenerHelper.removeAllChangeListener();
    }

    @Override
    public boolean hasChangeListeners() {
        return this.changeListenerHelper.hasChangeListeners();
    }
    
}
