/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.swing.impl;

import javax.swing.JPanel;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.gvsig.gui.beans.wizard.panel.NotContinueWizardException;
import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;

/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class WizardOptionPanelAdapter implements OptionPanel {

    private static final Logger LOG = LoggerFactory.getLogger(WizardOptionPanelAdapter.class);

    private final ExportPanel exportPanel;
    private final DefaultJExportProcessPanel processPanel;

    public WizardOptionPanelAdapter(DefaultJExportProcessPanel processPanel, ExportPanel exportPanel) {
        super();
        this.exportPanel = exportPanel;
        this.processPanel= processPanel;
    }
    
    @Override
    public String getPanelTitle() {
        return exportPanel.getTitlePanel();
    }

    @Override
    public void nextPanel() throws NotContinueWizardException {
        I18nManager i18n = ToolsLocator.getI18nManager();
        boolean valid;
        try {
            valid = this.exportPanel.validatePanel();
        } catch (ExportPanelValidationException ex) {
            LOG.info("Export form, invalid values.", ex);
            throw new NotContinueWizardException(
                    i18n.getTranslation("_Invalid_values_in_form"),
                    ex, 
                    exportPanel.asJComponent()
            );
        }
        if( !valid ) {
            throw new NotContinueWizardException(
                    i18n.getTranslation("_Invalid_values_in_form"),
                    exportPanel.asJComponent(),
                    true
            );
        }
        try {
            this.processPanel.setLastAcction(JExportProcessPanel.BUTTON_NEXT);
            this.exportPanel.nextPanel();
        } catch (Exception ex) {
            LOG.warn("Fail the call to nextPanel.", ex);
            MessagePanel.showMessage(
                    i18n.getTranslation("_Warning"),
                    i18n.getTranslation("_There_have_been_problems_retrieving_panel_data") +
                            " (" + this.exportPanel.getTitlePanel() +")" ,
                    ex,
                    null
            );
        }
    }

    @Override
    public void lastPanel() {
        try {
            this.processPanel.setLastAcction(JExportProcessPanel.BUTTON_BACK);
            this.exportPanel.previousPanel();
        } catch (Exception ex) {
            LOG.warn("Fail the call to previousPanel.", ex);
            I18nManager i18nManager = ToolsLocator.getI18nManager();
            MessagePanel.showMessage(
                    i18nManager.getTranslation("_Warning"),
                    i18nManager.getTranslation("_There_have_been_problems_filling_data_in_panel") +
                            " (" + this.exportPanel.getTitlePanel() +")" ,
                    ex,
                    null
            );
        }
    }

    @Override
    public void updatePanel() {
        try {
            this.exportPanel.enterPanel();
        } catch (Exception ex) {
            LOG.warn("Fail the call to enterPanel.", ex);
            I18nManager i18nManager = ToolsLocator.getI18nManager();
            MessagePanel.showMessage(
                    i18nManager.getTranslation("_Warning"),
                    i18nManager.getTranslation("_There_have_been_problems_filling_data_in_panel") +
                            " (" + this.exportPanel.getTitlePanel() +")" ,
                    ex,
                    null
            );
        }
    }

    @Override
    public JPanel getJPanel() {
        return (JPanel) exportPanel.asJComponent();
    }

}
