package org.gvsig.export.swing.impl.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComponent;
import org.gvsig.export.ExportParameters;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.export.ExportParametersGeometry;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.export.swing.spi.ExportPanelsManager;

public class CheckGeometriesPanel 
        extends CheckGeometriesPanelView 
        implements ExportPanel 
    {

    private final static Logger logger = LoggerFactory.getLogger(CheckGeometriesPanel.class);

    private final CheckAction[] checkActions = {
        new CheckAction("_Set_geometry_to_null", ExportParametersGeometry.CHECKGEOMETRYACTION_SET_GEOMETRY_TO_NULL),
        new CheckAction("_Skip_entire_feature", ExportParametersGeometry.CHECKGEOMETRYACTION_SKIP_FEATURE),
        new CheckAction("_Abort_process", ExportParametersGeometry.CHECKGEOMETRYACTION_ABORT)
    };
    private final ExportParametersGeometry parameters;
    private boolean updateComponents;
    
    private static class CheckAction {

        private final String label;
        private final int code;

        public CheckAction(String label, int code) {
            this.code = code;
            this.label = ToolsLocator.getI18nManager().getTranslation(label);
        }
        @Override
        public String toString() {
            return this.label;
        }
        public int getCode() {
            return this.code;
        }
    }
        
    public CheckGeometriesPanel(
            JExportProcessPanel processPanel,
            ExportParameters parameters
        ) {
        this.parameters = (ExportParametersGeometry) parameters;
        this.parameters.setGeometryChecks(ExportParametersGeometry.CHECKGEOMETRY_NONE);
        this.parameters.setGeometryChecksAction(ExportParametersGeometry.CHECKGEOMETRYACTION_SET_GEOMETRY_TO_NULL);
        this.parameters.setTryToFixGeometry(true);
        this.updateComponents = true;
        initComponents();
    }
    
    private void initComponents() {
        this.rdbCheckIsCorrupt.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
               onChangeRadioButtons();
            }
        });
        this.rdbCheckIsValid.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
               onChangeRadioButtons();
            }
        });
        this.rdbCheckNone.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
               onChangeRadioButtons();
            }
        });
        for (CheckAction checkAction : checkActions) {
            this.cboActionIsCorrupt.addItem(checkAction);
            this.cboActionIsValid.addItem(checkAction);            
        }
        this.rdbCheckNone.setSelected(true);
        this.rdbCheckIsValid.setSelected(true);
        this.cboActionIsValid.setSelectedItem(checkActions[2]);
        this.translate();
    }
    
    private void translate() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
                    
        this.lblHeader.setText("<html>"+i18nManager.getTranslation("_check_geometries_before_export_header")+"</html>");
        this.rdbCheckNone.setText(i18nManager.getTranslation("_No_realizar_ninguna_comprobacion_sobre_las_geometrias"));
        this.rdbCheckIsCorrupt.setText(i18nManager.getTranslation("_Comprobar_que_la_geometria_no_este_corrupta"));
        this.rdbCheckIsValid.setText(i18nManager.getTranslation("_Comprobar_la_validez_de_las_geometrias_antes_de_insertarlas"));
        this.lblCheckIsCorrupt.setText(i18nManager.getTranslation("_Cuando_la_geometria_este_corrupta"));
        this.lblCheckIsvalid.setText(i18nManager.getTranslation("_Cuando_la_geometria_no_sea_valida"));
        this.chkTryFixGeometry.setText(i18nManager.getTranslation("_Intentar_reparar_la_geometria"));
    }

    @Override
    public String getIdPanel() {
        return ExportPanelsManager.PANEL_CHECK_GEOMETRIES;
    }
    
    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_check_geometries_before_export");    
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void nextPanel() {
        this.parameters.setGeometryChecks(this.getGeometryChecks());
        this.parameters.setGeometryChecksAction(this.getGeometryChecksAction());
        this.parameters.setTryToFixGeometry(this.getTryToFixGeometry());
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void enterPanel() {
        if( this.updateComponents ) {
            switch(this.parameters.getGeometryChecks()) {
                case ExportParametersGeometry.CHECKGEOMETRY_NONE:
                    this.rdbCheckNone.setSelected(true);
                    this.rdbCheckIsCorrupt.setSelected(false);
                    this.cboActionIsCorrupt.setEnabled(false);
                    this.rdbCheckIsValid.setSelected(false);
                    this.cboActionIsValid.setEnabled(false);
                    this.chkTryFixGeometry.setEnabled(false);
                    break;
                case ExportParametersGeometry.CHECKGEOMETRY_CHECK_IF_VALID:
                    this.rdbCheckNone.setSelected(false);
                    this.rdbCheckIsCorrupt.setSelected(false);
                    this.cboActionIsCorrupt.setEnabled(false);
                    this.rdbCheckIsValid.setSelected(true);
                    this.cboActionIsValid.setEnabled(true);
                    this.chkTryFixGeometry.setEnabled(true);
                    break;
                case ExportParametersGeometry.CHECKGEOMETRY_CHECK_IF_CORRUPT:
                    this.rdbCheckNone.setSelected(false);
                    this.rdbCheckIsCorrupt.setSelected(true);
                    this.cboActionIsCorrupt.setEnabled(true);
                    this.rdbCheckIsValid.setSelected(false);
                    this.cboActionIsValid.setEnabled(false);
                    this.chkTryFixGeometry.setEnabled(false);
                    break;
            }
            this.chkTryFixGeometry.setSelected( this.parameters.getTryToFixGeometry() );
        }
        this.updateComponents = false;
        if( this.parameters.getSourceGeometryFieldName()==null ) {
            this.rdbCheckIsCorrupt.setEnabled(false);
            this.rdbCheckIsValid.setEnabled(false);
            this.rdbCheckNone.setEnabled(false);
        } else {
            this.rdbCheckIsCorrupt.setEnabled(true);
            this.rdbCheckIsValid.setEnabled(true);
            this.rdbCheckNone.setEnabled(true);
        }
    }
    
    @Override
    public JComponent asJComponent() {
        return this;
    }

    public int getGeometryChecks() {
        if( this.rdbCheckIsCorrupt.isSelected() ) {
            return ExportParametersGeometry.CHECKGEOMETRY_CHECK_IF_CORRUPT;
        } else if( this.rdbCheckIsValid.isSelected()) {
            return ExportParametersGeometry.CHECKGEOMETRY_CHECK_IF_VALID;
        } else {
            return ExportParametersGeometry.CHECKGEOMETRY_NONE;
        }
    }

    public int getGeometryChecksAction() {
        try {
            if( this.rdbCheckIsCorrupt.isSelected() ) {
                CheckAction checkAction = (CheckAction)(this.cboActionIsCorrupt.getSelectedItem());
                return checkAction.getCode();
            } else if( this.rdbCheckIsValid.isSelected()) {
                CheckAction checkAction = (CheckAction)(this.cboActionIsValid.getSelectedItem());
                return checkAction.getCode();
            } else {
                return ExportParametersGeometry.CHECKGEOMETRYACTION_ABORT;
            }
        } catch(Exception ex) {
            logger.warn("Can't get action for geometry check in export to JDBC.",ex);
            return ExportParametersGeometry.CHECKGEOMETRYACTION_ABORT;
        }
    }

    public boolean getTryToFixGeometry() {
        return this.chkTryFixGeometry.isSelected();
    }
    
    private void onChangeRadioButtons() {
        if( this.rdbCheckNone.isSelected() ) {
            this.cboActionIsCorrupt.setEnabled(false);
            this.cboActionIsValid.setEnabled(false);
            this.chkTryFixGeometry.setEnabled(false);
        } else if( this.rdbCheckIsCorrupt.isSelected() ) {
            this.cboActionIsCorrupt.setEnabled(true);
            this.cboActionIsValid.setEnabled(false);
            this.chkTryFixGeometry.setEnabled(false);
        } else if( this.rdbCheckIsValid.isSelected()) {
            this.cboActionIsCorrupt.setEnabled(false);
            this.cboActionIsValid.setEnabled(true);
            this.chkTryFixGeometry.setEnabled(true);
        }
    }
}
