package org.gvsig.export.swing.impl.panels;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.JComponent;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.bookmarksandhistory.Bookmarks;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue;
import org.gvsig.tools.swing.api.bookmarkshistory.BookmarksController;
import org.gvsig.tools.swing.api.bookmarkshistory.BookmarksController.BookmarkEvent;
import org.gvsig.tools.swing.api.task.JTaskStatus;
import org.gvsig.tools.swing.api.task.TaskStatusSwingManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author jjdelcerro
 */
public class ExportProgressPanel
        extends ExportProgressPanelView
        implements ExportPanel {

    private final static Logger LOGGER = LoggerFactory.getLogger(ExportProgressPanel.class);

    private final JExportProcessPanel processPanel;
    private final ExportParameters parameters;
    private JTaskStatus jTaskStatus;
    private BookmarksController bookmarksController;

    public ExportProgressPanel(
            JExportProcessPanel processPanel,
            ExportParameters parameters
    ) {
        this.processPanel = processPanel;
        this.parameters = parameters;
        this.initComponents();
    }

    private void initComponents() {
        TaskStatusSwingManager TaskStatusSwingManager = ToolsSwingLocator.getTaskStatusSwingManager();

        this.jTaskStatus = TaskStatusSwingManager.createJTaskStatus();
        this.jpnMain.setLayout(new BorderLayout());
        this.jpnMain.add(jTaskStatus, BorderLayout.NORTH);
        this.jTaskStatus.bind(this.processPanel.getProcess().getTaskStatus());

        Bookmarks<ExportParameters> bookmarks = ExportLocator.getManager().getBookmarks();
        this.bookmarksController = ToolsSwingLocator.getToolsSwingManager().createBookmarksController(bookmarks, this.btnBookmark);
        this.bookmarksController.setAllowAddBookmarks(true);
        this.bookmarksController.setFilter(null);
        this.bookmarksController.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    if (e.getID() == ActionEventWithCurrentValue.ID_GETVALUE) {
                        BookmarkEvent bookmarkEvent = (BookmarkEvent) e;
                        ExportParameters clone = parameters.clone();
                        clone.setCreationDate(new Date());
                        bookmarkEvent.setCurrentValue(clone);
                    }
                } catch (CloneNotSupportedException ex) {
                    LOGGER.warn("Can't clone export parameters.", ex);
                }
            }
        });
    }

    @Override
    public String getIdPanel() {
        return "ExportProgressPanel";
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("export_progress");
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void enterPanel() {
//        this.jTaskStatus.setCurValue(0);

    }

    @Override
    public void nextPanel() {
        processPanel.setButtonEnabled(JExportProcessPanel.BUTTON_FINISH, false);
    }

    @Override
    public void previousPanel() {
        processPanel.setButtonEnabled(JExportProcessPanel.BUTTON_FINISH, true);
        processPanel.setButtonEnabled(JExportProcessPanel.BUTTON_CANCEL, true);
        processPanel.setButtonText(JExportProcessPanel.BUTTON_CANCEL, ToolsLocator.getI18nManager().getTranslation("_Cancel"));
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

}
