/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.gvsig.export.swing.impl.panels;

import java.nio.charset.Charset;
import java.util.Map;
import java.util.SortedMap;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.export.ExportCharsetParameters;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynField;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.dynobject.exception.DynFieldNotFoundException;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author osc
 */
public class SelectCharsetPanel extends SelectCharsetPanelView
        implements ExportPanel {

    private static final Logger LOG = LoggerFactory.getLogger(SelectCharsetPanel.class);

    private final JExportProcessPanel processPanel;
    private final ExportParameters parameters;

    public SelectCharsetPanel(JExportProcessPanel processPanel, ExportParameters parameters) {
        super();
        this.processPanel = processPanel;
        this.parameters = parameters;
        this.initComponents();
    }

    private void initComponents() {
        DynStruct parametersDefinition = ToolsLocator.getPersistenceManager().getDefinition("DBFStoreParameters");
        DynField dynField = parametersDefinition.getDynField("Encoding");

        DefaultListModel model = new DefaultListModel();
        SortedMap<String, Charset> availableCharsets = Charset.availableCharsets();
        for (Map.Entry<String, Charset> charset : availableCharsets.entrySet()) {
            model.addElement(charset.getKey());
        }

        this.lstCharset.setModel(model);
        this.lstCharset.setSelectedIndex(0);

        this.translate();
    }

    private void translate() {
        ToolsSwingManager i18nc = ToolsSwingLocator.getToolsSwingManager();
        i18nc.translate(this.lblCharset);
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Select_encoding");
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void enterPanel() {
        ExportCharsetParameters params = (ExportCharsetParameters) this.parameters;
        if (!StringUtils.isEmpty(params.getCharset())) {
            String charset = params.getCharset();
            ListElement.setSelected(this.lstCharset, charset);
            return;
        }
        try {
            // TODO ?
            //CSVStoreParameters;
            String sourceCharset = (String) this.parameters.getSourceFeatureStore().getParameters().getDynValue("charset");
            if (sourceCharset != null) {
                this.lstCharset.setSelectedValue(sourceCharset, true);
                return;
            }
        } catch (DynFieldNotFoundException e) {
            LOG.info("Can't find charset dyn value from source store");
        }

        this.lstCharset.setSelectedValue(Charset.forName("UTF-8").displayName(), true);

    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        ExportCharsetParameters params = (ExportCharsetParameters) this.parameters;
        params.setCharset((String) this.lstCharset.getSelectedValue());
    }
}
