package org.gvsig.export.swing.impl.panels;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComponent;
import org.apache.commons.lang3.ArrayUtils;
import org.gvsig.export.ExportParameters;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.type.GeometryType;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.export.ExportParametersGeometry;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.export.swing.spi.ExportPanelsManager;

/**
 *
 * @author jjdelcerro
 */
public class SelectGeometryTypePanel 
        extends SelectGeometryTypePanelView 
        implements ExportPanel 
    {

    private final ExportParametersGeometry parameters;
    private int[] geometryTypes;
    private int[] geometrySubtypes;
    private final Map<Integer, String> geometryTypeNames;
    private final Map<Integer, String> geometrySubtypeNames;
    private boolean updateComponents;
    private final JExportProcessPanel processPanel;

            
    public SelectGeometryTypePanel(
            JExportProcessPanel processPanel,
            ExportParametersGeometry parameters
        ) {
        this(processPanel,parameters,null,null);
    }

    public SelectGeometryTypePanel(
            JExportProcessPanel processPanel,
            ExportParameters parameters, 
            int[] geometryTypes, 
            int[] geometrySubtypes
        ) {
        this.processPanel = processPanel;
        this.geometryTypeNames = new HashMap<>();
        this.geometrySubtypeNames = new HashMap<>();
        this.parameters = (ExportParametersGeometry) parameters;
        this.geometryTypes = geometryTypes;
        this.geometrySubtypes = geometrySubtypes;
        this.initData();
        this.initComponents();
    }

    private void initData() {
        FeatureType ft = this.parameters.getSourceFeatureType();
        FeatureAttributeDescriptor geomDescriptor = ft.getDefaultGeometryAttribute();
        
        if( geomDescriptor == null ) {
            this.parameters.setTargetGeometryType(Geometry.TYPES.GEOMETRY);
            this.parameters.setTargetGeometrySubtype(Geometry.SUBTYPES.GEOM2D);
        } else {
            GeometryType gt = geomDescriptor.getGeomType();
            this.parameters.setTargetGeometryType(gt.getType());
            this.parameters.setTargetGeometrySubtype(gt.getSubType());
        }
        
        if( ArrayUtils.isEmpty(this.geometryTypes) ) {
            this.geometryTypes = new int[] {
                Geometry.TYPES.GEOMETRY,
                Geometry.TYPES.POINT,
                Geometry.TYPES.CURVE,
                Geometry.TYPES.SURFACE,
                Geometry.TYPES.SOLID,
                Geometry.TYPES.AGGREGATE,
                Geometry.TYPES.MULTIPOINT,
                Geometry.TYPES.MULTICURVE,
                Geometry.TYPES.MULTISURFACE,
                Geometry.TYPES.MULTISOLID,
                Geometry.TYPES.CIRCLE,
                Geometry.TYPES.ARC,
                Geometry.TYPES.ELLIPSE,
                Geometry.TYPES.SPLINE,
                Geometry.TYPES.ELLIPTICARC,
                Geometry.TYPES.COMPLEX,
                Geometry.TYPES.LINE,
                Geometry.TYPES.POLYGON,
                //Geometry.TYPES.RING,
                Geometry.TYPES.MULTILINE,
                Geometry.TYPES.MULTIPOLYGON,
                Geometry.TYPES.CIRCUMFERENCE,
                Geometry.TYPES.PERIELLIPSE,
                Geometry.TYPES.FILLEDSPLINE
            };                    
        }
        if( ArrayUtils.isEmpty(this.geometrySubtypes) ) {
            this.geometrySubtypes = new int[] {
                Geometry.SUBTYPES.GEOM2D,
                Geometry.SUBTYPES.GEOM2DM,
                Geometry.SUBTYPES.GEOM3D,
                Geometry.SUBTYPES.GEOM3DM
            };
        }
        this.geometryTypeNames.put(Geometry.TYPES.GEOMETRY,"GEOMETRY");
        this.geometryTypeNames.put(Geometry.TYPES.POINT,"POINT");
        this.geometryTypeNames.put(Geometry.TYPES.CURVE,"CURVE");
        this.geometryTypeNames.put(Geometry.TYPES.SURFACE,"SURFACE");
        this.geometryTypeNames.put(Geometry.TYPES.SOLID,"SOLID");
        this.geometryTypeNames.put(Geometry.TYPES.AGGREGATE,"AGGREGATE");
        this.geometryTypeNames.put(Geometry.TYPES.MULTIPOINT,"MULTIPOINT");
        this.geometryTypeNames.put(Geometry.TYPES.MULTICURVE,"MULTICURVE");
        this.geometryTypeNames.put(Geometry.TYPES.MULTISURFACE,"MULTISURFACE");
        this.geometryTypeNames.put(Geometry.TYPES.MULTISOLID,"MULTISOLID");
        this.geometryTypeNames.put(Geometry.TYPES.CIRCLE,"CIRCLE");
        this.geometryTypeNames.put(Geometry.TYPES.ARC,"ARC");
        this.geometryTypeNames.put(Geometry.TYPES.ELLIPSE,"ELLIPSE");
        this.geometryTypeNames.put(Geometry.TYPES.SPLINE,"SPLINE");
        this.geometryTypeNames.put(Geometry.TYPES.ELLIPTICARC,"ELLIPTICARC");
        this.geometryTypeNames.put(Geometry.TYPES.COMPLEX,"COMPLEX");
        this.geometryTypeNames.put(Geometry.TYPES.LINE,"LINE");
        this.geometryTypeNames.put(Geometry.TYPES.POLYGON,"POLYGON");
        this.geometryTypeNames.put(Geometry.TYPES.NULL,"NULL");
        this.geometryTypeNames.put(Geometry.TYPES.RING,"RING");
        this.geometryTypeNames.put(Geometry.TYPES.MULTILINE,"MULTILINE");
        this.geometryTypeNames.put(Geometry.TYPES.MULTIPOLYGON,"MULTIPOLYGON");
        this.geometryTypeNames.put(Geometry.TYPES.CIRCUMFERENCE,"CIRCUMFERENCE");
        this.geometryTypeNames.put(Geometry.TYPES.PERIELLIPSE,"PERIELLIPSE");
        this.geometryTypeNames.put(Geometry.TYPES.FILLEDSPLINE,"FILLEDSPLINE");
        this.geometrySubtypeNames.put(Geometry.SUBTYPES.GEOM2D,"2D");
        this.geometrySubtypeNames.put(Geometry.SUBTYPES.GEOM2DM,"2DM");
        this.geometrySubtypeNames.put(Geometry.SUBTYPES.GEOM3D,"3D");
        this.geometrySubtypeNames.put(Geometry.SUBTYPES.GEOM3DM,"3DM");
        this.geometrySubtypeNames.put(Geometry.SUBTYPES.UNKNOWN,"UNKNOWN");
    }
    
    private void initComponents() {
        List<ListElement<Integer>> items;

        items = new ArrayList<>();
        for (int geometryType : this.geometryTypes) {
            items.add(new ListElement<>(
                    this.geometryTypeNames.getOrDefault(geometryType, "unknown ("+geometryType+")"),
                    geometryType
                )
            );
        }
        items.sort(new Comparator<ListElement<Integer>>() {
            @Override
            public int compare(ListElement<Integer> o1, ListElement<Integer> o2) {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });
        DefaultComboBoxModel model1 = new DefaultComboBoxModel();
        for (ListElement<Integer> item : items) {
            model1.addElement(item);
        }
        this.cboType.setModel(model1);


        items = new ArrayList<>();
        for (int geometrySubtype : this.geometrySubtypes) {
            items.add(new ListElement<>(
                    this.geometrySubtypeNames.getOrDefault(geometrySubtype, "unknown ("+geometrySubtype+")"),
                    geometrySubtype
                )
            );
        }
        items.sort(new Comparator<ListElement<Integer>>() {
            @Override
            public int compare(ListElement<Integer> o1, ListElement<Integer> o2) {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });
        model1 = new DefaultComboBoxModel();
        for (ListElement<Integer> item : items) {
            model1.addElement(item);
        }
        this.cboSubtype.setModel(model1);
        this.updateComponents=true;
        this.translate();
    }
    
    private void translate() {
        ToolsSwingManager i18nc = ToolsSwingLocator.getToolsSwingManager();
        i18nc.translate(this.lblSelectTargetGeometryType);
        i18nc.translate(this.lblSubtype);
        i18nc.translate(this.lblType);
    }

    @Override
    public String getIdPanel() {
        return ExportPanelsManager.PANEL_SELECT_GEOMETRY_TYPE;
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Select_geometry_type");    
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        Integer n = (Integer) ListElement.getSelected(cboType);
        if( n!=null ) {
            this.parameters.setTargetGeometryType(n);
        }
        n = (Integer) ListElement.getSelected(cboSubtype);
        if( n!=null ) {
            this.parameters.setTargetGeometrySubtype(n);
        }
    }

    @Override
    public void enterPanel() {
        if( this.updateComponents ) {
            ListElement.setSelected(cboType, this.parameters.getTargetGeometryTypeAsInt());
            ListElement.setSelected(cboSubtype, this.parameters.getTargetGeometrySubtype());
        }
        this.updateComponents=false;
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

}
