package org.gvsig.export.swing.impl.panels;

import java.awt.event.ActionEvent;
import java.io.File;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.export.swing.spi.ExportPanelsManager;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingLocator;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.pickercontroller.FilePickerController;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.util.HasAFile;

/**
 *
 * @author jjdelcerro
 */
public class SelectOutputFilePanel
        extends SelectOutputFilePanelView
        implements ExportPanel {

    private FilePickerController fileController;
    private final HasAFile parameters;
    private final JExportProcessPanel processPanel;

    public SelectOutputFilePanel(
            JExportProcessPanel processPanel,
            ExportParameters parameters
    ) {
        this.processPanel = processPanel;
        this.parameters = (HasAFile) parameters;
        this.initComponents();
    }

    @Override
    public String getIdPanel() {
        return ExportPanelsManager.PANEL_SELECT_OUTPUT_FILE;
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        return i18n.getTranslation("Seleccionar_fichero");
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
        if (this.fileController.isEmpty()) {
            throw new ExportPanelValidationException(
                    i18n.getTranslation("_File_cannot_be_empty"));
        }
        File file = this.fileController.get();
        File newFile = null;
        try {
            newFile = ExpressionUtils.evaluateFilename(file);
        } catch (Exception ex) {
            throw new ExportPanelValidationException(
                    i18n.getTranslation("_Evaluated_file_path_is_not_valid"));
        }
        if (newFile != null && newFile.exists()) {
            int resp = dialogs.confirmDialog(
                    i18n.getTranslation("fichero_ya_existe_seguro_desea_guardarlo"),
                    i18n.getTranslation("guardar"),
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    "OverwriteFileOnExport"
            );
            if (resp == JOptionPane.NO_OPTION) {
                return false;
            }
        }
        return true;
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        File file = this.fileController.get();
        this.parameters.setFile(file);
    }

    @Override
    public void enterPanel() {
        File file = this.parameters.getFile();
        if (file != null) {
            this.fileController.set(file);
        }
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    private void initComponents() {
        File initialPath = null;
        if (this.parameters.getFile() != null) {
            initialPath = this.parameters.getFile();
        }
        this.fileController = ToolsSwingLocator.getToolsSwingManager()
                .createFilePickerController(txtFile,
                        btnSelectFile,
                        null,
                        null, initialPath, true);
        JPopupMenu menu = this.txtFile.getComponentPopupMenu();
        menu.add(ExpressionEvaluatorSwingLocator.getManager().createJExpressionBuilderAction(this.txtFile));
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        Action textUniqueTempLayer;
        textUniqueTempLayer = new AbstractAction(i18nManager.getTranslation("_Temporal_layer_path")) {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (txtFile.isEditable() && txtFile.isEnabled()) {
                    String nameLayer;
                    try {
                        nameLayer = ((ExportParameters) parameters).getSourceFeatureStore().getName();
                    } catch (Exception ex) {
                        nameLayer = "temp";
                    }
                    txtFile.setText("<%=UNIQUETEMPORARYFILE('" + nameLayer + "')%>");
                }
            }
        };
        txtFile.getComponentPopupMenu().add(textUniqueTempLayer);
        this.translate();
    }

    private void translate() {
        ToolsSwingManager i18nc = ToolsSwingLocator.getToolsSwingManager();
        i18nc.translate(this.lblFile);
    }
}
