package org.gvsig.export.swing.impl.panels;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JComponent;
import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.ExportParametersGeometry;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.impl.pickercontroller.ProjectionPickerController;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.export.swing.spi.ExportPanelsManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;

/**
 *
 * @author jjdelcerro
 */
public class SelectTargetProjectionPanel
        extends SelectTargetProjectionPanelView
        implements ExportPanel {

    private final ExportParametersGeometry parameters;
    private final JExportProcessPanel processPanel;
    private ProjectionPickerController projectionController;

    public SelectTargetProjectionPanel(
            JExportProcessPanel processPanel,
            ExportParameters parameters
    ) {
        this.processPanel = processPanel;
        this.parameters = (ExportParametersGeometry) parameters;
        this.initComponents();
    }

    private void initComponents() {
        this.projectionController = new ProjectionPickerController(
                this.txtUserProjection,
                this.btnSelectUserProjection
        );
        this.translate();
        IProjection targetParams = this.parameters.getTargetProjection();
        if (this.parameters.getContextProjection() == this.parameters.getSourceProjection()) {
            this.rdbtnOriginalProjection.setEnabled(this.parameters.getSourceProjection() != null);
            this.txtOriginalProjection.setEnabled(this.parameters.getSourceProjection() != null);
            this.rdbtnViewProjection.setEnabled(false);
            this.txtViewProjection.setEnabled(false);
            this.rdbtnUserProjection.setEnabled(true);
            this.txtUserProjection.setEnabled(true);

        } else {
            this.rdbtnOriginalProjection.setEnabled(this.parameters.getSourceProjection() != null);
            this.txtOriginalProjection.setEnabled(this.parameters.getSourceProjection() != null);
            this.rdbtnViewProjection.setEnabled(this.parameters.getContextProjection() != null);
            this.txtViewProjection.setEnabled(this.parameters.getContextProjection() != null);
            this.rdbtnUserProjection.setEnabled(true);
            this.txtUserProjection.setEnabled(true);
        }
        if (this.parameters.getContextProjection() != null) {
            this.txtViewProjection.setText(this.parameters.getContextProjection().getAbrev());
        }
        if (this.parameters.getSourceProjection() != null) {
            this.txtOriginalProjection.setText(this.parameters.getSourceProjection().getAbrev());
        }
        if (this.parameters.getTargetProjection() == null) {
            this.projectionController.set(this.parameters.getSourceProjection());
        } else {
            this.projectionController.set(this.parameters.getTargetProjection());
        }
        this.rdbtnOriginalProjection.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                doRadioButonsStateChanged();
            }
        });
        this.rdbtnUserProjection.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                doRadioButonsStateChanged();
            }
        });
        this.rdbtnViewProjection.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                doRadioButonsStateChanged();
            }
        });
        if (this.parameters.getSourceProjection() == null) {
            this.rdbtnUserProjection.setSelected(true);
        } else if (this.parameters.getTargetProjection() == this.parameters.getSourceProjection()) {
            this.rdbtnOriginalProjection.setSelected(true);
        } else if (this.parameters.getTargetProjection() == this.parameters.getContextProjection()) {
            this.rdbtnViewProjection.setSelected(true);
        } else {
            this.rdbtnUserProjection.setSelected(true);
        }
    }

    private void doRadioButonsStateChanged() {
        if (this.rdbtnUserProjection.isSelected()) {
            this.projectionController.setEnabled(true);
            this.txtOriginalProjection.setEnabled(false);
            this.txtViewProjection.setEnabled(false);
            this.txtUserProjection.setEnabled(true);

        } else if (this.rdbtnOriginalProjection.isSelected()) {
            this.projectionController.setEnabled(false);
            this.txtOriginalProjection.setEnabled(true);
            this.txtViewProjection.setEnabled(false);
            this.txtUserProjection.setEnabled(false);

        } else if (this.rdbtnViewProjection.isSelected()) {
            this.projectionController.setEnabled(false);
            this.txtOriginalProjection.setEnabled(false);
            this.txtViewProjection.setEnabled(true);
            this.txtUserProjection.setEnabled(false);

        }
    }

    @Override
    public String getIdPanel() {
        return ExportPanelsManager.PANEL_SELECT_TARGET_PROJECTION;
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        return i18n.getTranslation("_Target_projection");
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        if (this.rdbtnUserProjection.isSelected()) {
            if (this.projectionController.isEmpty()) {
                return false;
            }
        }
        return true;
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void enterPanel() {
        IProjection targetProjection = this.parameters.getTargetProjection();
        IProjection contextProjection = this.parameters.getContextProjection();
        IProjection sourceProjection = this.parameters.getSourceProjection();

        ICoordTrans trans = this.parameters.getTargetTransformation();

        if (targetProjection != null) {
            this.projectionController.set(targetProjection);
        } else if (contextProjection != null) {
            this.projectionController.set(contextProjection);
        } else if (sourceProjection != null) {
            this.projectionController.set(sourceProjection);
        }

        if (targetProjection != contextProjection) {
            rdbtnUserProjection.setSelected(true);
        } else {
            if (targetProjection == sourceProjection) {
                rdbtnOriginalProjection.setSelected(true);
            } else if (targetProjection == contextProjection) {
                rdbtnViewProjection.setSelected(true);
            }
        }
    }

    @Override
    public void nextPanel() {
        if (this.rdbtnUserProjection.isSelected()) {
            this.parameters.setTargetProjection(this.projectionController.get());
            this.parameters.setTargetTransformation(this.projectionController.getCoordinateTransformation());

        } else if (this.rdbtnOriginalProjection.isSelected()) {
            this.parameters.setTargetProjection(null);
            this.parameters.setTargetTransformation(null);

        } else if (this.rdbtnViewProjection.isSelected()) {
            this.parameters.setTargetProjection(this.parameters.getContextProjection());
            ICoordTrans targetCT = this.parameters.getSourceProjection().getCT(this.parameters.getContextProjection());
            this.parameters.setTargetTransformation(targetCT);

        }
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    private void translate() {
        ToolsSwingManager manager = ToolsSwingLocator.getToolsSwingManager();
        manager.translate(this.lblSelectTargetProjection);
        manager.translate(this.rdbtnOriginalProjection);
        manager.translate(this.rdbtnUserProjection);
        manager.translate(this.rdbtnViewProjection);
    }

}
