package org.gvsig.export.csv.service;

import java.io.File;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.export.spi.AbstractExportParametersGeometry;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.folders.FoldersManager;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;

/**
 *
 * @author jjdelcerro
 */
public class ExportCSVParametersImpl
        extends AbstractExportParametersGeometry
        implements ExportCSVParameters {

    private String encoding;
    private File file;
    private String profile;
    private String charset;
    private String escapeCharacter;
    private String commentStartMarker;
    private String quoteCharacter;
    private String delimiter;
    private String recordSeparator;
    private String locale;
    private String quotePolicy;

    public ExportCSVParametersImpl(ExportCSVServiceFactory factory) {
        super(factory);
    }

    public ExportCSVParametersImpl() {
        super(null);
    }

    @Override
    public String getServiceName() {
        return ExportCSVServiceFactory.SERVICE_NAME;
    }

    @Override
    public File getFile() {
        return this.file;
    }

    @Override
    public void setFile(File file) {
        if (ExpressionUtils.isDynamicFilename(file)) {
            this.file = file;
        } else {
            if (file.isAbsolute()) {
                this.file = new File(FilenameUtils.removeExtension(file.getAbsolutePath()) + ".csv");
            } else {
                FoldersManager fm = ToolsLocator.getFoldersManager();
                this.file = new File(fm.getHome(), FilenameUtils.removeExtension(file.getPath()) + ".csv");
            }
        }
    }

    @Override
    public void setSourceFeatureType(FeatureType sourceFeatureType) {
        super.setSourceFeatureType(sourceFeatureType);
    }

    @Override
    public String getProfile() {
        return this.profile;
    }

    @Override
    public void setProfile(String profile) {
        this.profile = profile;
    }

    @Override
    public String getQuotePolicy() {
        return this.quotePolicy;
    }

    @Override
    public void setQuotePolicy(String quotePolicy) {
        this.quotePolicy = quotePolicy;
    }

    @Override
    public String getLocale() {
        return this.locale;
    }

    @Override
    public void setLocale(String locale) {
        this.locale = locale;
    }

    @Override
    public String getRecordSeparator() {
        return this.recordSeparator;
    }

    @Override
    public void setRecordSeparator(String recordSeparator) {
        this.recordSeparator = recordSeparator;
    }

    @Override
    public String getDelimiter() {
        return this.delimiter;
    }

    @Override
    public void setDelimiter(String delimiter) {
        this.delimiter = delimiter;
    }

    @Override
    public String getQuoteCharacter() {
        return this.quoteCharacter;
    }

    @Override
    public void setQuoteCharacter(String quoteCharacter) {
        this.quoteCharacter = quoteCharacter;
    }

    @Override
    public String getCommentStartMarker() {
        return this.commentStartMarker;
    }

    @Override
    public void setCommentStartMarker(String commentStartMarker) {
        this.commentStartMarker = commentStartMarker;
    }

    @Override
    public String getEscapeCharacter() {
        return this.escapeCharacter;
    }

    @Override
    public void setEscapeCharacter(String escapeCharacer) {
        this.escapeCharacter = escapeCharacer;
    }

    @Override
    public String getCharset() {
        return this.charset;
    }

    @Override
    public void setCharset(String charset) {
        this.charset = charset;
    }

    @Override
    public String toString() {
        String label = StringUtils.join(super.getLabel(), ",", this.getProfile());
        return label;
    }

    public static void registerPersistence() {

        PersistenceManager manager = ToolsLocator.getPersistenceManager();
        if (manager.getDefinition("ExportCSVParametersImpl") == null) {
            DynStruct definition = manager.addDefinition(ExportCSVParametersImpl.class,
                    "ExportCSVParametersImpl", "ExportCSVParametersImpl persistence definition", null, null);

            definition.addDynFieldString("encoding");
            definition.addDynFieldFile("file");
            definition.addDynFieldString("profile");
            definition.addDynFieldString("charset");
            definition.addDynFieldString("escapeCharacter");
            definition.addDynFieldString("commentStartMarker");
            definition.addDynFieldString("quoteCharacter");
            definition.addDynFieldString("delimiter");
            definition.addDynFieldString("recordSeparator");
            definition.addDynFieldString("locale");
            definition.addDynFieldString("quotePolicy");

            definition.extend(manager.getDefinition("AbstractExportParametersGeometry"));
        }
    }

    @Override
    public void saveToState(PersistentState state) throws PersistenceException {
        super.saveToState(state);
        state.set("encoding", encoding);
        state.set("file", file);
        state.set("profile", profile);
        state.set("charset", charset);
        state.set("escapeCharacter", escapeCharacter);
        state.set("commentStartMarker", commentStartMarker);
        state.set("quoteCharacter", quoteCharacter);
        state.set("delimiter", delimiter);
        state.set("recordSeparator", recordSeparator);
        state.set("locale", locale);
        state.set("quotePolicy", quotePolicy);
    }

    @Override
    public void loadFromState(PersistentState state) throws PersistenceException {
        super.loadFromState(state);
        this.encoding = state.getString("encoding");
        this.file = state.getFile("file");
        this.profile = state.getString("profile");
        this.charset = state.getString("charset");
        this.escapeCharacter = state.getString("escapeCharacter");
        this.commentStartMarker = state.getString("commentStartMarker");
        this.quoteCharacter = state.getString("quoteCharacter");
        this.delimiter = state.getString("delimiter");
        this.recordSeparator = state.getString("recordSeparator");
        this.locale = state.getString("locale");
        this.quotePolicy = state.getString("quotePolicy");
    }

}
