/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.csv.swing.panels;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynField;
import org.gvsig.tools.dynobject.DynObjectValueItem;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.export.csv.service.ExportCSVParameters;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.feature.FeatureStore;

/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class CSVExportParametersPanel
        extends CSVExportParametersPanelView
        implements ExportPanel {

    private static final Logger LOG = LoggerFactory.getLogger(CSVExportParametersPanel.class);

    private final ExportCSVParameters parameters;
    private final JExportProcessPanel processPanel;

    public CSVExportParametersPanel(JExportProcessPanel processPanel, ExportParameters parameters) {
        super();
        this.processPanel = processPanel;
        this.parameters = (ExportCSVParameters) parameters;
        this.initComponents();
    }

    class ComboItem {

        private String key;
        private String value;

        public ComboItem(String key, String value) {
            this.key = key;
            this.value = value;
        }

        @Override
        public String toString() {
            return key;
        }

        public String getKey() {
            return key;
        }

        public String getValue() {
            return value;
        }
    }

    private DefaultComboBoxModel fillComboBoxWithAvaibleParameters(String dynFieldValue) {
        DynStruct parametersDefinition = ToolsLocator.getPersistenceManager().getDefinition("CSVNewStoreParameters");
        DynField dynField = parametersDefinition.getDynField(dynFieldValue);

        DefaultComboBoxModel dcm = new DefaultComboBoxModel();
        DynObjectValueItem[] availableValues = dynField.getAvailableValues();
        for (DynObjectValueItem availableValue : availableValues) {

            Object item = new ComboItem(availableValue.getLabel(), (String) availableValue.getValue());
            dcm.addElement(item);
        }
        return dcm;

    }

    private void initComponents() {
        DynStruct parametersDefinition = ToolsLocator.getPersistenceManager().getDefinition("CSVNewStoreParameters");
        this.cmbProfile.setModel(this.fillComboBoxWithAvaibleParameters("profile"));
        this.cmbProfile.setSelectedIndex(0);
        
        //TODO: set params from previous source store in case of null values
        //it can be null
        //DataStoreParameters params = this.parameters.getSourceFeatureStore().getParameters();
//        if (params instanceof CSVStoreParameters) {
//            CSVStoreParameters sparams = (CSVStoreParameters) params;
//            CsvPreference profile = CSVStoreParameters.getPredefinedCSVPreferences(sparams);
//
//            CommentMatcher commentMarker = profile.getCommentMatcher();
//            this.txtCommentStartMarker.setText(commentMarker.toString());
//
//            int delimeterChar = profile.getDelimiterChar();
//            this.txtDelimiter.setText(String.valueOf(delimeterChar));
//
//            CsvEncoder encoder = profile.getEncoder();
//
//            String endOfLineSymbols = profile.getEndOfLineSymbols();
//            this.txtEscapeCharacter.setText(endOfLineSymbols);
//
//            int quoteChar = profile.getQuoteChar();
//            this.txtQuoteCharacter.setText(String.valueOf(quoteChar));
//
//            QuoteMode quoteMode = profile.getQuoteMode();
//            this.txtQuoteCharacter.setText(quoteMode.toString());
//        } else {
//            // default values
//
//            //this.cmbLocale.setSelectedItem(CSVStoreParameters.);
//        }

        this.cmbQuotePolicy.setModel(this.fillComboBoxWithAvaibleParameters("quotePolicy"));
        this.cmbQuotePolicy.setSelectedIndex(0);

        this.cmbLocale.setModel(this.fillComboBoxWithAvaibleParameters("locale"));
        this.cmbLocale.setSelectedIndex(0);

        this.translate();
    }

    private void translate() {
        ToolsSwingManager i18nc = ToolsSwingLocator.getToolsSwingManager();
        i18nc.translate(this.lblProfile);
        i18nc.translate(this.lblQuotePolicy);
        i18nc.translate(this.lblLocale);
        i18nc.translate(this.lblRecordSeparator);
        i18nc.translate(this.lblDelimiter);
        i18nc.translate(this.lblQuoteCharacter);
        i18nc.translate(this.lblCommentStartMarker);

    }

    public String getParameterLocale() {
        ComboItem item = (ComboItem) this.cmbLocale.getSelectedItem();
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public String getParameterQuotePolicy() {
        ComboItem item = (ComboItem) this.cmbQuotePolicy.getSelectedItem();
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public String getParameterProfile() {
        ComboItem item = (ComboItem) this.cmbProfile.getSelectedItem();
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public String getParameterRecordSeparator() {
        return this.txtRecordSeparator.getText();
    }

    public String getParameterDelimiter() {
        return this.txtDelimiter.getText();
    }

    public String getParameterQuoteCharacter() {
        return this.txtQuoteCharacter.getText();
    }

    public String getParameterCommentStartMarker() {
        return this.txtCommentStartMarker.getText();
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Select_encoding");
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void enterPanel() {
        FeatureStore csvStore = this.parameters.getSourceFeatureStore();
        DataStoreParameters sparams = csvStore.getParameters();

        this.setSelectedStringValue(cmbProfile, this.parameters.getProfile());
        this.setSelectedStringValue(cmbLocale, this.parameters.getLocale());
        this.setSelectedStringValue(cmbQuotePolicy, this.parameters.getQuotePolicy());

        this.txtRecordSeparator.setText(this.parameters.getRecordSeparator());
        this.txtDelimiter.setText(this.parameters.getDelimiter());
        this.txtQuoteCharacter.setText(this.parameters.getQuoteCharacter());
        this.txtCommentStartMarker.setText(this.parameters.getCommentStartMarker());
    }

    @Override
    public void previousPanel() {

    }

    private void setSelectedStringValue(JComboBox comboBox, String value) {

        ComboItem item;
        for (int i = 0; i < comboBox.getItemCount(); i++) {
            item = (ComboItem) comboBox.getItemAt(i);
            if (StringUtils.equals(item.getValue(),value)) {
                comboBox.setSelectedIndex(i);
                break;
            }
        }
    }

    @Override
    public void nextPanel() {
        this.parameters.setProfile(this.getParameterProfile());
        this.parameters.setQuotePolicy(this.getParameterQuotePolicy());
        this.parameters.setLocale(this.getParameterLocale());
        if (this.getParameterRecordSeparator() != null) {
            this.parameters.setRecordSeparator(this.getParameterRecordSeparator());
        }
        if (this.getParameterDelimiter() != null) {
            this.parameters.setDelimiter(this.getParameterDelimiter());
        }
        if (this.getParameterCommentStartMarker() != null) {
            this.parameters.setCommentStartMarker(this.getParameterCommentStartMarker());
        }
        if (this.getParameterQuoteCharacter()!=null) {
            this.parameters.setQuoteCharacter(this.getParameterQuoteCharacter());
        }
    }
}
