package org.gvsig.export.csv.swing.panels;
import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


public class CSVExportParametersPanelView extends JPanel
{
   JLabel lblQuotePolicy = new JLabel();
   JLabel lblProfile = new JLabel();
   JLabel lblLocale = new JLabel();
   JLabel lblRecordSeparator = new JLabel();
   JLabel lblDelimiter = new JLabel();
   JLabel lblQuoteCharacter = new JLabel();
   JLabel lblCommentStartMarker = new JLabel();
   JComboBox cmbProfile = new JComboBox();
   JComboBox cmbQuotePolicy = new JComboBox();
   JComboBox cmbLocale = new JComboBox();
   JTextField txtRecordSeparator = new JTextField();
   JTextField txtDelimiter = new JTextField();
   JTextField txtQuoteCharacter = new JTextField();
   JTextField txtCommentStartMarker = new JTextField();

   /**
    * Default constructor
    */
   public CSVExportParametersPanelView()
   {
      initializePanel();
   }

   /**
    * Main method for panel
    */
   public static void main(String[] args)
   {
      JFrame frame = new JFrame();
      frame.setSize(600, 400);
      frame.setLocation(100, 100);
      frame.getContentPane().add(new CSVExportParametersPanelView());
      frame.setVisible(true);

      frame.addWindowListener( new WindowAdapter()
      {
         public void windowClosing( WindowEvent evt )
         {
            System.exit(0);
         }
      });
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE,FILL:106DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE,CENTER:4PX:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblQuotePolicy.setName("lblQuotePolicy");
      lblQuotePolicy.setText("_Quote_Policy");
      jpanel1.add(lblQuotePolicy,cc.xy(2,4));

      lblProfile.setName("lblProfile");
      lblProfile.setText("_Profile");
      jpanel1.add(lblProfile,cc.xy(2,2));

      lblLocale.setName("lblLocale");
      lblLocale.setText("_Locale");
      jpanel1.add(lblLocale,cc.xy(2,6));

      lblRecordSeparator.setName("lblRecordSeparator");
      lblRecordSeparator.setText("_Record_separator");
      jpanel1.add(lblRecordSeparator,cc.xy(2,8));

      lblDelimiter.setName("lblDelimiter");
      lblDelimiter.setText("_Delimiter");
      jpanel1.add(lblDelimiter,cc.xy(2,10));

      lblQuoteCharacter.setName("lblQuoteCharacter");
      lblQuoteCharacter.setText("_Quote_character");
      jpanel1.add(lblQuoteCharacter,cc.xy(2,12));

      lblCommentStartMarker.setName("lblCommentStartMarker");
      lblCommentStartMarker.setText("_Comment_start_marker");
      jpanel1.add(lblCommentStartMarker,cc.xy(2,14));

      cmbProfile.setName("cmbProfile");
      jpanel1.add(cmbProfile,cc.xy(3,2));

      cmbQuotePolicy.setName("cmbQuotePolicy");
      jpanel1.add(cmbQuotePolicy,cc.xy(3,4));

      cmbLocale.setName("cmbLocale");
      jpanel1.add(cmbLocale,cc.xy(3,6));

      txtRecordSeparator.setName("txtRecordSeparator");
      jpanel1.add(txtRecordSeparator,cc.xy(3,8));

      txtDelimiter.setName("txtDelimiter");
      jpanel1.add(txtDelimiter,cc.xy(3,10));

      txtQuoteCharacter.setName("txtQuoteCharacter");
      jpanel1.add(txtQuoteCharacter,cc.xy(3,12));

      txtCommentStartMarker.setName("txtCommentStartMarker");
      jpanel1.add(txtCommentStartMarker,cc.xy(3,14));

      addFillComponents(jpanel1,new int[]{ 1,2,3,4 },new int[]{ 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16 });
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
