package org.gvsig.export.dbf.service;

import java.io.File;
import org.apache.commons.io.FilenameUtils;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.spi.AbstractExportParameters;
import org.gvsig.export.spi.AttributeNamesTranslator;
import org.gvsig.export.spi.CutAttributeNamesTranslator;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.folders.FoldersManager;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;

/**
 *
 * @author jjdelcerro
 */
public class ExportDBFParametersImpl
        extends AbstractExportParameters
        implements ExportDBFParameters {

    private String encoding;
    private File file;

    public ExportDBFParametersImpl(ExportDBFServiceFactory factory) {
        super(factory);
        AttributeNamesTranslator attr = new CutAttributeNamesTranslator(ExportDBFService.MAX_FIELD_NAME_LENGTH);
        this.exportAttributes.setNamesTranslator(attr);
    }

    public ExportDBFParametersImpl() {
        super(null);
    }

    public static void registerPersistence() {

        PersistenceManager manager = ToolsLocator.getPersistenceManager();
        if (manager.getDefinition("ExportDBFParametersImpl") == null) {
            DynStruct definition = manager.addDefinition(ExportDBFParametersImpl.class,
                    "ExportDBFParametersImpl", "ExportDBFParametersImpl persistence definition", null, null);

            definition.addDynFieldString("encoding");
            definition.addDynFieldFile("file");

            definition.extend(manager.getDefinition("AbstractExportParameters"));
        }
    }

    @Override
    public void saveToState(PersistentState state) throws PersistenceException {
        super.saveToState(state);
        state.set("encoding", encoding);
        state.set("file", file);
    }

    @Override
    public void loadFromState(PersistentState state) throws PersistenceException {
        super.loadFromState(state);
        this.encoding = state.getString("encoding");
        this.file = state.getFile("file");
    }

    @Override
    public String getServiceName() {
        return ExportDBFServiceFactory.SERVICE_NAME;
    }

    @Override
    public String getEncoding() {
        return this.encoding;
    }

    @Override
    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    @Override
    public File getFile() {
        return this.file;
    }

    @Override
    public void setFile(File file) {
        if (ExpressionUtils.isDynamicFilename(file)) {
            this.file = file;
        } else {
            if (file.isAbsolute()) {
                this.file = new File(FilenameUtils.removeExtension(file.getAbsolutePath()) + ".dbf");
            } else {
                FoldersManager fm = ToolsLocator.getFoldersManager();
                this.file = new File(fm.getHome(), FilenameUtils.removeExtension(file.getPath()) + ".dbf");
            }
        }
    }

    @Override
    public void setSourceFeatureType(FeatureType sourceFeatureType) {
        super.setSourceFeatureType(sourceFeatureType);

    }

}
