/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.dbf.service;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.io.FilenameUtils;
import org.gvsig.export.ExportException;

import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.NewFeatureStoreParameters;
import org.gvsig.fmap.dal.feature.OpenFeatureStoreParameters;
import org.gvsig.export.spi.ExportService;
import org.gvsig.export.spi.AbstractExportService;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.NewDataStoreParameters;
import org.gvsig.fmap.dal.OpenDataStoreParameters;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemServerExplorer;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemServerExplorerParameters;
import org.gvsig.tools.util.HasAFile;

/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class ExportDBFService
        extends AbstractExportService
        implements ExportService {

    public static final int MAX_FIELD_NAME_LENGTH = 10;
    public File targetFile = null;

    protected ExportDBFService(ExportServiceFactory factory, ExportDBFParameters parameters) {
        super(factory, parameters);
    }

    @Override
    public ExportDBFParameters getParameters() {
        return (ExportDBFParameters) super.getParameters();
    }

    @Override
    protected FilesystemServerExplorer createServerExplorer() throws ExportException {

        DataManager dataManager = DALLocator.getDataManager();

        FilesystemServerExplorerParameters explorerParams;
        try {
            explorerParams
                    = (FilesystemServerExplorerParameters) dataManager
                            .createServerExplorerParameters(FilesystemServerExplorer.NAME);
        } catch (Exception e) {
            throw new ExportException(e);
        }
        explorerParams.setRoot(this.getTargetFile().getParent());

        FilesystemServerExplorer explorer;
        try {
            explorer = (FilesystemServerExplorer) dataManager.openServerExplorer(
                    "FilesystemExplorer", explorerParams
            );
            return explorer;
        } catch (Exception e) {
            throw new ExportException(e);
        }
    }

    @Override
    protected NewDataStoreParameters createTargetNewStoreParameters() throws ExportException {
        try {
            FilesystemServerExplorer explorer = this.createServerExplorer();
            NewFeatureStoreParameters newStoreParameters = (NewFeatureStoreParameters) explorer.getAddParameters(
                    this.getTargetFile()
            );
            newStoreParameters.setDynValue("Encoding", this.getParameters().getEncoding());
            
//            AttributeNamesTranslator nameTranslator = this.getFactory().createAttributeNamesTranslator();
//            ExportAttributes exportAttr = this.getParameters().getExportAttributes();
//            exportAttr.setNamesTranslator(nameTranslator);

            FeatureType ft = this.getParameters().getExportAttributes().getTargetFeatureType();
            if (ft == null) {
                ft = this.getParameters().getSourceFeatureType();
            }

            // FIXME: check types of fields in ft (remove Geometry?)
            newStoreParameters.setDefaultFeatureType(ft.getEditable());
            return newStoreParameters;
        } catch (DataException ex) {
            throw new ExportException(ex);
        }
    }

    @Override
    public OpenDataStoreParameters createTargetOpenStoreParameters() throws ExportException {
        try {
            DataManager dataManager = DALLocator.getDataManager();
            OpenFeatureStoreParameters openStoreParameters = (OpenFeatureStoreParameters) dataManager.createStoreParameters("DBF");
            ((HasAFile) openStoreParameters).setFile(this.getTargetFile());
            openStoreParameters.setDynValue("Encoding", getParameters().getEncoding());
            return openStoreParameters;
        } catch (DataException ex) {
            throw new ExportException(ex);
        }
    }
    
    public File getTargetFile() {
        if (this.targetFile == null) {
            this.targetFile = ExpressionUtils.evaluateFilename(this.getParameters().getFile());
            this.targetFile = new File(FilenameUtils.removeExtension(this.targetFile.getAbsolutePath()) + ".dbf");
            return this.targetFile;
        }
        return this.targetFile;
    }
    @Override
    public List<OpenDataStoreParameters> getTargetOpenStoreParameters() throws ExportException {
        List<OpenDataStoreParameters> r = new ArrayList<>();
        r.add(this.createTargetOpenStoreParameters());
        return r;
    }
    
    @Override
    public void export(FeatureSet featureSet) throws ExportException {
        this.targetFile = null;
        super.export(featureSet);
    }

}
