/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.dbf.swing.panels;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;

import org.gvsig.fmap.dal.DataStoreParameters;

import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynField;
import org.gvsig.tools.dynobject.DynObjectValueItem;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.export.dbf.service.ExportDBFParameters;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class EncodingPanel
        extends EncodingPanelView
        implements ExportPanel {

    private static final Logger LOG = LoggerFactory.getLogger(EncodingPanel.class);

    private final ExportDBFParameters parameters;
    private final JExportProcessPanel processPanel;

    public EncodingPanel(JExportProcessPanel processPanel, ExportParameters parameters) {
        super();
        this.processPanel = processPanel;
        this.parameters = (ExportDBFParameters) parameters;
        this.initComponents();
    }

    private void initComponents() {
        DynStruct parametersDefinition = ToolsLocator.getPersistenceManager().getDefinition("DBFStoreParameters");
        DynField dynField = parametersDefinition.getDynField("Encoding");

        DefaultListModel model = new DefaultListModel();
        DynObjectValueItem[] encodings = dynField.getAvailableValues();
        for (DynObjectValueItem encoding : encodings) {
            model.addElement(new ListElement(encoding.getLabel(), encoding.getValue()));
        }
        this.lstEncoding.setModel(model);
        this.lstEncoding.setSelectedIndex(0);


        this.translate();
    }

    private void translate() {
        ToolsSwingManager i18nc = ToolsSwingLocator.getToolsSwingManager();
        i18nc.translate(this.lblEncoding);
    }

    public String getEncoding() {
        ListElement encoding = (ListElement) this.lstEncoding.getSelectedValue();
        if (encoding == null) {
            return null;
        }
        return (String) encoding.getValue();
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Select_encoding");
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void enterPanel() {
        String sourceProviderName = this.parameters.getSourceFeatureStore().getProviderName();
        if (sourceProviderName.equalsIgnoreCase("Shape") || sourceProviderName.equalsIgnoreCase("DBF")) {
            if (this.parameters.getEncoding() != null) {
                String encoding = this.parameters.getEncoding();
                ListElement.setSelected(this.lstEncoding, encoding);
            } else {
                DataStoreParameters params = this.parameters.getSourceFeatureStore().getParameters();
                String encoding = (String) params.getDynValue("Encoding");
                ListElement.setSelected(this.lstEncoding, encoding);
            }
        }
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        this.parameters.setEncoding(this.getEncoding());
    }
}
