/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.gvsig.export.jdbc.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.export.ExportAttributes;
import org.gvsig.export.spi.AttributeNamesTranslator;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;

/**
 *
 * @author osc
 */
public class ExportJDBCAttributeNamesTranslator implements AttributeNamesTranslator {

    private ExportJDBCParameters parameters;

    public ExportJDBCAttributeNamesTranslator(ExportJDBCParameters parameters) {
        this.parameters = parameters;
    }

    public ExportJDBCAttributeNamesTranslator() {

    }

    public void setParameters(ExportJDBCParameters parameters) {
        this.parameters = parameters;
    }

    public static void registerPersistence() {
        PersistenceManager manager = ToolsLocator.getPersistenceManager();
        if (manager.getDefinition("ExportJDBCAttributeNamesTranslator") == null) {
            DynStruct definition = manager.addDefinition(ExportJDBCAttributeNamesTranslator.class,
                    "ExportJDBCAttributeNamesTranslator", "ExportJDBCAttributeNamesTranslator persistence definition", null, null);
            definition.addDynFieldInt("a");
        }
    }

    @Override
    public void saveToState(PersistentState state) throws PersistenceException {
        state.set("a", 0);
    }

    @Override
    public void loadFromState(PersistentState state) throws PersistenceException {
        int a = state.getInt("a");
    }

    private String buildTranslatedName(String source_name, Collection<String> current_names) {

        String target_name = this.getNameSuggestion(source_name);
        if (!current_names.contains(target_name)) {
            return target_name;
        }

        for (int i = 0; i < 255; i++) {
            target_name = source_name + i;
            if (!current_names.contains(target_name)) {
                return target_name;
            }
        }
        return source_name.substring(0, 4) + "_" + (System.currentTimeMillis() % 1000000);
    }

    @Override
    public boolean isValidName(ExportAttributes attributes, int index, String name) {
        ExportAttributes.ExportAttribute selectedAttr = attributes.get(index);
        //
        String selfName = this.getNameSuggestion(attributes, index, name);
        if (!selfName.equals(name)) {
            return false;
        }

        // check with others values
        boolean valid = true;
        for (int i = 0; i < attributes.size(); i++) {
            if (i == index) {
                continue;
            }
            ExportAttributes.ExportAttribute attr = attributes.get(i);
            String newName = attr.getNewName();
            if (newName.equals(name)) {
                return false;
            }
            if (newName.equals(selfName)) {
                return false;
            }
            if (newName.isEmpty()) {
                return false;
            }
            if (this.parameters.getRemoveSpacesInIdentifiers()) {
                //if (newName.replaceAll("\\s","") != newName) {
                String s = StringUtils.normalizeSpace(newName).replace(" ", "_");
                if (!newName.equals(s)) {
                    return false;
                }
            }
            if (this.parameters.getTranslateHyphens()) {
                if (newName.contains("-")) {
                    return false;
                }
            }
            if (this.parameters.getTranslateIdentifiersToLowerCase()) {
                if (newName.toLowerCase() != newName) {
                    return false;
                }
            }
        }
        return valid;
    }

    @Override
    public String getNameSuggestion(ExportAttributes attributes, int index, String name) {
        List names = new ArrayList();
        boolean oneTime = true;
        for (ExportAttributes.ExportAttribute attribute : attributes) {
            if (attribute.getNewName().equals(name) && oneTime) {
                oneTime = false;
                continue;
            }
            names.add(attribute.getNewName());
        }
        String target_name = buildTranslatedName(name, names);
        return target_name;
    }

    @Override
    public AttributeNamesTranslator clone() throws CloneNotSupportedException {
        ExportJDBCAttributeNamesTranslator clone = (ExportJDBCAttributeNamesTranslator) super.clone();
        return clone;
    }

    @Override
    public String getNameSuggestion(String name) {
        String target_name = name;
        if (this.parameters.getRemoveSpacesInIdentifiers()) {
            //target_name = target_name.replaceAll("\\s","");
            target_name = StringUtils.normalizeSpace(target_name).replace(" ", "_");
        }
        if (this.parameters.getTranslateHyphens()) {
            target_name = target_name.replace("-", "_");
        }
        if (this.parameters.getTranslateIdentifiersToLowerCase()) {
            target_name = target_name.toLowerCase();
        }
        return target_name;

    }

}
