
package org.gvsig.export.jdbc.swing.panels;

import javax.swing.JComponent;
import org.gvsig.export.jdbc.service.ExportJDBCParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class GeometryIndexPanel 
        extends GeometryIndexPanelLayout 
        implements ExportPanel 
    {
    private static final long serialVersionUID = -3995015914295698209L;
    
    private static final Logger logger = LoggerFactory.getLogger(GeometryIndexPanel.class);

    private final ExportJDBCParameters parameters;
    private final JExportProcessPanel processPanel;

    public GeometryIndexPanel(
            JExportProcessPanel processPanel, 
            ExportJDBCParameters parameters
        ) {
        this.processPanel = processPanel;
        this.parameters = parameters;
        initComponents();
    }

    private void initComponents() {
        this.translate();
    }
    
    private void translate() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        
        this.lblHeader.setText("<html>"+i18nManager.getTranslation("_Create_spatial_index_header")+"</html>");
        this.chkCreateGeometryIndex.setText(i18nManager.getTranslation("_Crear_indice_espacial_sobre_las_columnas_de_tipo_geometria"));
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getCanonicalName();
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Create_spatial_index");    
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }
    
    @Override
    public void enterPanel() {
        if( this.parameters.canCreatetable() ) {
            int index = -1;
            index = this.parameters.getSourceFeatureType().getDefaultGeometryAttributeIndex();
            if( index <0 ) {
                this.chkCreateGeometryIndex.setEnabled(false);
                this.chkCreateGeometryIndex.setSelected(false);
            } else {
                this.chkCreateGeometryIndex.setEnabled(true);
                this.chkCreateGeometryIndex.setSelected(true);
            }
        } else {
            this.chkCreateGeometryIndex.setEnabled(false);
            this.chkCreateGeometryIndex.setSelected(false);
        }
        // set params
        this.chkCreateGeometryIndex.setSelected(this.parameters.getCreateIndexInGeometryRow());
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        this.parameters.setCreateIndexInGeometryRow(this.chkCreateGeometryIndex.isSelected());
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    public boolean getCreateIndexInGeometryRow() {
        return this.chkCreateGeometryIndex.isSelected();
    }
}
