package org.gvsig.export.jdbc.swing.panels;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;


public class IdentifiersOptionsPanelView extends JPanel
{
   JLabel lblHeader = new JLabel();
   JCheckBox chkTraslateToLowerCase = new JCheckBox();
   JCheckBox chkRemoveSpaces = new JCheckBox();
   JCheckBox chkTraslateHyphens = new JCheckBox();

   /**
    * Default constructor
    */
   public IdentifiersOptionsPanelView()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:8DLU:GROW(1.0),FILL:4DLU:NONE","CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:4DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblHeader.setName("lblHeader");
      lblHeader.setText("<html> Algunas bases de datos precisan tratar los identificadores de tabla o campo que tienen mayusculas y minusculas mezcladas, o espacios en blanco de forma especial a la hora de referirse a ellos.  Es recomendable que no los utilice para evitar problemas.<br> Puede indicar al asistente que traslade todos los nombres de campo a minusculas y que use \"_\" en lugar de espacios en blanco en el nombre de tabla si los hubiese. </html>");
      jpanel1.add(lblHeader,cc.xy(2,2));

      chkTraslateToLowerCase.setActionCommand("Trasladar identificadores de campo y tabla a minusculas");
      chkTraslateToLowerCase.setName("chkTraslateToLowerCase");
      chkTraslateToLowerCase.setSelected(true);
      chkTraslateToLowerCase.setText("Trasladar identificadores de campo y tabla a minusculas");
      jpanel1.add(chkTraslateToLowerCase,cc.xy(2,4));

      chkRemoveSpaces.setActionCommand("Trasladar espacios en blanco en los identificadores a  \"_\"");
      chkRemoveSpaces.setName("chkRemoveSpaces");
      chkRemoveSpaces.setSelected(true);
      chkRemoveSpaces.setText("Trasladar espacios en blanco en los identificadores a  \"_\"");
      jpanel1.add(chkRemoveSpaces,cc.xy(2,6));

      chkTraslateHyphens.setActionCommand("Trasladar guines, \"-\", en los identificadores a \"_\".");
      chkTraslateHyphens.setName("chkTraslateHyphens");
      chkTraslateHyphens.setSelected(true);
      chkTraslateHyphens.setText("Trasladar guines, \"-\", en los identificadores a \"_\".");
      jpanel1.add(chkTraslateHyphens,cc.xy(2,8));

      addFillComponents(jpanel1,new int[]{ 1,2,3 },new int[]{ 1,2,3,4,5,6,7,8,9 });
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
