/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.jdbc.swing.panels;

import javax.swing.JComponent;
import org.gvsig.export.jdbc.service.ExportJDBCParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.gvsig.fmap.dal.exception.ValidateDataParametersException;
import org.gvsig.fmap.dal.serverexplorer.db.DBServerExplorerParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;

/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class JDBCConnectionPanel
        implements ExportPanel {

    private static final long serialVersionUID = -3278172717881233447L;

    private static final Logger LOG = LoggerFactory.getLogger(JDBCConnectionPanel.class);
    private org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel connectionPanel;
    private final ExportJDBCParameters parameters;
    private final JExportProcessPanel processPanel;

    public JDBCConnectionPanel(
            JExportProcessPanel processPanel,
            ExportJDBCParameters parameters
    ) {
        this.processPanel = processPanel;
        this.parameters = parameters;
        initComponents();
    }

    private void initComponents() {
        DataSwingManager manager = DALSwingLocator.getSwingManager();
        this.connectionPanel = manager.createJDBCConnectionPanel();
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void enterPanel() {
        if (this.parameters.getExplorerParameters() != null) {
            this.connectionPanel.setServerExplorerParameters(this.parameters.getExplorerParameters());
        }
    }

    public JDBCServerExplorerParameters getServerExplorerParameters() {
        return this.connectionPanel.getServerExplorerParameters();
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getCanonicalName();
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("connection_params");
    }

    @Override
    public void nextPanel() {
        this.parameters.setExplorerParameters(this.getServerExplorerParameters());
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        DBServerExplorerParameters connection = this.connectionPanel.getServerExplorerParameters();
        if (connection == null) {
            return false;
        }
        try {
            connection.validate();
            this.parameters.setExplorerParameters(this.getServerExplorerParameters());
            return true;
        } catch (ValidateDataParametersException ex) {
            throw new ExportPanelValidationException(ex.getMessageStack(), ex);
        } catch (Exception ex) {
            return false;
        }
    }

    @Override
    public JComponent asJComponent() {
        return this.connectionPanel.asJComponent();
    }

}
