/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.jdbc.swing.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.ListModel;
import javax.swing.SwingUtilities;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.export.jdbc.service.ExportJDBCAttributeNamesTranslator;
import org.gvsig.export.jdbc.service.ExportJDBCParameters;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.SQLBuilder;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorer;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCStoreParameters;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.task.AbstractMonitorableTask;
import org.gvsig.tools.task.SimpleTaskStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.gvsig.fmap.dal.DataStoreParameters;

/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class SelectTableNamePanel
        extends SelectTableNamePanelLayout
        implements ExportPanel {

    private static final Logger logger = LoggerFactory.getLogger(SelectTableNamePanel.class);

    private static final long serialVersionUID = 6269512983586358017L;

    private static class TableItem {

        private JDBCStoreParameters params;
        private String label;

        public TableItem(String label, JDBCStoreParameters params) {
            this.params = params;
            this.label = label;
        }

        public TableItem(JDBCStoreParameters params) {
            this(StringUtils.isEmpty(params.getSchema())
                    ? params.getTable() : params.getSchema() + "." + params.getTable(),
                    params);
        }

        @Override
        public String toString() {
            return this.label;
        }

        public JDBCStoreParameters getParams() {
            return this.params;
        }
    }

    private FillTablesListTask task = null;
    private SQLBuilder sqlbuilder;
    private final ExportJDBCParameters parameters;
    private final JExportProcessPanel processPanel;

    @SuppressWarnings("OverridableMethodCallInConstructor")
    public SelectTableNamePanel(
            JExportProcessPanel processPanel,
            ExportJDBCParameters parameters
    ) {
        this.processPanel = processPanel;
        this.parameters = parameters;
        initComponents();
        this.addAncestorListener(new AncestorListener() {

            @Override
            public void ancestorAdded(AncestorEvent ae) {
            }

            @Override
            public void ancestorRemoved(AncestorEvent ae) {
                cancelTask();
            }

            @Override
            public void ancestorMoved(AncestorEvent ae) {
            }
        });
    }

    private void initComponents() {
        this.rdoCreateTable.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                onChangeRadioSelecion();
            }
        });
        this.rdoInsert.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                onChangeRadioSelecion();
            }
        });
        this.rdoCreateTable.setSelected(true);
        this.rdoInsert.setEnabled(false);
        this.lstTables.setEnabled(false);
        try {
            this.txtTableName.setText(this.parameters.getSourceFeatureStore().getName());
        } catch (Exception ex) {
            logger.warn("Can't set the default value for the table name", ex);
        }

        I18nManager i18nManager = ToolsLocator.getI18nManager();
        this.lblHeader.setText(i18nManager.getTranslation("_Indique_donde_desea_insertar_los_datos"));
        this.lblWarningUseExistingTable.setText(
                "<html>\n"
                + i18nManager.getTranslation("_Los_datos_se_insertaran_usando_los_nombres_de_columna_que_coincidan_con_la_tabla_origen_dejandose_al_valor_por_defecto_para_los_que_no_haya_valores_en_la_tabla_origen")
                + "\n</html>"
        );
        this.rdoInsert.setText(i18nManager.getTranslation("_Insertar_registros_en_una_tabla_existente"));
        this.lblSelectTableName.setText(i18nManager.getTranslation("_Seleccione_la_tabla_a_usar"));
        this.rdoCreateTable.setText(i18nManager.getTranslation("_Crear_una_tabla_nueva"));
        this.lblSchema.setText(i18nManager.getTranslation("_Indique_el_esquema_en_el_que_desea_crear_la_tabla"));
        this.lblTableName.setText(i18nManager.getTranslation("_Indique_el_nombre_de_la_tabla"));
    }

    private void cancelTask() {
        if (task != null) {
            task.cancelRequest();
            task.getSimpleTaskStatus().remove();
            task = null;
        }
    }

    public boolean canCreateTable() {
        return this.rdoCreateTable.isSelected();
    }

    public String getSchema() {
        if (this.canCreateTable()) {
            return StringUtils.defaultIfBlank(this.txtSchema.getText(), null);
        }
        TableItem item = (TableItem) this.lstTables.getSelectedValue();
        JDBCStoreParameters tableParameter = item.getParams();
        if (tableParameter == null) {
            return null;
        }
        return tableParameter.getSchema();
    }

    public String getTableName() {
        if (this.canCreateTable()) {
            return StringUtils.defaultIfBlank(this.txtTableName.getText(), null);
        }
        TableItem item = (TableItem) this.lstTables.getSelectedValue();
        if (item == null) {
            return null;
        }
        JDBCStoreParameters tableParameter = item.getParams();

        if (tableParameter == null) {
            return null;
        }
        return tableParameter.getTable();
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getCanonicalName();
    }

    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Tablename");
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        this.parameters.setSchema(this.getSchema());
        this.parameters.setTableName(this.getTableName());
        this.parameters.setCanCreatetable(this.canCreateTable());
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        String tablename = this.getTableName();
        if (tablename == null) {
            throw new ExportPanelValidationException(
                    i18nManager.getTranslation(
                            "_The_name_of_table_cannot_be_empty"
                    )
            );
        }
        String schema = this.getSchema();
        if (sqlbuilder.support_schemas()) {
            if (schema == null) {
                throw new ExportPanelValidationException(
                        i18nManager.getTranslation(
                                "_The_name_of_schema_cannot_be_empty"
                        )
                );
            }
        }
        if (this.rdoCreateTable.isSelected()) {
            ExportJDBCAttributeNamesTranslator nameTranslator
                    = (ExportJDBCAttributeNamesTranslator) this.parameters.getExportAttributes().getNamesTranslator();
            String tablename_tr = nameTranslator.getNameSuggestion(tablename);
            if (!tablename_tr.equals(tablename)) {
                String msg = i18nManager.getTranslation(
                        "Ha_utilizado_espacios_en_blanco_o_mayusculas_en_el_nombre_de_la_tabla_Desea_que_se_corrija_de_forma_automatica"
                );
                ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
                int resp = dialogs.confirmDialog(
                        msg,
                        i18nManager.getTranslation("_Warning"),
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.WARNING_MESSAGE,
                        "Exportto_Table_name_with_spaces_or_mixed_case"
                );
                if (resp != JOptionPane.YES_OPTION) {
                    msg = i18nManager.getTranslation(
                            "El_nombre_de_tabla_contiene_caracteres no_validos"
                    );
                    throw new ExportPanelValidationException(msg);
                }
                tablename = tablename_tr;
                this.txtTableName.setText(tablename);
            }
            ListModel model = this.lstTables.getModel();
            for (int i = 0; i < model.getSize(); i++) {
                TableItem item = (TableItem) model.getElementAt(i);
                if (StringUtils.equals(schema, item.getParams().getSchema())
                        && StringUtils.equals(tablename, item.getParams().getTable())) {
                    String msg = i18nManager.getTranslation(
                            "_La_tabla_{0}_{1}_ya_existe_en_la_base_de_datos_Seleccione_la_opcion_de_insertar_registros_en_una_tabla_existente_para_aadir_los_datos_a_esta_o_indique_otro_nombre",
                            new String[]{schema, tablename}
                    );
                    throw new ExportPanelValidationException(msg);
                }
            }
        }
        return true;
    }

    @Override
    public void enterPanel() {
        JDBCServerExplorerParameters explorerParameters = parameters.getExplorerParameters();
        if (explorerParameters == null) {
            this.processPanel.setButtonEnabled(JExportProcessPanel.BUTTON_NEXT, false);
            return;
        }
        this.processPanel.setButtonEnabled(JExportProcessPanel.BUTTON_NEXT, true);
        try {
            DataManager dataManager = DALLocator.getDataManager();
            JDBCServerExplorer explorer = (JDBCServerExplorer) dataManager.openServerExplorer(
                    explorerParameters.getExplorerName(),
                    explorerParameters
            );
            this.sqlbuilder = explorer.createSQLBuilder();
        } catch (Exception ex) {
            throw new RuntimeException("Can't retrieve the sqlbuilder", ex);
        }
        this.fillTablesList();

        // set values from params
        this.rdoCreateTable.setSelected(this.parameters.canCreatetable());
        if (this.parameters.canCreatetable() == true) {
            if (this.parameters.getSchema() != null) {
                this.txtSchema.setText(this.parameters.getSchema());
                this.txtTableName.setText(this.parameters.getTableName());
            }
        } else {
            String schema = this.parameters.getSchema();
            if (schema != null) {
                //this.lstTables;
                for (int i = 0; i < this.lstTables.getModel().getSize(); i++) {
                    TableItem item = (TableItem) this.lstTables.getModel().getElementAt(i);
                    JDBCStoreParameters tableParameter = item.getParams();
                    if (tableParameter.getSchema() == null ? schema == null : tableParameter.getSchema().equals(schema)) {
                        this.lstTables.setSelectedIndex(i);
                    }
                }
            }
        }
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    public void onChangeRadioSelecion() {
        if (this.rdoCreateTable.isSelected()) {
            this.txtSchema.setEnabled(true);
            this.txtTableName.setEnabled(true);
            this.lstTables.setEnabled(false);
        } else {
            this.txtSchema.setEnabled(false);
            this.txtTableName.setEnabled(false);
            this.lstTables.setEnabled(true);
        }
    }

    private void fillTablesList() {

        JDBCServerExplorerParameters explorerParameters = this.parameters.getExplorerParameters();
        if (explorerParameters == null) {
            return;
        }
        cancelTask();
        this.task = new FillTablesListTask();
        task.setDaemon(true);
        task.start();
    }

    private class FillTablesListTask extends AbstractMonitorableTask {

        public FillTablesListTask() {
            super("Export");
        }

        @Override
        protected SimpleTaskStatus getSimpleTaskStatus() {
            return (SimpleTaskStatus) this.getTaskStatus();
        }

        @Override
        public void run() {

            JDBCServerExplorerParameters explorerParameters = parameters.getExplorerParameters();
            if (parameters.getExplorerParameters() == null) {
                return;
            }
            final SimpleTaskStatus status = this.getSimpleTaskStatus();
            try {
                status.setAutoremove(true);

                DataManager dataManager = DALLocator.getDataManager();

                this.getSimpleTaskStatus().message("Connecting server");
                explorerParameters.setShowInformationDBTables(false);
                final JDBCServerExplorer explorer = (JDBCServerExplorer) dataManager.openServerExplorer(
                        explorerParameters.getExplorerName(),
                        explorerParameters
                );
                SwingUtilities.invokeAndWait(new Runnable() {

                    @Override
                    public void run() {
                        if (sqlbuilder.support_schemas()) {
                            txtSchema.setText(sqlbuilder.default_schema());
                        } else {
                            txtSchema.setText("");
                            txtSchema.setEnabled(false);
                        }
                    }
                });

                this.getSimpleTaskStatus().message("Retrieving tables");
                final List<DataStoreParameters> tables = explorer.list();

                this.getSimpleTaskStatus().message("Add tables");

                SwingUtilities.invokeAndWait(new Runnable() {
                    @Override
                    public void run() {
                        DefaultListModel lmodel = new DefaultListModel();
                        Iterator<DataStoreParameters> it = tables.iterator();
                        while (it.hasNext()) {
                            if (status.isCancelled()) {
                                status.cancel();
                                break;
                            }
                            JDBCStoreParameters table = (JDBCStoreParameters) it.next();
                            lmodel.addElement(new TableItem(table));
                        }
                        lstTables.setModel(lmodel);
                        //lstTables.setEnabled(true);
                        rdoInsert.setEnabled(true);
                    }
                });

                status.message("finish");
                status.terminate();

            } catch (final Exception ex) {
                logger.warn("Fail to fill tables list", ex);
                if (status.isCancellationRequested()) {
                    status.cancel();
                }
                if (status.isRunning()) {
                    I18nManager i18nManager = ToolsLocator.getI18nManager();
                    ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
                    dialogs.messageDialog(
                            i18nManager.getTranslation("_There_have_been_problems_filling_data_in_panel")
                            + " (" + getTitlePanel() + ")",
                            null,
                            i18nManager.getTranslation("_Warning"),
                            JOptionPane.WARNING_MESSAGE,
                            "ProblemsFillingTableNamePanel"
                    );
                }

            } finally {
                status.terminate();
                status.remove();
            }

        }

    }

}
