package org.gvsig.export.shape.service;

import java.io.File;
import org.apache.commons.io.FilenameUtils;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.dbf.service.ExportDBFService;
import org.gvsig.export.spi.AbstractExportParametersGeometry;
import org.gvsig.export.spi.AttributeNamesTranslator;
import org.gvsig.export.spi.CutAttributeNamesTranslator;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynStruct;
import org.gvsig.tools.folders.FoldersManager;
import org.gvsig.tools.persistence.PersistenceManager;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;

/**
 *
 * @author jjdelcerro
 */
public class ExportShapeParametersImpl
        extends AbstractExportParametersGeometry
        implements ExportShapeParameters {

    private String encoding;
    private File file;
    private boolean useSimplifiedExport = true;

    public ExportShapeParametersImpl(ExportShapeServiceFactory factory) {
        super(factory);
        AttributeNamesTranslator attr = new CutAttributeNamesTranslator(ExportDBFService.MAX_FIELD_NAME_LENGTH);
        this.exportAttributes.setNamesTranslator(attr);
    }

    public ExportShapeParametersImpl() {
        super(null);
    }

    @Override
    public String getServiceName() {
        return ExportShapeServiceFactory.SERVICE_NAME;
    }

    @Override
    public String getEncoding() {
        return this.encoding;
    }

    @Override
    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    @Override
    public File getFile() {
        return this.file;
    }

    @Override
    public void setFile(File file) {
        if (ExpressionUtils.isDynamicFilename(file)) {
            this.file = file;
        } else {
            if (file.isAbsolute()) {
                this.file = new File(FilenameUtils.removeExtension(file.getAbsolutePath()) + ".shp");
            } else {
                FoldersManager fm = ToolsLocator.getFoldersManager();
                this.file = new File(fm.getHome(), FilenameUtils.removeExtension(file.getPath()) + ".shp");
            }
        }
    }

    @Override
    public boolean getUseSimplifiedExport() {
        return this.useSimplifiedExport;
    }

    @Override
    public void setUseSimplifiedExport(boolean useSimplifiedExport) {
        this.useSimplifiedExport = useSimplifiedExport;
    }

    @Override
    public void setSourceFeatureType(FeatureType sourceFeatureType) {
        super.setSourceFeatureType(sourceFeatureType);

    }

    public static void registerPersistence() {

        PersistenceManager manager = ToolsLocator.getPersistenceManager();
        if (manager.getDefinition("ExportShapeParametersImpl") == null) {
            DynStruct definition = manager.addDefinition(ExportShapeParametersImpl.class,
                    "ExportShapeParametersImpl", "ExportShapeParametersImpl persistence definition", null, null);
            definition.addDynFieldString("encoding");
            definition.addDynFieldFile("file");
            definition.addDynFieldBoolean("useSimplifiedExport");
            definition.extend(manager.getDefinition("AbstractExportParametersGeometry"));
        }
    }

    @Override
    public void saveToState(PersistentState state) throws PersistenceException {
        super.saveToState(state);
        state.set("encoding", this.encoding);
        state.set("file", this.file);
        state.set("useSimplifiedExport", this.useSimplifiedExport);
    }

    @Override
    public void loadFromState(PersistentState state) throws PersistenceException {
        super.loadFromState(state);
        this.encoding = state.getString("encoding");
        this.file = state.getFile("file");
        this.useSimplifiedExport = state.getBoolean("useSimplifiedExport");
    }

}
