/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.shape.swing.panels;

import javax.swing.JComponent;


import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.shape.service.ExportShapeParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author gvSIG Team
 * @version $Id$
 * 
 */
public class SelectSimplifiedExport 
        extends SelectSimplifiedExportView 
        implements ExportPanel
    {

    private static final Logger LOG = LoggerFactory.getLogger(SelectSimplifiedExport.class);
    
    private final ExportShapeParameters parameters;
    private final JExportProcessPanel processPanel;

    public SelectSimplifiedExport(JExportProcessPanel processPanel, ExportParameters parameters) {
        super();
        this.processPanel = processPanel;
        this.parameters = (ExportShapeParameters) parameters;
        this.initComponents();
    }

    private void initComponents() {
        this.chkUseSimplifiedExport.setSelected(parameters.getUseSimplifiedExport());
        
        this.translate();
    }
    
    private void translate() {
        ToolsSwingManager i18nc = ToolsSwingLocator.getToolsSwingManager();
        I18nManager i18n = ToolsLocator.getI18nManager();

        i18nc.translate(this.lblUseSimplifiedExport);
        String s = i18n.getTranslation(this.lblUseSimplifiedExportDescription.getText());
        s = "<html>" + s.replace("\n","<br>\n") + "</html>";
        this.lblUseSimplifiedExportDescription.setText(s);
    }

    public boolean getUseSimplifiedExport() {
        return this.chkUseSimplifiedExport.isSelected();
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    @Override
    public String getIdPanel() {
        return this.getClass().getSimpleName();
    }
    
    @Override
    public String getTitlePanel() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Select_simplified_export");    
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        return true;
    }

    @Override
    public void enterPanel() {
        // Do nothing
    }

    @Override
    public void previousPanel() {

    }

    @Override
    public void nextPanel() {
        this.parameters.setUseSimplifiedExport(this.getUseSimplifiedExport());
    }
}
