package org.gvsig.export.shape.swing.panels;

import javax.swing.JComponent;
import org.gvsig.export.ExportParameters;
import org.gvsig.export.shape.service.ExportShapeParameters;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.export.swing.spi.ExportPanel;
import org.gvsig.export.swing.spi.ExportPanelValidationException;

/**
 *
 * @author jjdelcerro
 */
public class UseSimplifiedExportWrapper 
        implements ExportPanel
    {

    private final ExportPanel wraped;
    private final ExportShapeParameters parameters;
    private final JExportProcessPanel processPanel;
    
    public UseSimplifiedExportWrapper(ExportPanel wraped, JExportProcessPanel processPanel, ExportParameters parameters) {
        this.wraped = wraped;
        this.parameters = (ExportShapeParameters) parameters;
        this.processPanel = processPanel;
    }

    @Override
    public String getIdPanel() {
        return this.wraped.getIdPanel();
    }

    @Override
    public String getTitlePanel() {
        return this.wraped.getTitlePanel();
    }

    @Override
    public boolean validatePanel() throws ExportPanelValidationException {
        if( this.parameters.getUseSimplifiedExport() ) {
            return true;
        }
        return this.wraped.validatePanel();
    }

    @Override
    public void enterPanel() {
        if( this.parameters.getUseSimplifiedExport() ) {
            switch(this.processPanel.getLastAction()) {
                case JExportProcessPanel.BUTTON_BACK:
                    this.processPanel.previousPanel();
                    return;
                case JExportProcessPanel.BUTTON_NEXT:
                    this.processPanel.nextPanel();
                    return;
            }
        }
        this.wraped.enterPanel();
    }

    @Override
    public void nextPanel() {
        if( this.parameters.getUseSimplifiedExport() ) {
            return;
        }
        this.wraped.nextPanel();
    }

    @Override
    public void previousPanel() {
        if( this.parameters.getUseSimplifiedExport() ) {
            return;
        }
        this.wraped.previousPanel();
    }

    @Override
    public JComponent asJComponent() {
        return this.wraped.asJComponent();
    }
    
}
