package org.gvsig.expressionevaluator;

import org.gvsig.expressionevaluator.ExpressionBuilder.Value;
import org.gvsig.tools.visitor.Visitable;

public interface Code extends Visitable {

    public static final int UNDEFINED = -1;
    public static final int CONSTANT = 0;
    public static final int IDENTIFIER = 1;
    public static final int CALLER = 2;
    public static final int METHOD = 3;
    public static final int CODES = 4;
    
    public interface Constant extends Code {

        public Object value();

    }

    public interface Identifier extends Code {

        public String name();

    }

    public interface Caller extends Code {

        public static final int FUNCTION = 0;
        public static final int BINARY_OPERATOR = 1;
        public static final int UNARY_OPERATOR = 2;


        public String name();

        public Object call(Interpreter interpreter, Object[] args) throws Exception;

        public Function function();

        public Function function(Function function);

        public Codes parameters();

        public int type();

    }
    
    public interface Method extends Code {
        public Code instance();
        
        public String methodname();

        public Codes parameters();

        public Object call(Interpreter interpreter, Object[] args) throws Exception;
    }

    public static final Formatter<Code> EMPTY_FORMATTER = new Formatter<Code>() {
        @Override
        public boolean canApply(Code value) {
            return false;
        }

        @Override
        public String format(Code value) {
            return "";
        }
    };

    public int code();

    public Value toValue();

    public Value toValue(ExpressionBuilder builder);
    
    public String toString(Formatter<Code> formatter);
    
    public void link(SymbolTable symbolTable);
    
}
