package org.gvsig.expressionevaluator;

import org.gvsig.expressionevaluator.Code.Caller;
import org.gvsig.expressionevaluator.Code.Constant;
import org.gvsig.expressionevaluator.Code.Identifier;

import org.gvsig.tools.lang.Cloneable;

public interface CodeBuilder extends Cloneable {

    @Override
    public CodeBuilder clone() throws CloneNotSupportedException;    
    
    Constant constant(Object value);

    Identifier identifier(String name);

    Caller function(String name, int type, Codes args);

    Caller function(String name, Codes args);

    Caller operator(String name, Code arg1);

    Caller operator(String name, Code arg1, Code arg2);

    Codes args();

    Code add(Code op1, Code op2);

    Code subst(Code op1, Code op2);

    Code concat(Code op1, Code op2);

    Code mult(Code op1, Code op2);

    Code div(Code op1, Code op2);

    Code mod(Code op1, Code op2);

    Code or(Code op1, Code op2);

    Code and(Code op1, Code op2);

    Code like(Code op1, Code op2);

    Code ilike(Code op1, Code op2);

    Code regexp(Code op1, Code op2);

    Code lt(Code op1, Code op2);

    Code gt(Code op1, Code op2);

    Code le(Code op1, Code op2);

    Code ge(Code op1, Code op2);

    Code eq(Code op1, Code op2);

    Code ne(Code op1, Code op2);

    Code is(Code op1, Code op2);

    Code not(Code op1);

    Code negate(Code op1);

    Code getattr(Code obj, String attrname);

    Code getitem(Code obj, Code index);

    Code method(Code obj, String methodname, Codes args);
}
