package org.gvsig.expressionevaluator;

import java.util.List;
import java.util.Map;
import org.cresques.cts.IProjection;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.util.PropertiesSupport;
import org.gvsig.tools.util.PropertiesSupportHelper;

public interface ExpressionBuilder {

    public static final String OPERATOR_NOT = "NOT";
    public static final String OPERATOR_NEGATE = "NEGATE";
    public static final String OPERATOR_IS = "IS";
    public static final String OPERATOR_AND = "AND";
    public static final String OPERATOR_OR = "OR";
    public static final String OPERATOR_EQ = "=";
    public static final String OPERATOR_NE = "<>";
    public static final String OPERATOR_GT = ">";
    public static final String OPERATOR_GE = ">=";
    public static final String OPERATOR_LT = "<";
    public static final String OPERATOR_LE = "<=";
    public static final String OPERATOR_LIKE = "LIKE";
    public static final String OPERATOR_ILIKE = "ILIKE";
    public static final String OPERATOR_ADD = "+";
    public static final String OPERATOR_SUBST = "-";
    public static final String OPERATOR_MULT = "*";
    public static final String OPERATOR_DIV = "/";
    public static final String OPERATOR_MOD = "%";
    public static final String OPERATOR_CONCAT = "||";
    public static final String OPERATOR_REGEXP = "~";
    public static final String OPERATOR_AND2 = "&&";
    
    public static final String FUNCTION_ST_SRID = "ST_SRID";
    public static final String FUNCTION_ST_ASTEXT = "ST_AsText";
    public static final String FUNCTION_ST_ASBINARY = "ST_AsBinary";
    public static final String FUNCTION_ST_ASEWKB = "ST_AsEWKB";
    public static final String FUNCTION_ST_ENVELOPE = "ST_Envelope";
    public static final String FUNCTION_ST_GEOMFROMTEXT = "ST_GeomFromText";
    public static final String FUNCTION_ST_GEOMFROMWKB = "ST_GeomFromWKB";
    public static final String FUNCTION_ST_GEOMFROMEWKB = "ST_GeomFromEWKB";
    public static final String FUNCTION_ST_SIMPLIFY = "ST_Simplify";
    public static final String FUNCTION_ST_EQUALS = "ST_Equals";
    public static final String FUNCTION_ST_INTERSECTS = "ST_Intersects";
    public static final String FUNCTION_ST_CONTAINS = "ST_Contains";
    public static final String FUNCTION_ST_DISJOINT = "ST_Disjoint";
    public static final String FUNCTION_ST_CROSSES = "ST_Crosses";
    public static final String FUNCTION_ST_ISCLOSED = "ST_IsClosed";
    public static final String FUNCTION_ST_OVERLAPS = "ST_Overlaps";
    public static final String FUNCTION_ST_TOUCHES = "ST_Touches";
    public static final String FUNCTION_ST_WITHIN = "ST_Within";
    public static final String FUNCTION_ST_SETSRID = "ST_SetSRID";
    public static final String FUNCTION_ST_POINT = "ST_Point";
    public static final String FUNCTION_ST_MAKEPOINT = "ST_MakePoint";
    public static final String FUNCTION_ST_UNIONAGGREGATE = "ST_UnionAggregate";
    public static final String FUNCTION_ST_EXTENTAGGREGATE = "ST_ExtentAggregate";

    public static final String FUNCTION_IIF = "IIF";
    public static final String FUNCTION_IFNULL = "IFNULL";
    
    public static final String FUNCTION_LEFT = "LEFT";
    public static final String FUNCTION_RIGHT = "RIGHT";
    public static final String FUNCTION_LOCATE = "LOCATE";
    public static final String FUNCTION_POSITION = "POSITION";
    public static final String FUNCTION_LPAD = "LPAD";
    public static final String FUNCTION_RPAD = "RPAD";
    public static final String FUNCTION_LTRIM = "LTRIM";
    public static final String FUNCTION_RTRIM = "RTRIM";
    public static final String FUNCTION_TRIM = "TRIM";
    public static final String FUNCTION_REPEAT = "REPEAT";
    public static final String FUNCTION_REPLACE = "REPLACE";
    public static final String FUNCTION_ASCII = "ASCII";
    public static final String FUNCTION_LENGHT = "LENGHT";
    public static final String FUNCTION_INSTR = "INSTR";
    public static final String FUNCTION_LOWER = "LOWER";
    public static final String FUNCTION_UPPER = "UPPER";
    public static final String FUNCTION_SPACE = "SPACE";
    public static final String FUNCTION_SUBSTRING = "SUBSTRING";
    public static final String FUNCTION_ACOS = "ACOS";
    public static final String FUNCTION_ASIN = "ASIN";
    public static final String FUNCTION_ATAN = "ATAN";
    public static final String FUNCTION_COS = "COS";
    public static final String FUNCTION_COSH = "COSH";
    public static final String FUNCTION_COT = "COT";
    public static final String FUNCTION_BITAND = "BITAND";
    public static final String FUNCTION_BITOR = "BITOR";
    public static final String FUNCTION_BITXOR = "BITXOR";
    public static final String FUNCTION_CEIL = "CEIL";
    public static final String FUNCTION_DEGREES = "DEGREES";
    public static final String FUNCTION_EXP = "EXP";
    public static final String FUNCTION_FLOOR = "FLOOR";
    public static final String FUNCTION_LOG = "LOG";
    public static final String FUNCTION_LOG10 = "LOG10";
    public static final String FUNCTION_PI = "PI";
    public static final String FUNCTION_POWER = "POWER";
    public static final String FUNCTION_RADIANS = "RADIANS";
    public static final String FUNCTION_RAND = "RAND";
    public static final String FUNCTION_ROUND = "ROUND";
    public static final String FUNCTION_SQRT = "SQRT";
    public static final String FUNCTION_SIGN = "SIGN";
    public static final String FUNCTION_SIN = "SIN";
    public static final String FUNCTION_SINH = "SINH";
    public static final String FUNCTION_TAN = "TAN";
    public static final String FUNCTION_TANH = "TANH";
    public static final String FUNCTION_ZERO = "ZERO";
    public static final String FUNCTION_CHR = "CHR";
    public static final String FUNCTION_DATE = "DATE";
    public static final String FUNCTION_DATEDIFF = "DATEDIFF";
    public static final String FUNCTION_DATEADD = "DATEADD";
    public static final String FUNCTION_NOW = "NOW";
    public static final String FUNCTION_TIME = "TIME";
    public static final String FUNCTION_TIMESTAMP = "TIMESTAMP";

    public static final String FUNCTION_CAST = "CAST";
    public static final String FUNCTION_DECODE = "DECODE";
    public static final String FUNCTION_TODOUBLE = "TODOUBLE";
    public static final String FUNCTION_TOFLOAT = "TOFLOAT";
    public static final String FUNCTION_TOINTEGER = "TOINTEGER";
    public static final String FUNCTION_TOLONG = "TOLONG";
    public static final String FUNCTION_TOSTR = "TOSTR";
    
    public static final String FUNCTION_INCR = "INCR";
    public static final String FUNCTION_DECR = "DECR";
    
    public static final String FUNCTION_RANGE = "RANGE";
    public static final String FUNCTION_EVAL = "EVAL";
    public static final String FUNCTION_LIST = "LIST";
    public static final String FUNCTION_TUPLE = "TUPLE";
    
    public enum GeometrySupportType {
        WKT,
        WKB,
        EWKB,
        NATIVE
    }

    public enum ParameterType {
        Variable,
        Constant,
        Geometry
    }
    
    public interface VisitorFilter {
        boolean accept(Visitable visitable);
    }

     public interface Visitor {
        public void visit(Visitable value);
    }

    public interface Visitable {
        public void accept(Visitor visitor, VisitorFilter filter);
    }
        
    public interface Value extends Visitable, PropertiesSupport {
        public String toString(Formatter<Value> formatter);
        public void replace(Value target, Value replacement);
    }

    public interface Group extends Value {
        public Value value();
    }

    public interface Constant extends Value {
        public Object value();
    }

    public interface Custom extends Value {
        public Object value();
        public Custom add(Variable variable);
        public Custom add(Parameter parameter);
    }

    public interface Variable extends Value, Comparable<Variable> {
        public String name();
    }

    public interface Parameter extends Value {
        public Parameter as_constant();
        public Parameter as_variable();        
        public Parameter as_geometry_variable();
        public Parameter srs(Value srs);
        public Parameter srs(IProjection srs);
        public Parameter value(Object value);
        public Parameter name(String value);
        public String name();
        public Object value();
        public ParameterType type();
        public Value srs();
        public boolean is_constant();
        public boolean is_variable();
        public boolean is_geometry_variable();        
    }
    
    public interface Function extends Value {
        public String name();
        public List<Value> parameters();
        public Function parameter(Value parameter);
    }

    public interface Method extends Value {
        public String name();
        public Value instance();
        public List<Value> parameters();
        public Function parameter(Value parameter);
    }

    public interface BinaryOperator extends Value {
        public String name();
        public Value left();
        public Value right();
        public BinaryOperator left(Value operand);
        public BinaryOperator right(Value operand);
    }

    public abstract class AbstractValue implements Value {

        private PropertiesSupport properties;
        
        protected AbstractValue() {
            this.properties = new PropertiesSupportHelper();
        }
        
        @Override
        public void accept(Visitor visitor, VisitorFilter filter) {
            if (filter == null || filter.accept(this)) {
                visitor.visit(this);
            }
        }

        @Override
        public Map<String, Object> getProperties() {
            return this.properties.getProperties();
        }

        @Override
        public Object getProperty(String name) {
            return this.properties.getProperty(name);
        }

        @Override
        public void setProperty(String name, Object value) {
            this.properties.setProperty(name, value);
        }

        @Override
        public void replace(Value target, Value replacement) {
            // Do nothing
        }
    }

    public class ClassVisitorFilter implements VisitorFilter {

        private final Class classFilter;

        public ClassVisitorFilter(Class classFilter) {
            this.classFilter = classFilter;
        }

        @Override
        public boolean accept(Visitable visitable) {
            if( classFilter==null ) {
                return true;
            }
            return classFilter.isInstance(visitable);
        }

    }

    public static final Formatter<Value> EMPTY_FORMATTER = new Formatter<Value>() {
        @Override
        public boolean canApply(Value value) {
            return false;
        }

        @Override
        public String format(Value value) {
            return "";
        }
    };
            
    public String quote_for_identifiers();
    
    public String quote_for_strings();
    
    public void accept(Visitor visitor, VisitorFilter filter);

    public List<Variable> variables();
    
    public List<String> variables_names();
    
    public List<Parameter> parameters();
    
    public List<String> parameters_names();
    
    public GeometrySupportType geometry_support_type();
    
    public ExpressionBuilder geometry_support_type(GeometrySupportType geometrySupportType);

    public String identifier(String name);

    public String bytearray_hex(byte[] data);

    public String bytearray_0x(byte[] data);

    public String bytearray_x(byte[] data);

    public String string(String s);
   
    public Object srs_id(IProjection projection);
    
    public Value value();
    
    public ExpressionBuilder value(Value value);

    public Group group(Value group);
    
    public Constant bytearray(byte[] data);
    
    public Variable variable(String name);
    
    public Variable column(String name); // Alias for variable(name)

    public Parameter parameter(String name);

    public Parameter parameter();

    public Constant constant(Object value);
    
    public Constant geometry(Geometry geom, IProjection projection);

    public Constant geometry(Geometry geom);

    public Constant envelope(Envelope envelope, IProjection projection);

    public Constant envelope(Envelope envelope);

    public Constant srs(IProjection projection);

    public Custom custom(Object value);
    
    public Function as_geometry(Value value);

    public Method method(Value instance, String name, Value... values);

    public Function function(String name, Value... values);
    
    public BinaryOperator binaryOperator(String name, Value leftOperand, Value rightOperand);

    public Value toValue(String expression);
    
    public String toString(Formatter<Value> formatter);
    
    public Function not(Value value);

    public Function not_is_null(Value value);
    
    public Function is_null(Value value);

    
    public Function ST_Area(Value geom);

    public Function ST_AsBinary(Value geom);

    public Function ST_AsEWKB(Value geom);

    public Function ST_AsText(Value geom);

    public Function ST_Buffer(Value geom);

    public Function ST_Centroid(Value geom);

    public Function ST_Contains(Value geom1, Value geom2);

    public Function ST_CoveredBy(Value geom1, Value geom2);

    public Function ST_Covers(Value geom1, Value geom2);

    public Function ST_Crosses(Value geom1, Value geom2);

    public Function ST_Diference(Value geom1, Value geom2);

    public Function ST_Dimension(Value geom1);

    public Function ST_Disjoint(Value geom1, Value geom2);

    public Function ST_Distance(Value geom1, Value geom2);

    public Function ST_EndPoint(Value geom1);

    public Function ST_Envelope(Value geom);

    public Function ST_Equals(Value geom1, Value geom2);

    public Function ST_GeomFromText(Value geom, Value crs);
    
    public Function ST_GeomFromWKB(Value geom, Value crs);
            
    public Function ST_GeomFromEWKB(Value geom, Value crs);

    public Function ST_Intersection(Value geom1, Value geom2);

    public Function ST_Intersects(Value geom1, Value geom2);

    public Function ST_IsClosed(Value geom);

    public Function ST_IsSimple(Value geom);

    public Function ST_IsValid(Value geom);

    public Function ST_NumGeometries(Value geom);
    
    public Function ST_NumPoints(Value geom);

    public Function ST_Overlaps(Value geom1, Value geom2);

    public Function ST_Perimeter(Value geom);

    public Function ST_Point(Value x, Value y);

    public Function ST_MakePoint(Value x, Value y);

    public Function ST_MakePoint(Value x, Value y, Value z);

    public Function ST_MakePoint(Value x, Value y, Value z, Value m);

    public Function ST_PointN(Value geom, Value n);

    public Function ST_SRID(Value geom);

    public Function ST_SetSRID(Value geom, Value srid);
    
    public Function ST_StartPoint(Value geom);

    public Function ST_Simplify(Value geom, Value tolerance);

    public Function ST_Touches(Value geom1, Value geom2);

    public Function ST_Union(Value geom1, Value geom2);

    public Function ST_Within(Value geom1, Value geom2);

    public Function ST_X(Value geom);

    public Function ST_Y(Value geom);

    public Function ST_Z(Value geom);

    
    public Function ST_UnionAggregate(Value geom);
    
    public Function ST_ExtentAggregate(Value geom);


    
    public BinaryOperator and(Value op1, Value op2);

    public BinaryOperator and(Expression op1, Expression op2);

    public BinaryOperator and(Expression op1, Value op2);

    public BinaryOperator or(Value op1, Value op2);

    public BinaryOperator eq(Value op1, Value op2);
    
    public BinaryOperator ne(Value op1, Value op2);
    
    public BinaryOperator gt(Value op1, Value op2);
    
    public BinaryOperator ge(Value op1, Value op2);

    public BinaryOperator lt(Value op1, Value op2);
    
    public BinaryOperator le(Value op1, Value op2);

    public BinaryOperator like(Value op1, Value op2);

    public BinaryOperator ilike(Value op1, Value op2);

    public BinaryOperator add(Value op1, Value op2);

    public BinaryOperator subst(Value op1, Value op2);

    public BinaryOperator mult(Value op1, Value op2);

    public BinaryOperator div(Value op1, Value op2);

    public BinaryOperator concat(Value op1, Value op2);

    public ExpressionBuilder set(Value value);
    
    public ExpressionBuilder and(Value value);

    public ExpressionBuilder or(Value value);

    public Function iif(Value condition, Value iftrue, Value iffalse);
    
    public Function ifnull(Value value, Value iftrue, Value iffalse);

    public ExpressionBuilder createExpressionBuilder();
   
    public Function left(Value str, Value size);
    
    public Function right(Value str, Value len);
    
    public Function locate(Value search, Value str, Value start) ;
    
    public Function position(Value search, Value str);
    
    public Function lpad(Value str, Value len, Value padstr) ;
    
    public Function rpad(Value str, Value len, Value padstr);
    
    public Function ltrim(Value str);
    
    public Function rtrim(Value str);
    
    public Function trim(Value str);
    
    public Function repeat(Value str, Value size);
    
    public Function replace(Value str, Value search, Value replstr);
    
    public Function ascii(Value str);
    
    public Function lenght(Value str);

    public Function instr(Value str, Value search, Value start);
    
    public Function lower(Value str);
    
    public Function upper(Value str);
    
    public Function space(Value size);
    
    public Function substring(Value str, Value start, Value len);
    
    public Function acos(Value num);
    
    public Function asin(Value num);
    
    public Function atan(Value num);
    
    public Function cos(Value num);
    
    public Function cosh(Value num);
    
    public Function cot(Value num);
    
    public Function bitand(Value num1, Value num2);
    
    public Function bitor(Value num1, Value num2);
    
    public Function bitxor(Value num1, Value num2);
    
    public Function ceil(Value num);
    
    public Function degrees(Value num);
    
    public Function exp(Value num);
    
    public Function floor(Value num);
    
    public Function log(Value num);
    
    public Function log10(Value num);
    
    public Function pi(Value num);
    
    public Function power(Value num);
    
    public Function radians(Value num);
    
    public Function rand(Value num);
    
    public Function round(Value num);
    
    public Function sqrt(Value num);
    
    public Function sign(Value num);
    
    public Function sin(Value num);
    
    public Function sinh(Value num);
    
    public Function tan(Value num);
    
    public Function tanh(Value num);
    
    public Function zero();
    
    public Function chr(Value num);

    public Function decode(Value value, Value format);

    public Function cast(Value object, Value typeName);

    public Function toDouble(Value object);

    public Function toFloat(Value object);

    public Function toLong(Value object);

    public Function toInteger(Value object);

    public Function toStr(Value object);

    public Function list();
    
    public Function tuple();
    
    public Function tuple(Constant... values);
    
    public boolean isEmpty();
}
