
package org.gvsig.expressionevaluator;

import java.util.Collection;
import org.gvsig.tools.bookmarksandhistory.Bookmarks;
import org.gvsig.tools.bookmarksandhistory.History;
import org.gvsig.tools.resourcesstorage.ResourcesStorage;
import org.gvsig.tools.script.ScriptManager;


public interface ExpressionEvaluatorManager extends ScriptManager {
    
    public Object evaluate(String source);
    
    public Object evaluate(SymbolTable symbolTable, String source);
    
    public Object evaluate(SymbolTable symbolTable, Code code);
    
    public String evaluateDynamicText(String source);    
    
    public String evaluateDynamicText(SymbolTable symbolTable, String source);    
    
    public boolean isDynamicText(String source);
    
    public Code compile(String source);
    
    public Code compile(LexicalAnalyzer lex, String source);
    
    public Code optimize(SymbolTable symbolTable, Code code);
    
    public MutableSymbolTable createSymbolTable();
    
    public SymbolTable getInmutableSymbolTable();
    
    public SymbolTable getSymbolTable(String name);
    
    public Collection<SymbolTableFactory> getSymbolTableFactories();
    
    public void registerSymbolTable(SymbolTableFactory symbolTableFactory);
    
    public LexicalAnalyzer createLexicalAnalyzer();
    
    public CodeBuilder createCodeBuilder();

    public Compiler createCompiler();
    
    public Interpreter createInterpreter();
    
    public Optimizer createOptimizer();
    
    public Double getAccuracy();
    
    public void setAccuracy(Double accuracy);     

    public Expression createExpression();
    
    public ExpressionBuilder createExpressionBuilder();

    public void registerGrammar(GrammarFactory factory) ;

    public Collection<GrammarFactory> getGrammarFactories();

    public Grammar createGrammar(String name);
    
    public Bookmarks<Expression> getBookmarks();
    
    public History<Expression> getHistory();
    
    public ResourcesStorage getScriptsResourcesStorage();

    public void setScriptsResourcesStorage(ResourcesStorage scriptsResourcesStorage);
    
}
