package org.gvsig.expressionevaluator;

import org.apache.commons.lang3.StringUtils;

/**
 *
 * @author jjdelcerro
 */
public class ExpressionSyntaxException extends RuntimeException {

    private final int position;
    private final int line;
    private final int column;
    private final String phrase;
    private final String description;
    private String tip;
    
    public ExpressionSyntaxException() {
        super("Syntax error in expression.");
        this.phrase = null;
        this.position = -1;
        this.line = -1;
        this.column = -1;
        this.description = I18N.Syntax_error_in_expression();
        this.tip = null;
    }
    
    public ExpressionSyntaxException(LexicalAnalyzer lexer) {
        super("Syntax error in '"+getSource(lexer.getSource(), lexer.getPosition())+"' near character "+ lexer.getPosition()+" ("+lexer.getLine()+":"+lexer.getColumn()+").");
        this.phrase = lexer.getSource();
        this.position = lexer.getPosition();
        this.line = lexer.getLine();
        this.column = lexer.getColumn();
        this.description = I18N.Syntax_error_near_character_XPositionX(position);
    }
    
    public ExpressionSyntaxException(String msg, LexicalAnalyzer lexer) {
        super("Syntax error in '"+getSource(lexer.getSource(), lexer.getPosition()) +"' near character "+ lexer.getPosition()+" ("+lexer.getLine()+":"+lexer.getColumn()+"). "+msg);
        this.phrase = lexer.getSource();
        this.position = lexer.getPosition();
        this.line = lexer.getLine();
        this.column = lexer.getColumn();
        this.description = I18N.Syntax_error_near_character_XPositionX(position)+ " "+msg;
    }

    public ExpressionSyntaxException(String msg, LexicalAnalyzer lexer, String tip) {
        this(msg,lexer);
        this.tip = tip;
    }
    
    public ExpressionSyntaxException(String phrase, int position) {
        super("Syntax error in '"+getSource(phrase, position) +"' near character "+ position+".");
        this.phrase = phrase;
        this.position = position;
        this.line = -1;
        this.column = -1;
        this.description = I18N.Syntax_error_near_character_XPositionX(position);
    }
    
    public ExpressionSyntaxException(String msg, String phrase, int position) {
        super("Syntax error in '"+ getSource(phrase, position) +"' near character "+ position+". "+msg);
        this.phrase = phrase;
        this.position = position;
        this.line = -1;
        this.column = -1;
        this.description = I18N.Syntax_error_near_character_XPositionX(position)+" "+msg;
    }
    
    private static String getSource(String source, int position) {
        String s = StringUtils.left(source, position) + "[*]" + StringUtils.mid(source, position, 200);
        if( s.length()>200 ) {
            s = "..."+StringUtils.mid(s, position-100, 200)+"...";
        }
        return s;
    }
    
    public String getPhrase() {
        return this.phrase;
    }
    
    public int getPosition() {
        return this.position;
    }
    
    public int getLine() {
        return this.line;
    }
    
    public int getColumn() {
        return this.column;
    }
    
    public String getDescription() {
        return this.description;
    }
    
    public String getTip() {
        return this.tip;
    }
}
