package org.gvsig.expressionevaluator;

import java.util.List;
import org.apache.commons.lang3.Range;

public interface Function {

    public final String GROUP_NUMERIC = "Numeric";
    public final String GROUP_STRING = "String";
    public final String GROUP_DATETIME = "Datetime";
    public final String GROUP_BOOLEAN = "Boolean";
    public final String GROUP_OGC = "OGC";
    public final String GROUP_PROGRAMMING = "Programming";
    public final String GROUP_CONVERSION = "Conversion";
    public final String GROUP_OTHER = "Other";
   
    public String group();

    public String name();
    
    public String returnType();

    public Range argc();

    public String description();

    public String template();
    
    public String[] descriptionArgs();

    public Object call(Interpreter interpreter, Object[] args) throws Exception;
    
    public Object call(Interpreter interpreter, Codes args) throws Exception;
    
    public void addAlias(String name);
    
    public List<String> aliases();
    
    public boolean isOperator();
    
    public boolean useArgumentsInsteadObjects();
    
    public boolean isSQLCompatible();
    
    /**
     * Returns true if the function always returns the same value for 
     * the same parameters. If so, it is possible to optimize the 
     * generated code through the process of "constant folding".
     * 
     * https://en.wikipedia.org/wiki/Constant_folding
     * 
     * @return true if this function allow constant folding optimization.
     */
    public boolean allowConstantFolding();
}
