package org.gvsig.expressionevaluator;

import java.io.Writer;
import org.gvsig.tools.lang.Cloneable;
import org.gvsig.tools.resourcesstorage.ResourcesStorage;

public interface Interpreter extends Cloneable {

    public interface Cache {
        
        public Object get(Object context, Object key);
        
        public void put(Object context, Object key, Object value);
        
        public void remove(Object context, Object key);
        
        public void removeAll();
    }
    
    public void setSymbolTable(SymbolTable symbolTable);

    public SymbolTable getSymbolTable();

    public Object run(Code code);

    public void run(MutableSymbolTable symbolTable, Code code);

    public void link(Code code);

    public Double getAccuracy();

    public void setAccuracy(Double accuracy);
    
    public Code getCurrentCode();
    
    public Cache getCache();

    @Override
    public Interpreter clone() throws CloneNotSupportedException;
    
    public void setSQLCompatible(boolean sqlCompatible);
    
    public boolean isSQLCompatible();
    
    public Object call(String function, Object... args) throws Exception;
    
    public Object call(SymbolTable symbolTable, String funcname, Object... args) throws Exception ;

    public boolean hasFunction(String function);
    
    public Writer getWriter();
    
    public void setWriter(Writer writer);
    
    public void setResourcesStorage(ResourcesStorage resources);
    
    public ResourcesStorage getResourcesStorage();
}
