package org.gvsig.expressionevaluator;


import org.gvsig.tools.lang.Cloneable;


public interface LexicalAnalyzer extends Cloneable {

    public interface Token extends Cloneable {

        public static final int EOF = -1;
        public static final int IDENTIFIER = 0;
        public static final int STRING_LITERAL = 1;
        public static final int INTEGER_LITERAL = 2;
        public static final int FLOATING_POINT_LITERAL = 3;
        public static final int NULL = 4;
        public static final int TRUE = 5;
        public static final int FALSE = 6;
        public static final int OP_NOT = 9;
        public static final int OP_ADD = 10;
        public static final int OP_SUBST = 11;
        public static final int OP_MULT = 12;
        public static final int OP_DIV = 13;
        public static final int OP_MOD = 14;
        public static final int OP_REGEXP = 15;
        public static final int OP_LT = 16;
        public static final int OP_GT = 17;
        public static final int OP_LE = 18;
        public static final int OP_GE = 19;
        public static final int OP_EQ = 20;
        public static final int OP_NE = 21;
        public static final int PRED_BETWEEN = 22;
        public static final int PRED_LIKE = 23;
        public static final int PRED_ILIKE = 24;
        public static final int PRED_IS = 25;
        public static final int OP_OR = 26;
        public static final int OP_AND = 27;
        public static final int ISNULL = 28;
        public static final int NOTNULL = 29;
        public static final int PARENTHESIS_OPEN = 30;
        public static final int PARENTHESIS_CLOSE = 31;
        public static final int COMA = 32;
        public static final int OP_GETATTR = 33;
        public static final int CHAR = 34;
        public static final int OP_CONCAT = 35;
        public static final int OPEN_BRACKET = 36;
        public static final int CLOSED_BRACKET = 37;

        @Override
        public Token clone() throws CloneNotSupportedException;
        
        public void set(int type, String literal);

        public void set(int type, String literal, Object value);

        public int getType();

        public Object getValue();

        public String getLiteral();

        public void setLiteral(String value);

        public boolean is(String... value);
    }

    public void setSource(String source);
    
    public Token look();

    public Token next();

    @Override
    public LexicalAnalyzer clone() throws CloneNotSupportedException;

    public boolean isEOF();
    
    public int getPosition(); 
    
    public int getLine();
    
    public int getColumn();
    
    public String getSource();
    
    public void setUseBracketsForIdentifiers(boolean useBracketsForIdentifiers);
    
    public boolean getUseBracketsForIdentifiers();
}
