package org.gvsig.expressionevaluator;

/**
 *
 * @author jjdelcerro
 */
public interface Statement {

    public interface StatementContext {

        public Compiler getCompiler();

        public LexicalAnalyzer getLexicalAnalyzer();

        public void setCode(String id, Code code);

        public Code getCode(String id);

        public void setCodeClassifier(String classifier);

        public String getCodeClassifier();

        public CodeBuilder getCodeBuilder();

        public LexicalAnalyzer.Token look_token();

        public LexicalAnalyzer.Token next_token();

        public Code parse_expression();

        public Codes parse_expressions(String separator);
        
        public boolean isReservedWord(String s);

    }

    public interface Rule {

        public void parse(StatementContext context);
    }

    public interface CompoundRule extends Rule {

        public CompoundRule addRule(Rule rule);
    }

    public interface ConditionalRule extends Rule {

        public ConditionalRule addRuleOnTrue(Rule rule);

        public ConditionalRule addRuleOnFalse(Rule rule);
    }

    public interface ArgsBuilder {

        public Codes build(StatementContext context);
    }

    public interface StatementBuilder {
        
        public String getCodeID();
        
        public ArgsBuilder getArgsBuilder();
        
        public Code build(StatementContext context);

    }

    public String getName();

    public Statement addRule(Rule rule);

    public Rule require_any_token(String... token);

    public CompoundRule repeat_until_any_tokens(String... token);

    public Rule require_identifier(String id);

    public Rule require_literal_string(String id);

    public Rule set_expression(String id, Object value);

    public Rule require_expression(String id);

    public Rule require_expressions(String id, String separator);

    public ConditionalRule optional_any_token(String... id);

    public ConditionalRule optional_identifiers(String id, String separator);
    
    public ConditionalRule optional_literal_string(String id);
        
    public ArgsBuilder args_names(String... args);
    
    public ArgsBuilder args_expand(String... args);
    
    public void code(String id, ArgsBuilder argsBuilder);

    public void builder(StatementBuilder stmtBuilder);

    public Code parse(StatementContext context);

    public boolean isApplicable(StatementContext context);


}
