package org.gvsig.expressionevaluator.impl;

import org.gvsig.expressionevaluator.Grammar;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Statement;
import org.gvsig.expressionevaluator.Statement.StatementContext;

/**
 *
 * @author jjdelcerro
 */
public class DefaultGrammar implements Grammar {

    private final String name;
    private final List<Statement> statements;
    private final Set<String> reservedWords;
    
    public DefaultGrammar(String name) {
        this.name = name;
        this.statements = new ArrayList<>();
        this.reservedWords = new HashSet<>();
    }
    
    @Override
    public String getName() {
        return this.name;
    }

    @Override
    public List<Statement> getStatements() {
        return Collections.unmodifiableList(this.statements);
    }

    @Override
    public Statement createStatement(String name) {
        DefaultStatement stmt = new DefaultStatement(name);
        return stmt;
    }

    @Override
    public void addStatement(Statement statement) {
        this.statements.add(statement);
    }

    @Override
    public Statement getApplicableStatement(StatementContext context) {
        for (Statement statement : statements) {
            if( statement.isApplicable(context) ) {
                return statement;
            }
        }
        return null;
    }

    @Override
    public String toString() {
        return this.getName();
    }

    @Override
    public void addReservedWord(String id) {
        if( StringUtils.isBlank(id) ) {
            return;
        }
        this.reservedWords.add(id.toUpperCase());
    }

    @Override
    public boolean isReservedWord(String id) {
        if( StringUtils.isBlank(id) ) {
            return false;
        }
        return this.reservedWords.contains(id.toUpperCase());
    }

    @Override
    public Iterator<Statement> iterator() {
        return this.statements.iterator();
    }

}
