package org.gvsig.expressionevaluator.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Grammar;
import org.gvsig.expressionevaluator.GrammarSet;
import org.gvsig.expressionevaluator.Statement;
import org.gvsig.expressionevaluator.Statement.StatementContext;

/**
 *
 * @author jjdelcerro
 */
public class DefaultGrammarSet implements GrammarSet {

    private final List<Grammar> grammars;
    
    public DefaultGrammarSet() {
        this.grammars = new ArrayList<>();
    }
    
    @Override
    public Statement getApplicableStatement(StatementContext context) {
        for (Grammar grammar : this.grammars) {
            Statement stmt = grammar.getApplicableStatement(context);
            if( stmt != null ) {
                return stmt;
            }
        }
        return null;
    }

    @Override
    public boolean isReservedWord(String id) {
        for (Grammar grammar : this.grammars) {
            if( grammar.isReservedWord(id) ) {
                return true;
            }
        }
        return false;
    }

    public boolean contains(Grammar grammar) {
        if( grammar == null ) {
            return false;
        }
        for (Grammar theGrammar : this.grammars) {
            if( StringUtils.equalsIgnoreCase(theGrammar.getName(), grammar.getName()) ) {
                return true;
            }
        }
        return false;
    }
    
    @Override
    public void add(Grammar grammar) {
        if( grammar == null ) {
            return;
        }
        if( this.contains(grammar) ) {
            throw new IllegalArgumentException("The grammar set already contains the grammar '"+grammar+"'.");
        }
        this.grammars.add(grammar);
    }

    @Override
    public Grammar get(int i) {
        return this.grammars.get(i);
    }

    @Override
    public int size() {
        return this.grammars.size();
    }

    @Override
    public boolean isEmpty() {
        return this.grammars.isEmpty();
    }

    @Override
    public List<Grammar> toList() {
        return Collections.unmodifiableList(this.grammars);
    }

    @Override
    public Iterator<Grammar> iterator() {
        return this.grammars.iterator();
    }
    
}
