package org.gvsig.expressionevaluator.impl;

import org.gvsig.expressionevaluator.spi.AbstractSymbolTable;
import java.util.Collection;
import java.util.HashMap;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.MutableSymbolTable;

/**
 *
 * @author jjdelcerro
 */
public class DefaultSymbolTable 
        extends AbstractSymbolTable
        implements MutableSymbolTable 
    {


    public DefaultSymbolTable() {
        super();
        this.vars = new HashMap<>();
        this.functions = new HashMap<>();
//        this.scripts = new ArrayList<>();
        this.init();
    }
    
    private void init() {
        DefaultExpressionEvaluatorManager manager = (DefaultExpressionEvaluatorManager) ExpressionEvaluatorLocator.getManager();
        manager.populateSymbolTable(this);
    }
    
    @Override
    public void setVar(String name, Object value) {
        if( StringUtils.isEmpty(name) ) {
            throw new IllegalArgumentException("name can't be null");
        }
        this.vars.put(name.toUpperCase(), value);
    }

    @Override
    public void addFunction(Function function) {
        super.addFunction(function);
    }

    @Override
    public void addFunctions(Collection<Function> functions) {
        for (Function function : functions) {
            this.addFunction(function);
        }
    }

    @Override
    public void removeVar(String name) {
        if( StringUtils.isEmpty(name) ) {
            throw new IllegalArgumentException("name can't be null");
        }
        this.vars.remove(name.toUpperCase());
    }

    @Override
    public void removeFunction(String name) {
        if( StringUtils.isEmpty(name) ) {
            throw new IllegalArgumentException("name can't be null");
        }
        this.functions.remove(name.toUpperCase());
    }

//    @Override
//    public void addScript(Script script) {
//        if( script == null ) {
//            throw new IllegalArgumentException("script can't be null");
//        }
//        if (this.scripts.contains(script)) {
//            return;
//        }
//        this.scripts.add(script);
//    }
//
//    @Override
//    public void removeScript(Script script) {
//        if( script == null ) {
//            throw new IllegalArgumentException("script can't be null");
//        }
//        this.scripts.remove(script);
//    }

    @Override
    public MutableSymbolTable clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException("Not supported yet."); 
    }
}
