package org.gvsig.expressionevaluator.impl;

import org.cresques.ProjectionLibrary;
import org.gvsig.expressionevaluator.DataTypes;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLibrary;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.impl.coercion.CoerceToExpression;
import org.gvsig.expressionevaluator.impl.coercion.CoerceToString;
import org.gvsig.expressionevaluator.impl.grammars.BasicGrammarFactory;
import org.gvsig.expressionevaluator.impl.symboltable.BasicImageSymbolTableFactory;
import org.gvsig.expressionevaluator.impl.symboltable.FoldersSymbolTableFactory;
import org.gvsig.expressionevaluator.impl.symboltable.OGCSymbolTableFactory;
import org.gvsig.expressionevaluator.impl.symboltable.ProgrammingSymbolTableFactory;
import org.gvsig.expressionevaluator.impl.symboltable.SQLSymbolTableFactory;
import org.gvsig.expressionevaluator.impl.symboltable.UtilsSymbolTableFactory;
import org.gvsig.fmap.geom.GeometryLibrary;
import org.gvsig.tools.ToolsLibrary;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dataTypes.DataTypesManager;
import org.gvsig.tools.library.AbstractLibrary;
import org.gvsig.tools.library.LibraryException;

public class ExpressionEvaluatorImplLibrary extends AbstractLibrary {

    @Override
    public void doRegistration() {
        registerAsImplementationOf(ExpressionEvaluatorLibrary.class);
        require(ToolsLibrary.class);
        require(ProjectionLibrary.class);
        require(GeometryLibrary.class);        
    }

    @Override
    protected void doInitialize() throws LibraryException {
        ExpressionEvaluatorLocator.registerManager(DefaultExpressionEvaluatorManager.class);
    }

    @Override
    protected void doPostInitialize() throws LibraryException {
        ExpressionEvaluatorManager manager = ExpressionEvaluatorLocator.getManager();
        
        manager.registerSymbolTable(new SQLSymbolTableFactory());
        manager.registerSymbolTable(new OGCSymbolTableFactory());
        manager.registerSymbolTable(new UtilsSymbolTableFactory());
        manager.registerSymbolTable(new BasicImageSymbolTableFactory());
        manager.registerSymbolTable(new ProgrammingSymbolTableFactory());
        manager.registerSymbolTable(new FoldersSymbolTableFactory());

        manager.registerGrammar(new BasicGrammarFactory());
        
        DefaultExpression.registerPersistence();

        DataTypesManager dataTypesManager = ToolsLocator.getDataTypesManager();

        dataTypesManager.setCoercion(DataTypes.EXPRESSION, new CoerceToExpression());
        dataTypesManager.addCoercion(DataTypes.STRING,new CoerceToString());
    }
}

