package org.gvsig.expressionevaluator.impl.function.date;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.Period;
import org.apache.commons.lang3.Range;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_DATEDIFF;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class DateDiffFunction extends AbstractFunction {

    public DateDiffFunction() {
        super("Date", FUNCTION_DATEDIFF,Range.is(3),
            "Returns the difference between two dates",
            FUNCTION_DATEDIFF+"({{interval}}, date1, date2)",
            new String[]{
                "interval - a string with the value year,month,day,hour,minute,second or millisecond.",
                "date1 - a Date",
                "date2 - a Date",
            },
            "Long"
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        String interval = this.getStr(args, 0);
        LocalDateTime date1 = this.getLocalDateTime(args,1);
        LocalDateTime date2 = this.getLocalDateTime(args,2);
        
        Period p;
        Duration d;

        switch(interval.toLowerCase()) {
            case "millisecond":
            case "ms":
                d = Duration.between(date1, date2);
                return d.toMillis();
            case "second":
            case "ss":
            case "s":
                d = Duration.between(date1, date2);
                return d.getSeconds();
            case "minute":
            case "mi":
            case "n":
                d = Duration.between(date1, date2);
                return d.toMinutes();
            case "hour":
            case "hh":
                d = Duration.between(date1, date2);
                return d.toHours();
            case "day":
            case "dy":
            case "y":
                d = Duration.between(date1, date2);
                return d.toDays();
            case "month":
            case "mm":
            case "m":
                p = Period.between(date1.toLocalDate(), date2.toLocalDate());
                return (long) p.getMonths();
            case "year":
            case "yyyy":
            case "yy":
                p = Period.between(date1.toLocalDate(), date2.toLocalDate());
                return (long) p.getYears();
            default:
                throw new IllegalArgumentException("Unsupported interval '"+interval+"'.");
        } 
        
    }
    

}
