package org.gvsig.expressionevaluator.impl.function.date;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.apache.commons.lang3.Range;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_DATE;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class DateFunction extends AbstractFunction {

    public DateFunction() {
        super("Date", FUNCTION_DATE,Range.between(1,2),
            "Returns a date from the arguments",
            FUNCTION_DATE+"({{date}}, format)",
            new String[]{
                "date - a string with a date",
                "format - Optional. Format to use to parse the date"
            },
            "Date"
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        Object date_obj = getObject(args, 0);
        if( date_obj instanceof Date ) {
            Date x = new Date(((Date) date_obj).getTime());
            x.setHours(0);
            x.setMinutes(0);
            x.setSeconds(0);
            return x;
        }
        String date = getStr(args, 0);
        String format = null;
        if( args.length==2 ) {
            format = getStr(args, 1);
        }
        SimpleDateFormat df = new SimpleDateFormat();
        if( format != null ) {
            df.applyPattern(format);
        }
        try {
            Date x = df.parse(date);
            x.setHours(0);
            x.setMinutes(0);
            x.setSeconds(0);
            return x;
        } catch (ParseException ex) {
            throw new RuntimeException("Can't parse date value '"+date+"' with format '"+format==null?"":format+"'", ex);
        }
        
    }
    

}
