package org.gvsig.expressionevaluator.impl.function.date;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Objects;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.timesupport.DataTypes;
import org.gvsig.timesupport.RelativeInstant;
import org.gvsig.timesupport.TimeSupportLocator;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dataTypes.CoercionException;
import org.gvsig.tools.dataTypes.DataTypesManager;

public class RelativeInstantFunction extends AbstractFunction {

    private final DataTypesManager.Coercion toInstant;

    public RelativeInstantFunction() {
        super("Date", "RelativeInstant",Range.is(1),
            "Returns a relative instant from the arguments",
            "RelativeInstant({{date}}, format)",
            new String[]{
                "date - a string with a date",
                "format - Optional. Format to use to parse the date"
            },
            "RelativeInstant",
            false
        );
        this.toInstant = ToolsLocator.getDataTypesManager().getCoercion(DataTypes.INSTANT);
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        if( args.length==1 ) {
            Object arg0 = getObject(args, 0);
            try {
                Object instant = this.toInstant.coerce(arg0);
                return instant;
            } catch (CoercionException ex) {
                throw new RuntimeException("Can't convert value '"+Objects.toString(arg0, "")+"' to instant.", ex);
            }
        }
        String format = getStr(args, 1);
        String date = getStr(args, 0);
        SimpleDateFormat df = new SimpleDateFormat();
        if( format != null ) {
            df.applyPattern(format);
        }
        try {
            Date x = df.parse(date);
            RelativeInstant instant = TimeSupportLocator.getManager().createRelativeInstant(x);
            return instant;
        } catch (ParseException ex) {
            throw new RuntimeException("Can't parse date value '"+date+"' with format '"+format==null?"":format+"'", ex);
        }
        
    }
    

}
