package org.gvsig.expressionevaluator.impl.function.date;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.apache.commons.lang3.Range;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_TIMESTAMP;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class TimestampFunction extends AbstractFunction {

    public TimestampFunction() {
        super("Date", FUNCTION_TIMESTAMP,Range.between(1,2),
            "Returns a time stamp from the arguments",
            FUNCTION_TIMESTAMP+"({{timestamp}}, format)",
            new String[]{
                "timestamp - a string with a date/time",
                "format - Optional. Format to use to parse the timestamp"
            },
            "Timestamp"
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        Object date_obj = getObject(args, 0);
        if( date_obj instanceof Date ) {
            return date_obj;
        }
        String date = getStr(args, 0);
        String format = null;
        if( args.length==2 ) {
            format = getStr(args, 1);
        }
        SimpleDateFormat df = new SimpleDateFormat();
        if( format != null ) {
            df.applyPattern(format);
        }
        try {
            Date x = df.parse(date);
            return x;
        } catch (ParseException ex) {
            throw new RuntimeException("Can't parse date value '"+date+"' with format '"+format==null?"":format+"'", ex);
        }
        
    }
    

}
