package org.gvsig.expressionevaluator.impl.function.foldersmanager;

import java.io.File;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.folders.FoldersManager;

public class LastPathFunction extends AbstractFunction {

    public static final String NAME = "LASTPATH";
    
    public LastPathFunction() {
        super(
            "Folders", 
            NAME, 
            Range.between(1,2),
            "Retrieve the last value used to the indicated folder.", 
            NAME+"({{folderid}}, defaultValue)", 
            new String[] {
                "folderid - Identifier of the folder to retrieve",
                "defaultValue - Optional. value to get if the folder with id 'folderid' is not defined."
            }, 
            "File", 
            false
        );
    }


    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        FoldersManager folderManager = ToolsLocator.getFoldersManager();
        String id = getStr(args, 0);
        File defaultValue = getFile(args,1);
        File f = folderManager.getLastPath(id, defaultValue);
        return f;
    }
}
