package org.gvsig.expressionevaluator.impl.function.image;

import java.awt.geom.AffineTransform;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class AffineTransformFunction extends AbstractFunction {

    public AffineTransformFunction() {
        super(
            "Numeric", 
            "AffineTransform", 
            Range.between(0, 6),
            "With 0 parameters create an AffineTransform representing the Identity transformation\n" +
            "With 6 parameters create an AffineTransform with the 6 specifiable entries of the 3x3 transformation matrix.",
            "AffineTransform({{m00}}, m10, m01, m11, m02, m12)",
            new String[]{
                "m00 - the X coordinate scaling element of the 3x3 matrix",
                "m10 - the Y coordinate shearing element of the 3x3 matrix",
                "m01 - the X coordinate shearing element of the 3x3 matrix",
                "m11 - the Y coordinate scaling element of the 3x3 matrix",
                "m02 - the X coordinate translation element of the 3x3 matrix",
                "m12 - the Y coordinate translation element of the 3x3 matrix"
            },
            "AffineTransform",
            false
        );
    }

    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        AffineTransform at;
        switch(args.length) {
            case 0:
                at = new AffineTransform();
                return at;
            case 6:
                double m00 = getDouble(args, 0);
                double m10 = getDouble(args, 0);
                double m01 = getDouble(args, 0);
                double m11 = getDouble(args, 0);
                double m02 = getDouble(args, 0);
                double m12 = getDouble(args, 0);
                at = new AffineTransform(m00, m10, m01, m11, m02, m12);
                return at;
            default:
                throw new IllegalArgumentException("The AffinaTransform function requires 0 or 6 parameters.");
        }
    }
}
