package org.gvsig.expressionevaluator.impl.function.image;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import javax.imageio.ImageIO;
import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class ImageReadFunction extends AbstractFunction {

    public ImageReadFunction() {
        super(
                "Image", "IMAGEREAD", Range.is(1),
                "Read an image from a data source and return it as a 'Java BufferedImage' object.\n"
                + "The data source of the image can be:\n<ul>"
                + "<li>A byte array</li>"
                + "<li>A URL</li>"
                + "<li>A FILE</li>"
                + "<li>A hexadecimal coded string</li>"
                + "<li>A base64 coded string</li>"
                + "</ul>",
                "IMAGEREAD({{source}})",
                new String[]{
                    "Value from which to load the image"
                },
                "BufferedImage",
                false
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }

    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        Object arg0 = this.getObject(args, 0);
        SimpleImage image = ImageReadFunction.createSimpleImage(arg0);
        if( image.isEmpty() ) {
            return null;
        }
        return image.getBufferedImage();
    }

    public static SimpleImage createSimpleImage(Object source) {
        SimpleImage image = new SimpleImage(source);
        return image;
    }
}
