package org.gvsig.expressionevaluator.impl.function.image;

import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class ImageResizeFunction extends AbstractFunction {

    public ImageResizeFunction() {
        super(
                "Image", "IMAGERESIZE", Range.between(1, 3),
                "Resize an image, specifing a scale factor or a with and height",
                "IMAGERESIZE({{image}}, factor)",
                new String[]{
                    "image",
                    "factor",
                    "width",
                    "height"
                },
                "Image",
                false
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }

    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        Object arg0 = this.getObject(args, 0);
        SimpleImage image = ImageReadFunction.createSimpleImage(arg0);
        if( image.isEmpty() ) {
            return null;
        }
        switch(args.length) {
            case 2:
                double factor = this.getDouble(args, 1);
                image = image.resize(factor);
                break;
            case 3:
                int width = this.getInt(args, 1);
                int height = this.getInt(args, 2);
                image = image.resize(width, height);
                break;
            default:
                throw new ExpressionRuntimeException(
                        null, 
                        "Incorrect number of arguments", 
                        "The IMAGERESIZE function has 2 arguments, the image and a scale factor, or 3, the image, width and height"
                );
        }
        return image.getBufferedImage();
    }

}
