package org.gvsig.expressionevaluator.impl.function.image;

import java.awt.geom.AffineTransform;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class ImageTransformFunction extends AbstractFunction {

    public ImageTransformFunction() {
        super(
                "Image", "IMAGETRANSFORM", Range.between(2, 4),
                "",
                "IMAGETRANSFORM({{image}})",
                new String[]{
                    "image",
                    "transform",
                    "width - Optional.",
                    "height - Optional."
                },
                "Image",
                false
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }

    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        Object source = this.getObject(args, 0);
        AffineTransform transform = (AffineTransform) this.getObject(args, 1);
        SimpleImage image = ImageReadFunction.createSimpleImage(source);
        if( image.isEmpty() ) {
            return null;
        }
        switch(args.length) {
            case 2:
                image = image.transform(transform);
                break;
            case 4:
                int width = this.getInt(args, 2);
                int height = this.getInt(args, 3);
                image = image.transform(transform, width, height);
                break;
            default:
                throw new ExpressionRuntimeException(
                        null, 
                        "Incorrect number of arguments", 
                        "The IMAGETRANSFORM function has 2 arguments, image and transform, or 4 arguments, image, transform, width and height."
                );
        }
        return image.getBufferedImage();
    }

}
