package org.gvsig.expressionevaluator.impl.function.image;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import javax.imageio.ImageIO;
import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;

/**
 *
 * @author jjdelcerro
 */
public class SimpleImage {
    
    BufferedImage image = null;

    public SimpleImage() {
    }

    public SimpleImage(BufferedImage image) {
        this.image = image;
    }

    public SimpleImage(Object source) {
        this.image = this.load(source);
    }
    
    public boolean isEmpty() {
        return this.image == null;
    }

    public BufferedImage getBufferedImage() {
        return this.image;
    }

    public void set(BufferedImage image) {
        this.image = image;
    }

    public void set(Object source) {
        this.image = this.load(source);
    }

    private BufferedImage load(Object source) {
        BufferedImage theImage = null;
        try {
            if (source instanceof BufferedImage) {
                return (BufferedImage) source;
            } else if (source instanceof Image) {
                return null;
            } else if (source instanceof byte[]) {
                ByteArrayInputStream is = new ByteArrayInputStream((byte[]) source);
                theImage = ImageIO.read(is);
            } else if (source instanceof File) {
                theImage = ImageIO.read((File) source);
            } else if (source instanceof URL) {
                theImage = ImageIO.read((URL) source);
            } else if (source instanceof String) {
                String s = (String) source;
                byte[] data;
                try {
                    data = Hex.decodeHex(s.toCharArray());
                } catch (DecoderException ex) {
                    data = Base64.decodeBase64(s);
                }
                ByteArrayInputStream is = new ByteArrayInputStream(data);
                theImage = ImageIO.read(is);
            }
        } catch (IOException ex) {
            return null;
        }
        return theImage;
    }

    public int getWidth() {
        return this.image.getWidth();
    }
    
    public int getHeight() {
        return this.image.getHeight();
    }
    
    public SimpleImage resize(double factor) {
        BufferedImage theImage = this.getBufferedImage();
        int w = theImage.getWidth();
        int h = theImage.getHeight();
        int width = (int) (w * factor);
        int height = (int) (h * factor);
        BufferedImage newImage = new BufferedImage(width, height, theImage.getType());
        Graphics2D g = newImage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.drawImage(theImage, 0, 0, width, height, 0, 0, w, h, null);
        g.dispose();
        return new SimpleImage(newImage);
    }

    public SimpleImage resize(int width, int height) {
        BufferedImage theImage = this.getBufferedImage();
        int w = theImage.getWidth();
        int h = theImage.getHeight();
        BufferedImage newImage = new BufferedImage(width, height, theImage.getType());
        Graphics2D g = newImage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.drawImage(theImage, 0, 0, width, height, 0, 0, w, h, null);
        g.dispose();
        return new SimpleImage(newImage);
    }

    public SimpleImage rotate(int angle) {
        BufferedImage theImage = this.getBufferedImage();
        int w = theImage.getWidth();
        int h = theImage.getHeight();
        BufferedImage newImage = new BufferedImage(w, h, theImage.getType());
        Graphics2D g = newImage.createGraphics();
        g.rotate(Math.toRadians(angle), w / 2, h / 2);
        g.drawImage(theImage, null, 0, 0);
        g.dispose();
        return new SimpleImage(newImage);
    }

    public SimpleImage horizontalflip() {
        BufferedImage img = this.getBufferedImage();
        int w = img.getWidth();
        int h = img.getHeight();
        BufferedImage newImage = new BufferedImage(w, h, img.getType());
        Graphics2D g = newImage.createGraphics();
        g.drawImage(img, 0, 0, w, h, w, 0, 0, h, null);
        g.dispose();
        return new SimpleImage(newImage);
    }

    public SimpleImage verticalflip() {
        BufferedImage img = this.getBufferedImage();
        int w = img.getWidth();
        int h = img.getHeight();
        BufferedImage newImage = new BufferedImage(w, h, img.getColorModel().getTransparency());
        Graphics2D g = newImage.createGraphics();
        g.drawImage(img, 0, 0, w, h, 0, h, w, 0, null);
        g.dispose();
        return new SimpleImage(newImage);
    }

    public SimpleImage transform(AffineTransform transform, int width, int height) {
        BufferedImage img = this.getBufferedImage();
        BufferedImage newImage = new BufferedImage(width, height, img.getColorModel().getTransparency());
        Graphics2D g = newImage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.setTransform(transform);
        g.drawImage(img, 0, 0, null);
        g.dispose();
        return new SimpleImage(newImage);
    }

    public SimpleImage transform(AffineTransform transform) {
        BufferedImage img = this.getBufferedImage();
        int w = img.getWidth();
        int h = img.getHeight();
        BufferedImage newImage = new BufferedImage(w, h, img.getColorModel().getTransparency());
        Graphics2D g = newImage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.setTransform(transform);
        g.drawImage(img, 0, 0, null);
        g.dispose();
        return new SimpleImage(newImage);
    }
    
}
