package org.gvsig.expressionevaluator.impl.function.numeric;

import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class BitXorFunction extends AbstractFunction {

    public BitXorFunction() {
        super("Numeric", "BITXOR", Range.is(2),
            "The BITXOR function treats its inputs and its output as vectors of bits; the output is the bitwise XOR of the inputs.\n" +
            "\n" +
            "The types of expr1 and expr2 are INTEGER or LONG, and the result is of type INTEGER or LONG.\n" +
            "If the types of expr1 or expr2 are not INTEGER or LONG a error is raised.\n",
            "BITXOR({{expr1}}, expr2)",
            new String[]{
                "expr1 - the fisrt argument",
                "expr2 - the second argument"
            },
            "Long",
            true
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        Object op1 = args[0];
        Object op2 = args[1];
        
        if( op1 instanceof Integer ) {
            int value = ((Number) op1).intValue() ^ ((Number) op2).intValue();
            return value;
        } 
        if( op1 instanceof Number ) {
            long value = ((Number) op1).longValue() ^ ((Number) op2).longValue();
            return value;
        } 
        
        throw new IllegalArgumentException("Types not allowed in '"+name()+"' function.");
    }

}
