package org.gvsig.expressionevaluator.impl.function.numeric;

import org.apache.commons.lang3.Range;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_DECR;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.MutableSymbolTable;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class DecrFunction extends AbstractFunction {

    public DecrFunction() {
        super("Numeric", FUNCTION_DECR,Range.between(1,3),
            "Decrease the value of the indicated identifier and return its value.",
            FUNCTION_DECR+"('CONTADOR#1')",
            new String[]{
                "identifier name - the identifier used as counter.",
                "first value - Optional. The value with which the identifier must be initialized in its first use. By default zero",
                "step - Optional. The value used to decrease the identifier. By default one",
            },
            "Long",
            false
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        String identifierName = this.getStr(args, 0);
        long firstValue = 0;
        long increment = 1; 
        long value;
        switch(args.length) {
            default:
            case 1:
                break;
            case 2:
                firstValue = this.getLong(args, 1);
                break;
            case 3:
                firstValue = this.getLong(args, 1);
                increment = this.getLong(args, 2);
                break;
        }
        MutableSymbolTable symbolTable = (MutableSymbolTable) interpreter.getSymbolTable();
        if( symbolTable.exists(identifierName) ) {
            value = ((Number) symbolTable.value(identifierName)).longValue();
        } else {
            value = firstValue;
        }
        value = value-increment;
        symbolTable.setVar(identifierName, value);
        return value;
    }
    

}
