package org.gvsig.expressionevaluator.impl.function.operator;

import java.util.Date;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;


public abstract class AbstractBinaryOperator extends AbstractFunction implements BinaryOperator {
    
    protected AbstractBinaryOperator(String group, String name, String description, boolean isSQLCompatible) {
        super(group, name, Range.is(2), description, " {{oper1}} " + name + " oper2 ", null, null, isSQLCompatible);
    }    

    protected AbstractBinaryOperator(String group, String name, boolean isSQLCompatible) {
        this(group, name, null, isSQLCompatible);
    }

    @Override
    public Object call(Interpreter interpreter, Object[] args) {
        return call(interpreter, args[0], args[1]);
        
    }

    @Override
    public boolean isOperator() {
        return true;
    }
    
    protected static final int TYPE_INT =     0b0000001;
    protected static final int TYPE_LONG =    0b0000010;
    protected static final int TYPE_FLOAT =   0b0000100;
    protected static final int TYPE_DOUBLE =  0b0001000;
    protected static final int TYPE_BOOLEAN = 0b0010000;
    protected static final int TYPE_STRING =  0b0100000;
    protected static final int TYPE_DATE =    0b1000000;
    
    protected int getType(Object op1, Object op2) {
        int r = 0;
        if( op1 instanceof Double ) {
            r |= TYPE_DOUBLE;
        } else if( op1 instanceof Float ) {
            r |= TYPE_FLOAT;
        } else if( op1 instanceof Long ) {
            r |= TYPE_LONG;
        } else if( op1 instanceof Integer ) {
            r |= TYPE_INT;
        } else if( op1 instanceof Boolean ) {
            r |= TYPE_BOOLEAN;
        } else if( op1 instanceof String ) {
            r |= TYPE_STRING;
        } else if( op1 instanceof Date ) {
            r |= TYPE_DATE;
        }
        if( op2 instanceof Double ) {
            r |= TYPE_DOUBLE;
        } else if( op2 instanceof Float ) {
            r |= TYPE_FLOAT;
        } else if( op2 instanceof Long ) {
            r |= TYPE_LONG;
        } else if( op2 instanceof Integer ) {
            r |= TYPE_INT;
        } else if( op2 instanceof Boolean ) {
            r |= TYPE_BOOLEAN;
        } else if( op2 instanceof String ) {
            r |= TYPE_STRING;
        } else if( op2 instanceof Date ) {
            r |= TYPE_DATE;
        }
        return r;
    }
    
}
