package org.gvsig.expressionevaluator.impl.function.operator;

import java.util.Objects;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Code;
import org.gvsig.expressionevaluator.Code.Caller;
import org.gvsig.expressionevaluator.Code.Constant;
import static org.gvsig.expressionevaluator.ExpressionBuilder.OPERATOR_CONCAT;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.Optimizer;
import org.gvsig.expressionevaluator.Optimizer.FunctionOptimizer;

public class ConcatOperator 
        extends AbstractBinaryOperator 
        implements FunctionOptimizer
    {
    
    public ConcatOperator() {
        super(Function.GROUP_STRING, OPERATOR_CONCAT, true);
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object op1, Object op2) {       
        return Objects.toString(op1, "")+Objects.toString(op2, "");
    }

    @Override
    public Code optimize(Optimizer optimizer, Caller caller) {
        Code op1 = optimizer.optimize(caller.parameters().get(0));
        Code op2 = optimizer.optimize(caller.parameters().get(1));
        
        if ( op1.code() == Code.CONSTANT && op2.code() == Code.CONSTANT) {
            Code newCode = optimizer.getCodeBuilder().constant(
                    call(
                            optimizer.getInterpreter(), 
                            ((Constant)op1).value(), 
                            ((Constant)op2).value()
                    )
            );
            return newCode;
            
        } else if ( op1.code() == Code.CONSTANT ) {
            String value = Objects.toString(((Constant)op1).value(), "");
            if( StringUtils.isBlank(value) ) {
                return op2;
            }
            Code newCode = optimizer.getCodeBuilder().operator(caller.name(), op1, op2);
            return newCode;

        } else if ( op2.code() == Code.CONSTANT ) {
            String value = Objects.toString(((Constant)op2).value(), "");
            if( StringUtils.isBlank(value) ) {
                return op1;
            }
            Code newCode = optimizer.getCodeBuilder().operator(caller.name(), op1, op2);
            return newCode;
        }

        return caller;
    }
    
}
