package org.gvsig.expressionevaluator.impl.function.operator;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.math.util.MathUtils;
import static org.gvsig.expressionevaluator.ExpressionBuilder.OPERATOR_LE;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;

public class LeOperator extends AbstractBinaryOperator {

    public LeOperator() {
        super(Function.GROUP_BOOLEAN, OPERATOR_LE, true);
    }
    
    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object op1, Object op2) {       
        int type = this.getType(op1, op2);
        if( (type & TYPE_DOUBLE) == TYPE_DOUBLE ) {
            boolean value = MathUtils.compareTo(
                ((Number) op1).doubleValue(), 
                ((Number) op2).doubleValue(),
                interpreter.getAccuracy()==null? MathUtils.EPSILON:interpreter.getAccuracy()
            ) <= 0;
            return value;
        }
        if( (type & TYPE_FLOAT) == TYPE_FLOAT ) {
            boolean value = MathUtils.compareTo(
                ((Number) op1).floatValue(), 
                ((Number) op2).floatValue(),
                interpreter.getAccuracy()==null? MathUtils.EPSILON:interpreter.getAccuracy()
            ) <= 0;
            return value;
        }
        if( (type & TYPE_LONG) == TYPE_LONG ) {
            boolean value = ((Number) op1).longValue() <= ((Number) op2).longValue();
            return value;
        }
        if( (type & TYPE_INT) == TYPE_INT ) {
            boolean value = ((Number) op1).intValue() <= ((Number) op2).intValue();
            return value;
        }        
        if( op2 instanceof Comparable && op2 instanceof Comparable ) {
            boolean value = ObjectUtils.compare((Comparable)op1,(Comparable)op2) <= 0;
            return value;
        }
        throw new IllegalArgumentException("Types not allowed in '"+name()+"' operand.");
    }
    

}
