package org.gvsig.expressionevaluator.impl.function.operator;

import org.apache.commons.math.util.MathUtils;
import static org.gvsig.expressionevaluator.ExpressionBuilder.OPERATOR_NE;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;

public class NeOperator extends AbstractBinaryOperator {

    public NeOperator() {
        super(Function.GROUP_BOOLEAN, OPERATOR_NE, true);
        this.addAlias("!=");
    }
    
    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object op1, Object op2) {       
        if( op1==op2 ) {
            return false;
        }
        if( op1==null || op2==null ) {
            return true;
        }
        int type = this.getType(op1, op2);
        if( (type & TYPE_DOUBLE) == TYPE_DOUBLE ) {
            boolean value = MathUtils.compareTo(
                ((Number) op1).doubleValue(), 
                ((Number) op2).doubleValue(),
                interpreter.getAccuracy()==null? MathUtils.EPSILON:interpreter.getAccuracy()
            ) != 0;
            return value;
        }
        if( (type & TYPE_FLOAT) == TYPE_FLOAT ) {
            boolean value = MathUtils.compareTo(
                ((Number) op1).floatValue(), 
                ((Number) op2).floatValue(),
                interpreter.getAccuracy()==null? MathUtils.EPSILON:interpreter.getAccuracy()
            ) != 0;
            return value;
        }
        if( (type & TYPE_LONG) == TYPE_LONG ) {
            boolean value = Long.compare(
                ((Number) op1).longValue(), 
                ((Number) op2).longValue()
            ) != 0;
            return value;
        }
        if( (type & TYPE_INT) == TYPE_INT ) {
            boolean value = Integer.compare(
                ((Number) op1).intValue(), 
                ((Number) op2).intValue()
            ) != 0;
            return value;
        }
        if( op2 instanceof Comparable && op2 instanceof Comparable ) {
            // boolean value = ObjectUtils.compare((Comparable)op1,(Comparable)op2) != 0;
            boolean value =   ((Comparable)op1).compareTo((Comparable)op2) != 0;
            return value;
        }
        throw new IllegalArgumentException("Types not allowed in '"+name()+"' operand.");
    }
    

}
