package org.gvsig.expressionevaluator.impl.function.programming;

import java.lang.reflect.Method;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class CallStaticMethodFunction extends AbstractFunction {

    public static final String NAME = "CALL_STATIC";
    
    public CallStaticMethodFunction() {
        super(
            Function.GROUP_PROGRAMMING, 
            NAME, 
            Range.between(2, Integer.MAX_VALUE),
            "The "+NAME+"() function call a static mehod of a class.",
            NAME+"( className, methodName, arguments... )",
            null,
            "Object",
            false
        );
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        String theClassName = getStr(args,0);
        String theMethodName = getStr(args,1);
        int firstArg = 1;
        if( theClassName == null ) {
            throw new NullPointerException("A class name name was expected to invoke and a null class name was received");
        }
        if( theMethodName == null ) {
            throw new NullPointerException("A method name was expected to invoke and a null methd name was received");
        }
        Class[] parameterTypes = new Class[args.length-firstArg];
        Object[] parameters = new Object[args.length-firstArg];
        for (int i = 0; i < parameters.length; i++) {
            Object parameter = args[i+firstArg];
            parameters[i] = parameter;
            if( parameter==null ) {
                parameterTypes[i] = null;
            } else {
                parameterTypes[i] = parameter.getClass();
            }
        }
        Class<?> theClass = Class.forName(theClassName);
        Method method = theClass.getMethod(theMethodName, parameterTypes);
        Object value = method.invoke(null, parameters);
        return value;
    }
    
}
