package org.gvsig.expressionevaluator.impl.function.programming;

import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Codes;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dataTypes.DataTypes;
import org.gvsig.tools.dataTypes.DataTypesManager.Coercion;

public class CaseFunction extends AbstractFunction {

    public static final String NAME = "CASE";

    public CaseFunction() {
        super(Function.GROUP_PROGRAMMING, 
                NAME, 
                Range.between(2, Integer.MAX_VALUE),
                "",
                null,
                null,
                "Object",
                false
        );
    }

    @Override
    public boolean useArgumentsInsteadObjects() {
        return true;
    }

    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
    
    @Override
    public Object call(Interpreter interpreter, Codes args) throws Exception {
        // CASE(cond1, cond2, cond3,... condN, expr1, expr2, expr3... exprN, exprElse)
        //      0      1      2                3      4      5               6
        Coercion toBoolean = ToolsLocator.getDataTypesManager().getCoercion(DataTypes.BOOLEAN);
        int count;
        int firstValue;
        int elseValue;
        
        if( (args.size() % 2) == 0) {
            count = args.size() / 2;
            firstValue = count;
            elseValue = -1;
        } else {
            count = (args.size()-1) / 2;
            firstValue = count;
            elseValue = args.size()-1;
        }
        boolean needElse = true;
        Object value = null;
        for( int n=0; n<count; n++ ) {
            Object condition_o = getObject(interpreter, args, n);
            Boolean condition = (Boolean) toBoolean.coerce(condition_o);
            if( condition ) {
                needElse = false;
                value = getObject(interpreter, args, n+firstValue);
            }
        }
        if( needElse && elseValue>0 ) {
            value = getObject(interpreter, args, elseValue);        
        }
        return value;
    }
    
}
