package org.gvsig.expressionevaluator.impl.function.programming;

import java.lang.reflect.Field;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.tools.dynobject.DynObject;

public class GetattrFunction extends AbstractFunction {

    public static final String NAME = "GETATTR";

    public GetattrFunction() {
        super(Function.GROUP_PROGRAMMING, NAME, Range.is(2));
    }
    
    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        Object obj = getObject(args, 0);
        String attrname = getStr(args,1);
        if( attrname == null ) {
            throw new NullPointerException("A attribute name was expected and a null was received");
        }
        if( obj==null ) {
            throw new NullPointerException("An object pointer was expected to access attribute "+attrname+" and a null was received");
        }
        Object value;
        if( obj instanceof DynObject ) {
            value = ((DynObject) obj).getDynValue(attrname);
        } else {
            Class<?> theClass = obj.getClass();
            Field field = theClass.getField(attrname);
            value = field.get(obj);
        }
        return value;
    }
    
}
