package org.gvsig.expressionevaluator.impl.function.programming;

import java.util.List;
import java.util.Map;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.tools.util.GetItem;
import org.gvsig.tools.util.GetItem64;
import org.gvsig.tools.util.GetItemByKey;
import org.gvsig.tools.util.PropertiesSupport;

public class GetitemFunction extends AbstractFunction {

    public static final String NAME = "GETITEM";
    
    public GetitemFunction() {
        super(Function.GROUP_PROGRAMMING, 
            NAME, 
            Range.is(2),
            null,
            null,
            null,
            "Object",
            false
        );
    }
    
    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        Object value;
        Object obj = getObject(args, 0);
        if( obj == null ) {
            throw new ExpressionRuntimeException("Null is not valid in "+NAME+" function.");
        }
        
        if( obj instanceof List ) {
            List l = ((List)obj);
            int index = getInt(args,1);
            if( index<0 ) {
                index = l.size() + index;
            }
            value = l.get(index);
        
        } else if( obj instanceof CharSequence ) {
            CharSequence s = (CharSequence)obj;
            int index = getInt(args,1);
            if( index<0 ) {
                index = s.length() + index;
            }
            value = s.subSequence(index, index+1);

        } else if( obj instanceof Map ) {
            Map m = (Map)obj;
            Object index = getObject(args,1);
            value = m.get(index);
            
        } else if( obj instanceof GetItem ) {
            GetItem l = (GetItem)obj;
            int index = getInt(args,1);
            value = l.get(index);
            
        } else if( obj instanceof GetItem64 ) {
            GetItem64 l = (GetItem64)obj;
            long index = getLong(args,1);
            value = l.get64(index);
            
        } else if( obj instanceof GetItemByKey ) {
            GetItemByKey l = (GetItemByKey)obj;
            Object key = getObject(args,1);
            value = l.get(key);
            
        } else if( obj instanceof PropertiesSupport ) {
            PropertiesSupport l = (PropertiesSupport)obj;
            String propName = getStr(args,1);
            value = l.getProperty(propName);
            
        } else {
            throw new ExpressionRuntimeException("The "+NAME+" function require a String, List or Map and a received a '"+obj.getClass().getSimpleName()+"'.");
        }
        return value;
    }
    
}
