package org.gvsig.expressionevaluator.impl.function.programming;

import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.MutableSymbolTable;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class LetFunction extends AbstractFunction {

    public static final String NAME = "LET";
    
    public LetFunction() {
        super(Function.GROUP_PROGRAMMING, 
            NAME, 
            Range.is(2),
            "Assigns the value indicated in the second argument to the variable indicated in the first argument.",
            NAME+"({{identifier_name}},value)",
            new String[]{
                "identifier_name - Name of the variable",
                "value - Value to assign to the variable"
            },
            "Object",
            false
        );
        this.addAlias("SET");
    }
    
    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        String name = getStr(args,0);
        Object obj = getObject(args, 1);
        if( name == null ) {
            throw new NullPointerException("A string with a variable name was expected and a null was received");
        }
        SymbolTable symbolTable = interpreter.getSymbolTable();
        if( symbolTable instanceof MutableSymbolTable ) {
            ((MutableSymbolTable)symbolTable).setVar(name, obj);
        } else {
            throw new RuntimeException("A MutableSymbolTable is required for use let function.");
        }
        return obj;
    }
    
}
