package org.gvsig.expressionevaluator.impl.function.programming;

import java.util.Iterator;
import org.apache.commons.lang3.Range;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_RANGE;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class RangeFunction extends AbstractFunction {

    public RangeFunction() {
        super(
            Function.GROUP_PROGRAMMING, 
            FUNCTION_RANGE, 
            Range.between(1,3),
            "",
            FUNCTION_RANGE+"({{start}}, stop, step)",
            null,
            "Iterable",
            false
        );
    }
    
    @Override
    public boolean allowConstantFolding() {
        return false;
    }    
    
    @Override
    public Object call(Interpreter interpreter, final Object[] args) throws Exception {
        switch(args.length) {
            case 1:
                return new Iterable<Long>() {
                    @Override
                    public Iterator<Long> iterator() {
                        return new XRangeIterator(0, getLong(args, 0), 1);
                    }
                };
            case 2:
                return new Iterable<Long>() {
                    @Override
                    public Iterator<Long> iterator() {
                        return new XRangeIterator(getLong(args, 0), getLong(args, 1), 1);
                    }
                };
            case 3:
                return new Iterable<Long>() {
                    @Override
                    public Iterator<Long> iterator() {
                        return new XRangeIterator(getLong(args, 0), getLong(args, 1), getLong(args, 2));
                    }
                };
            default:
                // Aqui no deberia llegar, el interprete deberia producir un error
                // antes, al comprobar el numero de argumentos de esta funcion.
                throw new ExpressionRuntimeException("Incorrect number of arguments in function range.");
        }
    }
    
    private static class XRangeIterator implements Iterator<Long> {

        private final long start;
        private final long stop;
        private final long step;
        private long current;

        public XRangeIterator(long start, long stop, long step) {
            this.start = start;
            this.stop = stop;
            this.step = step;
            this.current = start;
        }
        
        @Override
        public boolean hasNext() {
            return this.current < this.stop;
        }

        @Override
        public Long next() {
            long n = this.current;
            this.current += this.step;
            return n;
        }
        
    }

}
