package org.gvsig.expressionevaluator.impl.function.programming;

import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class ReturnFunction extends AbstractFunction {
    
    public static final String NAME = "RETURN";

    public ReturnFunction() {
        super(Function.GROUP_PROGRAMMING, 
                NAME, 
                Range.between(0, Integer.MAX_VALUE),
                null,
                null,
                null,
                "Object",
                false
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        
        switch(args.length) {
            case 0:
                throw new ReturnException(null);
            case 1:
                throw new ReturnException(args[0]);
            default:
                throw new ReturnException(args);
        }
    }

    public static class ReturnException extends RuntimeException {

        private final Object value;
        
        public ReturnException(Object value) {
            this.value = value;
        }
        
        public Object getValue() {
            return this.value;
        }
    }
}
