package org.gvsig.expressionevaluator.impl.function.programming;

import java.io.File;
import java.net.URI;
import java.net.URL;
import org.apache.commons.lang3.Range;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import static org.gvsig.tools.dataTypes.DataTypes.URL;

public class URLFunction extends AbstractFunction {

    public static final String NAME = "URL";
    
    public URLFunction() {
        super(Function.GROUP_PROGRAMMING, 
            NAME, 
            Range.is(1),
            null,
            null,
            null,
            "Object",
            false
        );
    }
    
    @Override
    public boolean allowConstantFolding() {
        return false;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        Object value;
        Object obj = getObject(args, 0);
        if( obj == null ) {
            return null;
        }        
        if( obj instanceof URL ) {
            return obj;
        } else if( obj instanceof CharSequence ) {
            value = new URL(((CharSequence)obj).toString());
        } else if( obj instanceof File ) {
            value = ((File)obj).toURI().toURL();
        } else if( obj instanceof URI ) {
            value = ((URI)obj).toURL();
        } else {
            throw new ExpressionRuntimeException("The "+NAME+" function require a File, URI or a String and a received a '"+obj.getClass().getSimpleName()+"'.");
        }
        return value;
    }
    
}
