package org.gvsig.expressionevaluator.impl.function.spatial;

import org.apache.commons.lang3.Range;
import org.cresques.cts.IProjection;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_ST_SETSRID;
import org.gvsig.expressionevaluator.ExpressionRuntimeException;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;
import org.gvsig.fmap.crs.CRSFactory;
import org.gvsig.fmap.geom.Geometry;

public class STSetSRIDFunction extends AbstractFunction {

    public STSetSRIDFunction() {
        super("OGC", FUNCTION_ST_SETSRID, Range.is(2));
    }
    
    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        Geometry geom = this.getGeom(args, 0);
        int srid = this.getInt(args, 1);
        IProjection proj = CRSFactory.getCRS("EPSG:"+srid);
        if( proj==null ) {
            throw new ExpressionRuntimeException("Invalid SRID '"+srid+"'.");
        }
        geom.setProjection(proj);
        return geom;
    }
    
}
