package org.gvsig.expressionevaluator.impl.function.typeconversion;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.Range;
import static org.gvsig.expressionevaluator.ExpressionBuilder.FUNCTION_DECODE;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.spi.AbstractFunction;

public class DecodeFunction extends AbstractFunction {

    public DecodeFunction() {
        super(
            GROUP_CONVERSION, FUNCTION_DECODE, Range.is(2),
            "Decode binary data from textual representation in string.\nSupported formats are: base64, hex.",
            FUNCTION_DECODE+"({{data}}, 'hex')",
            new String[]{
                "data - A string value with the data to be converted",
                "format - A string value indicating the format of the input data. The default value is 'hex'."
            },
            "Byte array",
            true
        );
    }

    @Override
    public boolean allowConstantFolding() {
        return true;
    }
    
    @Override
    public Object call(Interpreter interpreter, Object[] args) throws Exception {
        String data_s = getStr(args,0);
        String type = getStr(args,1);
        if( "hex".equalsIgnoreCase(type) ) {
            byte[] data = Hex.decodeHex(data_s.toCharArray());
            return data;
        } else if( "base64".equalsIgnoreCase(type) ) {
            byte[] data = Base64.decodeBase64(data_s);
            return data;
        } else if( "escape".equalsIgnoreCase(type) ) {
            // Not supported
        }
        throw new IllegalArgumentException("Unsupported format type '"+type+"'.");
    }

}
