package org.gvsig.expressionevaluator.impl.javascripting;

import javax.script.Bindings;
import javax.script.CompiledScript;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptException;
import org.gvsig.expressionevaluator.Code;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.SymbolTable;

/**
 *
 * @author jjdelcerro
 */
public class CosaCompiledScript extends CompiledScript {

    private final CosaScriptEngine engine;
    private final Code code;

    public CosaCompiledScript(CosaScriptEngine engine, Code code) {
        this.engine = engine;
        this.code = code;
    }
    
    @Override
    public Object eval(ScriptContext context) throws ScriptException {
        ExpressionEvaluatorManager manager = ExpressionEvaluatorLocator.getManager();
        Bindings bindings = context.getBindings(ScriptContext.ENGINE_SCOPE); 
        SymbolTable symbolTable;
        if( bindings instanceof SymbolTableToBindingsAdapter ) {
            symbolTable = ((SymbolTableToBindingsAdapter)bindings).getSymbolTable();
        } else {
            symbolTable = new BindingsToSymbolTableAdapter(bindings);
        }
        Object x = manager.evaluate(symbolTable, this.code);
        return x;
    }

    @Override
    public ScriptEngine getEngine() {
        return this.engine;
    }
    
}
