package org.gvsig.expressionevaluator.impl.symboltable;

import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.impl.function.date.DateAddFunction;
import org.gvsig.expressionevaluator.impl.function.date.DateDiffFunction;
import org.gvsig.expressionevaluator.impl.function.date.DateFunction;
import org.gvsig.expressionevaluator.impl.function.date.NowFunction;
import org.gvsig.expressionevaluator.impl.function.date.TimeFunction;
import org.gvsig.expressionevaluator.impl.function.date.TimestampFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.ACosFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.ASinFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.ATanFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.AbsFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.BitAndFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.BitGetFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.BitOrFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.BitXorFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.CeilFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.CosFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.CoshFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.CotFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.DegreesFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.ExpFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.FloorFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.Log10Function;
import org.gvsig.expressionevaluator.impl.function.numeric.LogFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.PiFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.PowFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.RadiansFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.RandFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.RoundFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.SignFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.SinFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.SinhFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.SqrtFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.TanFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.TanhFunction;
import org.gvsig.expressionevaluator.impl.function.numeric.ZeroFunction;
import org.gvsig.expressionevaluator.impl.function.programming.IFNULLFunction;
import org.gvsig.expressionevaluator.impl.function.programming.IIFFunction;
import org.gvsig.expressionevaluator.impl.function.operator.AddOperator;
import org.gvsig.expressionevaluator.impl.function.operator.AndOperator;
import org.gvsig.expressionevaluator.impl.function.operator.ConcatOperator;
import org.gvsig.expressionevaluator.impl.function.operator.DivOperator;
import org.gvsig.expressionevaluator.impl.function.operator.EqOperator;
import org.gvsig.expressionevaluator.impl.function.operator.GeOperator;
import org.gvsig.expressionevaluator.impl.function.operator.GtOperator;
import org.gvsig.expressionevaluator.impl.function.operator.IsOperator;
import org.gvsig.expressionevaluator.impl.function.operator.LeOperator;
import org.gvsig.expressionevaluator.impl.function.operator.LtOperator;
import org.gvsig.expressionevaluator.impl.function.operator.ModOperator;
import org.gvsig.expressionevaluator.impl.function.operator.MulOperator;
import org.gvsig.expressionevaluator.impl.function.operator.NeOperator;
import org.gvsig.expressionevaluator.impl.function.operator.NegOperator;
import org.gvsig.expressionevaluator.impl.function.operator.NotOperator;
import org.gvsig.expressionevaluator.impl.function.operator.OrOperator;
import org.gvsig.expressionevaluator.impl.function.operator.RegExpOperator;
import org.gvsig.expressionevaluator.impl.function.operator.SubstOperator;
import org.gvsig.expressionevaluator.impl.function.predicate.IlikeOperator;
import org.gvsig.expressionevaluator.impl.function.predicate.LikeOperator;
import org.gvsig.expressionevaluator.impl.function.string.AsciiFunction;
import org.gvsig.expressionevaluator.impl.function.string.CharFunction;
import org.gvsig.expressionevaluator.impl.function.string.ConcatFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.DecodeFunction;
import org.gvsig.expressionevaluator.impl.function.string.FormatFunction;
import org.gvsig.expressionevaluator.impl.function.string.InstrFunction;
import org.gvsig.expressionevaluator.impl.function.string.LTrimFunction;
import org.gvsig.expressionevaluator.impl.function.string.LeftFunction;
import org.gvsig.expressionevaluator.impl.function.string.LengthFunction;
import org.gvsig.expressionevaluator.impl.function.string.LocateFunction;
import org.gvsig.expressionevaluator.impl.function.string.LowerFunction;
import org.gvsig.expressionevaluator.impl.function.string.LpadFunction;
import org.gvsig.expressionevaluator.impl.function.string.MidFunction;
import org.gvsig.expressionevaluator.impl.function.string.PositionFunction;
import org.gvsig.expressionevaluator.impl.function.string.RTrimFunction;
import org.gvsig.expressionevaluator.impl.function.string.RepeatFunction;
import org.gvsig.expressionevaluator.impl.function.string.ReplaceFunction;
import org.gvsig.expressionevaluator.impl.function.string.RightFunction;
import org.gvsig.expressionevaluator.impl.function.string.RpadFunction;
import org.gvsig.expressionevaluator.impl.function.string.SpaceFunction;
import org.gvsig.expressionevaluator.impl.function.string.SubstringFunction;
import org.gvsig.expressionevaluator.impl.function.string.TrimFunction;
import org.gvsig.expressionevaluator.impl.function.string.UpperFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.CastFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.ToDoubleFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.ToFloatFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.ToIntegerFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.ToLongFunction;
import org.gvsig.expressionevaluator.impl.function.typeconversion.ToStrFunction;
import org.gvsig.expressionevaluator.spi.AbstractSymbolTable;

/**
 *
 * @author jjdelcerro
 */
public class SQLSymbolTable extends AbstractSymbolTable {
    static final String NAME = "SQL";
    
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public SQLSymbolTable() {
        super(NAME);
        
        this.addOperator(new AndOperator());
        this.addOperator(new AddOperator());
        this.addOperator(new DivOperator());
        this.addOperator(new EqOperator());
        this.addOperator(new IsOperator());
        this.addOperator(new GeOperator());
        this.addOperator(new GtOperator());
        this.addOperator(new LeOperator());
        this.addOperator(new LtOperator());
        this.addOperator(new ModOperator());
        this.addOperator(new NeOperator());
        this.addOperator(new ModOperator());
        this.addOperator(new MulOperator());
        this.addOperator(new NeOperator());
        this.addOperator(new NotOperator());
        this.addOperator(new OrOperator());
        this.addOperator(new SubstOperator());
        this.addOperator(new ConcatOperator());
        
        this.addOperator(new IlikeOperator());
        this.addOperator(new LikeOperator());
        this.addOperator(new RegExpOperator());
        this.addOperator(new NegOperator());
        
        this.addFunction(new ACosFunction());
        this.addFunction(new ASinFunction());
        this.addFunction(new ATanFunction());
        this.addFunction(new AbsFunction());
        this.addFunction(new BitAndFunction());
        this.addFunction(new BitOrFunction());
        this.addFunction(new BitXorFunction());
        this.addFunction(new BitGetFunction());
        this.addFunction(new CeilFunction());
        this.addFunction(new CosFunction());
        this.addFunction(new CoshFunction());
        this.addFunction(new CotFunction());
        this.addFunction(new DegreesFunction());
        this.addFunction(new ExpFunction());
        this.addFunction(new FloorFunction());
        this.addFunction(new Log10Function());
        this.addFunction(new LogFunction());
        this.addFunction(new PiFunction());
        this.addFunction(new PowFunction());
        this.addFunction(new RadiansFunction());
        this.addFunction(new RandFunction());
        this.addFunction(new RoundFunction());
        this.addFunction(new SignFunction());
        this.addFunction(new SinFunction());
        this.addFunction(new SinhFunction());
        this.addFunction(new SqrtFunction());
        this.addFunction(new TanFunction());
        this.addFunction(new TanhFunction());
        this.addFunction(new ZeroFunction());
        
        this.addFunction(new AsciiFunction());
        this.addFunction(new CharFunction());
        this.addFunction(new ConcatFunction());
        this.addFunction(new InstrFunction());
        this.addFunction(new LTrimFunction());
        this.addFunction(new LeftFunction());
        this.addFunction(new LengthFunction());
        this.addFunction(new LocateFunction());
        this.addFunction(new LowerFunction());
        this.addFunction(new LpadFunction());
        this.addFunction(new PositionFunction());
        this.addFunction(new RTrimFunction());
        this.addFunction(new RepeatFunction());
        this.addFunction(new ReplaceFunction());
        this.addFunction(new RightFunction());
        this.addFunction(new RpadFunction());
        this.addFunction(new SpaceFunction());
        this.addFunction(new SubstringFunction());
        this.addFunction(new TrimFunction());
        this.addFunction(new UpperFunction());
        this.addFunction(new MidFunction());
        
        this.addFunction(new NowFunction());
        this.addFunction(new DateFunction());
        this.addFunction(new DateDiffFunction());
        this.addFunction(new DateAddFunction());
        this.addFunction(new TimeFunction());
        this.addFunction(new TimestampFunction());

        this.addFunction(new IIFFunction());
        this.addFunction(new IFNULLFunction());
        this.addFunction(new FormatFunction());

        this.addFunction(new CastFunction());
        this.addFunction(new DecodeFunction());
        this.addFunction(new ToStrFunction());
        this.addFunction(new ToDoubleFunction());
        this.addFunction(new ToFloatFunction());
        this.addFunction(new ToIntegerFunction());
        this.addFunction(new ToLongFunction());
        
        
    }

    private void addOperator(Function operator) {
        this.addFunction(operator);
    }
    
}
