package org.gvsig.expresionevaluator.impl;

import static junit.framework.Assert.assertEquals;
import junit.framework.TestCase;
import org.cresques.cts.IProjection;
import org.gvsig.expressionevaluator.Code;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.expressionevaluator.ExpressionBuilder.GeometrySupportType;
import org.gvsig.expressionevaluator.LexicalAnalyzer;
import org.gvsig.expressionevaluator.impl.DefaultCompiler;
import org.gvsig.expressionevaluator.impl.DefaultExpressionBuilder;
import org.gvsig.expressionevaluator.impl.SQLLexicalAnalyzer;
import org.gvsig.fmap.crs.CRSFactory;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.primitive.Point;
import org.gvsig.fmap.geom.primitive.Polygon;
import org.gvsig.tools.library.impl.DefaultLibrariesInitializer;
import org.gvsig.expressionevaluator.Compiler;
import org.gvsig.expressionevaluator.ExpressionBuilder.Value;
import org.gvsig.expressionevaluator.Interpreter;
import org.gvsig.expressionevaluator.MutableSymbolTable;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.impl.DefaultInterpreter;
import org.gvsig.expressionevaluator.impl.DefaultSymbolTable;
import org.gvsig.fmap.geom.exception.CreateGeometryException;

public class TestCase1 extends TestCase {

    public TestCase1(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
        new DefaultLibrariesInitializer().fullInitialize();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    // TODO add test methods here. The name must begin with 'test'. For example:
    // public void testHello() {}
    
    protected LexicalAnalyzer createLexicalAnalyzer() {
        SQLLexicalAnalyzer lexer = new SQLLexicalAnalyzer();
        return lexer;
    }
    
    protected Compiler createCompiler() {
        Compiler compiler = new DefaultCompiler();
        compiler.setLexicalAnalyzer(createLexicalAnalyzer());
        return compiler;
    }
    protected MutableSymbolTable createSymbolTable() {
        DefaultSymbolTable symbolTable = new DefaultSymbolTable();
        return symbolTable;
    }
    
    protected Interpreter createInterpreter(SymbolTable symbolTable) {
        Interpreter interpreter = new DefaultInterpreter();
        interpreter.setSymbolTable(symbolTable);
        return interpreter;
    }
    
    private String createExpr1() {
        ExpressionBuilder builder = new DefaultExpressionBuilder();
        
        Value expr1 = builder.gt(
                builder.column("campo1"),
                builder.constant(10)
        );
        System.out.println(expr1.toString());
        assertEquals(
                "( (\"campo1\") > (10) )",
                expr1.toString()
        );
        return expr1.toString();
    }

    private Polygon createPolygon() throws CreateGeometryException {
        GeometryManager geometryManager = GeometryLocator.getGeometryManager();
        IProjection proj = CRSFactory.getCRS("EPSG:4326");

        Polygon polygon = geometryManager.createPolygon(Geometry.SUBTYPES.GEOM2D);
        polygon.addVertex(0, 0);
        polygon.addVertex(0, 100);
        polygon.addVertex(100, 100);
        polygon.addVertex(100, 0);
        polygon.addVertex(0, 0);

        polygon.setProjection(proj);
        return polygon;
    }
    
    private Geometry createPoint(double x, double y) throws CreateGeometryException {
        GeometryManager geometryManager = GeometryLocator.getGeometryManager();
        IProjection proj = CRSFactory.getCRS("EPSG:4326");
        Point point = geometryManager.createPoint(x, y, Geometry.SUBTYPES.GEOM2D);
        point.setProjection(proj);
        return point;
    }
    
    public void testBuildExpression() throws Exception {
        ExpressionBuilder builder = new DefaultExpressionBuilder();
        
        Value expr2 = builder.and(
                builder.custom(this.createExpr1()),
                builder.ST_Intersects(
                        builder.geometry(createPolygon()),
                        builder.column("GEOMETRY")
                )
        );
        builder.geometry_support_type(GeometrySupportType.WKT);
        System.out.println(expr2.toString());
        assertEquals(
                "(( (\"campo1\") > (10) ) AND ST_Intersects((ST_GeomFromText(('POLYGON ((0 0, 0 100, 100 100, 100 0, 0 0))'), (4326))), (\"GEOMETRY\")))",
                expr2.toString()
        );
        builder.geometry_support_type(GeometrySupportType.WKB);
        System.out.println(expr2.toString());
        assertEquals(
                "(( (\"campo1\") > (10) ) AND ST_Intersects((ST_GeomFromWKB((DECODE('000000000300000001000000050000000000000000000000000000000000000000000000004059000000000000405900000000000040590000000000004059000000000000000000000000000000000000000000000000000000000000','hex')), (4326))), (\"GEOMETRY\")))",
                expr2.toString()
        );
    }

    public void testCompileExpression() throws Exception {
        IProjection proj = CRSFactory.getCRS("EPSG:4326");
        ExpressionBuilder builder = new DefaultExpressionBuilder();
        
        Value expr2 = builder.and(
              builder.custom(this.createExpr1()),
              builder.ST_Intersects(
                builder.geometry(createPolygon()),
                builder.column("GEOMETRY")
              )
        );
        
        Compiler compiler = createCompiler();
        Code code;
        
        builder.geometry_support_type(GeometrySupportType.WKT);
        code = compiler.compileExpression(expr2.toString());
        System.out.println(code.toString());
        assertEquals(
                "((\"campo1\" > 10) AND ST_Intersects(ST_GeomFromText('POLYGON ((0 0, 0 100, 100 100, 100 0, 0 0))', 4326), \"GEOMETRY\"))", 
                code.toString()
        );

        builder.geometry_support_type(GeometrySupportType.WKB);
        code = compiler.compileExpression(expr2.toString());
        System.out.println(code.toString());
        assertEquals(
                "((\"campo1\" > 10) AND ST_Intersects(ST_GeomFromWKB(DECODE('000000000300000001000000050000000000000000000000000000000000000000000000004059000000000000405900000000000040590000000000004059000000000000000000000000000000000000000000000000000000000000', 'hex'), 4326), \"GEOMETRY\"))", 
                code.toString()
        );
        
    }

    public void testRunExpression() throws Exception {

        ExpressionBuilder builder = new DefaultExpressionBuilder();
        builder.geometry_support_type(GeometrySupportType.WKB);
        
        String expr2 = builder.and(
            builder.custom(this.createExpr1()),
            builder.ST_Intersects(
              builder.geometry(createPolygon()),
              builder.column("GEOMETRY")
            )
        ).toString();
        
        Compiler compiler = createCompiler();
        Code code = compiler.compileExpression(expr2);

        MutableSymbolTable symbolTable = createSymbolTable();
        Interpreter interpreter = createInterpreter(symbolTable);
        Object v;
        
        symbolTable.setVar("campo1", 11);
        symbolTable.setVar("GEOMETRY", createPoint(50, 50));
        v = interpreter.run(code);
        assertEquals(true, ((Boolean)v).booleanValue());
        
        symbolTable.setVar("campo1", 9);
        symbolTable.setVar("GEOMETRY", createPoint(50, 50));
        v = interpreter.run(code);
        assertEquals(false, ((Boolean)v).booleanValue());

        symbolTable.setVar("campo1", 11);
        symbolTable.setVar("GEOMETRY", createPoint(150, 50));
        v = interpreter.run(code);
        assertEquals(false, ((Boolean)v).booleanValue());
    }

}
