package org.gvsig.expresionevaluator.impl;

import java.io.File;
import junit.framework.TestCase;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.expressionevaluator.MutableSymbolTable;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.library.impl.DefaultLibrariesInitializer;

public class TestEvaluateDynamicText extends TestCase {
    
    public TestEvaluateDynamicText(String testName) {
        super(testName);
    }
    
    @Override
    protected void setUp() throws Exception {
        super.setUp();
        new DefaultLibrariesInitializer().fullInitialize();
    }
    
    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    // TODO add test methods here. The name must begin with 'test'. For example:
    // public void testHello() {}
    
    protected SymbolTable createSymbolTable() {
        ExpressionEvaluatorManager manager = ExpressionEvaluatorLocator.getManager();
        MutableSymbolTable symbolTable = manager.createSymbolTable();
        symbolTable.setVar("precio", 200);
        symbolTable.setVar("PI", Math.PI);
        return symbolTable;
    }
    
    public void testSimple1() {
        
        SymbolTable symbolTable = createSymbolTable();
        String s = ExpressionUtils.evaluateDynamicText(symbolTable, "el precion es <%=precio%>");

        assertEquals("el precion es 200", s);
    }
    
    public void testSimple2() {
        String s = ExpressionUtils.evaluateDynamicText("el home es <%=HOME()%>");

        assertEquals("el home es "+ToolsLocator.getFoldersManager().getHome().toString(), s);
    }

    public void testSimple3() {
        String s = ExpressionUtils.evaluateDynamicText("un fichero temporal <%=UNIQUETEMPORARYFILE('nombre.txt')%>");
        
        // El resultado viene a ser algo como:
        //   "un fichero temporal /tmp/tmp-gvsig/nombre-1-16d2f3e4f44.txt"
        if( !s.startsWith("un fichero temporal ") || !s.contains("nombre-") || !s.endsWith(".txt") ) {
            assertTrue("Expected 'un fichero temporal *nombre-*.txt' and found '"+s+"'", false);
        }
    }

    public void test1() {
        
        SymbolTable symbolTable = createSymbolTable();
        String s = ExpressionUtils.evaluateDynamicText(symbolTable, "el precion es <%=precio%> y PI*100 es <%=TOINTEGER(PI*100)%>");

        assertEquals("el precion es 200 y PI*100 es 314", s);
    }
    
    public void testFilename1() {
        File source = new File("<%=UNIQUETEMPORARYFILE('nombre.txt')%>");
        File target = ExpressionUtils.evaluateFilename(source);
        
        // El resultado viene a ser algo como:
        //   /tmp/tmp-gvsig/nombre-1-16d2f3e4f44.txt
        String s = target.getAbsolutePath();
        if( !s.contains("nombre-") || !s.endsWith(".txt") ) {
            assertTrue("Expected '*nombre-*.txt' and found '"+s+"'", false);
        }
    }

    public void testFilename2() {
        File source = new File("/tmp/prueba.txt");
        File target = ExpressionUtils.evaluateFilename(source);
        
        String s = target.getAbsolutePath();
        assertEquals("/tmp/prueba.txt", s);
    }


}