package org.gvsig.expresionevaluator.impl;

import junit.framework.TestCase;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.MutableSymbolTable;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.spi.AbstractSymbolTable;
import org.gvsig.tools.library.impl.DefaultLibrariesInitializer;


public class TestSymbolTable extends TestCase {
    
    public TestSymbolTable(String testName) {
        super(testName);
    }
    
    @Override
    protected void setUp() throws Exception {
        super.setUp();
        new DefaultLibrariesInitializer().fullInitialize();
    }
    
    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    // TODO add test methods here. The name must begin with 'test'. For example:
    // public void testHello() {}

    
    private MutableSymbolTable createMutableSymbolTable() {
        ExpressionEvaluatorManager manager = ExpressionEvaluatorLocator.getManager();
        MutableSymbolTable t = manager.createSymbolTable();
        return  t;
    }
    
    private SymbolTable createSymbolTable() {
        SymbolTable t = new AbstractSymbolTable() {
        };
        return t;
    }
    
    public void testGetter() {
        SymbolTable symbolTable = createSymbolTable();
        
        assertNull(symbolTable.value("number"));
        assertFalse(symbolTable.exists("number"));
        
        assertNull(symbolTable.function("abs"));

        assertEquals(0, symbolTable.variables().size());
        assertEquals(0, symbolTable.functions().size());
//        assertEquals(0, symbolTable.scripts().size());
    }

    public void testGetter2() {
        MutableSymbolTable symbolTable = createMutableSymbolTable();
        
        assertNull(symbolTable.value("number"));
        assertFalse(symbolTable.exists("number"));

        assertNotNull(symbolTable.function("abs"));
        
        assertEquals(0, symbolTable.variables().size());
        assertFalse(symbolTable.functions().isEmpty());
//        assertEquals(0, symbolTable.scripts().size());
    }

    public void testSetter1() {
        MutableSymbolTable symbolTable = createMutableSymbolTable();
        
        symbolTable.setVar("number", 10);
        
        assertEquals(10, symbolTable.value("number"));
        assertTrue(symbolTable.exists("number"));

        assertEquals(1, symbolTable.variables().size());
        assertFalse(symbolTable.functions().isEmpty());
//        assertEquals(0, symbolTable.scripts().size());
    }

    public void testDelegated1() {
        
        MutableSymbolTable symbolTable1 = createMutableSymbolTable();

        MutableSymbolTable symbolTable2 = createMutableSymbolTable();
        symbolTable2.setVar("number", 10);
        
        symbolTable1.addSymbolTable(symbolTable2);
        assertEquals(10, symbolTable1.value("number"));
        assertTrue(symbolTable1.exists("number"));
        assertNotNull(symbolTable1.function("abs"));
        
        assertEquals(symbolTable2.functions().size(), symbolTable1.functions().size());
    }

    public void testDelegated2() {
        
        SymbolTable symbolTable1 = createSymbolTable();

        MutableSymbolTable symbolTable2 = createMutableSymbolTable();
        symbolTable2.setVar("number", 10);
        
        symbolTable1.addSymbolTable(symbolTable2);
        assertEquals(10, symbolTable1.value("number"));
        assertTrue(symbolTable1.exists("number"));
        assertNotNull(symbolTable1.function("abs"));
        
        assertEquals(symbolTable2.functions().size(), symbolTable1.functions().size());
    }

}
