package org.gvsig.expressionevaluator.swing.impl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.JTextComponent;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.DataTypes;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.ExpressionUtils;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dataTypes.CoercionException;
import org.gvsig.tools.dataTypes.DataTypesManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.pickercontroller.AbstractPickerController;
import org.gvsig.tools.swing.icontheme.IconTheme;
import org.gvsig.tools.swing.icontheme.IconThemeManager;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.expressionevaluator.swing.ExpressionPickerController;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue;
import static org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue.ID_GETVALUE;
import static org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue.ID_SETVALUE;
import org.gvsig.tools.swing.api.bookmarkshistory.BookmarksController;
import org.gvsig.tools.swing.api.bookmarkshistory.BookmarksController.BookmarkEvent;
import org.gvsig.tools.swing.api.bookmarkshistory.HistoryController;
import org.gvsig.tools.swing.api.bookmarkshistory.HistoryController.HistoryEvent;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;

/**
 *
 * @author jjdelcerro
 */
public class DefaultExpressionPickerController 
        extends AbstractPickerController<Expression>
        implements ExpressionPickerController 
    {

    public static void selfRegister() {
        URL imageResource = DefaultExpressionPickerController.class.getClassLoader().getResource("org/gvsig/tools/swing/picker/picker-expression.png");
        if (imageResource != null) {
            IconThemeManager iconThemeManager = ToolsSwingLocator.getIconThemeManager();
            IconTheme theme = iconThemeManager.getCurrent();
            ImageIcon icon = new ImageIcon(imageResource);
            theme.registerDefault("tools", "picker", "picker-expression", icon, imageResource);
        }
    }
    
    private final JButton button;
    private final JTextComponent text;
    private Expression value;
    private final ExpressionBuilderConfig config;
    private final ExpressionEvaluatorSwingManager manager;
    private final ActionListener buttonAction;
    private final JMenuItem menuItem;
    private final AbstractAutomaticExpressionChecker automaticExpressionChecker;
    private boolean returnNullWhenEmptyExpression;
    private final HistoryController<Expression> historyController;
    private final BookmarksController<Expression> bookmarksController;
    
    public DefaultExpressionPickerController(ExpressionEvaluatorSwingManager manager, 
            JTextComponent text, 
            JButton button,
            JButton btnHistory,
            JButton btnBookmarks
        ) {
        ToolsSwingManager swingManager = ToolsSwingLocator.getToolsSwingManager();
        I18nManager i18n = ToolsLocator.getI18nManager();

        this.manager = manager;
        this.config = new DefaultExpressionBuilderConfig(manager);
        this.text = text;
        this.button = button;
        if(  "...".equals(this.button.getText()) ) {
            this.button.setText("");
        }
        this.value = null;
        this.returnNullWhenEmptyExpression = true;
        this.button.setIcon(this.getIcon("picker-expression"));
        this.buttonAction = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doSelectExpression();
            }
        };
        this.button.addActionListener(this.buttonAction);

        swingManager.addClearButton(this.text);
        swingManager.setDefaultPopupMenu(this.text);
        JPopupMenu popupMenu = this.text.getComponentPopupMenu();
        this.menuItem = new JMenuItem(i18n.getTranslation("_Show_expression_builder"));
        this.menuItem.addActionListener(buttonAction);
        popupMenu.add(this.menuItem);
        
        ActionListener historyAndBookmarkListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ActionEventWithCurrentValue<Expression> ee = (ActionEventWithCurrentValue<Expression>)e;
                switch(ee.getID()) {
                    case ID_SETVALUE:
                        set(ee.getCurrentValue());
                        break;
                    case ID_GETVALUE:
                        ee.setCurrentValue(get());
                        break;
                }
            }
        };
        this.historyController = swingManager.createHistoryController(
                ExpressionEvaluatorLocator.getManager().getHistory(), 
                btnHistory
        );
        this.historyController.addActionListener(historyAndBookmarkListener);
        popupMenu.add(this.historyController.createMenu());
        
        this.bookmarksController = swingManager.createBookmarksController(
                ExpressionEvaluatorLocator.getManager().getBookmarks(), 
                btnBookmarks
        );
        this.bookmarksController.addActionListener(historyAndBookmarkListener);        
        popupMenu.add(this.bookmarksController.createMenu());

        this.automaticExpressionChecker = new AbstractAutomaticExpressionChecker(config, text) {
            @Override
            protected Expression getExpression() {
                return getWithOutHistory();
            }
        };
    }
    
    private void doSelectExpression() {
        Expression theValue = this.get();

        DefaultJExpressionBuilder panel = new DefaultJExpressionBuilder(this.manager, this.config);
        panel.setExpression(theValue);
        
        WindowManager_v2 windowManager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
        Dialog dialog = windowManager.createDialog(
                panel.asJComponent(),
                "Expression builder",
                null, 
                WindowManager_v2.BUTTONS_OK_CANCEL
        );
        dialog.show(WindowManager.MODE.DIALOG);
        if( dialog.getAction()==WindowManager_v2.BUTTON_OK ) {
            this.set(panel.getExpression());
        }
    }
    
    @Override
    public Expression get() {
        Expression expression = this.getWithOutHistory();
        ExpressionEvaluatorManager theManager = ExpressionEvaluatorLocator.getManager();
        if (expression != null && !expression.isEmpty()) {
            theManager.getHistory().add(expression);
        }
        return expression;
    }
    
    @Override
    public Expression getWithOutHistory() {
        String s = this.text.getText();
        if( this.returnNullWhenEmptyExpression ) {
            if( ExpressionUtils.isEmpty(this.value) &&  StringUtils.isBlank(s) ) {
                return null;
            }
        }
        if( this.value == null ) {
            this.value = ExpressionEvaluatorLocator.getManager().createExpression();
        }
        this.value.setPhrase(StringUtils.defaultIfBlank(this.text.getText(),null));
        return this.value;
    }

    @Override
    public void set(Expression value) {
        if( value==null ) {
            this.text.setText("");
            this.value = null;
            this.returnNullWhenEmptyExpression = true;
        } else {
            this.value = value;
            this.text.setText(this.value.getPhrase());
            this.returnNullWhenEmptyExpression = false;
        }
    }

    @Override
    public boolean isEmpty() {
        boolean empty = StringUtils.isBlank(this.text.getText());
        return empty;
    }

    
    @Override
    public void coerceAndSet(Object value) {
        try {
            DataTypesManager.Coercion toExpression = ToolsLocator.getDataTypesManager().getCoercion(DataTypes.EXPRESSION);
            this.set((Expression) toExpression.coerce(value));
        } catch (CoercionException ex) {
            throw new IllegalArgumentException("Can't converto to color", ex);
        }
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.text.setEnabled(enabled);
        this.button.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled() {
        return this.text.isEnabled();
    }

    @Override
    public void setEditable(boolean editable) {
        super.setEditable(editable);
        this.text.setEditable(editable);
        this.button.setEnabled(editable);
    }

    @Override
    public Preferences getPreferences() {
        return this.config.getPreferences();
    }
    
    @Override
    public List<Element> getElements() {
        return this.config.getElements();
    }

    @Override
    public void addElement(Element element) {
        this.config.addElement(element);
    }
    
    @Override
    public Collection<SymbolTable> getSymbolTables() {
        return this.config.getSymbolTables();
    }

    @Override
    public void addSymbolTable(String name) {
        this.config.addSymbolTable(name);
    }
    
    @Override
    public void addSymbolTable(SymbolTable symbolTable) {
        this.config.addSymbolTable(symbolTable);
    }

    @Override
    public void removeAllSymbolTables() {
        this.config.removeAllSymbolTables();
    }

    @Override
    public void removeSymbolTable(String name) {
        this.config.removeSymbolTable(name);
    }

    @Override
    public SymbolTable getPreviewSymbolTable() {
        return this.config.getPreviewSymbolTable();
    }

    @Override
    public void setPreviewSymbolTable(SymbolTable symbolTable) {
        this.config.setPreviewSymbolTable(symbolTable);
    }

    @Override
    public Object getProperty(String name) {
        return this.config.getProperty(name);
    }

    @Override
    public void setProperty(String name, Object value) {
        this.config.setProperty(name, value);
    }

    @Override
    public Map<String, Object> getProperties() {
        return this.config.getProperties();
    }
}
