package org.gvsig.expressionevaluator.swing.impl;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.configurableactions.ConfigurableActionsMamager;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.GroupElement;
import org.gvsig.expressionevaluator.swing.Element.SimpleElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.expressionevaluator.swing.JExpressionBuilder;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.script.Script;
import org.gvsig.tools.swing.api.Component;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue;
import static org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue.ID_GETVALUE;
import static org.gvsig.tools.swing.api.bookmarkshistory.ActionEventWithCurrentValue.ID_SETVALUE;
import org.gvsig.tools.swing.api.bookmarkshistory.BookmarksController;
import org.gvsig.tools.swing.api.bookmarkshistory.HistoryController;
import org.gvsig.tools.swing.api.pickercontroller.PickerController;
import org.gvsig.tools.swing.api.script.ScriptSwingManager;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.swing.icontheme.IconTheme;
import org.gvsig.tools.util.ToolsUtilLocator;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class DefaultJExpressionBuilder
        extends DefaultJExpressionBuilderView2
        implements JExpressionBuilder {

    private final ExpressionEvaluatorSwingManager manager;
    private Element currentElement;
    private PickerController<List<Script>> scriptPicker;
    private ScriptSwingManager.ScriptEditor scriptEditor;
    
    private final ExpressionBuilderConfig builderConfig;
    private ElementsTreeModel treeModel;
    private ImageIcon defaultIconGroup;
    private ImageIcon defaultIconElement;
    private InfoPanel infoPanel;
    private AbstractAutomaticExpressionChecker automaticExpressionChecker;
    private HistoryController<Expression> historyController;
    private BookmarksController<Expression> bookmarksController;

    private static class InfoPanel implements Component {

        private final JEditorPane txtDescription;
        private final JScrollPane scrDescription;
        private final JTabbedPane tab;

        private final String labelDescription;
        private final String labelAdditionalPanel;
        private final JPanel pnlContainer;
        
        private final String defautltDescription;

        private String description; 
        private Component additionalPanel;
        
        public InfoPanel(JPanel pnlContainer, String defautltDescription) {
            ToolsSwingManager toolsSwingManager = ToolsSwingLocator.getToolsSwingManager();
            I18nManager i18n = ToolsLocator.getI18nManager();
            
            this.defautltDescription = defautltDescription;
            this.pnlContainer = pnlContainer;
            this.labelDescription = i18n.getTranslation("_Description");
            this.labelAdditionalPanel = i18n.getTranslation("_Assistant");
            
            this.txtDescription = new JEditorPane();
            this.scrDescription = new JScrollPane(this.txtDescription);
            this.tab = new JTabbedPane();
            
            toolsSwingManager.setDefaultPopupMenu(this.txtDescription);
            this.txtDescription.setPreferredSize(new Dimension(200,200));
        }
        
        public void clean() {
            this.description = null;
            this.additionalPanel = null;
        }
        
        public void setDescription(String description) {
            this.description = description;
        }
        
        public void setAdditionalPanel(Component panel) {
            this.additionalPanel = panel;
        }

        @Override
        public JComponent asJComponent() {
            this.txtDescription.setContentType("text/html");
            this.txtDescription.setText(this.description);
            this.txtDescription.setCaretPosition(0);

            if( StringUtils.isBlank(this.description) ) {
                if( this.additionalPanel==null ) {
                    // Sin descripcion ni panel adicional, mostramos
                    // la descripcion por defecto.
                    this.txtDescription.setText(this.defautltDescription);
                    this.txtDescription.setCaretPosition(0);
                    return this.scrDescription;
                }
                // Sin descripcion pero con panel adicional, mostramos el 
                // panel adicional.
                return this.additionalPanel.asJComponent();
            }
            if( this.additionalPanel==null ) {
                // Con descripcion y sin panel adicional, mostramos
                // la descripcion.
                return this.scrDescription;                
            } 
            // Con descripcion y panel adicional, mostramos un tab con los dos.
            this.tab.removeAll();
            this.tab.add(
                    this.labelDescription, 
                    this.scrDescription
            );
            this.tab.add(
                    this.labelAdditionalPanel, 
                    this.additionalPanel.asJComponent()
            );
            this.tab.setSelectedIndex(1);
            return this.tab;
        }

        public void repaint() {
            this.pnlContainer.removeAll();
            this.pnlContainer.add(this.asJComponent(), BorderLayout.CENTER);
            this.pnlContainer.revalidate();
            this.pnlContainer.repaint();
        }
    }

    private class ElementsTreeModel implements TreeModel {

        private final GroupElement root;
        private final HashSet<TreeModelListener> listeners;

        public ElementsTreeModel(GroupElement root) {
            this.root = root;
            this.listeners = new HashSet<>();
        }

        @Override
        public Object getRoot() {
            return this.root;
        }

        public void reload() {
            this.reload(this.root);
            this.fireTreeChanged();
        }
        
        private void reload(Element element) {
            if (element instanceof Element) {
//                if (element instanceof GroupElement) {
                    reload(element);
//                }
                element.reload();
            }
        }
        
        private List<Element> getChilds(final GroupElement node) {
            List<Element> x = new ArrayList<>();
            for (Element element : node.getElements()) {
//                if (element instanceof GroupElement) {
                    x.add(element);
//                }
            }
            return x; //node.getElements();
        }

        @Override
        public Object getChild(Object parent, int index) {
            List<Element> childs = getChilds((GroupElement) parent);
            Element child = childs.get(index);
            return child;
        }

        @Override
        public int getChildCount(Object parent) {
            List<Element> childs = getChilds((GroupElement) parent);
            return childs.size();
        }

        @Override
        public boolean isLeaf(Object node) {
            return !(node instanceof GroupElement);
        }

        @Override
        public void valueForPathChanged(TreePath path, Object newValue) {
        }

        @Override
        public int getIndexOfChild(Object parent, Object child) {
            List<Element> childs = getChilds((GroupElement) parent);
            int n = childs.indexOf(child);
            return n;
        }

        @Override
        public void addTreeModelListener(TreeModelListener l) {
            this.listeners.add(l);
        }

        @Override
        public void removeTreeModelListener(TreeModelListener l) {
            this.listeners.remove(l);
        }

        protected void fireTreeChanged() {
            for (TreeModelListener listener : listeners) {
                TreeModelEvent e = new TreeModelEvent(this, new Object[] {this.root});
                listener.treeNodesChanged(e);
            }
        }
    }
  
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public DefaultJExpressionBuilder(ExpressionEvaluatorSwingManager manager, ExpressionBuilderConfig config) {
        this.manager = manager;
        if( config == null ) {
            this.builderConfig = new DefaultExpressionBuilderConfig(manager);
        } else {
            this.builderConfig = config;
        }
        this.initComponents();
    }
    
    @Override
    public ImageIcon loadImage(String imageName) {
        String name = FilenameUtils.getBaseName(imageName);
        IconTheme theme = ToolsSwingLocator.getIconThemeManager().getDefault();
        if (theme.exists(name)) {
            return theme.get(name);
        }
        URL url = this.getClass().getResource(name + ".png");
        if (url == null) {
            return null;
        }
        return new ImageIcon(url);
    }

    @SuppressWarnings("Convert2Lambda")
    private void initComponents() {
        ConfigurableActionsMamager cfgActionsManager = ToolsUtilLocator.getConfigurableActionsMamager();
        JComponent c = cfgActionsManager.getConfigurableActionsComponent(CONFIGURABLE_PANEL_ID, this);
        pnlCfgActions.setLayout(new BorderLayout(0,0));
        pnlCfgActions.add(c, BorderLayout.LINE_END);
        
        final ToolsSwingManager toolsSwingManager = ToolsSwingLocator.getToolsSwingManager();
        final IconTheme theme = ToolsSwingLocator.getIconThemeManager().getCurrent();
        final I18nManager i18n = ToolsLocator.getI18nManager();
        
        this.automaticExpressionChecker = new AbstractAutomaticExpressionChecker(
                builderConfig, txtExpression
            ) {
            @Override
            protected Expression getExpression() {
                return DefaultJExpressionBuilder.this.getExpressionWithOutHistory();
            }

            @Override
            protected void setMessage(int mode, String text) {
                super.setMessage(mode, text); 
                lblMsg.setText("<html>"+StringUtils.abbreviate(text,100)+"</html>");
            }

            @Override
            protected void setPreview(Object value) {
                super.setPreview(value);
                lblMsg.setText("<html>"+StringUtils.abbreviate(this.getPreview(),100)+"</html>");
            }

            @Override
            protected void setTip(String theTip) {
                super.setTip(theTip); 
                btnTip.setToolTipText(this.getTip());
                btnTip.setVisible(this.isTipEnabled());
            }
            
        };
        this.infoPanel = new InfoPanel(this.pnlDescription, load_description_from_resource());
        
        this.btnTip.setVisible(false);
        this.btnTip.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
                dialogs.messageDialog(
                        btnTip.getToolTipText(), 
                        "Tip", 
                        JOptionPane.INFORMATION_MESSAGE
                );
            }
        });

        this.defaultIconGroup = theme.get("expressionbuilder-element-group");
        this.defaultIconElement = theme.get("expressionbuilder-element");
        
        toolsSwingManager.translate(this.tabExpressionBuilder);
        toolsSwingManager.setDefaultPopupMenu(this.txtGroupElement);
        toolsSwingManager.setDefaultPopupMenu(this.txtSimpleElementFilter);
        toolsSwingManager.addClearButton(this.txtSimpleElementFilter, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                txtSimpleElementFilter.setText("");
                doFilter();
            }
        });
        JMenuItem msgMenuTextEditor = new JMenuItem(i18n.getTranslation("text_editor"));
        msgMenuTextEditor.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                automaticExpressionChecker.showMessageDialog();
            }
        });
        JMenuItem msgMenuCopy = new JMenuItem(i18n.getTranslation("copy"));
        msgMenuTextEditor.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                StringSelection selection = new StringSelection(automaticExpressionChecker.getMessage());
                Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                clipboard.setContents(selection, selection);                
            }
        });
        
        JPopupMenu msgMenu = new JPopupMenu();
        msgMenu.add(msgMenuTextEditor);
        msgMenu.addSeparator();
        msgMenu.add(msgMenuCopy);
        this.lblMsg.setComponentPopupMenu(msgMenu);

        this.pnlDescription.setLayout(new BorderLayout());
        this.txtExpression.addCaretListener(new CaretListener() {
            @Override
            public void caretUpdate(CaretEvent e) {
                lblColumn.setText(Integer.toString(e.getDot()));
            }
        });
        this.treeElements.setRowHeight(0);
        this.treeElements.setCellRenderer(new DefaultTreeCellRenderer() {
            @Override
            public java.awt.Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                JLabel component = (JLabel) super.getTreeCellRendererComponent(tree,value, selected, expanded, leaf, row, hasFocus);
                JLabel text = component;
                JLabel icon = component;
                
                if( value instanceof Element ) {
                    Element element = (Element) value;
                    if( element.getIconName()==null ) {
                        if( element instanceof GroupElement ) {
                            icon.setIcon(defaultIconGroup);
                        } else {
                            icon.setIcon(defaultIconElement);
                        }
                    } else {
                        icon.setIcon(element.getIcon());
                    }
                    text.setText(element.getLabel());
                    text.setEnabled(element.isEnabled());
                } else {
                    icon.setIcon(null);
                    text.setText(Objects.toString(value, ""));
                }
                return component;
            }
        });
        
        this.treeElements.addTreeSelectionListener(new TreeSelectionListener() {
            @Override
            public void valueChanged(TreeSelectionEvent e) {
                TreePath path = e.getPath();
                Element element = (Element) path.getLastPathComponent();
                doSelectElement(element);
                updateSimpleElementsList(element);
            }
        });
        this.treeElements.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if( e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()==2 ) {
                    TreePath path = treeElements.getSelectionPath();
                    if( path!=null ) {
                        Element element = (Element) path.getLastPathComponent();
                        if( element instanceof SimpleElement ) {
                            doInsert((SimpleElement) element);
                        }
                    }
                }
            }
        });
        this.btnGroupElementInsert.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if( currentElement!=null ) {
                    doInsert(currentElement.getRenderedValue());
                } else {
                    doInsert(txtGroupElement.getText());
                }
            }
        });
        this.btnSimpleElementInsert.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                Element element = (Element) lstSimpleElement.getSelectedValue();
                if (element != null) {
                    doInsert(element);
                }
            }
        });
        this.btnSimpleElementFilter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doFilter();
            }
        });
        
        this.lstSimpleElement.setModel(new FilteredListModel());
        this.lstSimpleElement.setCellRenderer(new DefaultListCellRenderer() {
            @Override
            public java.awt.Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                JLabel label = (JLabel)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
                if( value instanceof Element ) {
                    Element element = (Element) value;
                    if( element.getIconName()==null ) {
                        if( element instanceof GroupElement ) {
                            label.setIcon(defaultIconGroup);
                        } else {
                            label.setIcon(defaultIconElement);
                        }
                    } else {
                        label.setIcon(element.getIcon());
                    }
                    label.setText(element.getLabel());
                    label.setEnabled(element.isEnabled());
                } else {
                    label.setIcon(null);
                    label.setText(Objects.toString(value, ""));
                }
                return label;
            }
        });
        this.lstSimpleElement.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                Element element = (Element) lstSimpleElement.getSelectedValue();
                if (element != null) {
                    doSelectElement(element);
                }
            }
        });
        this.lstSimpleElement.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if( e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()==2 ) {
                    Element element = (Element) lstSimpleElement.getSelectedValue();
                    if (element != null) {
                        doSelectElement(element);
                        doInsert(element);
                    }
                }
            }
        });
        this.lstSimpleElement.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                if( e.getKeyCode()==KeyEvent.VK_ENTER ) {
                    Element element = (Element) lstSimpleElement.getSelectedValue();
                    if (element != null) {
                        doSelectElement(element);
                        doInsert(element);
                    }
                }
            }
        });

        this.btnSimpleElementTimeLimit.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doSetTimeLimit();
            }
        });
        this.btnSimpleElementSortDown.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                FilteredListModel model = (FilteredListModel) lstSimpleElement.getModel();
                model.sort(false);
            }
        });
        this.btnSimpleElementSortUp.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                FilteredListModel model = (FilteredListModel) lstSimpleElement.getModel();
                model.sort(true);
            }
        });
        this.txtSimpleElementFilter.addKeyListener(new KeyAdapter() {
            @Override
            public void keyTyped(KeyEvent e) {
                if (e.getKeyChar() == '\n') {
                    doFilter();
                }
            }
        });
        this.btnAdd.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" + ");
            }
        });
        this.btnDiv.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" / ");
            }
        });
        this.btnEq.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" = ");
            }
        });
        this.btnMult.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" * ");
            }
        });
        this.btnNeq.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" <> ");
            }
        });
        this.btnParentClose.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" ) ");
            }
        });
        this.btnParentOpen.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" ( ");
            }
        });
        this.btnSubst.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsert(" - ");
            }
        });
//        ScriptSwingManager scriptSwingManager = ToolsSwingLocator.getScriptSwingManager();
//        this.scriptPicker = scriptSwingManager.createListScriptPickerController(
//                cboPickerScripts,
//                btnPickerRemove,
//                btnPickerSelectScript
//        );
//        this.scriptEditor = scriptSwingManager.createScriptEditor();
//        this.pnlScriptEditorContainer.setLayout(new BorderLayout());
//        this.pnlScriptEditorContainer.add(this.scriptEditor.asJComponent(), BorderLayout.CENTER);
//    
//        this.tabExpressionBuilder.setEnabledAt(0, this.builderConfig.getPreferences().getScriptsEnabled());
        
        ActionListener historyAndBookmarkListener = new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ActionEventWithCurrentValue<Expression> ee = (ActionEventWithCurrentValue<Expression>)e;
                switch(ee.getID()) {
                    case ID_SETVALUE:
                        setExpression(ee.getCurrentValue());
                        break;
                    case ID_GETVALUE:
                        ee.setCurrentValue(getExpressionWithOutHistory());
                        break;
                }
            }
        };
        this.historyController = toolsSwingManager.createHistoryController(
                ExpressionEvaluatorLocator.getManager().getHistory(), 
                btnHistory
        );
        this.historyController.addActionListener(historyAndBookmarkListener);
        
        this.bookmarksController = toolsSwingManager.createBookmarksController(
                ExpressionEvaluatorLocator.getManager().getBookmarks(), 
                btnBookmarks
        );
        this.bookmarksController.addActionListener(historyAndBookmarkListener);        
        
        this.infoPanel.repaint();
        
        this.setPreferredSize(new Dimension(750, 450));
        

        Dimension dim = this.getPreferredSize();
        this.spnlExpression.setDividerLocation(dim.height/5);
        this.spnlBottom.setDividerLocation(dim.width/3);
        this.spnlItem.setDividerLocation(dim.width/3);
    }

    private String removeCursorMark(String s) {
        if( s == null ) {
            return null;
        }
        s = s.replace("{{", "");
        s = s.replace("}}", "");
        return s;
    }

    @Override
    public void insertText(String text) {
        doInsert(text);
    }

    @Override
    public String getText() {
        return this.txtExpression.getText();
    }
    
    @Override
    public void setText(String text) {
        this.txtExpression.setText(text);
    }
    
    private void doInsert(String s) {
        this.txtExpression.replaceSelection(s);
        String ss = this.txtExpression.getText();
        int start = ss.indexOf("{{");
        int end = ss.indexOf("}}");
        if (start > 0 && end > 0) {
            this.txtExpression.replaceRange("", end, end + 2);
            this.txtExpression.replaceRange("", start, start + 2);
            this.txtExpression.setSelectionStart(start);
            this.txtExpression.setSelectionEnd(end - 2);
        }
        this.txtExpression.requestFocusInWindow();
    }

    private void doInsert(Element element) {
        doInsert(element.getRenderedValue());
        element.used();
    }

    private void doFilter() {
        FilteredListModel model = (FilteredListModel) this.lstSimpleElement.getModel();
        model.setFilter(this.txtSimpleElementFilter.getText());
    }

    private void doSelectElement(Element element) {
//        if( !element.isEnabled() ) {
//            this.infoPanel.clean();
//        } else {
            this.infoPanel.setDescription(removeCursorMark(element.getDescription()));
            this.infoPanel.setAdditionalPanel(element.getAditionalPanel(this));
            this.infoPanel.repaint();
//        }        
    }

    private void updateSimpleElementsList(Element element) {
        FilteredListModel model = new FilteredListModel();
        model.addElement("Loading...");
        this.currentElement = element;
        this.txtGroupElement.setText(element.getName());
        this.txtSimpleElementFilter.setText("");
//        this.infoPanel.clean();
//        this.infoPanel.repaint();
        if (element instanceof SimpleElement) {
            this.btnGroupElementInsert.setEnabled(true);
        } else {
            this.btnGroupElementInsert.setEnabled(false);
        }
        this.lstSimpleElement.setEnabled(false);
        this.lstSimpleElement.setModel(model);

        Thread th = new Thread(new Runnable() {
            @Override
            public void run() {
                simpleElementsUpdater();
            }
        }, "ExpressionBuilderSimpleElementsUpdater");
        th.start();
    }

    private void simpleElementsUpdater() {
        final Element element = this.currentElement;
        final FilteredListModel model = new FilteredListModel();
        for (Element value : element.getValues()) {
            if (element != this.currentElement) {
                // Como estamos en otro hilo, cuando el usuario cambia de elemento,
                // cancelamos las actualizaciones que teniamos en curso.
                return;
            }
            if (value instanceof SimpleElement) {
                model.addElement(value);
            }
        }
        model.sort(true);
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                if (element != currentElement) {
                    return;
                }
                if( element.hasMoreValues()) {
                    lblSimpleElementsMsg.setText("More elements...");
                } else {
                    lblSimpleElementsMsg.setText("");
                }
                lstSimpleElement.setEnabled(true);
                lstSimpleElement.setModel(model);
            }
        });

    }

    @Override
    public Expression getExpression() {
        Expression expression = this.getExpressionWithOutHistory();
        ExpressionEvaluatorManager theManager = ExpressionEvaluatorLocator.getManager();
        theManager.getHistory().add(expression);
        return expression;
    }
    
    @Override
    public Expression getExpressionWithOutHistory() {
        String phrase = this.txtExpression.getText();
        if (StringUtils.isEmpty(phrase)) {
            return null;
        }
        Expression expression = ExpressionEvaluatorLocator.getManager().createExpression();
        expression.setPhrase(phrase);
//        Script script = ToolsLocator.getScriptManager().createScript("UserScript", "", "python");
//        this.scriptEditor.fetch(script);
//        if( !StringUtils.isEmpty(script.getCode()) ) {
//            expression.setUserScript(script);
//        }
        return expression;
    }

    @Override
    public void setExpression(Expression expression) {
        if( expression==null ) {
            this.txtExpression.setText("");
        } else {
            this.txtExpression.setText(expression.getPhrase());
        }
//            try { this.scriptEditor.set(null); } catch(Exception ex) {}
//            this.scriptPicker.set(null);
//        } else {
//            this.txtExpression.setText(expression.getPhrase());
//            if( expression.getUserScript()!=null ) {
//                this.scriptEditor.set(expression.getUserScript());
//            }
//            if( expression.getScripts()!=null ) {
//                this.scriptPicker.set(expression.getScripts().toList());
//            }
//        }
    }

    @Override
    public Preferences getPreferences() {
        return this.builderConfig.getPreferences();
    }
    
    @Override
    public List<Element> getElements() {
        return this.builderConfig.getElements();
    }
    
    @Override
    public void addElement(Element element) {
        this.builderConfig.addElement(element);
    }
    
    @Override
    public Collection<SymbolTable> getSymbolTables() {
        return this.builderConfig.getSymbolTables();
    }
    
    @Override
    public void addSymbolTable(String name) {
        this.builderConfig.addSymbolTable(name);
    }
    
    @Override
    public void addSymbolTable(SymbolTable symbolTable) {
        this.builderConfig.addSymbolTable(symbolTable);
    }

    @Override
    public void removeAllSymbolTables() {
        this.builderConfig.removeAllSymbolTables();
    }

    @Override
    public void removeSymbolTable(String name) {
        this.builderConfig.removeSymbolTable(name);
    }

    @Override
    public SymbolTable getPreviewSymbolTable() {
        return this.builderConfig.getPreviewSymbolTable();
    }

    @Override
    public void setPreviewSymbolTable(SymbolTable symbolTable) {
        this.builderConfig.setPreviewSymbolTable(symbolTable);
    }
    
    @Override
    public JComponent asJComponent() {
        this.initializeTree();
        return this;
    }
    
    private void initializeTree() {
        if( this.treeModel == null ) {
            I18nManager i18n = ToolsLocator.getI18nManager();
            this.treeModel = new ElementsTreeModel( (GroupElement)
                    this.manager.createGroupElement(
                            i18n.getTranslation("_Elements"),
                            this.getElements()
                    ).setConfig(this));
            this.treeElements.setModel(this.treeModel);
        }
    }
    
    private void doSetTimeLimit() {
        ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
        String s = dialogs.inputDialog(
                "Indicate the time limit in seconds for calculating the elements", 
                "Time limit", 
                JOptionPane.QUESTION_MESSAGE, 
                String.valueOf(this.builderConfig.getPreferences().getSimpleElementsLimit())
        );
        if( StringUtils.isBlank(s) ) {
            return;
        }
        try {
            int n = Integer.parseInt(s);
            this.builderConfig.getPreferences().setSimpleElementsLimit(n);
            this.treeModel.reload();
        } catch(Exception ex) {
            dialogs.messageDialog(
                "Invalid time limit '"+s+"'.", 
                "Time limit", 
                JOptionPane.WARNING_MESSAGE
            );                    
        }
    }

    private String load_description_from_resource() {
        String lang = Locale.getDefault().getLanguage();
        URL url = this.getClass().getResource("/org/gvsig/expressionevaluator/swing/jexpressionbuilder/"+lang+"/description.html");
        if( url == null ) {
            url = this.getClass().getResource("/org/gvsig/expressionevaluator/swing/jexpressionbuilder/en/description.html");
            if( url == null ) {
                return null;
            }
        }
        InputStream is = null;
        try {
            is = url.openStream();
            List<String> lines = IOUtils.readLines(is);
            return StringUtils.join(lines,  "\n");
        } catch (Exception ex) {
            return null;
        } finally {
            IOUtils.closeQuietly(is);
        }
    }
    
    @Override
    public void expandElement(Element element) {
        this.initializeTree();
        for (int i = 0; i < this.treeElements.getRowCount(); i++) {
            TreePath path = this.treeElements.getPathForRow(i);
            if( path.getLastPathComponent()==element ) {
                // FIXME: habria que expandir los padres hasta llegar a este.
                this.treeElements.expandPath(path);
                return;
            }
        }
    }

    @Override
    public Object getProperty(String name) {
        return this.builderConfig.getProperty(name);
    }

    @Override
    public void setProperty(String name, Object value) {
        this.builderConfig.setProperty(name, value);
    }

    @Override
    public Map<String, Object> getProperties() {
        return this.builderConfig.getProperties();
    }
}
