package org.gvsig.expressionevaluator.swing.impl;

import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.border.EmptyBorder;


public class DefaultJExpressionBuilderView extends JPanel
{
   JTabbedPane tabExpressionBuilder = new JTabbedPane();
   JList lstSimpleElement = new JList();
   JTree treeElements = new JTree();
   JTextArea txtExpression = new JTextArea();
   JButton btnEq = new JButton();
   JButton btnNeq = new JButton();
   JButton btnAdd = new JButton();
   JButton btnSubst = new JButton();
   JButton btnMult = new JButton();
   JButton btnDiv = new JButton();
   JButton btnParentOpen = new JButton();
   JButton btnParentClose = new JButton();
   JLabel lblMsg = new JLabel();
   JButton btnTip = new JButton();
   JLabel lblColumn = new JLabel();
   JTextField txtGroupElement = new JTextField();
   JButton btnGroupElementInsert = new JButton();
   JButton btnSimpleElementInsert = new JButton();
   JButton btnSimpleElementSortDown = new JButton();
   JButton btnSimpleElementSortUp = new JButton();
   JLabel lblSimpleElementsMsg = new JLabel();
   JButton btnSimpleElementTimeLimit = new JButton();
   JTextField txtSimpleElementFilter = new JTextField();
   JButton btnSimpleElementFilter = new JButton();
   JPanel pnlDescription = new JPanel();
   JPanel pnlScriptEditorContainer = new JPanel();
   JComboBox cboPickerScripts = new JComboBox();
   JButton btnPickerRemove = new JButton();
   JButton btnPickerSelectScript = new JButton();

   /**
    * Default constructor
    */
   public DefaultJExpressionBuilderView()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE","CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:2DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      tabExpressionBuilder.setName("tabExpressionBuilder");
      tabExpressionBuilder.addTab("_Expression",null,createPanel1());
      tabExpressionBuilder.addTab("_Scripts",null,createPanel8());
      jpanel1.add(tabExpressionBuilder,cc.xy(2,2));

      addFillComponents(jpanel1,new int[]{ 1,2,3 },new int[]{ 1,2,3 });
      return jpanel1;
   }

   public JPanel createPanel1()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:80DLU:GROW(0.2),FILL:4DLU:NONE,FILL:100DLU:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(0.8),FILL:4DLU:NONE","CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,FILL:DEFAULT:NONE,CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lstSimpleElement.setName("lstSimpleElement");
      JScrollPane jscrollpane1 = new JScrollPane();
      jscrollpane1.setViewportView(lstSimpleElement);
      jscrollpane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      jpanel1.add(jscrollpane1,cc.xy(4,8));

      treeElements.setName("treeElements");
      JScrollPane jscrollpane2 = new JScrollPane();
      jscrollpane2.setViewportView(treeElements);
      jscrollpane2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane2.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      jpanel1.add(jscrollpane2,cc.xywh(2,4,1,6));

      jpanel1.add(createPanel2(),cc.xywh(2,2,5,1));
      jpanel1.add(createPanel5(),cc.xy(4,4));
      jpanel1.add(createPanel6(),cc.xy(4,9));
      jpanel1.add(createPanel7(),cc.xy(4,6));
      pnlDescription.setName("pnlDescription");
      jpanel1.add(pnlDescription,cc.xywh(6,4,1,6));

      addFillComponents(jpanel1,new int[]{ 1,2,3,4,5,6,7 },new int[]{ 1,2,3,4,5,6,7,8,9 });
      return jpanel1;
   }

   public JPanel createPanel2()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","FILL:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtExpression.setLineWrap(true);
      txtExpression.setName("txtExpression");
      txtExpression.setRows(2);
      JScrollPane jscrollpane1 = new JScrollPane();
      jscrollpane1.setViewportView(txtExpression);
      jscrollpane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      jpanel1.add(jscrollpane1,cc.xy(1,1));

      jpanel1.add(createPanel3(),new CellConstraints(3,1,1,3,CellConstraints.DEFAULT,CellConstraints.TOP));
      jpanel1.add(createPanel4(),cc.xy(1,3));
      addFillComponents(jpanel1,new int[]{ 2,3 },new int[]{ 2,3 });
      return jpanel1;
   }

   public JPanel createPanel3()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      btnEq.setActionCommand("+");
      btnEq.setName("btnEq");
      btnEq.setText("=");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnEq.setBorder(emptyborder1);
      jpanel1.add(btnEq,cc.xy(1,1));

      btnNeq.setActionCommand("+");
      btnNeq.setName("btnNeq");
      btnNeq.setText("<>");
      EmptyBorder emptyborder2 = new EmptyBorder(2,2,2,2);
      btnNeq.setBorder(emptyborder2);
      jpanel1.add(btnNeq,cc.xy(3,1));

      btnAdd.setActionCommand("+");
      btnAdd.setName("btnAdd");
      btnAdd.setText("+");
      EmptyBorder emptyborder3 = new EmptyBorder(2,2,2,2);
      btnAdd.setBorder(emptyborder3);
      jpanel1.add(btnAdd,cc.xy(5,1));

      btnSubst.setActionCommand("+");
      btnSubst.setName("btnSubst");
      btnSubst.setText("-");
      EmptyBorder emptyborder4 = new EmptyBorder(2,2,2,2);
      btnSubst.setBorder(emptyborder4);
      jpanel1.add(btnSubst,cc.xy(7,1));

      btnMult.setActionCommand("+");
      btnMult.setName("btnMult");
      btnMult.setText("*");
      EmptyBorder emptyborder5 = new EmptyBorder(2,2,2,2);
      btnMult.setBorder(emptyborder5);
      jpanel1.add(btnMult,cc.xy(1,3));

      btnDiv.setActionCommand("+");
      btnDiv.setName("btnDiv");
      btnDiv.setText("/");
      EmptyBorder emptyborder6 = new EmptyBorder(2,2,2,2);
      btnDiv.setBorder(emptyborder6);
      jpanel1.add(btnDiv,cc.xy(3,3));

      btnParentOpen.setActionCommand("+");
      btnParentOpen.setName("btnParentOpen");
      btnParentOpen.setText("(");
      EmptyBorder emptyborder7 = new EmptyBorder(2,2,2,2);
      btnParentOpen.setBorder(emptyborder7);
      jpanel1.add(btnParentOpen,cc.xy(5,3));

      btnParentClose.setActionCommand("+");
      btnParentClose.setName("btnParentClose");
      btnParentClose.setText(")");
      EmptyBorder emptyborder8 = new EmptyBorder(2,2,2,2);
      btnParentClose.setBorder(emptyborder8);
      jpanel1.add(btnParentClose,cc.xy(7,3));

      addFillComponents(jpanel1,new int[]{ 2,4,6,8 },new int[]{ 2,4 });
      return jpanel1;
   }

   public JPanel createPanel4()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblMsg.setName("lblMsg");
      jpanel1.add(lblMsg,cc.xy(1,1));

      btnTip.setActionCommand("+");
      btnTip.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-tip.png.png"));
      btnTip.setName("btnTip");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnTip.setBorder(emptyborder1);
      jpanel1.add(btnTip,cc.xy(5,1));

      lblColumn.setName("lblColumn");
      lblColumn.setText("0");
      jpanel1.add(lblColumn,cc.xy(3,1));

      addFillComponents(jpanel1,new int[]{ 2,4 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel5()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtGroupElement.setBackground(new Color(236,233,216));
      txtGroupElement.setEditable(false);
      txtGroupElement.setName("txtGroupElement");
      jpanel1.add(txtGroupElement,cc.xy(1,1));

      btnGroupElementInsert.setActionCommand("+");
      btnGroupElementInsert.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-insert-text.png"));
      btnGroupElementInsert.setName("btnGroupElementInsert");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnGroupElementInsert.setBorder(emptyborder1);
      jpanel1.add(btnGroupElementInsert,cc.xy(3,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel6()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:PREF:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      btnSimpleElementInsert.setActionCommand("+");
      btnSimpleElementInsert.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-insert-text.png"));
      btnSimpleElementInsert.setName("btnSimpleElementInsert");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnSimpleElementInsert.setBorder(emptyborder1);
      jpanel1.add(btnSimpleElementInsert,cc.xy(8,1));

      btnSimpleElementSortDown.setActionCommand("+");
      btnSimpleElementSortDown.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-sortdown.png"));
      btnSimpleElementSortDown.setName("btnSimpleElementSortDown");
      EmptyBorder emptyborder2 = new EmptyBorder(2,2,2,2);
      btnSimpleElementSortDown.setBorder(emptyborder2);
      jpanel1.add(btnSimpleElementSortDown,cc.xy(6,1));

      btnSimpleElementSortUp.setActionCommand("+");
      btnSimpleElementSortUp.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-sortup.png"));
      btnSimpleElementSortUp.setName("btnSimpleElementSortUp");
      EmptyBorder emptyborder3 = new EmptyBorder(2,2,2,2);
      btnSimpleElementSortUp.setBorder(emptyborder3);
      jpanel1.add(btnSimpleElementSortUp,cc.xy(4,1));

      lblSimpleElementsMsg.setName("lblSimpleElementsMsg");
      jpanel1.add(lblSimpleElementsMsg,cc.xy(1,1));

      btnSimpleElementTimeLimit.setActionCommand("+");
      btnSimpleElementTimeLimit.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-time-limit.png"));
      btnSimpleElementTimeLimit.setName("btnSimpleElementTimeLimit");
      EmptyBorder emptyborder4 = new EmptyBorder(2,2,2,2);
      btnSimpleElementTimeLimit.setBorder(emptyborder4);
      jpanel1.add(btnSimpleElementTimeLimit,cc.xy(2,1));

      addFillComponents(jpanel1,new int[]{ 3,5,7 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel7()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtSimpleElementFilter.setName("txtSimpleElementFilter");
      jpanel1.add(txtSimpleElementFilter,cc.xy(1,1));

      btnSimpleElementFilter.setActionCommand("+");
      btnSimpleElementFilter.setIcon(loadImage("datos/devel/org.gvsig.desktop/org.gvsig.desktop.library/org.gvsig.expressionevaluator/org.gvsig.expressionevaluator.swing/org.gvsig.expressionevaluator.swing.impl/src/main/resources/org/gvsig/expressionevaluator/swing/impl/expressionbuilder-filter-values.png"));
      btnSimpleElementFilter.setName("btnSimpleElementFilter");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnSimpleElementFilter.setBorder(emptyborder1);
      jpanel1.add(btnSimpleElementFilter,cc.xy(3,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel8()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE","CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      pnlScriptEditorContainer.setName("pnlScriptEditorContainer");
      jpanel1.add(pnlScriptEditorContainer,cc.xy(2,2));

      jpanel1.add(createPanel9(),cc.xy(2,4));
      addFillComponents(jpanel1,new int[]{ 1,2,3 },new int[]{ 1,2,3,4,5 });
      return jpanel1;
   }

   public JPanel createPanel9()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      cboPickerScripts.setName("cboPickerScripts");
      jpanel1.add(cboPickerScripts,cc.xy(1,1));

      btnPickerRemove.setActionCommand("+");
      btnPickerRemove.setName("btnPickerRemove");
      btnPickerRemove.setText("R");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnPickerRemove.setBorder(emptyborder1);
      jpanel1.add(btnPickerRemove,cc.xy(3,1));

      btnPickerSelectScript.setActionCommand("+");
      btnPickerSelectScript.setName("btnPickerSelectScript");
      btnPickerSelectScript.setText("S");
      EmptyBorder emptyborder2 = new EmptyBorder(2,2,2,2);
      btnPickerSelectScript.setBorder(emptyborder2);
      jpanel1.add(btnPickerSelectScript,cc.xy(5,1));

      addFillComponents(jpanel1,new int[]{ 2,4 },new int[0]);
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
