package org.gvsig.expressionevaluator.swing.impl;

import java.awt.BorderLayout;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.expressionevaluator.swing.ExpressionPickerController;
import org.gvsig.expressionevaluator.swing.JExpressionPicker;

/**
 *
 * @author jjdelcerro
 */
public class DefaultJExpressionPicker 
        extends JPanel
        implements JExpressionPicker 
    {

    private DefaultExpressionPickerController controller;
    private final ExpressionEvaluatorSwingManager manager;

    
    public DefaultJExpressionPicker(ExpressionEvaluatorSwingManager manager) {
        this.manager = manager;
        this.initComponents();
    }
    
    private void initComponents() {
        JButton button = new JButton();
        JTextField text = new JTextField();
        this.controller = new DefaultExpressionPickerController(manager, text, button, null, null);
        this.setLayout(new BorderLayout(2, 2));
        this.add(text, BorderLayout.CENTER);
        this.add(button, BorderLayout.LINE_END);
    }

    @Override
    public Preferences getPreferences() {
        return this.controller.getPreferences();
    }
    
    @Override
    public ExpressionPickerController getController() {
        return this.controller;
    }

    @Override
    public Expression getExpression() {
        return this.controller.get();
    }
    
    @Override
    public Expression getExpressionWithOutHistory() {
        return this.controller.getWithOutHistory();
    }

    @Override
    public void setExpression(Expression expression) {
        this.controller.set(expression);
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    @Override
    public List<Element> getElements() {
        return this.controller.getElements();
    }

        @Override
    public void addElement(Element element) {
        this.controller.addElement(element);
    }
    
    @Override
    public Collection<SymbolTable> getSymbolTables() {
        return this.controller.getSymbolTables();
    }
    
    @Override
    public void addSymbolTable(String name) {
        this.controller.addSymbolTable(name);
    }
    
    @Override
    public void addSymbolTable(SymbolTable symbolTable) {
        this.controller.addSymbolTable(symbolTable);
    }

    @Override
    public void removeAllSymbolTables() {
        this.controller.removeAllSymbolTables();
    }

    @Override
    public void removeSymbolTable(String name) {
        this.controller.removeSymbolTable(name);
    }

    @Override
    public SymbolTable getPreviewSymbolTable() {
        return this.controller.getPreviewSymbolTable();
    }

    @Override
    public void setPreviewSymbolTable(SymbolTable symbolTable) {
        this.controller.setPreviewSymbolTable(symbolTable);
    }

    @Override
    public Object getProperty(String name) {
        return this.controller.getProperty(name);
    }

    @Override
    public void setProperty(String name, Object value) {
        this.controller.setProperty(name, value);
    }

    @Override
    public Map<String, Object> getProperties() {
        return this.controller.getProperties();
    }

}
