package org.gvsig.expressionevaluator.swing.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import javax.swing.AbstractListModel;
import javax.swing.SwingUtilities;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.util.LabeledValueImpl;

/**
 *
 * @author jjdelcerro
 */
class FilteredListModel extends AbstractListModel {

    private List<LabeledValue> elements;
    private List<LabeledValue> filteredElements;
    private String filter;

    public FilteredListModel() {
        this.elements = new ArrayList<>();
        this.filteredElements = this.elements;
        this.filter = null;
    }

    @Override
    public int getSize() {
        return this.filteredElements.size();
    }

    @Override
    public LabeledValue getElementAt(int index) {
        return this.filteredElements.get(index);
    }

    void addElement(LabeledValue element) {
        this.elements.add(element);
        if (this.elements != this.filteredElements) {
            this.filteredElements.add(element);
        }
    }

    void addElement(Object element) {
        this.addElement(new LabeledValueImpl<>(Objects.toString(element, ""), element));
    }

    public void setFilter(String filter) {
        if (StringUtils.isBlank(filter)) {
            this.filter = null;
            this.filteredElements = this.elements;
        } else {
            this.filter = filter;
            this.filteredElements = new ArrayList<>();
            for (LabeledValue element : elements) {
                if (FilenameUtils.wildcardMatch(element.getValue().toString().toLowerCase(), this.filter.toLowerCase())) {
                    this.filteredElements.add(element);
                }
            }
        }
        fireContentsChanged();
    }

    private void fireContentsChanged() {
        if( !SwingUtilities.isEventDispatchThread() ) {
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    fireContentsChanged();
                }
            });
            return;
        }
        this.fireContentsChanged(this, 0, this.filteredElements.size());
     }

    public String getFilter() {
        return this.filter;
    }

    public void sort(final boolean ascending) {
        this.sortBackground(ascending);
    }
    
    private void sortBackground(final boolean ascending) {
        Thread th = new Thread(new Runnable() {
            @Override
            public void run() {
                sortForeground(ascending);
            }
        }, "FilteredListModelSorter");
        th.start();
    }

    private void sortForeground(final boolean ascending) {
        final List items = new ArrayList<>(elements);
        items.sort(new Comparator<Comparable>() {
            @Override
            public int compare(Comparable o1, Comparable o2) {
                int r = ObjectUtils.compare(o1, o2);
                if (ascending) {
                    return r;
                }
                return r * -1;
            }
        });
        elements = items;
        filteredElements = elements;
        if (filter != null) {
            setFilter(filter);
        } else {
            fireContentsChanged();
        }
    }

}
