package org.gvsig.expressionevaluator.swing.impl;

import java.util.Comparator;
import java.util.Date;
import java.util.Objects;
import org.apache.commons.lang.StringEscapeUtils;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.timesupport.Time;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author jjdelcerro
 */
public class Utils {

    public static Comparator naturalComparator() {
        return new Comparator() {
            @Override
            public int compare(Object o1, Object o2) {
                if (o1 instanceof Integer && o2 instanceof Number ) {
                    return Long.compare((Integer) o1, ((Number)o2).intValue());
                    
                } else if (o1 instanceof Long && o2 instanceof Number ) {
                    return Long.compare((Long) o1, ((Number)o2).longValue());
                    
                } else if (o1 instanceof Double && o2 instanceof Number ) {
                    return Double.compare((Double) o1, ((Number)o2).doubleValue());
                    
                } else if (o1 instanceof Float && o2 instanceof Number) {
                    return Double.compare((Float) o1, ((Number)o2).floatValue());
                    
                } else if (o1 instanceof Number && o2 instanceof Number) {
                    return Double.compare( getDouble((Number) o1), getDouble((Number) o2));
                    
                } else if (o1 instanceof String && o2 instanceof String) {
                    return ((String) o1).compareToIgnoreCase((String)o2);
                    
                } else {
                    try {
                        return ((Comparable) o1).compareTo((Comparable)o2);
                    } catch(Exception ex) {
                        return Objects.toString(o1, "").compareTo(Objects.toString(o2, ""));
                    }
                }
            }
        };
    }
    
    private static double getDouble(Number n) {
        if( n==null ) {
            return 0;
        }
        return n.doubleValue();
    }

    public static String repr(Object value)  {
        if (value == null) {
            return "NULL";
        } else if (value instanceof Date) {
            return "'" + StringEscapeUtils.escapeSql(((Date) value).toString()) + "'";
//            return "DATE('" + value + "')";
        } else if (value instanceof Boolean) {
            return value.toString();
        } else if (value instanceof String) {
            return "'" + StringEscapeUtils.escapeSql((String) value) + "'";
        } else if (value instanceof Geometry) {
            ExpressionBuilder builder = ExpressionEvaluatorLocator.getManager().createExpressionBuilder();
            return builder.geometry((Geometry)value).toString();
        } else if (value instanceof Envelope) {
            ExpressionBuilder builder = ExpressionEvaluatorLocator.getManager().createExpressionBuilder();
            return builder.envelope((Envelope)value).toString();
        } else if (value instanceof Time) {
            return "'" + StringEscapeUtils.escapeSql(((Date) value).toString()) + "'";
        } else if (value instanceof Number) {
            return ((Number)value).toString();
        } else {
            return StringEscapeUtils.escapeSql(value.toString());
        }
        
    }
    
    public static String toString(Object value)  {
        if (value == null) {
            return "NULL";
        } else if (value instanceof Date) {
            return "'" + StringEscapeUtils.escapeSql(((Date) value).toString()) + "'";
//            return "DATE('" + value + "')";
        } else if (value instanceof Boolean) {
            return value.toString();
        } else if (value instanceof String) {
            return "'" + StringEscapeUtils.escapeSql((String) value) + "'";
        } else if (value instanceof Geometry) {
            try {
                return ((Geometry)value).convertToWKT();
            } catch (Exception ex) {
                return "[ERROR] "+ex.getMessage();
            }
        } else if (value instanceof Envelope) {
            try {
                return ((Envelope)value).getGeometry().convertToWKT();
            } catch (Exception ex) {
                return "[ERROR] "+ex.getMessage();
            }
        } else if (value instanceof Time) {
            return "'" + StringEscapeUtils.escapeSql(((Date) value).toString()) + "'";
        } else if (value instanceof Number) {
            return ((Number)value).toString();
        } else {
            return StringEscapeUtils.escapeSql(value.toString());
        }
        
    }
    
}
