package org.gvsig.expressionevaluator.swing.impl.elements;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;

/**
 *
 * @author jjdelcerro
 */
public class DefaultAggregateElements 
        extends AbstractElement
        implements Element.GroupElement 
    {
    
    private final List<Element> elements;
    private final ExpressionEvaluatorSwingManager manager;
    private List<Element> aggregateds;

    public DefaultAggregateElements(ExpressionEvaluatorSwingManager manager, String name) {
        super(name, name, "expressionbuilder-element-group");
        this.manager = manager;
        this.elements = new ArrayList<>();
    }

    @Override
    public List<Element> getElements() {
        return this.elements;
    }

    @Override
    public Element get(int index) {
        return this.elements.get(index);
    }

    @Override
    public int size() {
        return this.elements.size();
    }

    @Override
    public Iterator<Element> iterator() {
        return this.elements.iterator();
    }

    @Override
    public Collection<Element> getValues() {
        if( aggregateds==null ) {
            this.aggregateds = buildChilds(null, elements);
        }
        return this.aggregateds;
    }
    
    private List<Element>  buildChilds(List<Element> theAggregateds, List<Element> elements) {
        if( theAggregateds==null ) {
            theAggregateds = new ArrayList<>();
        }
        for (Element element : elements) {
            theAggregateds.add(element);
            if( element instanceof GroupElement ) {
                buildChilds(theAggregateds, ((GroupElement)element).getElements());
            }
        }
        return theAggregateds;
    }
    
    @Override
    public void addElement(Element element) {
        this.elements.add(element);
    }

    @Override
    public void addElement(Function function) {
        Element item = this.manager.createConstantElement(function, function.name());
        item.setConfig(this.getConfig());
        this.addElement(item);
    }

    @Override
    public Element setConfig(ExpressionBuilderConfig config) {
        super.setConfig(config);
        if( this.elements!=null ) {
            for (Element element : this.elements) {
                element.setConfig(config);
            }
        }
        return this;
    }

}
