/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.gvsig.expressionevaluator.swing.impl.elements;

import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element.FunctionElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingLocator;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;
import org.gvsig.expressionevaluator.swing.FunctionAdditionalPanelFactory;
import org.gvsig.expressionevaluator.swing.JExpressionBuilder;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.Component;

/**
 *
 * @author jjdelcerro
 */
public class DefaultFunctionElement
        extends AbstractElement
        implements FunctionElement {

    private final Function function;

    public DefaultFunctionElement(Function function) {
        super(
                function.name(),
                function.name(),
                function.isSQLCompatible()
                ? "expressionbuilder-element-function-sql"
                : "expressionbuilder-element-function"
        );
        this.function = function;
    }

    @Override
    public Function getFunction() {
        return this.function;
    }

    @Override
    public boolean isEnabled() {
        try {
            if( this.getConfig().getPreferences().isSQLCompatible() ) {
                return this.function.isSQLCompatible();
            }
        } catch(Exception ex) {
            
        }
        return true;
    }

    @Override
    public Component getAditionalPanel(JExpressionBuilder expressionBuilder) {
        ExpressionEvaluatorSwingManager manager = ExpressionEvaluatorSwingLocator.getManager();
        FunctionAdditionalPanelFactory factory = manager.getFunctionAdditionalPanelFactory(this.function.name());
        if (factory == null) {
            return null;
        }
        return factory.createAdditionalPanel(expressionBuilder);
    }

    @Override
    public String getName() {
        return this.function.name();
    }

    @Override
    public Object getValue() {
        return this.function.name();
    }

    @Override
    public String getLabel() {
        return "<html><b>" + function.name() + "</b> return " + function.returnType() + "</html>";
    }

    @Override
    public String getRenderedValue() {
        return function.template() == null ? function.name() + "({{}})" : function.template();
    }

    @Override
    public String getDescription() {
        I18nManager i18n = ToolsLocator.getI18nManager();

        StringBuilder html = new StringBuilder();
        html.append("<html>\n");

        Function theFunction = this.getFunction();
        html.append("<b>").append(i18n.getTranslation("_Function")).append("</b> ").append(this.getName()).append("<br>\n<br>\n");
        List<String> aliases = theFunction.aliases();
        if( aliases!=null && !aliases.isEmpty() ) {
            html.append("<b>").append(i18n.getTranslation("_Aliases")).append(":</b> ").append("<ul>\n");
            for (String alias : aliases) {
                html.append("<li>").append(alias).append("</li>\n");
            }
            html.append("</ul>\n");
        }
        html.append("<b>").append(i18n.getTranslation("_Return")).append("</b> ");
        html.append(StringUtils.isEmpty(theFunction.returnType()) ? "Objec" : theFunction.returnType());
        html.append("<br>\n");
        html.append("<b>").append(i18n.getTranslation("_Template")).append("</b> ");
        html.append(StringUtils.isEmpty(theFunction.template()) ? this.getName() + "()" : theFunction.template());
        html.append("<br>\n");

        String[] args = theFunction.descriptionArgs();
        if (args != null) {
            html.append("<b>").append(i18n.getTranslation("_Arguments")).append(":</b> ").append("<ul>\n");
            for (String arg : args) {
                html.append("<li>").append(arg).append("</li>\n");
            }
            html.append("</ul>\n");
            html.append("<br>\n");
        }
        String functionDescription = theFunction.description();
        if( !StringUtils.isBlank(functionDescription) ) {
            html.append("<b>").append(i18n.getTranslation("_Description")).append("</b><br>\n");
            html.append(functionDescription.replace("\n", "<br>")).append("<br>\n");
        }            

        html.append("</html>\n");
        return html.toString();
    }

}
