package org.gvsig.expressionevaluator.swing.impl.elements;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.GroupElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.expressionevaluator.swing.ExpressionEvaluatorSwingManager;

/**
 *
 * @author jjdelcerro
 */
public class DefaultGroupElement 
        extends AbstractElement
        implements GroupElement 
    {
    
    final private List<Element> elements;
    private ExpressionEvaluatorSwingManager manager;

    public DefaultGroupElement(ExpressionEvaluatorSwingManager manager, String name, List<Element> elements) {
        super(name, name, "expressionbuilder-element-group");
        this.manager = manager;
        this.elements = elements;
    }

    public DefaultGroupElement(String name) {
        super(name, name, "expressionbuilder-element-group");
        this.elements = new ArrayList<>();
    }

    @Override
    public List<Element> getElements() {
        return this.elements;
    }

    @Override
    public Collection<Element> getValues() {
        return this.elements;
    }
    
    @Override
    public Element get(int index) {
        return this.elements.get(index);
    }

    @Override
    public int size() {
        return this.elements.size();
    }

    @Override
    public Iterator<Element> iterator() {
        return this.elements.iterator();
    }

    @Override
    public void addElement(Element element) {
        this.elements.add(element);
    }

    @Override
    public void addElement(Function function) {
        Element item = this.manager.createConstantElement(function, function.name());
        item.setConfig(this.getConfig());
        this.addElement(item);
    }

    @Override
    public Element setConfig(ExpressionBuilderConfig config) {
        super.setConfig(config);
        if( this.elements!=null ) {
            for (Element element : this.elements) {
                element.setConfig(config);
            }
        }
        return this;
    }

    
}
