/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
/*
 * AUTHORS (In addition to CIT):
 * 2010 {Prodevelop}   {Task}
 */
package org.gvsig.installer.swing.impl.execution;

import java.awt.BorderLayout;
import java.io.File;
import java.util.List;
import javax.swing.JComponent;

import org.gvsig.installer.lib.api.InstallerLocator;
import org.gvsig.installer.lib.api.PackageInfo;
import org.gvsig.installer.lib.api.execution.InstallPackageService;
import org.gvsig.installer.lib.api.execution.InstallPackageServiceException;
import org.gvsig.installer.swing.api.SwingInstallerLocator;
import org.gvsig.installer.swing.api.SwingInstallerManager.UrlAndLabel;
import org.gvsig.installer.swing.api.execution.PackageFilter;
import org.gvsig.installer.swing.impl.execution.panel.SelectBundlesPanel;
import org.gvsig.installer.swing.impl.execution.wizard.DownloadProgressWizardPage;
import org.gvsig.installer.swing.impl.execution.wizard.AddBundlesWizardPage;
import org.gvsig.installer.swing.impl.execution.wizard.InstallersWizardPage;
import org.gvsig.installer.swing.impl.execution.wizard.ProgressWizardPage;
import org.gvsig.installer.swing.impl.execution.wizard.SelectBundlesWizardPage;
import org.gvsig.installer.swing.impl.execution.wizard.SelectPackagesWizardPage;
import org.gvsig.installer.swing.impl.execution.wizard.TypicalOrAdvancedWizardPage;
import org.gvsig.installer.swing.impl.wizard.AbstractInstallerWizardPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DefaultInstallWizardPanel
        extends
        AbstractInstallerWizardPanel
        implements
        InstallWizardPanel_ext {

    private static final Logger LOG = LoggerFactory.getLogger(DefaultInstallWizardPanel.class);
            
    private static final long serialVersionUID = 554068938842141497L;
    private static final int TYPICAL_MODE = 0;
    private static final int ADVANCED_MODE = 1;

    private int installationMode = ADVANCED_MODE;

    private boolean skipTypicalOrAdvancedWizardPage = false;
    private boolean skipSelectBundleWizardPage = false;

    private boolean selectDefaultPackages;

    private final I18nManager i18nManager;
    private PackageFilter packageFilter;

    
    private InstallPackageService installerExecutionService = null;

    // Wizard pages
    private SelectBundlesWizardPage selectInstallersWizard = null;
    private InstallersWizardPage installersWizard = null;
    private SelectPackagesWizardPage selectPluginsWizard = null;
    private ProgressWizardPage progressWizard = null;
    private DownloadProgressWizardPage downloadProgressWizard = null;
    private TypicalOrAdvancedWizardPage typicalOrAdvancedWizard = null;
    private AddBundlesWizardPage addBundlesWizard;


    public DefaultInstallWizardPanel(File applicationFolder,
            File installFolder)  {
        super(applicationFolder, installFolder);
        this.i18nManager = ToolsLocator.getI18nManager();
    }

    @Override
    public JComponent asJComponent() {
        if( needInitilization() ) {
            initComponents();
        }
        return this;
    }

    /**
     * @return the installerExecutionService
     */
    @Override
    public InstallPackageService getInstallerExecutionService() {
        if( needInitilization() ) {
            initComponents();
        }
        return installerExecutionService;
    }

    private boolean needInitilization() {
        return selectPluginsWizard == null;
    }
    
    @Override
    protected void initComponents() {
        if( needInitilization() ) {
            try {
                installerExecutionService = InstallerLocator.getInstallerManager()
                        .getInstallPackageService();
            } catch (InstallPackageServiceException ex) {
                LOG.warn("Can't create the InstallerExecutionService.",ex);
                throw new RuntimeException(ex);
            }
            List<UrlAndLabel> downloadURLs = SwingInstallerLocator
                    .getSwingInstallerManager().getDefaultDownloadUrlAndLabels();
            selectInstallersWizard = new SelectBundlesWizardPage(this, downloadURLs);
            addBundlesWizard = new AddBundlesWizardPage(this);
            typicalOrAdvancedWizard = new TypicalOrAdvancedWizardPage(this);
            selectPluginsWizard = new SelectPackagesWizardPage(this);
            progressWizard = new ProgressWizardPage(this);
            downloadProgressWizard = new DownloadProgressWizardPage(this);
            installersWizard = new InstallersWizardPage(this);

            addWizards();

            setFinishButtonEnabled(false);

            this.setLayout(new BorderLayout());
            super.initComponents();        
        }
    }
    
    private void addWizards() {
        this.addOptionPanel(typicalOrAdvancedWizard);
        this.addOptionPanel(selectInstallersWizard);
        this.addOptionPanel(addBundlesWizard);
        this.addOptionPanel(installersWizard);
        this.addOptionPanel(selectPluginsWizard);
        this.addOptionPanel(downloadProgressWizard);
        this.addOptionPanel(progressWizard);
    }

    @Override
    public SelectBundlesPanel getBundlesPanel() {
        if( needInitilization() ) {
            initComponents();
        }
        return this.selectInstallersWizard;
    }

    @Override
    public void installFromDefaultDirectory()
            throws InstallPackageServiceException {
        if( needInitilization() ) {
            initComponents();
        }
        getWizardComponents().removeWizardPanel(0);
        installerExecutionService.addBundlesFromDirectory(getInstallFolder(), null);
        selectPluginsWizard.updatePanel();
    }
    
    /**
     * @return the installersToInstall
     */
    @Override
    public List<PackageInfo> getPackagesToInstall() {
        if( needInitilization() ) {
            initComponents();
        }
        return this.selectPluginsWizard.getPackagesToInstall();
    }

    @Override
    public void setSelectDefaultPackages(boolean isActivated) {
        this.selectDefaultPackages = isActivated;
    }

    @Override
    public boolean getSelectDefaultPackages() {
        return this.selectDefaultPackages;
    }

    @Override
    public void setSkipSelectPackagesWizardPage(boolean skip) {
        if (skip) {
            installationMode = TYPICAL_MODE;
            selectDefaultPackages = true;
        } else {
            installationMode = ADVANCED_MODE;
        }
    }

    @Override
    public boolean getSkipSelectPackagesWizardPage() {
        return installationMode == TYPICAL_MODE;
    }

    @Override
    public boolean getSkipTypicalOrAdvancedWizardPage() {
        return skipTypicalOrAdvancedWizardPage;
    }

    @Override
    public void setSkipTypicalOrAdvancedWizardPage(boolean skip) {
        skipTypicalOrAdvancedWizardPage = skip;
        if (skip) {
            this.installationMode = TYPICAL_MODE;
        }
    }

    @Override
    public void setSkipSelectBundleWizardPage(boolean skip) {
        this.skipSelectBundleWizardPage = skip;
    }

    @Override
    public boolean getSkipSelectBundleWizardPage() {
        if ( !getSkipTypicalOrAdvancedWizardPage() && installationMode == TYPICAL_MODE) {
            return true;
        }
        return this.skipSelectBundleWizardPage;
    }

    @Override
    public boolean needsToRestartApplicationAfterFinish() {
        // installers need to restart
        return true;
    }

    @Override
    public String getTranslation(String key) {
        return this.i18nManager.getTranslation(key);
    }

    @Override
    public PackageFilter getPackageFilter() {
        return this.packageFilter;
    }

    @Override
    public void setPackageFilter(PackageFilter type) {
        this.packageFilter = type;
    }
}
