/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
/*
 * AUTHORS (In addition to CIT):
 * 2010 {Prodevelop}   {Task}
 */

package org.gvsig.installer.swing.impl.execution.panel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.InputStream;
import java.net.URL;
import java.util.List;
import java.util.Locale;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;

import org.gvsig.installer.lib.api.PackageInfo;
import org.gvsig.installer.swing.api.SwingInstallerLocator;
import org.gvsig.installer.swing.api.SwingInstallerManager;
import org.gvsig.installer.swing.impl.execution.panel.filters.AndFilter;
import org.gvsig.installer.swing.impl.execution.panel.filters.NameDescriptionOrCodeFilter;
import org.gvsig.installer.swing.api.execution.PackageFilter;
import org.gvsig.installer.swing.impl.execution.panel.model.PackagesTableModel;
import org.gvsig.installer.swing.impl.execution.panel.model.PackagesTableModel.PackageOfficialRecommended;
import org.gvsig.installer.swing.impl.execution.panel.model.PackagesTableModel.PackageOsAndArchitecture;
import org.gvsig.installer.swing.impl.execution.panel.model.PackagesTableModel.PackageStatus;
import org.gvsig.installer.swing.impl.execution.panel.renderers.AbstractTablePackageInfoCellRenderer;
import org.gvsig.installer.swing.impl.execution.panel.renderers.InstallStatusCellEditor;
import org.gvsig.installer.swing.impl.execution.panel.renderers.InstallStatusCellRenderer;
import org.gvsig.installer.swing.impl.execution.panel.renderers.InstalledPackageCellRenderer;
import org.gvsig.installer.swing.impl.execution.panel.renderers.OfficialRecommendedCellRenderer;
import org.gvsig.installer.swing.impl.execution.panel.renderers.OsAndArchitectureCellRenderer;

/**
 * @author gvSIG Team
 * @version $Id$
 */
public class PackagesTablePanel extends JPanel implements ListSelectionListener {

	private static final long serialVersionUID = 8156088357208685689L;
	protected SwingInstallerManager swingInstallerManager = null;

	private JScrollPane descriptionScrollPane;
	private JEditorPane descriptionTextEditor;
	private JScrollPane pluginsScrollPane;
	private JTable pluginsTable;
	private PackagePropertiesFilterPanel filterPanel;
	private FastFilterButtonsPanel fastFilterPanel;
	private SelectPackagesPanel selectPackagesPanel;
        
        private PackageFilter baseFilter = null;

	public PackagesTablePanel(SelectPackagesPanel selectPackagesPanel) {
		super();
		swingInstallerManager = SwingInstallerLocator
				.getSwingInstallerManager();
		this.selectPackagesPanel = selectPackagesPanel;
		initComponents();
		pluginsTable.getSelectionModel().addListSelectionListener(this);
		pluginsTable.setDefaultRenderer(PackageOfficialRecommended.class,
				new OfficialRecommendedCellRenderer());
		pluginsTable.setDefaultRenderer(PackageStatus.class,
				new InstallStatusCellRenderer());
		pluginsTable.setDefaultEditor(PackageStatus.class,
				new InstallStatusCellEditor(this.selectPackagesPanel));
		AbstractTablePackageInfoCellRenderer ipcr = new InstalledPackageCellRenderer();
		pluginsTable.setDefaultRenderer(PackageOsAndArchitecture.class,
				new OsAndArchitectureCellRenderer());
		pluginsTable.setDefaultRenderer(String.class, ipcr);
		pluginsTable.setDefaultRenderer(Number.class, ipcr);
		pluginsTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
	}

	private void initComponents() {
		java.awt.GridBagConstraints gridBagConstraints;

		pluginsTable = new JTable();

		MouseListener mouseListener = new MyMouseListener();
		pluginsTable.addMouseListener(mouseListener);

		pluginsScrollPane = new JScrollPane(pluginsTable);

		descriptionTextEditor = new JEditorPane();
		descriptionTextEditor.setBackground(Color.WHITE);
		descriptionTextEditor.setEditable(false);
		descriptionTextEditor.setContentType("text/html");

		descriptionScrollPane = new JScrollPane(descriptionTextEditor);

		filterPanel = new PackagePropertiesFilterPanel(this);
		filterPanel.setVisible(true);

		fastFilterPanel = new FastFilterButtonsPanel(this);

                JSplitPane rightPanel = new JSplitPane(
                        JSplitPane.VERTICAL_SPLIT,
                        pluginsScrollPane,
                        descriptionScrollPane
                );
                rightPanel.setResizeWeight(0.8);
                rightPanel.setOneTouchExpandable(true);

		setLayout(new GridBagLayout());

		// left panel filter
		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.weightx = 0.13;
		gridBagConstraints.weighty = 1;
		gridBagConstraints.gridheight = 2;
		gridBagConstraints.insets = new Insets(0, 2, 2, 2);
		add(filterPanel, gridBagConstraints);

		// fast filters buttons panel
		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.weightx = 0.75;
		gridBagConstraints.weighty = 0;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(fastFilterPanel, gridBagConstraints);

		// plugins scroll panel
		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.weightx = 0.75;
		gridBagConstraints.weighty = 0.7;
		gridBagConstraints.insets = new Insets(2, 2, 2, 2);
		add(rightPanel, gridBagConstraints);

	}

	public void resetFilter() {
		filterPanel.resetPanel();
		valueChanged();
	}
	
	public void selectPackages() {
		PackagesTableModel pluginsTableModel = (PackagesTableModel) pluginsTable
				.getModel();
		pluginsTableModel.selectDefaultPackages();
	}

	public void setTableModel(PackagesTableModel pluginsTableModel) {
		pluginsTable.setModel(pluginsTableModel);
		pluginsTableModel.fireTableDataChanged();
		TableColumnModel tableColumnModel = pluginsTable.getColumnModel();
		tableColumnModel.getColumn(0).setPreferredWidth(20);
		tableColumnModel.getColumn(1).setPreferredWidth(20);
		tableColumnModel.getColumn(2).setPreferredWidth(20);
		tableColumnModel.getColumn(3).setPreferredWidth(225);
		tableColumnModel.getColumn(4).setPreferredWidth(130);
		tableColumnModel.getColumn(5).setPreferredWidth(53);

	}

	public List<PackageInfo> getPackagesToInstall() {
		return ((PackagesTableModel) pluginsTable.getModel())
				.getPackagesToInstall();
	}

	public boolean isPackageSelected() {
		return ((PackagesTableModel) pluginsTable.getModel())
				.hasAnyPackageSelected();
	}

	private class HTMLCoder {
		private StringBuffer buffer = new StringBuffer();
	
		public String toString() {
			return buffer.toString();
		}
		
		public void addTitle(String title) {
			if( title == null ) {
				return;
			}
			buffer.append("<h1>");
			buffer.append(title);
			buffer.append("</h1>");
		}
		
		public void addParagraph(Object text) {
			if( text == null ) {
				return;
			}
			String t = text.toString().replaceAll("\\n", "<br>\n");
			buffer.append("<p>");
			buffer.append(t);
			buffer.append("</p>");
		}
		
		public void addBeginList() {
			buffer.append("<ul>");
		}
		
		public void addEndList() {
			buffer.append("</ul>");
		}
		
		public void addListItem(String label, Object value) {
			if( value == null ) {
				return;
			}
			buffer.append("<li>");
			buffer.append(label);
			buffer.append(": <i>");
			buffer.append(value.toString());
			buffer.append("</i></li>");
		}
		
		public void addListItem(String label, URL value) {
			if( value == null ) {
				return;
			}
			buffer.append("<li>");
			buffer.append(label);
			buffer.append(": <a href=\"");
			buffer.append(value.toString());
			buffer.append("\">");
			buffer.append(value.toString());
			buffer.append("</a></li>");
		}
		
	}
	public void valueChanged(ListSelectionEvent e) {
		valueChanged();
	}

	public void valueChanged() {
		int row = pluginsTable.getSelectedRow();
		if (row < 0) {
			descriptionTextEditor.setText("<p></p>");
			return;
		}

		PackageInfo pkginfo = ((PackagesTableModel) pluginsTable.getModel()).getPackageInfoAt(row);
		descriptionTextEditor.setText(getFullDescriptionAsHTML(pkginfo));

		// get the view area to the top-left corner
		descriptionTextEditor.setCaretPosition(0);

	}
        
        public String getFullDescriptionAsHTML(PackageInfo pkginfo) {
            String html = null;
            String lang = Locale.getDefault().getLanguage();
            InputStream res = this.getClass().getClassLoader().getResourceAsStream("org/gvsig/installer/swing/impl/pkg_description_"+lang+".html");
            if( res == null ) {
                res = this.getClass().getClassLoader().getResourceAsStream("org/gvsig/installer/swing/impl/pkg_description_en.html");
            }
            if( res != null ) {
                try {
                    List<String> lines = IOUtils.readLines(res);
                    html = StringUtils.join(lines,null);
                    res.close();
                } catch (Exception ex) {
                    // Do nothing
                }
            }
            if( html == null ) {
		HTMLCoder coder = new HTMLCoder();
		coder.addTitle(pkginfo.getName());
		coder.addParagraph(pkginfo.getDescription());
		coder.addBeginList();
		coder.addListItem("code", pkginfo.getCode());
		coder.addListItem("Organization", pkginfo.getOwner());
		coder.addListItem("Organization URL", pkginfo.getOwnerURL());
		coder.addListItem("Sources", pkginfo.getSourcesURL());
		coder.addListItem("Dependencies", pkginfo.getDependencies());
		coder.addListItem("Categories", pkginfo.getCategoriesAsString());
		coder.addListItem("Official", pkginfo.isOfficial());
		coder.addEndList();
                return coder.toString();
            }
            
            html = html.replace("${gvsig.package.info.name}",pkginfo.getName());
            html = html.replace("${gvsig.package.info.description}",pkginfo.getDescription());
            html = html.replace("${gvsig.package.info.official}",pkginfo.isOfficial()?"true":"false");
            html = html.replace("${gvsig.package.info.official.label}",pkginfo.isOfficial()?"official":"community");
            html = html.replace("${gvsig.package.info.owner}",pkginfo.getOwner());
            html = html.replace("${gvsig.package.info.webURL}",pkginfo.getWebURL()==null? "":pkginfo.getWebURL().toString());
            html = html.replace("${gvsig.package.info.categories}",pkginfo.getCategoriesAsString());
            html = html.replace("${gvsig.package.info.sourcesURL}",pkginfo.getSourcesURL()==null?"":pkginfo.getSourcesURL().toString());
            html = html.replace("${gvsig.package.info.code}",pkginfo.getCode());
            html = html.replace("${gvsig.package.info.dependencies}",pkginfo.getDependencies()==null?"":pkginfo.getDependencies().toString());
            html = html.replace("${gvsig.package.info.version}",pkginfo.getVersion()==null?"":pkginfo.getVersion().toString());
            html = html.replace("${gvsig.package.info.state}",pkginfo.getState());
            html = html.replace("${gvsig.package.info.operatingSystem}",pkginfo.getOperatingSystem());
            html = html.replace("${gvsig.package.info.architecture}",pkginfo.getArchitecture());
            html = html.replace("${gvsig.package.info.javaVM}",pkginfo.getJavaVM());
            return html;
        }

	public void setFilter(PackageFilter filter) {
            PackagesTableModel pluginsTableModel = (PackagesTableModel) pluginsTable.getModel();
            if( baseFilter == null ) {
                pluginsTableModel.setFilter(filter);
            } else {
                pluginsTableModel.setFilter(new AndFilter().add(this.baseFilter).add(filter));
            }
            selectPackagesPanel.updatePanel();
	}

        public void setBaseFilter(PackageFilter filter) {
            this.baseFilter = filter;
            this.setFilter(null);
        }

        public void setEnabledTypeFilter(boolean enabled) {
            this.filterPanel.setEnabledTypeFilter(enabled);            
        }
        
	private class MyMouseListener implements MouseListener {

		public void mouseClicked(MouseEvent e) {
			selectPackagesPanel.checkIfPluginSelected();
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}
	}

	public SelectPackagesPanel getSelectPackagesPanel() {
		return this.selectPackagesPanel;
	}

	public void clearAllPanels() {
		filterPanel.resetPanel();
		descriptionTextEditor.setText("");
		fastFilterPanel.resetPanel();
	}

	public void resetPanel() {
		filterPanel.resetPanel();
		descriptionTextEditor.setText("");
	}

    /**
     * 
     */
    public void setInitialFilter() {
        filterPanel.setInitialFilter();
    }

}
