/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
/*
 * AUTHORS (In addition to CIT):
 * 2010 {Prodevelop}   {Task}
 */
package org.gvsig.installer.swing.impl.execution.wizard;

import javax.swing.JOptionPane;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.gvsig.gui.beans.wizard.WizardPanelWithLogo;

import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.installer.lib.api.execution.InstallPackageService;
import org.gvsig.installer.swing.api.SwingInstallerLocator;
import org.gvsig.installer.swing.api.SwingInstallerManager;
import org.gvsig.installer.swing.impl.execution.InstallWizardPanel_ext;
import org.gvsig.installer.swing.impl.execution.panel.SelectBundlesPanel;
import org.gvsig.installer.swing.impl.panel.DefaultProgressPanel;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.task.AbstractMonitorableTask;
import org.gvsig.tools.task.SimpleTaskStatus;

public class AddBundlesWizardPage extends DefaultProgressPanel implements OptionPanel {

    private static final long serialVersionUID = 8531884535246881448L;
    private final InstallWizardPanel_ext wizardPanel;

    public AddBundlesWizardPage(InstallWizardPanel_ext installerExecutionWizard) {
        super();
        this.wizardPanel = installerExecutionWizard;
    }

    @Override
    public JPanel getJPanel() {
        return this;
    }

    @Override
    public String getPanelTitle() {
        return wizardPanel.getTranslation("_Loading_packages");
    }

    @Override
    public void lastPanel() {
        wizardPanel.setFinishButtonEnabled(false);
        wizardPanel.setCancelButtonEnabled(true);
    }

    @Override
    public void nextPanel() {
    }

    @Override
    public void updatePanel() {
        if( wizardPanel.getDirection() == WizardPanelWithLogo.ACTION_PREVIOUS ) {
            wizardPanel.skip();
            return;
        }
        wizardPanel.setFinishButtonEnabled(false);
        wizardPanel.setBackButtonEnabled(false);
        wizardPanel.setNextButtonEnabled(false);
        wizardPanel.setCancelButtonEnabled(true);

        UpdatePanel task = new UpdatePanel(this);
        this.wizardPanel.addCancellableTask(task);
        bind(task.getTaskStatus());
        task.setDaemon(true);
        task.start();
    }

    private class UpdatePanel extends AbstractMonitorableTask {

        private final AddBundlesWizardPage panel;

        public UpdatePanel(AddBundlesWizardPage panel) {
            super(panel.wizardPanel.getTranslation("_Loading_packages"));
            this.panel = panel;
        }

        @Override
        public synchronized void run() {

            try {

                logger.info("loading packages");
                SimpleTaskStatus taskStatus = (SimpleTaskStatus) this.getTaskStatus();
                taskStatus.setIndeterminate();

                InstallPackageService installerExecutionService = wizardPanel.getInstallerExecutionService();
                SelectBundlesPanel bundlesPanel = wizardPanel.getBundlesPanel();

                if (bundlesPanel.isStandardSelected()) {
                    installerExecutionService.addBundlesFromDirectory(
                            wizardPanel.getInstallFolder(), taskStatus
                    );

                } else if (bundlesPanel.isURLSelected() && bundlesPanel.getSelectedURL() != null) {
                    installerExecutionService.addBundle(bundlesPanel.getSelectedURL(), taskStatus);

                } else if (bundlesPanel.isFileSelected() && bundlesPanel.getSelectedFile() != null && bundlesPanel.getSelectedFile().exists()) {
                    installerExecutionService.addBundle(bundlesPanel.getSelectedFile(), taskStatus);

                }

                taskStatus.message(panel.wizardPanel.getTranslation("_Finished"));

                taskStatus.terminate();
                SwingUtilities.invokeAndWait(new Runnable() {

                    @Override
                    public void run() {
                        wizardPanel.setFinishButtonEnabled(false);
                        wizardPanel.setBackButtonEnabled(true);
                        wizardPanel.setNextButtonEnabled(true);
                        wizardPanel.setCancelButtonEnabled(true);
                        wizardPanel.doAction(WizardPanelWithLogo.ACTION_NEXT);
                    }
                });
                logger.info("Loading packages finished");

            } catch (Throwable th) {
                logger.warn("Problems loading packages.", th);
                ToolsSwingLocator.getThreadSafeDialogsManager().messageDialog(
                        panel.wizardPanel.getTranslation("_Cant_load_packages"), 
                        panel.wizardPanel.getTranslation("_Warning"), 
                        JOptionPane.WARNING_MESSAGE
                );

            }
        }

    }
}
