/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
/*
 * AUTHORS (In addition to CIT):
 * 2010 {Prodevelop}   {Task}
 */
package org.gvsig.installer.swing.impl.execution.wizard;

import javax.swing.JOptionPane;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.gvsig.gui.beans.wizard.WizardPanelWithLogo;

import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.installer.lib.api.execution.InstallPackageService;
import org.gvsig.installer.swing.api.SwingInstallerLocator;
import org.gvsig.installer.swing.api.SwingInstallerManager;
import org.gvsig.installer.swing.impl.DefaultSwingInstallerManager;
import org.gvsig.installer.swing.impl.execution.InstallWizardPanel_ext;
import org.gvsig.installer.swing.impl.panel.DefaultProgressPanel;
import org.gvsig.tools.task.AbstractMonitorableTask;
import org.gvsig.tools.task.SimpleTaskStatus;

public class InstallersWizardPage extends DefaultProgressPanel implements OptionPanel {

    private static final long serialVersionUID = 8531884535246881448L;
    private final InstallWizardPanel_ext wizardPanel;

    public InstallersWizardPage(InstallWizardPanel_ext installerExecutionWizard) {
        super();
        this.wizardPanel = installerExecutionWizard;
    }

    @Override
    public JPanel getJPanel() {
        return this;
    }

    @Override
    public String getPanelTitle() {
        return wizardPanel.getTranslation("_Install_required_addons");
    }

    @Override
    public void lastPanel() {
        wizardPanel.setFinishButtonEnabled(false);
        wizardPanel.setCancelButtonEnabled(true);
    }

    @Override
    public void nextPanel() {
    }

    @Override
    public void updatePanel() {
        if( wizardPanel.getDirection() == WizardPanelWithLogo.ACTION_PREVIOUS ) {
            wizardPanel.skip();
            return;
        }
        InstallPackageService bundle = wizardPanel.getInstallerExecutionService();
        if ( !wizardPanel.getSelectDefaultPackages() || ! bundle.needInstallPackageProviders()) {
            wizardPanel.skip();
            return;
        }

        wizardPanel.setFinishButtonEnabled(false);
        wizardPanel.setBackButtonEnabled(false);
        wizardPanel.setNextButtonEnabled(false);
        wizardPanel.setCancelButtonEnabled(true);

        UpdatePanel task = new UpdatePanel(this);
        this.wizardPanel.addCancellableTask(task);
        bind(task.getTaskStatus());
        task.setDaemon(true);
        task.start();
    }

    private class UpdatePanel extends AbstractMonitorableTask {

        private final InstallersWizardPage panel;

        public UpdatePanel(InstallersWizardPage panel) {
            super(panel.wizardPanel.getTranslation("_Installing_prerequisites_three_dots"));
            this.panel = panel;
        }

        private void showWarning(final String msg) {
            if (!SwingUtilities.isEventDispatchThread()) {
                try {
                    SwingUtilities.invokeAndWait(new Runnable() {
                        @Override
                        public void run() {
                        }
                    });
                } catch (Exception ex) {
                    logger.warn("Can't show message to the user. " + msg);
                }
                return;
            }
            JOptionPane.showMessageDialog(panel, msg, "Warning", JOptionPane.WARNING_MESSAGE);
        }

        @Override
        public synchronized void run() {

            try {

                logger.info("Package installation initiated");
                SimpleTaskStatus taskStatus = (SimpleTaskStatus) this.getTaskStatus();

                InstallPackageService bundle = wizardPanel.getInstallerExecutionService();
                if (bundle.needInstallPackageProviders()) {
                    bundle.installPackageProviders(taskStatus);
                }

                taskStatus.message(panel.wizardPanel.getTranslation("_Finished"));

                taskStatus.terminate();
                SwingUtilities.invokeAndWait(new Runnable() {

                    @Override
                    public void run() {
                        wizardPanel.setFinishButtonEnabled(false);
                        wizardPanel.setBackButtonEnabled(true);
                        wizardPanel.setNextButtonEnabled(true);
                        wizardPanel.setCancelButtonEnabled(true);
                        wizardPanel.doAction(WizardPanelWithLogo.ACTION_NEXT);
                    }
                });
                logger.info("Package installation finished");
            } catch (Throwable th) {
                logger.warn("Problems install packages.", th);
                showWarning(panel.wizardPanel.getTranslation("_Cant_install_package"));
            }
        }

    }
}
