/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.installer.swing.impl.execution.wizard;

import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.gvsig.gui.beans.wizard.panel.NotContinueWizardException;
import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.installer.lib.api.Dependencies;
import org.gvsig.installer.lib.api.DependenciesCalculator;
import org.gvsig.installer.lib.api.InstallerLocator;
import org.gvsig.installer.lib.api.InstallerManager;
import org.gvsig.installer.lib.api.PackageInfo;
import org.gvsig.installer.swing.api.SwingInstallerLocator;
import org.gvsig.installer.swing.api.SwingInstallerManager;
import org.gvsig.installer.swing.api.execution.JShowPackageStatusAndAskContinuePanel;
import org.gvsig.installer.swing.api.execution.JShowPackagesAndAskContinuePanel;
import org.gvsig.installer.swing.api.execution.JShowRequiredPackagesAndAskContinuePanel;
import org.gvsig.installer.swing.api.execution.JShowUnresolvedDependenciesAndAskContinuePanel;
import org.gvsig.installer.swing.api.execution.PackageFilter;
import org.gvsig.installer.swing.impl.execution.InstallWizardPanel_ext;
import org.gvsig.installer.swing.impl.execution.panel.SelectPackagesPanel;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SelectPackagesWizardPage implements OptionPanel {

    private static final long serialVersionUID = -4678172923039393643L;

    private static final Logger LOG = LoggerFactory.getLogger(SelectPackagesWizardPage.class);

    private final InstallWizardPanel_ext wizardPanel;

    private SelectPackagesPanel selectPackagesPanel = null;

    public SelectPackagesWizardPage(
            InstallWizardPanel_ext wizardPanel) {
        super();
        this.wizardPanel = wizardPanel;
        this.selectPackagesPanel = new SelectPackagesPanel(this);
    }
    
    @Override
    public JPanel getJPanel() {
        return selectPackagesPanel;
    }

    @Override
    public String getPanelTitle() {
        return wizardPanel.getTranslation("_select_plugins");
    }

    @Override
    public void lastPanel() {
        selectPackagesPanel.clearPanel();
    }

    @Override
    public void nextPanel() throws NotContinueWizardException {
        SwingInstallerManager swingManager = SwingInstallerLocator.getSwingInstallerManager();
        /*
         * First warn about packages in development or unofficial
         */
        checkDevelAndUnoficialPackages(swingManager);
        /*
         * Then check dependencies
         */
        checkDependencies(swingManager);
        wizardPanel.setNextButtonEnabled(false);
    }

    private void checkDependencies(SwingInstallerManager swingManager) throws NotContinueWizardException {
        List<PackageInfo> requiredPackages = null;
        List<PackageInfo> troubledPackages = null;
        Dependencies unresolvedDependencies = null;
        try {
            InstallerManager manager = InstallerLocator.getInstallerManager();
            // Creamos el calculador de dependencias
            DependenciesCalculator calculator = manager.createDependenciesCalculator(
                wizardPanel.getInstallerExecutionService()
            );

            // Le indicamos los paquetes que queremos instalar
            calculator.addPackageToInstall(
                    this.wizardPanel.getPackagesToInstall()
            );

            // Le aadimos los paquetes que ya hay instalados.
            PackageInfo[] pkgs = manager.getInstalledPackages();
            calculator.addInstalledPackage(pkgs);

            // Calculamos las dependencias
            calculator.calculate();

            requiredPackages = calculator.getRequiredPackages();
            unresolvedDependencies = calculator.getUnresolvedDependencies();
            troubledPackages = calculator.getConflictPackages();
        } catch (Throwable e) {
            LOG.warn(
                    wizardPanel.getTranslation("_an_error_has_occurred_when_calculating_the_dependecies_of_the_packages_to_install"),
                    e
            );
            int resp = JOptionPane.showConfirmDialog(
                null,
                wizardPanel.getTranslation("_an_error_has_occurred_when_calculating_the_dependecies_of_the_selected_packages") +
                "\n" +
                wizardPanel.getTranslation("_it_is_not_possible_to_verify_if_any_additional_package_is_needed_to_install_the_selected_packages") +
                "\n" +
                wizardPanel.getTranslation("_if_you_want_you_can_select_again_manually_the_packets_that_might_be_necessary") +
                wizardPanel.getTranslation("_do_you_want_to_continue_and_install_the_packages_you_already_have_selected"), 
                wizardPanel.getTranslation("_a_problem_has_occurred_when_calculating_the_dependencies"),
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE
            );
            if (resp == JOptionPane.NO_OPTION) {
                throw new NotContinueWizardException("", null, false);
            }
        }

        if (troubledPackages != null && troubledPackages.size() > 0) {
            JShowPackagesAndAskContinuePanel dlg = swingManager
                    .createJShowTroubledPackagesAndAskContinuePanel(
                            troubledPackages,
                            wizardPanel.getTranslation("_Some_of_the_selected_addons_should_not_be_installed_simultaneously")
                            + ". "
                            + wizardPanel.getTranslation("_Installing_these_addons_may_cause_errors")
                            + "\n"
                            + wizardPanel.getTranslation("_do_you_want_to_continue")
                    );
            if (dlg.needShow()) {
                WindowManager wm = ToolsSwingLocator.getWindowManager();
                wm.showWindow(
                    dlg, 
                    wizardPanel.getTranslation("_do_you_want_to_continue"),
                    WindowManager.MODE.DIALOG
                );
                if (dlg.cancelled()) {
                    throw new NotContinueWizardException("", null, false);
                }
            }
        }

        if (requiredPackages != null && requiredPackages.size() > 0) {
            JShowRequiredPackagesAndAskContinuePanel dlg = swingManager
                    .createJShowRequiredPackagesAndAskContinuePanel(
                            requiredPackages,
                            wizardPanel.getTranslation("_the_selected_packages_require_to_install_or_update_the_following_packages")
                            + ":\n"
                            + wizardPanel.getTranslation("_do_you_want_to_continue"));
            if (dlg.needShow()) {
                WindowManager wm = ToolsSwingLocator.getWindowManager();
                wm.showWindow(
                    dlg, 
                    wizardPanel.getTranslation("_do_you_want_to_continue"),
                    WindowManager.MODE.DIALOG
                );
                if (dlg.cancelled()) {
                    throw new NotContinueWizardException("", null, false);
                }
            }
        }

        if (unresolvedDependencies != null && unresolvedDependencies.size() > 0) {
            JShowUnresolvedDependenciesAndAskContinuePanel dlg = swingManager
                .createJShowUnresolvedDependenciesAndAskContinuePanel(
                    unresolvedDependencies,
                    wizardPanel.getTranslation("_the_following_dependencies_have_not_been_possible_to_resolve")
                    + ". "
                    + wizardPanel.getTranslation("_This_may_cause_issues_in_gvSIG")
                    + "\n"
                    + wizardPanel.getTranslation("_do_you_want_to_continue_anyway")
                );
            if (dlg.needShow()) {
                WindowManager wm = ToolsSwingLocator.getWindowManager();
                wm.showWindow(
                    dlg, 
                    wizardPanel.getTranslation("_do_you_want_to_continue"),
                    WindowManager.MODE.DIALOG
                );
                if (dlg.cancelled()) {
                    throw new NotContinueWizardException("", null, false);
                }
            }
        }

        Iterator<PackageInfo> it = requiredPackages.iterator();
        while (it.hasNext()) {
            PackageInfo pkg = it.next();
            selectPackagesPanel.getModel().selectPackage(pkg);
            LOG.info("Automatic selection of required package " + pkg.getCode());
        }
    }

    private void checkDevelAndUnoficialPackages(SwingInstallerManager swingManager)
            throws NotContinueWizardException {
        List<PackageInfo> packagesToInstall = this.wizardPanel.getPackagesToInstall();

        JShowPackageStatusAndAskContinuePanel dlg = swingManager
                .createJShowPackageStatusAndAskContinuePanel(
                        packagesToInstall,
                        wizardPanel.getTranslation("_you_have_selected_in_development_or_not_official_versions")
                        + ". "
                        + wizardPanel.getTranslation("_Installing_devel_addons_may_cause_errors")
                        + "\n"
                        + wizardPanel.getTranslation("_do_you_want_to_continue_anyway"));
        if (dlg.needShow()) {
            WindowManager wm = ToolsSwingLocator.getWindowManager();
            wm.showWindow(
                    dlg, 
                    wizardPanel.getTranslation("_do_you_want_to_continue"),
                    WindowManager.MODE.DIALOG
            );
            if (dlg.cancelled()) {
                throw new NotContinueWizardException("", null, false);
            }
        }
    }

    @Override
    public void updatePanel() {
        selectPackagesPanel.updatePanel();
        selectPackagesPanel.setInitialFilter();

        // if default packages must be selected or not
        if (wizardPanel.getSelectDefaultPackages()) {
            selectPackagesPanel.selectPackages();
        }
        wizardPanel.setFinishButtonEnabled(true);
        wizardPanel.setBackButtonEnabled(true);
        wizardPanel.setNextButtonEnabled(false);
        wizardPanel.setCancelButtonEnabled(true);

        this.selectPackagesPanel.setBaseFilter(this.wizardPanel.getPackageFilter());

        if (wizardPanel.getSkipSelectPackagesWizardPage() && !this.getPackagesToInstall().isEmpty()) {
            wizardPanel.skip();
            return;
        }

        checkIfPluginSelected();
    }

    public void checkIfPluginSelected() {
        wizardPanel.setNextButtonEnabled(selectPackagesPanel.isPackageSelected());
    }

    public List<PackageInfo> getPackagesToInstall() {
        return selectPackagesPanel.getPackagesToInstall();
    }

    public Boolean isDefaultPackagesSelectionSet() {
        return wizardPanel.getSelectDefaultPackages();
    }

    public InstallWizardPanel_ext getWizardPanel() {
        return this.wizardPanel;
    }

}
