/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
/*
 * AUTHORS (In addition to CIT):
 * 2010 {Prodevelop}   {Task}
 */
package org.gvsig.installer.swing.impl.wizard;

import java.awt.BorderLayout;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JButton;

import javax.swing.JPanel;
import jwizardcomponent.DefaultJWizardComponents;
import org.gvsig.gui.beans.wizard.WizardPanel;
import org.gvsig.gui.beans.wizard.WizardPanelActionListener;
import org.gvsig.gui.beans.wizard.WizardPanelWithLogo;
import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.installer.swing.api.wizard.InstallerWizardPanel;
import org.gvsig.installer.swing.api.wizard.InstallerWizardActionListener;

import org.gvsig.tools.task.CancellableTask;

public abstract class AbstractInstallerWizardPanel extends JPanel implements
        InstallerWizardPanel, WizardPanel {

    private static final long serialVersionUID = 5609799260650093733L;
    private InstallerWizardActionListener wizardActionListener = null;
    private final File applicationFolder;
    private final File installFolder;
    private final List<CancellableTask> cancelableTasks;
    private WizardPanelWithLogo wizardPanelWithLogo = null;
    private WizardListenerAdapter wizardListenerAdapter = null;

    public AbstractInstallerWizardPanel(File applicationFolder, File installFolder) {
        super();
        this.wizardPanelWithLogo = new WizardPanelWithLogo();
        this.applicationFolder = applicationFolder;
        this.installFolder = installFolder;
        this.cancelableTasks = new ArrayList<>();
        this.wizardPanelWithLogo.setWizardListener(this);

    }

    protected void initComponents() {
        this.add(this.wizardPanelWithLogo, BorderLayout.CENTER);
        this.wizardPanelWithLogo.updatePanel();
    }

    public void addOptionPanel(OptionPanel wizardPage) {
        this.wizardPanelWithLogo.addOptionPanel(wizardPage);
    }

    public DefaultJWizardComponents getWizardComponents() {
        return wizardPanelWithLogo.getWizardComponents();
    }

    @Override
    public void addCancellableTask(CancellableTask task) {
        this.cancelableTasks.add(task);
    }

    @Override
    public List<CancellableTask> getCancellableTasks() {
        return this.cancelableTasks;
    }

    @Override
    public void setWizardActionListener(
            InstallerWizardActionListener wizardActionListener) {
        this.wizardActionListener = wizardActionListener;
    }

    @Override
    public InstallerWizardActionListener getWizardActionListener() {
        return wizardActionListener;
    }

    @Override
    public File getApplicationFolder() {
        return applicationFolder;
    }

    @Override
    public File getInstallFolder() {
        return installFolder;
    }

    @Override
    public boolean needsToRestartApplicationAfterFinish() {
        // false by default
        // to be overriden by subclasses
        return false;
    }

    @Override
    public void setNextButtonEnabled(boolean isEnabled) {
        JButton button = getWizardComponents().getNextButton();
        button.setEnabled(isEnabled);
        button.repaint();
    }

    @Override
    public void setFinishButtonEnabled(boolean isEnabled) {
        JButton button = getWizardComponents().getFinishButton();
        button.setEnabled(isEnabled);
        button.repaint();
    }

    @Override
    public void setCancelButtonEnabled(boolean isEnabled) {
        JButton button = getWizardComponents().getCancelButton();
        button.setEnabled(isEnabled);
        button.repaint();
    }

    @Override
    public void setBackButtonEnabled(boolean isEnabled) {
        JButton button = getWizardComponents().getBackButton();
        button.setEnabled(isEnabled);
        button.repaint();
    }

    @Override
    public void doAction(int action) {
        this.wizardPanelWithLogo.doAction(action);
    }

    @Override
    public void skip() {
        this.wizardPanelWithLogo.skip();
    }

    @Override
    public int getDirection() {
        return this.wizardPanelWithLogo.getDirection();
    }

    @Override
    public WizardPanelActionListener getWizardPanelActionListener() {
        if (((wizardListenerAdapter == null) && (getWizardActionListener() != null))) {
            wizardListenerAdapter = new WizardListenerAdapter(this);
        }
        return wizardListenerAdapter;
    }

    @Override
    public void setWizardPanelActionListener(
            WizardPanelActionListener wizardActionListener) {
        // TODO Auto-generated method stub
    }
}
