/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.app.gui.styling;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import org.gvsig.svgsupport.SVGRenderer;
import org.gvsig.tools.util.ToolsUtilLocator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SLDListBoxCellRenderer extends JPanel implements ListCellRenderer {

    private static final Logger LOG = LoggerFactory.getLogger(SLDListBoxCellRenderer.class);
    /**
     *
     */
    private static final long serialVersionUID = 1458566622433802796L;
    private String str;
    private final SVGRenderer renderer;

    BufferedImage image;

    /**
     * Constructor method
     *
     */
    public SLDListBoxCellRenderer() {
        setOpaque(true);
        setLayout(new BorderLayout(3, 3));
        renderer = ToolsUtilLocator.getSVGSupportManager().createRenderer();
    }

    @Override
    public Component getListCellRendererComponent(JList list, Object value,
            int index, boolean isSelected, boolean cellHasFocus) {
        String selectedSymbol = value.toString();
        str = selectedSymbol;

        if (isSelected) {
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
        } else {
            setBackground(list.getBackground());
            setForeground(list.getForeground());
        }

        try {
            renderer.setSource(new File(str));
        } catch (Exception ex) {
            LOG.warn("Can't draw svg from '" + str + "'.", ex);
        }

        String pet = selectedSymbol;
        setFont(list.getFont());
        repaint();

        return this;
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        try {
            renderer.drawInsideRectangle(
                    (Graphics2D) g,
                    new Rectangle2D.Double(0.0, 0.0, getWidth() - 1.0, getHeight() - 1.0),
                    true
            );
        } catch (Exception e) {
            Rectangle bounds = getBounds();
            JLabel errLbl = new JLabel("<html><b>" + "Cannot render north" +/*PluginServices.getText(this, "cannot_render_north")+*/ "</b></html>");
            errLbl.setPreferredSize(bounds.getSize());
            errLbl.setSize(bounds.getSize());
            errLbl.setBounds(bounds);
            errLbl.paint(g);
        }
    }

}
