/* gvSIG. Geographic Information System of the Valencian Government
 *
 * Copyright (C) 2007-2008 Infrastructures and Transports Department
 * of the Valencian Government (CIT)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 */
package org.gvsig.timesupport.animation;

import org.gvsig.timesupport.Interval;
import org.gvsig.timesupport.RelativeInterval;
import org.gvsig.tools.observer.Observable;

public interface TimeAnimation extends Observable {

    public static final int YEAR = 0;
    public static final int MONTH = 1;
    public static final int WEEK = 2;
    public static final int DAY = 3;
    public static final int HOUR = 4;
    public static final int MINUTE = 5;
    public static final int SECOND = 6;
    public static final int MILLISECOND = 7;

    public static final int STOPED = 0;
    public static final int PLAYING = 1;
    public static final int PAUSED = 2;

    public interface TimeAnimationRenderer {
        public void begin();
        public void render(Interval time);
        public void end();
    }
    /**
     * Stops the animation
     */
    public void stop();

    /**
     * Pauses the animation
     */
    public void pause();

    /**
     * Play the animation
     */
    public void play();

    /**
     * Return is the animation is playing.
     *
     * @return
     */
    public boolean isPlaying();
    
    public int getStatus();

    public RelativeInterval getInterval();

    public void setInterval(RelativeInterval interval);
    
    public void inferStepScale();
    
    public void setPauseTime(long time);
    
    public long getPauseTime();

    public void setFramesPerSecond(double fps);
    
    public double getFramesPerSecond();
    
    public long getStep();

    public int getStepScale();

    public void setStep(long step);

    public long getStep(int scale);

    public void setStep(long step, int scale);

    public int getStepCount();
    
    public RelativeInterval getCurrentTime();
    
    public int getCurrentPosition();
    
    public void setCurrentPosition(int position);
    
    public int getIterations();
    
    public void setIterations(int iterations);

    public int getCurrentIteration();
 
    public String getStatusLabel(int status);   
    
    public String getScaleLabel(int scale);
}
