package org.gvsig.timesupport.impl.coercion;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Locale;
import org.apache.commons.lang3.math.NumberUtils;
import org.gvsig.timesupport.RelativeInstant;
import org.gvsig.timesupport.TimeSupportLocator;
import org.gvsig.timesupport.TimeSupportManager;
import org.gvsig.tools.dataTypes.CoercionException;
import org.gvsig.tools.dataTypes.DataTypesManager.CoercionWithLocale;

/**
 *
 * @author jjdelcerro
 */
public class CoerceToInstant implements CoercionWithLocale {

    public CoerceToInstant() {
        
    }
    
    @Override
    public Object coerce(Object o) throws CoercionException {
        return this.coerce(o, null);
    }
    
    public Object coerce(Object o, Locale locale) throws CoercionException {
        if( o == null ) {
            return null;
        }
        if( o instanceof RelativeInstant ) {
            return o;
        }
        TimeSupportManager manager = TimeSupportLocator.getManager();
        if( o instanceof Date ) {
            RelativeInstant instant = manager.createRelativeInstant((Date)o);
            return instant;
        }
        if( o instanceof Number ) {
            RelativeInstant instant = manager.createRelativeInstant( ((Number)o).longValue());
            return instant;
        }
        if( o instanceof String ) {
            long milis_l = NumberUtils.toLong((String) o, Long.MIN_VALUE);
            if( milis_l != Long.MIN_VALUE ) {
                RelativeInstant instant = manager.createRelativeInstant(milis_l);
                return instant;
            }
            double milis_d = NumberUtils.toDouble((String) o, Double.NaN);
            if( milis_d != Double.NaN ) {
                RelativeInstant instant = manager.createRelativeInstant((long)milis_d);
                return instant;
            }
            DateFormat df = DateFormat.getDateTimeInstance(
                    DateFormat.DEFAULT,
                    DateFormat.DEFAULT,
                    locale==null? Locale.getDefault():locale
            );
            try {
                Date date = df.parse((String) o);
                RelativeInstant instant = manager.createRelativeInstant(date);
                return instant;
            } catch (ParseException ex) {
            }
        }
        throw new CoercionException("Can't coerce ("+o.getClass().getSimpleName()+") '"+o.toString()+"' to Instant.");
    }
}
