package org.gvsig.timesupport.impl.coercion;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Locale;
import org.apache.commons.lang3.math.NumberUtils;
import org.gvsig.timesupport.RelativeInstant;
import org.gvsig.timesupport.RelativeInterval;
import org.gvsig.timesupport.TimeSupportLocator;
import org.gvsig.timesupport.TimeSupportManager;
import org.gvsig.tools.dataTypes.CoercionException;
import org.gvsig.tools.dataTypes.DataTypesManager.CoercionWithLocale;

/**
 *
 * @author jjdelcerro
 */
public class CoerceToInterval implements CoercionWithLocale {

    public CoerceToInterval() {
        
    }
    
    @Override
    public Object coerce(Object o) throws CoercionException {
        return this.coerce(o, null);
    }
    
    public Object coerce(Object o, Locale locale) throws CoercionException {
        if( o == null ) {
            return null;
        }
        if( o instanceof RelativeInterval ) {
            return o;
        }
        TimeSupportManager manager = TimeSupportLocator.getManager();
        if( o instanceof RelativeInstant ) {
            RelativeInterval interval = manager.createRelativeInterval(((RelativeInstant) o), ((RelativeInstant) o));
            return interval;
        }
        if( o instanceof Date ) {
            RelativeInterval interval = manager.createRelativeInterval(((Date) o).getTime(), ((Date) o).getTime());
            return interval;
        }
        if( o instanceof Number ) {
            RelativeInterval interval = manager.createRelativeInterval(((Number)o).longValue(), ((Number)o).longValue());
            return interval;
        }
        if( o instanceof String ) {
            long milis_l = NumberUtils.toLong((String) o, Long.MIN_VALUE);
            if( milis_l != Long.MIN_VALUE ) {
            RelativeInterval interval = manager.createRelativeInterval(milis_l, milis_l);
            return interval;
            }
            double milis_d = NumberUtils.toDouble((String) o, Double.NaN);
            if( milis_d != Double.NaN ) {
                RelativeInterval interval = manager.createRelativeInterval((long)milis_d, (long)milis_d);
                return interval;
            }
            DateFormat df = DateFormat.getDateTimeInstance(
                    DateFormat.DEFAULT,
                    DateFormat.DEFAULT,
                    locale==null? Locale.getDefault():locale
            );
            try {
                Date date = df.parse((String) o);
                RelativeInterval interval = manager.createRelativeInterval(date.getTime(), date.getTime());
                return interval;
            } catch (ParseException ex) {
            }
        }
        throw new CoercionException("Can't coerce ("+o.getClass().getSimpleName()+") '"+o.toString()+"' to Interval.");
    }
}
