/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2017 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.timesupport.swing.api.panel;

import java.util.List;

import org.gvsig.timesupport.Instant;
import org.gvsig.timesupport.Interval;
import org.gvsig.timesupport.Time;
import org.gvsig.tools.observer.Observable;
import org.gvsig.tools.swing.api.Component;

public interface TimeSelectorPanel extends Component, Observable {

    public interface TimeSelectorPanelNotification {

        public static final String UPDATE_TIME = "update";

        public Time getTime();

        /**
         * Returns the type of this notification, represented by one of the
         * constants defined in this interface.
         *
         * @return a String containing this notification's type
         */
        public String getType();
    }

    /**
     * Set the interval of panel. If the interval is absolute, the control shows
     * absolute intervals and if is relative, the control only shows relative
     * intervals
     *
     * @param interval the maximum and minimum instants that can be selected
     */
    public void setIntervals(Interval interval);

    /**
     * Set valid instants
     *
     * @param instants
     */
    public void setInstants(List<Instant> instants);

    public boolean isInterval();
    
    public boolean isInstant();

    public boolean isNone();

}
