/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2017 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.timesupport.swing.impl.components;

import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JLabel;

import org.gvsig.timesupport.AbsoluteInstant;
import org.gvsig.timesupport.AbsoluteInstantType;
import org.gvsig.timesupport.AbsoluteInstantTypeNotRegisteredException;
import org.gvsig.timesupport.Time;
import org.gvsig.timesupport.TimeSupportLocator;
import org.gvsig.timesupport.TimeSupportManager;
import org.gvsig.timesupport.swing.impl.rdv.TimeAdjustmentListener;
import org.gvsig.timesupport.swing.impl.rdv.TimeEvent;

/**
 * Panel to manipulate AbsoluteInstants / AbsoluteIntervals
 *
 */
public class AbsoluteTemporalComponent extends javax.swing.JPanel implements TemporalComponent, DateTextListener {

    private static final long serialVersionUID = 4486793564004694709L;
//    private static TimeSupportManager timeSupportManager = TimeSupportLocator
//	.getManager();
    private TimeAdjustmentListener listener;
    private AbsoluteInstant initTime;
    private JTemporalDateText jDateText;
    private JLabel lblLabel = null;

    public AbsoluteTemporalComponent() {
        super();
        initComponents();
    }

    public TimeAdjustmentListener getListener() {
        return listener;
    }

    public void setListener(TimeAdjustmentListener listener) {
        this.listener = listener;
    }

    public void setTimes(String label, Time init, Time lower, Time upper) {
        initTime = (AbsoluteInstant) init;
        getTemporalDateText().setInstant(initTime);
        getLabel().setText(label + " " + initTime.toString());
    }

    public void setCurrentInstant(Time date) {
        getTemporalDateText().setInstant((AbsoluteInstant) date);
    }

    public void setStartTime(Time date) {

    }

    public void setEndTime(Time date) {

    }

    public Time getTime() {
        ArrayList<Integer> types = new ArrayList<Integer>();
        ArrayList<Integer> values = new ArrayList<Integer>();

        try {
            values.add(getTemporalDateText().getYears());
            types.add(AbsoluteInstantType.YEAR);
        } catch (NumberFormatException e) {
        }

        try {
            values.add(getTemporalDateText().getWeekOfWeekYear());
            types.add(AbsoluteInstantType.MONTH_OF_YEAR);
        } catch (NumberFormatException e) {
        }
        try {
            values.add(getTemporalDateText().getMonthOfYear());
            types.add(AbsoluteInstantType.WEEK_OF_WEEKYEAR);
        } catch (NumberFormatException e) {
        }
        try {
            values.add(getTemporalDateText().getDayOfMonth());
            types.add(AbsoluteInstantType.DAY_OF_MONTH);
        } catch (NumberFormatException e) {
        }
        try {
            values.add(getTemporalDateText().getHourOfDay());
            types.add(AbsoluteInstantType.HOUR_OF_DAY);
        } catch (NumberFormatException e) {
        }
        try {
            values.add(getTemporalDateText().getMinuteOfHour());
            types.add(AbsoluteInstantType.MINUTE_OF_HOUR);
        } catch (NumberFormatException e) {
        }
        try {
            values.add(getTemporalDateText().getSecondOfMinute());
            types.add(AbsoluteInstantType.SECOND_OF_MINUTE);
        } catch (NumberFormatException e) {
        }
        try {
            values.add(getTemporalDateText().getMillisOfSecond());
            types.add(AbsoluteInstantType.MILLIS_OF_SECOND);
        } catch (NumberFormatException e) {
        }
        int[] vTypes = new int[types.size()];
        int[] vValues = new int[types.size()];
        for( int i = 0; i < vTypes.length; i++ ) {
            vTypes[i] = (Integer) types.get(i);
            vValues[i] = (Integer) values.get(i);
        }

        try {
            TimeSupportManager timeSupportManager = TimeSupportLocator.getManager();
            return timeSupportManager.createAbsoluteInstant(vTypes, vValues);
        } catch (AbsoluteInstantTypeNotRegisteredException e) {
            e.printStackTrace();
            return null;
        }

    }

    public void resetLowertTime() {

    }

    public void resetUpperTime() {

    }

    public void resetValues() {
        getTemporalDateText().setInstant(initTime);
    }

    public void focusGained(FocusEvent e) {

    }

    public void focusLost(FocusEvent e) {
        if( listener != null ) {
            listener.timeChanged(new TimeEvent(this));
        }
    }

    public void actionPerformed(ActionEvent e) {
        if( listener != null ) {
            listener.timeChanged(new TimeEvent(this));
        }

    }

    private void initComponents() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        add(getLabel());
        getTemporalDateText().setListener(this);
        add(getTemporalDateText());
    }

    private JTemporalDateText getTemporalDateText() {
        if( jDateText == null ) {
            jDateText = new JTemporalDateText();
        }
        return jDateText;
    }

    private JLabel getLabel() {
        if( lblLabel == null ) {
            lblLabel = new JLabel();
        }
        return lblLabel;
    }

}
