package org.gvsig.timesupport.swing.impl.panel.animation;

import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;


public class AnimationBarPanelImplLayout extends JPanel
{
   JTextField textWindowTime = new JTextField();
   JSlider sliderSpeed = new JSlider();
   JButton buttonPlay = new JButton();
   JButton buttonStop = new JButton();
   JButton buttonPause = new JButton();
   JLabel labelMinInterval = new JLabel();
   JLabel labelMaxInterval = new JLabel();
   JLabel timeStep = new JLabel();
   JTextField textTimeStep = new JTextField();
   JLabel speedOfProgress = new JLabel();
   JTextField textTimeSpeed = new JTextField();

   /**
    * Default constructor
    */
   public AnimationBarPanelImplLayout()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE","CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      textWindowTime.setBackground(new Color(236,233,216));
      textWindowTime.setEditable(false);
      textWindowTime.setName("textWindowTime");
      textWindowTime.setOpaque(false);
      textWindowTime.setScrollOffset(1);
      textWindowTime.setSelectionEnd(50);
      textWindowTime.setSelectionStart(50);
      textWindowTime.setText("04/08/2017 23:11:33:626 to 04/08/2017 23:11:33:626");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      textWindowTime.setBorder(emptyborder1);
      jpanel1.add(textWindowTime,new CellConstraints(2,1,1,1,CellConstraints.CENTER,CellConstraints.DEFAULT));

      sliderSpeed.setMajorTickSpacing(20);
      sliderSpeed.setName("sliderSpeed");
      jpanel1.add(sliderSpeed,cc.xy(2,2));

      jpanel1.add(createPanel1(),cc.xy(2,3));
      jpanel1.add(createPanel2(),cc.xy(2,5));
      addFillComponents(jpanel1,new int[]{ 1,3 },new int[]{ 1,2,3,4,5 });
      return jpanel1;
   }

   public JPanel createPanel1()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0)","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      buttonPlay.setActionCommand("JButton");
      buttonPlay.setName("buttonPlay");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      buttonPlay.setBorder(emptyborder1);
      jpanel1.add(buttonPlay,cc.xy(3,1));

      buttonStop.setActionCommand("JButton");
      buttonStop.setName("buttonStop");
      EmptyBorder emptyborder2 = new EmptyBorder(2,2,2,2);
      buttonStop.setBorder(emptyborder2);
      jpanel1.add(buttonStop,cc.xy(5,1));

      buttonPause.setActionCommand("JButton");
      buttonPause.setName("buttonPause");
      EmptyBorder emptyborder3 = new EmptyBorder(2,2,2,2);
      buttonPause.setBorder(emptyborder3);
      jpanel1.add(buttonPause,cc.xy(7,1));

      labelMinInterval.setName("labelMinInterval");
      labelMinInterval.setText("04/08/2017 23:11:33:626");
      jpanel1.add(labelMinInterval,cc.xy(1,1));

      labelMaxInterval.setName("labelMaxInterval");
      labelMaxInterval.setText("04/08/2017 23:11:33:626");
      jpanel1.add(labelMaxInterval,new CellConstraints(9,1,1,1,CellConstraints.RIGHT,CellConstraints.DEFAULT));

      addFillComponents(jpanel1,new int[]{ 2,4,6,8 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel2()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0)","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      timeStep.setName("timeStep");
      timeStep.setText("Paso del tiempo");
      jpanel1.add(timeStep,cc.xy(1,1));

      textTimeStep.setBackground(new Color(236,233,216));
      textTimeStep.setEditable(false);
      textTimeStep.setName("textTimeStep");
      textTimeStep.setOpaque(false);
      textTimeStep.setSelectionEnd(5);
      textTimeStep.setSelectionStart(5);
      textTimeStep.setText("1 dia");
      EmptyBorder emptyborder1 = new EmptyBorder(1,1,1,1);
      textTimeStep.setBorder(emptyborder1);
      jpanel1.add(textTimeStep,cc.xy(3,1));

      speedOfProgress.setName("speedOfProgress");
      speedOfProgress.setText("Velocidad de progreso");
      jpanel1.add(speedOfProgress,cc.xy(5,1));

      textTimeSpeed.setBackground(new Color(236,233,216));
      textTimeSpeed.setEditable(false);
      textTimeSpeed.setName("textTimeSpeed");
      textTimeSpeed.setOpaque(false);
      textTimeSpeed.setSelectionEnd(4);
      textTimeSpeed.setSelectionStart(4);
      textTimeSpeed.setText("1000");
      EmptyBorder emptyborder2 = new EmptyBorder(1,1,1,1);
      textTimeSpeed.setBorder(emptyborder2);
      jpanel1.add(textTimeSpeed,cc.xy(7,1));

      addFillComponents(jpanel1,new int[]{ 2,4,6 },new int[0]);
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
