/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
/*
 * AUTHORS (In addition to CIT):
 * 2010 {Prodevelop}   {Task}
 */

package org.gvsig.gui.beans.wizard.panel;

import java.awt.BorderLayout;
import javax.swing.BorderFactory;

import javax.swing.JOptionPane;

import jwizardcomponent.JWizardPanel;
import org.gvsig.gui.beans.wizard.WizardPanelWithLogo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * This class is a wizard panel that displays the panel returned by the
 * {@link OptionPanel#getJPanel()} method int the top of the wizard. It also
 * call the other methods when the user clicks a button.
 * <p>
 * 
 * @author <a href="mailto:jpiera@gvsig.org">Jorge Piera Llodr&aacute;</a>
 */
public class OptionPanelContainer extends JWizardPanel {

    private static final long serialVersionUID = 3947658150325230122L;
    private static final Logger LOG = LoggerFactory.getLogger(OptionPanelContainer.class);

    private OptionPanel optionPanel = null;
    private WizardPanelWithLogo wizardPanel;
    
    public OptionPanelContainer(WizardPanelWithLogo wizardPanel,
        OptionPanel optionPanel) {
        super(wizardPanel.getWizardComponents());
        this.optionPanel = optionPanel;
        this.wizardPanel = wizardPanel;
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createTitledBorder(
                BorderFactory.createEmptyBorder(20, 5, 5, 5),
                optionPanel.getPanelTitle()
            )
        );
//        setBorder(BorderFactory.createCompoundBorder(
//                BorderFactory.createTitledBorder(optionPanel.getPanelTitle()), 
//                BorderFactory.createEmptyBorder(5, 5, 5, 5)
//            )
//        );
        add(optionPanel.getJPanel(), BorderLayout.CENTER);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jwizardcomponent.JWizardPanel#back()
     */
    @Override
    public void back() {
        this.wizardPanel.setDirection(WizardPanelWithLogo.ACTION_PREVIOUS);
        LOG.debug("WizarPanel '"+this.optionPanel.getPanelTitle()+"' back.");
        optionPanel.lastPanel();
        super.back();
    }

    /*
     * (non-Javadoc)
     * 
     * @see jwizardcomponent.JWizardPanel#next()
     */
    @Override
    public void next() {
        try {
            this.wizardPanel.setDirection(WizardPanelWithLogo.ACTION_NEXT);
            LOG.debug("WizarPanel '"+this.optionPanel.getPanelTitle()+"' next.");
            optionPanel.nextPanel();
            super.next();
        } catch (NotContinueWizardException e) {
            // this is not an error and not need to raise a error or
            // warning in the log.
            LOG.info("It is not possible to continue with the wizard. "+e.displayMessage());
            if (e.displayMessage()) {
                JOptionPane.showMessageDialog(e.getComponent(), e
                    .getLocalizedMessageStack());
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see jwizardcomponent.JWizardPanel#update()
     */
    @Override
    public void update() {
        LOG.debug("WizarPanel '"+this.optionPanel.getPanelTitle()+"' update, direction "+this.wizardPanel.getDirection()+".");
        optionPanel.updatePanel();
        super.update();
    }
}
