package org.gvsig.webbrowser.jfx;

import com.jeta.forms.components.image.ImageComponent;
import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;


public class JFXWebBrowserPanelView extends JPanel
{
   JTextField txtURL = new JTextField();
   JPanel containerBrowser = new JPanel();
   JTextField txtSearch = new JTextField();
   JLabel lblStatus = new JLabel();
   ImageComponent imgSearch = new ImageComponent();
   JButton btnSearchNext = new JButton();
   JButton btnSearchPrevious = new JButton();
   JButton btnPrevious = new JButton();
   JButton btnNext = new JButton();
   JButton btnRefresh = new JButton();
   JButton btnConfig = new JButton();

   /**
    * Default constructor
    */
   public JFXWebBrowserPanelView()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE","CENTER:4DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:4DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtURL.setName("txtURL");
      jpanel1.add(txtURL,cc.xy(8,2));

      containerBrowser.setName("containerBrowser");
      jpanel1.add(containerBrowser,new CellConstraints(2,4,9,1,CellConstraints.FILL,CellConstraints.FILL));

      jpanel1.add(createPanel1(),cc.xywh(2,6,9,1));
      btnPrevious.setName("btnPrevious");
      EmptyBorder emptyborder1 = new EmptyBorder(0,0,0,0);
      btnPrevious.setBorder(emptyborder1);
      jpanel1.add(btnPrevious,new CellConstraints(2,2,1,1,CellConstraints.FILL,CellConstraints.FILL));

      btnNext.setName("btnNext");
      EmptyBorder emptyborder2 = new EmptyBorder(0,0,0,0);
      btnNext.setBorder(emptyborder2);
      jpanel1.add(btnNext,new CellConstraints(4,2,1,1,CellConstraints.FILL,CellConstraints.FILL));

      btnRefresh.setEnabled(false);
      btnRefresh.setName("btnRefresh");
      EmptyBorder emptyborder3 = new EmptyBorder(0,0,0,0);
      btnRefresh.setBorder(emptyborder3);
      jpanel1.add(btnRefresh,new CellConstraints(6,2,1,1,CellConstraints.FILL,CellConstraints.FILL));

      btnConfig.setEnabled(false);
      btnConfig.setName("btnConfig");
      EmptyBorder emptyborder4 = new EmptyBorder(0,0,0,0);
      btnConfig.setBorder(emptyborder4);
      jpanel1.add(btnConfig,new CellConstraints(10,2,1,1,CellConstraints.FILL,CellConstraints.FILL));

      addFillComponents(jpanel1,new int[]{ 1,2,3,4,5,6,7,8,9,10,11 },new int[]{ 1,2,3,4,5,6,7 });
      return jpanel1;
   }

   public JPanel createPanel1()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:MIN(12DLU;DEFAULT):GROW(0.3),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(0.7)","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtSearch.setName("txtSearch");
      jpanel1.add(txtSearch,cc.xy(3,1));

      lblStatus.setName("lblStatus");
      LineBorder lineborder1 = new LineBorder(new Color(189,190,176),1,false);
      lblStatus.setBorder(lineborder1);
      jpanel1.add(lblStatus,new CellConstraints(9,1,1,1,CellConstraints.FILL,CellConstraints.FILL));

      imgSearch.setName("imgSearch");
      jpanel1.add(imgSearch,cc.xy(1,1));

      btnSearchNext.setName("btnSearchNext");
      EmptyBorder emptyborder1 = new EmptyBorder(0,0,0,0);
      btnSearchNext.setBorder(emptyborder1);
      jpanel1.add(btnSearchNext,cc.xy(5,1));

      btnSearchPrevious.setName("btnSearchPrevious");
      EmptyBorder emptyborder2 = new EmptyBorder(0,0,0,0);
      btnSearchPrevious.setBorder(emptyborder2);
      jpanel1.add(btnSearchPrevious,cc.xy(7,1));

      addFillComponents(jpanel1,new int[]{ 2,4,6,8 },new int[0]);
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
