package org.gvsig.app.extension;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.table.TableDocument;
import org.gvsig.app.project.documents.table.TableManager;
import org.gvsig.app.project.documents.table.gui.FeatureTableDocumentPanel;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.searchpanel.FeatureStoreSearchPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;

/**
 *
 * @author jjdelcerro
 */
public class SearchByAttributesInTableExtension extends Extension {

    @Override
    public void initialize() {
        IconThemeHelper.registerIcon("action", "search-by-attributes-table", this);                
    }

    @Override
    public void execute(String actionCommand) {
        ApplicationManager application = ApplicationLocator.getManager();

        if ("search-by-attributes-table".equals(actionCommand)) {
            DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();
            WindowManager winManager = ToolsSwingLocator.getWindowManager();
            
            TableDocument tabledoc = (TableDocument) application.getActiveDocument(TableManager.TYPENAME);
            if( tabledoc == null ) {
                return;
            }
            I18nManager i18n = ToolsLocator.getI18nManager();
            final FeatureTableDocumentPanel tablePanel = (FeatureTableDocumentPanel) tabledoc.getMainComponent();
            final FeatureStore featureStore = tablePanel.getFeatureStore();
            String title = i18n.getTranslation("search")+ ": " + featureStore.getName();
            FeatureStoreSearchPanel panel = dataSwingManager.createFeatureStoreSearchPanel(featureStore);
            winManager.showWindow(
                    panel.asJComponent(), 
                    title, 
                    WindowManager.MODE.WINDOW
            );
            
        }
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean isVisible() {
        ApplicationManager application = ApplicationLocator.getManager();

        TableDocument tabledoc = (TableDocument) application.getActiveDocument(TableManager.TYPENAME);
        if( tabledoc == null ) {
            return false;
        }
        return true;
    }
}
