/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.app.extension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.AbstractAction;
import javax.swing.Action;
import static javax.swing.Action.ACTION_COMMAND_KEY;
import static javax.swing.Action.NAME;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.table.TableDocument;
import org.gvsig.app.project.documents.table.TableManager;
import org.gvsig.app.project.documents.table.gui.FeatureTableDocumentPanel;
import org.gvsig.configurableactions.ConfigurableActionsMamager;
import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.fmap.dal.feature.EditableFeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.EditableFeatureType;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.featuretype.FeatureTypeAttributePanel;
import org.gvsig.fmap.dal.swing.featuretype.FeatureTypePanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;
import org.gvsig.tools.util.ToolsUtilLocator;

/**
 * Extensin que abre la ventana para cambiar la configuracin de la estructura
 * de la tabla.
 * 
 */
@SuppressWarnings("UseSpecificCatch")
public class TableEditAttributes extends Extension {

    @Override
    public void initialize() {
        IconThemeHelper.registerIcon("action", "table-column-manager", this);
        IconThemeHelper.registerIcon("action", "table-add-column", this);
    }

    @Override
    public void postInitialize() {
        ConfigurableActionsMamager cfgActionsManager = ToolsUtilLocator.getConfigurableActionsMamager();
        cfgActionsManager.addConfigurableAction(FeatureTypePanel.CONFIGURABLE_PANEL_ID, new ShowAsTableAction());
    }

    @Override
    public boolean canQueryByAction() {
        return true;
    }

    @Override
    public void execute(String s) {
        final I18nManager i18n = ToolsLocator.getI18nManager();
        final WindowManager_v2 winManager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
        final DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();
        final ApplicationManager application = ApplicationLocator.getManager();

        TableDocument tabledoc = (TableDocument) application.getActiveDocument(TableManager.TYPENAME);
        if( tabledoc == null ) {
            return;
        }
//        final FeatureStore featureStore = tabledoc.getStore();
        final FeatureTableDocumentPanel tablePanel = (FeatureTableDocumentPanel) tabledoc.getMainComponent();
        final FeatureStore featureStore = tablePanel.getFeatureStore();

    	if( "table-column-manager".equalsIgnoreCase(s)) {
            try {
                final FeatureType featureType = featureStore.getDefaultFeatureType();
                final EditableFeatureType editableFeatureType = featureType.getEditable();
                final FeatureTypePanel panel = dataSwingManager.createFeatureTypePanel();
                panel.put(editableFeatureType);
                panel.setMode(
                        featureStore.isEditing()? 
                                FeatureTypePanel.MODE_EDIT_ALL:
                                FeatureTypePanel.MODE_EDIT_ONLY_METADATA
                );
                final Dialog dialog = winManager.createDialog(
                        panel.asJComponent(),
                        i18n.getTranslation("_Column_manager") + " - " + tabledoc.getName(),
                        null,
                       WindowManager_v2.BUTTONS_OK_CANCEL
                );
                panel.addChangeListener(new ChangeListener() {
                    @Override
                    public void stateChanged(ChangeEvent e) {
                        if( panel.isModifyingAField() ) {
                            dialog.setButtonEnabled(WindowManager_v2.BUTTON_OK, false);
                        } else {
                            dialog.setButtonEnabled(WindowManager_v2.BUTTON_OK, true);
                        }
                    }
                });
                dialog.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        if( dialog.getAction()!= WindowManager_v2.BUTTON_OK ) {
                            return;
                        }
                        try {
                            panel.fetch(editableFeatureType);
                            featureStore.update(editableFeatureType);
                        } catch (Exception ex) {
                            logger.warn("Problems updating featuretype.", ex);
                        }
                    }
                });
                dialog.show(WindowManager.MODE.WINDOW);
            } catch (Exception ex) {
                logger.warn("Problems show FeatureTypePanel.", ex);
            }

        } else if ("table-add-column".equals(s)) {
            try {
                final FeatureType featureType = featureStore.getDefaultFeatureType();
                final FeatureTypeAttributePanel panel = dataSwingManager.createFeatureTypeAttributePanel();
                panel.clean();
                panel.setNameAsNew(featureType);
                final Dialog dialog = winManager.createDialog(
                        panel.asJComponent(),
                        i18n.getTranslation("_New_field") + " - " + tabledoc.getName(),
                        null,
                        WindowManager_v2.BUTTONS_OK_CANCEL
                );
                dialog.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        if( dialog.getAction()!=WindowManager_v2.BUTTONS_OK ) {
                            return;
                        }
                        if( !featureStore.isEditing() ) {
                            return;
                        }
                        try {
                            EditableFeatureType editableFeatureType = featureType.getEditable();
                            EditableFeatureAttributeDescriptor descriptor = editableFeatureType.add(
                                    panel.getName(),
                                    DataTypes.STRING
                            );
                            panel.fetch(descriptor);
                            featureStore.update(editableFeatureType);
                        } catch (Exception ex) {
                            logger.warn("Problems updating featuretype.", ex);
                        }
                    }
                });
                dialog.show(WindowManager.MODE.WINDOW);
            } catch (Exception ex) {
                logger.warn("Problems show FeatureTypeAttributePanel.", ex);
            }
    		
    	}
    }

    public static class ShowAsTableAction extends AbstractAction {

        @SuppressWarnings("OverridableMethodCallInConstructor")
        public ShowAsTableAction() {
            I18nManager i18n = ToolsLocator.getI18nManager();

            this.putValue(NAME, i18n.getTranslation("_Show_as_table"));
            this.putValue(ACTION_COMMAND_KEY, "ShowAsTable");
        this.putValue(
                Action.SMALL_ICON, 
                ToolsSwingLocator.getIconThemeManager().getCurrent().get("layer-show-attributes-table")
        );
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            FeatureTypePanel panel = (FeatureTypePanel) ae.getSource();
            EditableFeatureType featureType = panel.fetch(null);
            FeatureStore store = featureType.getAsFeatureStore();

            ProjectManager projectManager = ApplicationLocator.getManager().getProjectManager();
            Project project = projectManager.getCurrentProject();
            TableDocument tableDoc = (TableDocument) project.getDocument(store.getName(),TableManager.TYPENAME);
            if( tableDoc == null ) {
                tableDoc = (TableDocument) projectManager.createDocument(TableManager.TYPENAME);
                tableDoc.setStore(store);
                tableDoc.setName(store.getName());
                project.addDocument(tableDoc);
            }
            ApplicationLocator.getManager().getUIManager().addWindow(tableDoc.getMainWindow());
            }
    }

    @Override
    public boolean isEnabled() {
        ApplicationManager application = ApplicationLocator.getManager();

        TableDocument tabledoc = (TableDocument) application.getActiveDocument(TableManager.TYPENAME);
        if( tabledoc == null ) {
            return false;
        }
        return tabledoc.getStore().isEditing();
    }

    @Override
    public boolean isVisible() {
        ApplicationManager application = ApplicationLocator.getManager();

        TableDocument tabledoc = (TableDocument) application.getActiveDocument(TableManager.TYPENAME);
        return tabledoc!=null;
    }

    @Override
    public boolean isVisible(String action) {
        return this.isVisible();
    }

    @Override
    public boolean isEnabled(String action) {
        ApplicationManager application = ApplicationLocator.getManager();

        TableDocument tabledoc = (TableDocument) application.getActiveDocument(TableManager.TYPENAME);
        if( tabledoc == null ) {
            return false;
        }
    	if( "table-column-manager".equalsIgnoreCase(action)) {
            return true;
            
        } else if ("table-add-column".equals(action)) {
            return tabledoc.getStore().isEditing();
            
        }
        return false;
    }
    
}
