
package org.gvsig.app.extension;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.Document;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.SpatialIndex;
import org.gvsig.fmap.geom.SpatialIndexFactory;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.tools.dataTypes.DataTypes;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.exception.BaseException;
import org.gvsig.tools.service.Service;
import org.gvsig.tools.service.ServiceException;
import org.gvsig.tools.util.HasAFile;
import org.gvsig.tools.visitor.VisitCanceledException;
import org.gvsig.tools.visitor.Visitor;


public class CreateSpatialIndexExtension extends Extension {

    @Override
    public void initialize() {
    }

    @Override
    public void execute(String actionCommand) {
        if( "layer-create-spatialindex".equalsIgnoreCase(actionCommand) ) {
            ApplicationManager application = ApplicationLocator.getManager();
            ViewDocument view = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
            if( view ==null ) {
                return;
            }
            FLayer[] layers = view.getMapContext().getLayers().getActives();
            if( !ArrayUtils.isEmpty(layers) ) {
                for( FLayer layer : layers ) {
                    if( layer instanceof FLyrVect ) {
                        FeatureStore store = ((FLyrVect) layer).getFeatureStore();
                        DataStoreParameters params = store.getParameters();
                        if( params instanceof HasAFile) {
                            createSpatialIndex(store);
                        }
                    }
                }
            }
        }
    }

    @Override
    public boolean isEnabled() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument view = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( view ==null ) {
            return false;
        }
        FLayer[] layers = view.getMapContext().getLayers().getActives();
        if( !ArrayUtils.isEmpty(layers) ) {
            for( FLayer layer : layers ) {
                if( layer instanceof FLyrVect ) {
                    DataStoreParameters params = ((FLyrVect) layer).getFeatureStore().getParameters();
                    if( params instanceof HasAFile) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public boolean isVisible() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument view = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( view ==null ) {
            return false;
        }
        return view.getMapContext().hasVectorLayers();
    }

    private void createSpatialIndex(FeatureStore store) {

        try {
            HasAFile storeParams = (HasAFile) store.getParameters();
            File f = new File(FilenameUtils.removeExtension(storeParams.getFile().getAbsolutePath()) + ".jqt");

            GeometryManager geomManager = GeometryLocator.getGeometryManager();
            DynObject indexParams = geomManager.createServiceParameters(geomManager.SPATIALINDEX_DEFAULT_QUADTREE);
            ((HasAFile)indexParams).setFile(f);
            final SpatialIndex index = store.wrapSpatialIndex(geomManager.createSpatialIndex(geomManager.SPATIALINDEX_DEFAULT_QUADTREE, indexParams));

            store.accept(new Visitor() {
                @Override
                public void visit(Object o) throws VisitCanceledException, BaseException {
                    Feature f = (Feature) o;
                    Geometry g = f.getDefaultGeometry();
                    index.insert(g,f);
                }
            });

        } catch (Exception ex) {
        }


    }

}
