/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.app.extension;

import java.awt.Dimension;
import java.util.Locale;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.LocaleManager;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.PluginsLocator;
import org.gvsig.andami.PluginsManager;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.webbrowser.WebBrowserFactory;
import org.gvsig.webbrowser.WebBrowserPanel;


public class HelpExtension extends Extension {

    @Override
    public void initialize() {
        IconThemeHelper.registerIcon("action", "help-contents", this);
        IconThemeHelper.registerIcon("action", "help-search", this);
    }

    @Override
    public void execute(String actionCommand) {
        WindowManager winmanager = ToolsSwingLocator.getWindowManager();
        I18nManager i18n = ToolsLocator.getI18nManager();
        WebBrowserPanel webbrowser;
        
        LocaleManager localeManager = PluginsLocator.getLocaleManager();
        PluginsManager pluginsManager = PluginsLocator.getManager();
        PluginServices plugin = pluginsManager.getPlugin(this);
        String baseHelpURL = (String) plugin.getPluginProperties().getDynValue("baseHelpURL");        
        if( !baseHelpURL.endsWith("/") ) {
            baseHelpURL = baseHelpURL + "/";
        }
        
        Locale locale = localeManager.getCurrentLocale();
        String localeid = locale.getLanguage();
        Locale[] localeAlternatives = localeManager.getLocaleAlternatives(locale);
        if( !ArrayUtils.isEmpty(localeAlternatives) ) {
            localeid = localeAlternatives[0].getLanguage();
        }
        switch( StringUtils.defaultIfEmpty(actionCommand, "").toLowerCase() ) {
        case "help-contents":
            webbrowser = WebBrowserFactory.createWebBrowserPanel();
            webbrowser.asJComponent().setPreferredSize(new Dimension(900,500));
            webbrowser.setPage(baseHelpURL + localeid +"/build/html/user_manual/2.3/index.html");
            winmanager.showWindow( 
                webbrowser.asJComponent(), 
                i18n.getTranslation("_Help_contents"), 
                WindowManager.MODE.WINDOW
            );
            break;
        case "help-search":
            webbrowser = WebBrowserFactory.createWebBrowserPanel();
            webbrowser.asJComponent().setPreferredSize(new Dimension(900,500));
            webbrowser.setPage(baseHelpURL + localeid +"/build/html/gvsigsearch_all.html");
            winmanager.showWindow( 
                webbrowser.asJComponent(), 
                i18n.getTranslation("_Help_contents"), 
                WindowManager.MODE.WINDOW
            );
            break;
        }
    }
    
    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean isVisible() {
        return true;
    }
    
}
