package org.gvsig.app.extension;

import java.awt.GridBagConstraints;
import java.beans.PropertyVetoException;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.cresques.cts.IProjection;
import org.gvsig.andami.Arguments;
import org.gvsig.andami.PluginsLocator;
import org.gvsig.andami.PluginsManager;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.DocumentManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.app.project.documents.view.gui.IView;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataServerExplorer;
import org.gvsig.fmap.dal.DataServerExplorerParameters;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemFileFilter;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemServerExplorer;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.tools.dynobject.DynField;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OpenLayerOnStart implements Runnable {

    private static final Logger logger = LoggerFactory.getLogger(OpenLayerOnStart.class);

    @Override
    public void run() {
        PluginsManager PluginManager = PluginsLocator.getManager();
        DataManager dataManager = DALLocator.getDataManager();
        ApplicationManager application = ApplicationLocator.getManager();
        ProjectManager projectManager = application.getProjectManager();
        DocumentManager viewManager = projectManager.getDocumentManager(ViewManager.TYPENAME);
        MapContextManager mapContextManager = MapContextLocator.getMapContextManager();
        Project project = application.getProjectManager().getCurrentProject();

        Arguments args = PluginManager.getArguments();
        List<String> params = new ArrayList<>(args.parameters());
        params.remove(0);
        params.remove(0);
        for( String param : params ) {
            if( !StringUtils.isEmpty(param) && param.endsWith(Project.FILE_EXTENSION) ) {
                try {
                    File f = new File(param);
                    application.getUIManager().closeAllWindows();
                    project = projectManager.createProject();
                    project.loadState(f);
                    projectManager.setCurrentProject(project);
                } catch (Throwable th) {
                    logger.warn("Can't load project '" + param + "'.", th);
                }
                break;
            }
        }

        ViewDocument viewDoc = null;
        FilesystemServerExplorer explorer = null;
        FilesystemFileFilter filter = null;
        IProjection proj = null;
        for( String param : params ) {
            try {
                if( FilenameUtils.isExtension(param, "gvproj") ) {
                    continue;
                }
                if( explorer == null ) {
                    DataServerExplorerParameters explorerparams = dataManager.createServerExplorerParameters(FilesystemServerExplorer.NAME);
                    explorer = (FilesystemServerExplorer) dataManager.openServerExplorer(FilesystemServerExplorer.NAME, explorerparams);
                    filter = explorer.getFilter(DataServerExplorer.MODE_GEOMETRY | DataServerExplorer.MODE_RASTER, "All formats");
                }

                File f = new File(param);
                if( filter.accept(f) ) {
                    if( viewDoc == null ) {
                        viewDoc = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
                        if( viewDoc==null || !viewDoc.getMapContext().getLayers().isEmpty() ) {
                            application.getUIManager().closeAllWindows();
                            viewDoc = (ViewDocument) viewManager.createDocument();
                            project.addDocument(viewDoc);
                        }
                        proj = viewDoc.getMapContext().getProjection();
                        IView viewWindow = (IView) viewManager.getMainWindow(viewDoc);
                        application.getUIManager().addWindow(viewWindow, GridBagConstraints.CENTER);
                        try {
                            application.getUIManager().setMaximum((IWindow) viewWindow, true);
                        } catch (PropertyVetoException e) {

                        }
                    }
                    DataStoreParameters storeParams = explorer.createStoreParameters(f);
                    // Buscamos por el parametro de la proyeccion
                    // que sean obligatorios y estn a null
                    // y le ponemos la proyeccion de la vista
                    DynField[] fields = storeParams.getDynClass().getDynFields();
                    for (DynField field : fields) {
                        if (field.getType() == DataTypes.CRS && field.isMandatory()) {
                            if (storeParams.getDynValue(field.getName()) == null) {
                                storeParams.setDynValue(field.getName(), proj);
                            }
                        }
                    }                    
                    DataStore store = dataManager.openStore(storeParams.getDataStoreName(), storeParams);
                    FLayer layer = mapContextManager.createLayer(store.getName(), store);
                    viewDoc.getMapContext().getLayers().add(layer);
                }
            } catch (Throwable th) {
                logger.warn("Can't load layer '" + param + "'.", th);
            }
        }

    }
}
