package org.gvsig.app.extension;

import javax.swing.JOptionPane;
import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.gui.IView;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.searchpanel.FeatureStoreSearchPanel;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;

/**
 *
 * @author jjdelcerro
 */
public class SearchByAttributesExtension extends Extension {

    @Override
    public void initialize() {
        IconThemeHelper.registerIcon("action", "search-by-attributes-layer", this);                
    }

    @Override
    public void execute(String actionCommand) {
        ApplicationManager application = ApplicationLocator.getManager();

        IView view = (IView) application.getActiveComponent(ViewDocument.class);
        if (view == null) {
            return;
        }
        if ("search-by-attributes-layer".equals(actionCommand)) {
            I18nManager i18n = ToolsLocator.getI18nManager();
            DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();
            WindowManager winManager = ToolsSwingLocator.getWindowManager();
            
            ViewDocument document = view.getViewDocument();

            FLayer layer = document.getMapContext().getLayers().getActives()[0];
            String title = i18n.getTranslation("search")+ ": " + layer.getName();
            FeatureStore featureStore = ((FLyrVect) layer).getFeatureStore();
            
            try {
                FeatureType type = featureStore.getDefaultFeatureType();
                if( ! type.supportReferences() ) {
                    ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
                    dialogs.messageDialog(
                           i18n.getTranslation("_The_table_has_no_primary_key_or_OID") +"\n" +
                                   i18n.getTranslation("_Many_features_selection_deletion_modification_will_not_be_available_as_they_require_it_for_proper_operation"),
                           null, 
                           i18n.getTranslation("_Warning"),
                            JOptionPane.WARNING_MESSAGE, 
                           "TableDoNotSupportReferences"
                   );
                }
            } catch (Exception ex) {
            }
            
            FeatureStoreSearchPanel panel = dataSwingManager.createFeatureStoreSearchPanel(featureStore);
            winManager.showWindow(
                    panel.asJComponent(), 
                    title, 
                    WindowManager.MODE.WINDOW
            );
            
        }
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean isVisible() {
        ApplicationManager application = ApplicationLocator.getManager();

        IView view = (IView) application.getActiveComponent(ViewDocument.class);
        if (view == null) {
            return false;
        }
        ViewDocument document = view.getViewDocument();
        return document.getMapContext().hasActiveVectorLayers();
    }
}
