package org.gvsig.app.gui.preferencespage;

import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import org.gvsig.andami.preferences.AbstractPreferencePage;


public abstract class ViewPageView extends AbstractPreferencePage
{
   JLabel lblDefaultProjection = new JLabel();
   JButton btnChangeProjection = new JButton();
   JTextField txtDefaultProjection = new JTextField();
   JLabel lblZoomInFactor = new JLabel();
   JCheckBox chkInvisibleNewLayers = new JCheckBox();
   JCheckBox chkKeepScaleOnResizing = new JCheckBox();
   JCheckBox chkOpenWithANewMaximizedView = new JCheckBox();
   JLabel lblZoomOutFactor = new JLabel();
   JLabel lblDefaultViewBackColor = new JLabel();
   JLabel lblDefaultSelectionColor = new JLabel();
   JLabel lblMapUnits = new JLabel();
   JLabel lblDistanceUnits = new JLabel();
   JLabel lblDistanceArea = new JLabel();
   JComboBox jCmbMapUnits = new JComboBox();
   JComboBox jCmbDistanceUnits = new JComboBox();
   JComboBox jCmbDistanceArea = new JComboBox();
   JButton btnDefaultViewBackColor = new JButton();
   JTextField txtDefaultViewBackColor = new JTextField();
   JTextField txtDefaultSelectionColor = new JTextField();
   JSlider sldDefaultSelectionColor = new JSlider();
   JButton btnDefaultSelectionColor = new JButton();
   JLabel lblAlpha = new JLabel();
   JSpinner txtZoomInFactor = new JSpinner();
   JSpinner txtZoomOutFactor = new JSpinner();
   JLabel lblToC = new JLabel();
   JCheckBox chkHideLegendOfNonVisibleLayers = new JCheckBox();

   /**
    * Default constructor
    */
   public ViewPageView()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:8DLU:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:GROW(1.0),FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE","CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:4DLU:NONE,CENTER:DEFAULT:NONE,CENTER:4DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:4DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:4DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,FILL:DEFAULT:GROW(1.0)");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblDefaultProjection.setName("lblDefaultProjection");
      lblDefaultProjection.setText("Default proyection");
      jpanel1.add(lblDefaultProjection,cc.xy(2,2));

      jpanel1.add(createPanel1(),cc.xy(4,2));
      lblZoomInFactor.setName("lblZoomInFactor");
      lblZoomInFactor.setText("Zoom in factor");
      jpanel1.add(lblZoomInFactor,cc.xy(2,6));

      jpanel1.add(createPanel2(),cc.xy(4,4));
      lblZoomOutFactor.setName("lblZoomOutFactor");
      lblZoomOutFactor.setText("Zoom out factor");
      jpanel1.add(lblZoomOutFactor,cc.xy(2,8));

      lblDefaultViewBackColor.setName("lblDefaultViewBackColor");
      lblDefaultViewBackColor.setText("Default view back color");
      jpanel1.add(lblDefaultViewBackColor,cc.xy(2,10));

      lblDefaultSelectionColor.setName("lblDefaultSelectionColor");
      lblDefaultSelectionColor.setText("Default selection color");
      jpanel1.add(lblDefaultSelectionColor,new CellConstraints(2,12,1,1,CellConstraints.DEFAULT,CellConstraints.CENTER));

      lblMapUnits.setName("lblMapUnits");
      lblMapUnits.setText("Map units");
      jpanel1.add(lblMapUnits,cc.xy(2,14));

      lblDistanceUnits.setName("lblDistanceUnits");
      lblDistanceUnits.setText("Measuring units");
      jpanel1.add(lblDistanceUnits,cc.xy(2,16));

      lblDistanceArea.setName("lblDistanceArea");
      lblDistanceArea.setText("Area units");
      jpanel1.add(lblDistanceArea,cc.xy(2,18));

      jCmbMapUnits.setName("jCmbMapUnits");
      jpanel1.add(jCmbMapUnits,cc.xy(4,14));

      jCmbDistanceUnits.setName("jCmbDistanceUnits");
      jpanel1.add(jCmbDistanceUnits,cc.xy(4,16));

      jCmbDistanceArea.setName("jCmbDistanceArea");
      jpanel1.add(jCmbDistanceArea,cc.xy(4,18));

      jpanel1.add(createPanel3(),cc.xy(4,10));
      jpanel1.add(createPanel4(),cc.xy(4,12));
      txtZoomInFactor.setName("txtZoomInFactor");
      jpanel1.add(txtZoomInFactor,cc.xy(4,6));

      txtZoomOutFactor.setName("txtZoomOutFactor");
      jpanel1.add(txtZoomOutFactor,cc.xy(4,8));

      lblToC.setName("lblToC");
      lblToC.setText("ToC");
      jpanel1.add(lblToC,new CellConstraints(2,20,1,1,CellConstraints.DEFAULT,CellConstraints.TOP));

      jpanel1.add(createPanel5(),cc.xy(4,20));
      addFillComponents(jpanel1,new int[]{ 1,2,3,4,5,6,7,8 },new int[]{ 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21 });
      return jpanel1;
   }

   public JPanel createPanel1()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      btnChangeProjection.setActionCommand("Change");
      btnChangeProjection.setName("btnChangeProjection");
      btnChangeProjection.setText("Change");
      jpanel1.add(btnChangeProjection,cc.xy(3,1));

      txtDefaultProjection.setBackground(new Color(236,233,216));
      txtDefaultProjection.setEditable(false);
      txtDefaultProjection.setName("txtDefaultProjection");
      jpanel1.add(txtDefaultProjection,cc.xy(1,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel2()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0)","CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      chkInvisibleNewLayers.setActionCommand("Add new layers in invisible mode");
      chkInvisibleNewLayers.setName("chkInvisibleNewLayers");
      chkInvisibleNewLayers.setText("Add new layers in invisible mode");
      jpanel1.add(chkInvisibleNewLayers,cc.xy(1,3));

      chkKeepScaleOnResizing.setActionCommand("Keep scale on resizing");
      chkKeepScaleOnResizing.setName("chkKeepScaleOnResizing");
      chkKeepScaleOnResizing.setText("Keep scale on resizing");
      jpanel1.add(chkKeepScaleOnResizing,cc.xy(1,5));

      chkOpenWithANewMaximizedView.setActionCommand("Open with a new maximized  view ");
      chkOpenWithANewMaximizedView.setName("chkOpenWithANewMaximizedView");
      chkOpenWithANewMaximizedView.setText("Open with a new maximized  view ");
      jpanel1.add(chkOpenWithANewMaximizedView,cc.xy(1,1));

      addFillComponents(jpanel1,new int[0],new int[]{ 2,4 });
      return jpanel1;
   }

   public JPanel createPanel3()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      btnDefaultViewBackColor.setActionCommand("...");
      btnDefaultViewBackColor.setName("btnDefaultViewBackColor");
      btnDefaultViewBackColor.setText("...");
      jpanel1.add(btnDefaultViewBackColor,cc.xy(3,1));

      txtDefaultViewBackColor.setEnabled(false);
      txtDefaultViewBackColor.setName("txtDefaultViewBackColor");
      jpanel1.add(txtDefaultViewBackColor,cc.xy(1,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel4()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,LEFT:50DLU:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtDefaultSelectionColor.setBackground(new Color(255,255,0));
      txtDefaultSelectionColor.setEnabled(false);
      txtDefaultSelectionColor.setName("txtDefaultSelectionColor");
      jpanel1.add(txtDefaultSelectionColor,cc.xy(1,1));

      sldDefaultSelectionColor.setMajorTickSpacing(10);
      sldDefaultSelectionColor.setMaximum(255);
      sldDefaultSelectionColor.setName("sldDefaultSelectionColor");
      sldDefaultSelectionColor.setToolTipText("Alpha");
      sldDefaultSelectionColor.setValue(255);
      jpanel1.add(sldDefaultSelectionColor,cc.xy(7,1));

      btnDefaultSelectionColor.setActionCommand("...");
      btnDefaultSelectionColor.setName("btnDefaultSelectionColor");
      btnDefaultSelectionColor.setText("...");
      jpanel1.add(btnDefaultSelectionColor,cc.xy(3,1));

      lblAlpha.setName("lblAlpha");
      lblAlpha.setText("Alpha");
      jpanel1.add(lblAlpha,cc.xy(5,1));

      addFillComponents(jpanel1,new int[]{ 2,4,6 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel5()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:12DLU:NONE,FILL:DEFAULT:GROW(1.0)","CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      chkHideLegendOfNonVisibleLayers.setActionCommand("Hide legend of non visible layers");
      chkHideLegendOfNonVisibleLayers.setName("chkHideLegendOfNonVisibleLayers");
      chkHideLegendOfNonVisibleLayers.setText("Hide legend of non visible layers");
      jpanel1.add(chkHideLegendOfNonVisibleLayers,cc.xywh(1,1,2,1));

      JLabel jlabel1 = new JLabel();
      jpanel1.add(jlabel1,cc.xy(1,2));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
