package org.gvsig.app.gui.preferencespage.dialogreminder;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.preferences.IPreference;
import org.gvsig.andami.preferences.StoreException;

import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.JListWithCheckbox;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.reminder.DialogReminder;
import org.gvsig.tools.swing.api.reminder.DialogReminderManager;


public class DialogReminderPage extends DialogReminderView implements IPreference {

    private static final long serialVersionUID = 3811078625224506453L;
    //
    // The class DialogReminderView can extend from AbstractPreferencePage, not JPanel,
    // if recreate it from the GUI builder change te class declaration to:
    //    public abstract class DialogReminderView extends AbstractPreferencePage

    private JListWithCheckbox lstCheckdsRemembers;

    public static String id = DialogReminderPage.class.getName();
    private ImageIcon icon;

    public DialogReminderPage() {
        //setParentID(GeneralPage.class.getName());
        initComponents();
    }

    private void initComponents() {
        icon = IconThemeHelper.getImageIcon("dialog-remember-preferences");
        ToolsSwingManager swingManager = ToolsSwingLocator.getToolsSwingManager();
        this.lstCheckdsRemembers = swingManager.createJListWithCheckbox(this.lstRemembers);
        final ListSelectionModel checkModel = this.lstCheckdsRemembers.getCheckedModel();
        checkModel.addListSelectionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent e) {
                if( e.getValueIsAdjusting() ) {
                    return;
                }
                int n = e.getFirstIndex();
                if( n<=lstCheckdsRemembers.getModel().getSize() ) {
                    return;
                }
                ListElement<DialogReminder> element = (ListElement) lstCheckdsRemembers.getModel().getElementAt(n);
                if( element==null ) {
                    return;
                }
                if( checkModel.isSelectedIndex(n) && !element.getValue().hasValue() ) {
                    checkModel.removeIndexInterval(n, n);
                }
            }
        });
        translateAll();
    }

    private void translateAll() {
        I18nManager i18n = ToolsLocator.getI18nManager();

        this.lblTitle.setText(i18n.getTranslation(this.lblTitle.getText()));
    }

    @Override
    public boolean isResizeable() {
        return true;
    }

    @Override
    public void initializeValues() {
        DialogReminderManager remembers = ToolsSwingLocator.getDialogReminderManager();

        DefaultListModel model = new DefaultListModel();
        ListSelectionModel checkModel = this.lstCheckdsRemembers.getCheckedModel();
        checkModel.clearSelection();
        int n=0;
        for(DialogReminder remember : remembers.reminders() ) {
            model.addElement(new ListElement<>(remember.getId(), remember));
            if( remember.hasValue() ) {
                checkModel.addSelectionInterval(n, n);
            }
            n++;
        }
        this.lstCheckdsRemembers.setModel(model);
    }

    @Override
    public void initializeDefaults() {
        DialogReminderManager remembers = ToolsSwingLocator.getDialogReminderManager();

        DefaultListModel model = new DefaultListModel();
        ListSelectionModel checkModel = this.lstCheckdsRemembers.getCheckedModel();
        for(DialogReminder remember : remembers.reminders() ) {
            model.addElement(new ListElement<>(remember.getId(), remember));
        }
        this.lstCheckdsRemembers.setModel(model);
        checkModel.clearSelection();
    }

    @Override
    public String getID() {
        return id;
    }

    @Override
    public String getTitle() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("_Remembered_dialogs");
    }

    @Override
    public JPanel getPanel() {
        return this;
    }


    @Override
    public void storeValues() throws StoreException {
        DialogReminderManager remembers = ToolsSwingLocator.getDialogReminderManager();

        ListSelectionModel checkModel = this.lstCheckdsRemembers.getCheckedModel();
        int n=0;
        for(DialogReminder remember : remembers.reminders() ) {
            if( ! checkModel.isSelectedIndex(n)  ) {
                remember.reset();
            }
            n++;
        }
    }

    @Override
    public ImageIcon getIcon() {
        return icon;
    }

    @Override
    public boolean isValueChanged() {
        DialogReminderManager remembers = ToolsSwingLocator.getDialogReminderManager();

        ListSelectionModel checkModel = this.lstCheckdsRemembers.getCheckedModel();
        int n=0;
        for(DialogReminder remember : remembers.reminders() ) {
            if( checkModel.isSelectedIndex(n) != remember.hasValue() ) {
                return true;
            }
            n++;
        }
        return false;
    }

    @Override
    public void setChangesApplied() {
        // Do nothing
    }


}
