/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.app.project.documents.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.ui.mdiManager.SingletonWindow;
import org.gvsig.andami.ui.mdiManager.WindowInfo;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.documents.gui.projectpanel.ProjectPanelPage;
import org.gvsig.propertypage.PropertiesPage;
import org.gvsig.propertypage.PropertiesPageManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.util.ToolsUtilLocator;

public class ProjectWindow  
        extends JPanel
        implements SingletonWindow 
    {

    private Project project;
    private WindowInfo viewInfo;
    private final List<PropertiesPage> pages;
    
    public ProjectWindow() {
        PropertiesPageManager manager = ToolsUtilLocator.getPropertiesPageManager();
        this.pages = manager.getPages(Project.PROJECT_PANEL_PAGE_GROUP,this, null);        
        initComponents();
    }
    
    private void initComponents() {
        this.setLayout(new BorderLayout());
        Component contents = createPagesPanel();
        this.add(contents, BorderLayout.CENTER);
    }
    
    protected Component createPagesPanel() {
        if( this.pages.size()==1 && !useTabsAlwais() ) {
            PropertiesPage page = this.pages.get(0);
            return page.asJComponent();
        } else {
            JTabbedPane theTabbedPane = new JTabbedPane();
            for( int i=0; i<this.pages.size(); i++ ) {
                PropertiesPage page = this.pages.get(i);
                theTabbedPane.addTab(page.getTitle(), page.asJComponent());
            }
            return theTabbedPane;
        }
    }
    
    protected boolean useTabsAlwais() {
        return false;
    }

    @Override
    public Object getWindowModel() {
        return project;
    }

    public Project getProject() {
        return project;
    }
    
    public void setProject(Project project) {
        this.project = project;
        for (PropertiesPage page : this.pages) {
            if( page instanceof ProjectPanelPage ) {
                ((ProjectPanelPage)page).setProject(project);
            }
        }
    }
    
    @Override
    public Object getWindowProfile() {
        return WindowInfo.PROJECT_PROFILE;
    }
    
    @Override
    public WindowInfo getWindowInfo() {
        if (viewInfo == null) {
            I18nManager i18n = ToolsLocator.getI18nManager();
            viewInfo = new WindowInfo(WindowInfo.MAXIMIZABLE | WindowInfo.RESIZABLE);
            viewInfo.setWidth(this.getWidth());
            viewInfo.setHeight(this.getHeight());
            int w = this.getPreferredSize().width;
            if( w < 450 ) {
                w = 450;
            }
            viewInfo.setNormalWidth(w);
            viewInfo.setNormalHeight(this.getPreferredSize().height);
            viewInfo.setMinimumSize(getPreferredSize());
            viewInfo.setTitle(i18n.getTranslation("_Project_manager"));
        }
        return viewInfo;
    }    
    
}
