package org.gvsig.app.project.documents.gui.projectpanel;

import java.awt.BorderLayout;
import org.gvsig.app.project.documents.gui.*;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.andami.ui.mdiManager.MDIManager;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.Document;
import org.gvsig.app.project.documents.DocumentManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.identitymanagement.SimpleIdentityManager;
import org.gvsig.tools.observer.Observable;
import org.gvsig.tools.observer.Observer;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.utils.DefaultListModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ProjectDocumentsPanelPage
        extends ProjectDocumentsPanelPageView
        implements ProjectPanelPage {
    
    private final Logger LOG = LoggerFactory.getLogger(ProjectGeneralPropertiesPage.class);

    private Project project;
    private DocumentManager currentDocumentType;
    private final PropertyChangeListener projectPropertyChangeListener;
    private final Observer projectObserver;

    public ProjectDocumentsPanelPage() {
        this.project = null;
        this.projectPropertyChangeListener = new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                updateProject();
            }
        };
        this.projectObserver = new Observer() {
            @Override
            public void update(Observable observable, Object notification) {
                updateProject();
            }
        };
        this.initComponents();
    }

    @Override
    public void setData(Object data) {
    }
    
    @Override
    public String getTitle() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        return i18n.getTranslation("_Documents");
    }

    @Override
    public int getPriority() {
        return 1000;
    }

    @Override
    public boolean whenAccept() {
        return true;
    }

    @Override
    public boolean whenApply() {
        return true;
    }

    @Override
    public boolean whenCancel() {
        return true;
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    private void initComponents() {
        this.initDocumentTypes();
        this.btnDocumentNew.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doDocumentNew();
            }
        });
        this.btnDocumentOpen.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doDocumentOpen();
            }
        });
        this.btnDocumentRename.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doDocumentRename();
            }
        });
        this.btnDocumentDelete.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doDocumentDelete();
            }
        });
        this.btnDocumentProperties.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doDocumentProperties();
            }
        });
        this.btnProjectProperties.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doProjectProperties();
            }
        });
        this.lstDocuments.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }
                doDocumentsSelectionChanged();
            }
        });
        this.lstDocuments.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if( e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()==2 ) {
                    doDocumentOpen();
                } 
            }
        });
        this.translate();
    }

    private void translate() {
        ToolsSwingManager manager = ToolsSwingLocator.getToolsSwingManager();
        manager.translate(this.lblCreationDate);
        manager.translate(this.lblSavedIn);
        manager.translate(this.lblSessionName);
        manager.translate(this.lblDocument);
        manager.translate(this.lblDocumentTypes);
        manager.translate(this.lblSessionProperties);
        manager.translate(this.lblCreationDate);
        manager.translate(this.lblCreationDate);
        manager.translate(this.lblCreationDate);
        manager.translate(this.lblCreationDate);
        manager.translate(this.lblCreationDate);
        manager.translate(this.lblCreationDate);

        manager.translate(this.btnDocumentDelete);
        manager.translate(this.btnDocumentNew);
        manager.translate(this.btnDocumentOpen);
        manager.translate(this.btnDocumentProperties);
        manager.translate(this.btnDocumentRename);
        manager.translate(this.btnProjectProperties);

    }

    @Override
    public void setProject(Project project) {
        if( this.project!=null ) {
            this.project.removePropertyChangeListener(this.projectPropertyChangeListener);
            this.project.deleteObserver(this.projectObserver);
        }
        this.project = project;
        if(project!=null){
            project.addPropertyChangeListener(this.projectPropertyChangeListener);
            project.addObserver(this.projectObserver);
        }
        updateProject();
    }

    private void updateProject() {
        if( !SwingUtilities.isEventDispatchThread() ) {
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    updateProject();
                }
            });
            return;
        }
        updateProjectDocumens();
        updateProjectProperties();
    }
    
    private void updateProjectProperties() {
        if (project != null) {
            this.txtSessionName.setText(project.getName());
            File path = this.project.getFile();
            if (path != null) {
                this.txtSavedIn.setText(path.toString());
            } else {
                this.txtSavedIn.setText("");
            }
            this.txtCreationDate.setText(project.getCreationDate());
        }
    }    
    
    private void initDocumentTypes() {
        SimpleIdentityManager identityManager = ToolsLocator.getIdentityManager();

        this.documentTypesContainer.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

        List<DocumentManager> factories = ProjectManager.getInstance().getDocumentManagers();
        Collections.sort(factories, new Comparator<DocumentManager>() {

            @Override
            public int compare(DocumentManager arg0, DocumentManager arg1) {
                return arg0.getPriority() - arg1.getPriority();
            }

        });
        this.currentDocumentType = factories.get(0);
        JRadioButton selectButton = null;
        ButtonGroup group = new ButtonGroup();
        JPanel panel = new JPanel();
        for (DocumentManager documentFactory : factories) {
            if (!identityManager.getCurrentIdentity().isAuthorized(
                    Document.ACCESS_DOCUMENT_AUTHORIZATION,
                    documentFactory,
                    documentFactory.getTypeName())) {
                continue;
            }
            JRadioButton button = this.createDocumentTypeButton(documentFactory);
            group.add(button);
            panel.add(button);
            if( documentFactory == this.currentDocumentType ) {
                selectButton = button;
            }
        }
     
        JScrollPane scroll = new JScrollPane(panel);
        Dimension dim = panel.getPreferredSize();
        scroll.setMinimumSize(new Dimension(dim.width+2, dim.height + 35));
        scroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        this.documentTypesContainer.setLayout(new BorderLayout());
        this.documentTypesContainer.add(scroll, BorderLayout.CENTER);
        selectButton.setSelected(true);
    }
   
    private JRadioButton createDocumentTypeButton(final DocumentManager documentFactory) {
        final JRadioButton rb = new JRadioButton();

        rb.setIcon(documentFactory.getIcon());
        rb.setSelectedIcon(documentFactory.getIconSelected());
        rb.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        rb.setText(documentFactory.getTitle());
        rb.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        rb.setName(documentFactory.getTypeName());
        rb.addChangeListener(new javax.swing.event.ChangeListener() {
            @Override
            public void stateChanged(javax.swing.event.ChangeEvent e) {
                if (rb.isSelected()) {
                    currentDocumentType = documentFactory;
                    updateProjectDocumens();
                }
            }
        });
        return rb;
    }

    private void openDocumentWindow(Document doc) {
        if (doc == null) {
            return;
        }
        I18nManager i18n = ToolsLocator.getI18nManager();
        ApplicationManager application = ApplicationLocator.getManager();
        IWindow window = doc.getFactory().getMainWindow(doc);
        if (window == null) {
            application.message(
                    i18n.getTranslation("error_opening_the_document"),
                    JOptionPane.WARNING_MESSAGE
            );
            return;
        }
        application.getUIManager().addWindow(
                window,
                MDIManager.ALIGN_FIRST_LINE_END_CASCADE
        );
        project.setModified(true);
    }

    private void updateProjectDocumens() {
        if (project == null) {
            return;
        }
        I18nManager i18n = ToolsLocator.getI18nManager();
        this.lblDocument.setText(i18n.getTranslation("documentos_existentes"));

        String documentsTypeName = this.currentDocumentType.getTypeName();
        List<Document> documents = project.getDocuments(documentsTypeName);
        DefaultListModel model = new DefaultListModel(documents);
        this.lstDocuments.setModel(model);
        this.lblDocument.setText(currentDocumentType.getTitle());
        updateDocumentButtons();
    }

    private void updateDocumentButtons() {
        this.btnDocumentNew.setEnabled(true);
        if (this.lstDocuments.getSelectedIndex() != -1) {
            this.btnDocumentOpen.setEnabled(true);
            this.btnDocumentDelete.setEnabled(true);
            this.btnDocumentRename.setEnabled(true);
            this.btnDocumentProperties.setEnabled(true);
        } else {
            btnDocumentOpen.setEnabled(false);
            btnDocumentDelete.setEnabled(false);
            btnDocumentRename.setEnabled(false);
            btnDocumentProperties.setEnabled(false);
        }
    }

    private void doDocumentNew() {
        String documentsTypeName = this.currentDocumentType.getTypeName();
        ProjectManager projectManager = ProjectManager.getInstance(); 
        Iterator<? extends Document> documents = projectManager.createDocumentsByUser(documentsTypeName);
        while ( documents != null && documents.hasNext() ) {
            try {
                Document document = documents.next();
                project.addDocument(document);
                openDocumentWindow(document);
            } catch (Exception e) {
                LOG.warn("Can't open document", e);
            }
        }
        this.lstDocuments.requestFocus();        
        this.lstDocuments.setSelectedIndex(this.lstDocuments.getModel().getSize()-1);
    }

    private void doDocumentOpen() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ApplicationManager application = ApplicationLocator.getManager();

        int[] indexes = this.lstDocuments.getSelectedIndices();
        for (int i = indexes.length - 1; i >= 0; i--) {
            int index = indexes[i];
            if (index == -1) {
                return;
            }
            String documentsTypeName = this.currentDocumentType.getTypeName();
            List<Document> documents = project.getDocuments(documentsTypeName);
            Document doc = documents.get(index);
            if (!doc.isAvailable()) {
                application.messageDialog(
                        i18n.getTranslation("document_not_available"),
                        i18n.getTranslation("abrir"),
                        JOptionPane.WARNING_MESSAGE
                );
            }
            IWindow win = doc.getMainWindow();
            openDocumentWindow(doc);
        }

    }

    private void doDocumentRename() {
        ApplicationManager application = ApplicationLocator.getManager();
        I18nManager i18n = ToolsLocator.getI18nManager();

        int index = this.lstDocuments.getSelectedIndex();
        if (index == -1) {
            return;
        }
        String DoumentTypeName = this.currentDocumentType.getTypeName();
        List<Document> documents = project.getDocuments(DoumentTypeName);
        Document doc = documents.get(index);
        String name = application.inputDialog(
                i18n.getTranslation("introduce_nombre"),
                i18n.getTranslation("renombrar"),
                JOptionPane.QUESTION_MESSAGE,
                doc.getName()
        );
        for (Document document : documents) {
            if (StringUtils.equals(document.getName(), name)) {
                application.messageDialog(
                        i18n.getTranslation("elemento_ya_existe"),
                        i18n.getTranslation("warning"),
                        JOptionPane.WARNING_MESSAGE
                );
                return;
            }
        }
        doc.setName(name);
        updateProjectDocumens();
        this.project.setModified(true);
    }

    private void doDocumentDelete() {
        ApplicationManager application = ApplicationLocator.getManager();
        I18nManager i18n = ToolsLocator.getI18nManager();

        int res = application.confirmDialog(
                i18n.getTranslation("confirmar_borrar"),
                i18n.getTranslation("borrar"),
                JOptionPane.YES_NO_OPTION,
                JOptionPane.INFORMATION_MESSAGE,
                "Confirm remove document from project"
        );
        int lastremoved = 0;
        if (res == JOptionPane.YES_OPTION) {
            String DoumentTypeName = this.currentDocumentType.getTypeName();
            int[] indexes = this.lstDocuments.getSelectedIndices();
            for (int i = indexes.length - 1; i >= 0; i--) {
                int index = indexes[i];
                List<Document> documents = project.getDocuments(DoumentTypeName);
                Document doc = documents.get(index);
                if (doc.isLocked()) {
                    application.messageDialog(
                            i18n.getTranslation("locked_element_it_cannot_be_deleted"),
                            i18n.getTranslation("Information"),
                            JOptionPane.INFORMATION_MESSAGE
                    );
                    return;
                }
                PluginServices.getMDIManager().closeSingletonWindow(doc);
                project.removeDocument(doc);
                lastremoved = index;
            }
            if (lastremoved >= this.lstDocuments.getModel().getSize()) {
                this.lstDocuments.setSelectedIndex(this.lstDocuments.getModel().getSize() - 1);
            } else if (lastremoved >= 0) {
                this.lstDocuments.setSelectedIndex(lastremoved);
            }
            project.setModified(true);
        }

    }

    private void doDocumentProperties() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        ApplicationManager application = ApplicationLocator.getManager();

        int index = this.lstDocuments.getSelectedIndex();
        if (index == -1) {
            return;
        }
        String DoumentTypeName = this.currentDocumentType.getTypeName();
        List<Document> documents = project.getDocuments(DoumentTypeName);
        Document doc = documents.get(index);
        if (!doc.isAvailable()) {
            application.messageDialog(
                i18nManager.getTranslation("document_not_available"),
                i18nManager.getTranslation("properties"),
                JOptionPane.WARNING_MESSAGE
            );
            return;
        }
        IWindow properties = doc.getFactory().getPropertiesWindow(doc);
        application.getUIManager().addWindow(
                properties
        );
        this.lstDocuments.setSelectedIndex(index);
        project.setModified(true);
    }

    private void doProjectProperties() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        WindowManager winmgr = ToolsSwingLocator.getWindowManager();
        ProjectProperties panel = new ProjectProperties(project);
        panel.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentHidden(ComponentEvent e) {
                updateProjectProperties();
            }
        });
        winmgr.showWindow(
                panel,
                i18nManager.getTranslation("project_properties"),
                WindowManager.MODE.WINDOW
        );
    }

    private void doDocumentsSelectionChanged() {
        this.updateDocumentButtons();
    }
}
