package org.gvsig.app.project.documents.view.dalactions;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import static javax.swing.Action.ACTION_COMMAND_KEY;
import static javax.swing.Action.NAME;
import static javax.swing.Action.SHORT_DESCRIPTION;
import static javax.swing.Action.SMALL_ICON;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureSet;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.swing.AbstractDALActionFactory;
import org.gvsig.fmap.dal.swing.DALActionFactory.DALActionContext;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.GeometryUtils;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.icontheme.IconTheme;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class ViewZoomAction extends AbstractAction {

    public static class ViewZoomActionFactory extends AbstractDALActionFactory {

        public static final String ACTION_NAME = "ViewZoom";
    
        public ViewZoomActionFactory() {
            super(ACTION_NAME);
        }
        
        @Override
        public Action createAction(DALActionContext context) {
            return new ViewZoomAction(context);
        }
        
        public static void selfRegister() {
            DataSwingManager dalSwingManager = DALSwingLocator.getSwingManager();
            dalSwingManager.registerStoreAction(new ViewZoomActionFactory());
        }
        
    }

    private final DALActionContext context;

    public ViewZoomAction(DALActionContext context) {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        IconTheme iconTheme = ToolsSwingLocator.getIconThemeManager().getDefault();

        this.context = context;
        this.putValue(NAME, null);
        this.putValue(SHORT_DESCRIPTION, i18nManager.getTranslation("_Zoom"));
        this.putValue(SMALL_ICON, iconTheme.get("view-navigation-zoom-to-selection"));
        this.putValue(ACTION_COMMAND_KEY, ViewZoomActionFactory.ACTION_NAME);
        DataStore store = this.context.getStore();
        this.setEnabled(store instanceof FeatureStore);
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument view = (ViewDocument) application.getActiveDocument(ViewDocument.class);
        if( view == null ) {
            return;
        }
        Expression filter = this.context.getFilterForSelecteds();
        if (filter == null) {
            filter = this.context.getFilter();
        }
        if (filter == null) {
            return;
        }
        try {
            FeatureStore store = (FeatureStore) this.context.getStore();
            FeatureSet set = store.getFeatureSet(filter);
            Envelope envelope = GeometryUtils.createEnvelope(Geometry.SUBTYPES.GEOM2D);
            for (Feature feature : set) {
                envelope.add(feature.getDefaultGeometry());
            }
            if( !envelope.isEmpty() ) {
                view.getMapContext().getViewPort().setEnvelope(envelope);
            }
        } catch(Exception ex) {
                
        }
    }

}
