package org.gvsig.app.project.documents.view.expressionevaluator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.Document;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.app.project.documents.view.expressionevaluator.viewcapturepoint.ViewCapturePointElement;
import org.gvsig.app.project.documents.view.expressionevaluator.viewcapturerectangle.ViewCaptureRectangleElement;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.GroupElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.tools.ToolsLocator;

/**
 *
 * @author jjdelcerro
 */
public class ViewElement 
    extends AbstractElement
    implements GroupElement 
    {

    private final List<Element> children;
    private final List<Element> values;
    
    
    public ViewElement() {
        super(
                "Views",
                ToolsLocator.getI18nManager().getTranslation("Views"), 
                "expressionbuilder-element-view"
        );
        this.children = new ArrayList<>();
        this.values = new ArrayList<>();
        
        this.values.add(new ViewCaptureRectangleElement());
        this.values.add(new ViewCapturePointElement());
        this.values.add(new ViewTakeProjectionCodeElement());

        ProjectManager manager = ApplicationLocator.getProjectManager();
        List<Document> views = manager.getCurrentProject().getDocuments(ViewManager.TYPENAME);
        for (Document view : views) {
            Element e = new ViewLayersElement((ViewDocument) view);
            this.children.add(e);
        }
    }

    @Override
    public Collection<Element> getValues() {
        return this.values;
    }

    @Override
    public Element setConfig(ExpressionBuilderConfig config) {
        super.setConfig(config); 
        for (Element child : children) {
            child.setConfig(config);
        }
        return this;
    }
    
    @Override
    public Element get(int index) {
        return this.children.get(index);
    }

    @Override
    public int size() {
        return this.children.size();
    }

    @Override
    public Iterator<Element> iterator() {
        return this.children.iterator();
    }

    @Override
    public void addElement(Element element) {
        this.children.add(element);
    }

    @Override
    public void addElement(Function function) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public List<Element> getElements() {
        return this.children;
    }

}
