package org.gvsig.app.project.documents.view.expressionevaluator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.expressionevaluator.Function;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.Element.GroupElement;
import org.gvsig.expressionevaluator.swing.Element.SimpleElement;
import org.gvsig.expressionevaluator.swing.ExpressionBuilderConfig;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.expressionevaluator.FeatureStoreElement;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.operations.SingleLayer;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;

/**
 *
 * @author jjdelcerro
 */
public class ViewLayersElement
        extends AbstractElement
        implements GroupElement, SimpleElement {

    private class LayerEntryElement
            extends AbstractElement
            implements ConstantElement {

        private final FLayer layer;
        
        public LayerEntryElement(ViewDocument view, FLayer layer) {
            super(layer.getName(), layer.getName(), "expressionbuilder-element-constant");
            this.layer = layer;
        }

        @Override
        public Object getValue() {
            return layer.getName();
        }
    }

    private final List<Element> children;

    public ViewLayersElement(ViewDocument view) {
        super(view.getName());
        
        DataSwingManager dalSwingManager = DALSwingLocator.getSwingManager();

        this.children = new ArrayList<>();
        Iterator<FLayer> it = view.deepiterator();
        while( it.hasNext() ) {
            FLayer layer = it.next();
            if( layer instanceof SingleLayer ) {
                if( layer instanceof FLyrVect ) {
                    FeatureStoreElement e = dalSwingManager.createFeatureStoreElement();
                    e.setFeatureStore(((FLyrVect) layer).getFeatureStore());
                    e.setName(layer.getName());
                    this.children.add(e);
                } else {
                    ConstantElement e = new LayerEntryElement(view, layer);
                    this.children.add(e);
                }
            }
        }
    }

    @Override
    public Element setConfig(ExpressionBuilderConfig config) {
        super.setConfig(config); 
        for (Element child : this.children) {
            child.setConfig(config);
        }
        return this;
    }

    @Override
    public Collection<Element> getValues() {
        return this.children;
    }
    
    @Override
    public Element get(int index) {
        return this.children.get(index);
    }

    @Override
    public int size() {
        return this.children.size();
    }

    @Override
    public Iterator<Element> iterator() {
        return this.children.iterator();
    }

    @Override
    public void addElement(Element element) {
        this.children.add(element);
    }

    @Override
    public void addElement(Function function) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public List<Element> getElements() {
        return this.children;
    }

}
