package org.gvsig.app.project.documents.view.expressionevaluator.viewcapturerectangle;

import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.expressionevaluator.ExpressionBuilder;
import org.gvsig.expressionevaluator.ExpressionBuilder.GeometrySupportType;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.swing.Element.CalculatedElement;
import org.gvsig.expressionevaluator.swing.Element.SimpleElement;
import org.gvsig.expressionevaluator.swing.JExpressionBuilder;
import org.gvsig.expressionevaluator.swing.spi.AbstractElement;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.swing.api.Component;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class ViewCaptureRectangleElement 
        extends AbstractElement
        implements SimpleElement, CalculatedElement
{
    private ViewCaptureRectanglePanel aditionalPanel;

    public ViewCaptureRectangleElement() {
        super(
                "ViewCaptureRectangle",
                ToolsLocator.getI18nManager().getTranslation("_Capture_rectangle"), 
                "expressionbuilder-element-view-capture-bbox"
        );
    }

    @Override
    public Object getValue() {
        if (this.aditionalPanel == null) {
            return null;
        }
        try {
            Geometry geom = this.aditionalPanel.getGeometry();
            if (geom == null) {
                return null;
            }
            ExpressionBuilder builder = ExpressionEvaluatorLocator.getManager().createExpressionBuilder();
            if (this.aditionalPanel.insertAsWKT()) {
                if (this.aditionalPanel.insertAsGeometry()) {
                    builder.geometry_support_type(GeometrySupportType.WKT);
                    return builder.geometry(geom).toString();
                } else {
                    String wkt = geom.convertToWKT();
                    return "'" + wkt + "'";
                }
            } else {
                if (this.aditionalPanel.insertAsGeometry()) {
                    builder.geometry_support_type(GeometrySupportType.WKB);
                    return builder.geometry(geom).toString();
                } else {
                    byte[] wkb = geom.convertToWKB();
                    return builder.bytearray_hex(wkb);
                }
            }
        } catch (Exception ex) {
            LOGGER.warn("Can't get point value.", ex);
            return null;
        }
    }

    @Override
    public String getRenderedValue() {
        Object v = this.getValue();
        if( v == null ) {
            return super.getRenderedValue();
        }
        return v.toString();
    }
    
    @Override
    public Component getAditionalPanel(JExpressionBuilder expressionBuilder) {
        if (aditionalPanel == null) {
            this.aditionalPanel = new ViewCaptureRectanglePanel();
        }
        return this.aditionalPanel;
    }

    @Override
    public boolean isEnabled() {
        ApplicationManager application = ApplicationLocator.getManager();
        return ! application.getUIManager().isModalDialogShowing();
    }

}
