package org.gvsig.app.project.documents.view.legend.gui.legendactions;

import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import org.gvsig.app.project.documents.view.legend.gui.LegendFileFilter;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;

public abstract class AbstractLoadSaveLegendAction extends AbstractAction {

    private static final long serialVersionUID = 3282139517245723693L;

    protected AbstractLoadSaveLegendAction() {
        super();
    }
    
    protected List<LegendFileFilter> getLegendFileFilters(boolean writing) {

        List<LegendFileFilter> resp = new ArrayList<>();
        MapContextManager mcoman = MapContextLocator.getMapContextManager();
        List wformats;

        if( writing ) {
            wformats = mcoman.getLegendWritingFormats();
        } else {
            wformats = mcoman.getLegendReadingFormats();
        }

        String fmt;
        for( int i = 0; i < wformats.size(); i++ ) {
            fmt = (String) wformats.get(i);
            fmt = getFileExtension(fmt);
            if( fmt != null ) {
                resp.add(new LegendFileFilter(fmt));
            }
        }
        return resp;
    }

    /**
     * Returns null if mime format is not parsed properly
     *
     * @param fmt MIME format
     *
     * @return
     */
    protected String getFileExtension(String fmt) {
        // Example: "text/xml; subtype=gml/2.1.2" => "gml"
        if( fmt == null ) {
            return null;
        }
        String[] parts = fmt.split(";");
        String aux;
        if( parts.length > 1 ) {
            aux = parts[1].trim();
        } else {
            aux = parts[0].trim();
        }
        parts = aux.split("=");
        if( parts.length > 1 ) {
            aux = parts[1].trim();
            // Example: aux = "gml/2.1.2"
            parts = aux.split("/");
            return parts[0].length() == 0 ? null : parts[0].toLowerCase();
        } else {
            aux = parts[0].trim();
            // Example: "text/xml"
            parts = aux.split("/");
            if( parts.length > 1 ) {
                return parts[1].length() == 0 ? null : parts[1].toLowerCase();
            } else {
                return aux.length() == 0 ? null : aux.toLowerCase();
            }
        }
    }

}
