package org.gvsig.app.project.documents.view.legend.gui.legendactions;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.gvsig.andami.PluginServices;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.project.documents.view.legend.gui.LegendFileFilter;
import org.gvsig.app.project.documents.view.legend.gui.LegendsPanel;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;
import org.gvsig.fmap.mapcontext.rendering.legend.ILegend;
import org.gvsig.fmap.mapcontext.rendering.legend.driver.ILegendReader;
import org.gvsig.i18n.Messages;

public class LoadLegendAction extends AbstractLoadSaveLegendAction {

    private static final long serialVersionUID = 8023893921676942206L;

    public LoadLegendAction() {
        super();
        this.putValue(Action.NAME, PluginServices.getText(this, "Recuperar_leyenda") + "...");
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if( ! (e.getSource() instanceof LegendsPanel) ) {
            return;
        }
        try {
            loadLegend((LegendsPanel) e.getSource());
        } catch (Exception e1) {
            JOptionPane.showMessageDialog(
                ApplicationLocator.getManager().getRootComponent(),
                Messages.getText("_Unable_to_read_legend")
                + "\n\n" + e1.getMessage(),
                Messages.getText("legend"),
                JOptionPane.WARNING_MESSAGE);
        }
    }

    private void loadLegend(LegendsPanel legends) throws Exception {

        List<LegendFileFilter> filters = getLegendFileFilters(false);
        if( filters.isEmpty() ) {
            JOptionPane.showMessageDialog(
                ApplicationLocator.getManager().getRootComponent(),
                Messages.getText("_No_input_format_available"),
                Messages.getText("legend"),
                JOptionPane.WARNING_MESSAGE);
            return;
        }

        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setAcceptAllFileFilterUsed(false);

        for( int i = 0; i < filters.size(); i++ ) {
            fileChooser.addChoosableFileFilter(filters.get(i));
        }

        // open a file
        if( fileChooser.showOpenDialog((Component) PluginServices
            .getMainFrame()) == JFileChooser.APPROVE_OPTION ) {

            File file = fileChooser.getSelectedFile();
            LegendFileFilter lff = (LegendFileFilter) fileChooser.getFileFilter();
            MapContextManager mcoman = MapContextLocator.getMapContextManager();
            ILegendReader lr = mcoman.createLegendReader(lff.getFileExtension());

            ILegend leg = lr.read(file, Geometry.TYPES.GEOMETRY);
            legends.setLegend(leg);
        }
    }

}
