/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.app.project.documents.view.toc.actions;

import java.awt.Component;

import javax.swing.JOptionPane;
import org.apache.commons.lang3.StringUtils;

import org.gvsig.andami.PluginServices;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.extension.ProjectExtension;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.documents.view.toc.AbstractTocContextMenuAction;
import org.gvsig.app.project.documents.view.toc.ITocItem;
import org.gvsig.fmap.mapcontext.layers.CancelationException;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;

public class EliminarCapaTocMenuEntry extends AbstractTocContextMenuAction {

    public String getGroup() {
        return "group3"; //FIXME
    }

    public int getGroupOrder() {
        return 30;
    }

    public int getOrder() {
        return 0;
    }

    public String getText() {
        return PluginServices.getText(this, "eliminar_capa");
    }

    public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
        return true;
    }

    public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
        if (isTocItemBranch(item)) {
            return true;
        }
        return false;

    }

    public void execute(ITocItem item, FLayer[] selectedItems) {
        ApplicationManager application = ApplicationLocator.getManager();
        I18nManager i18n = ToolsLocator.getI18nManager();
        FLayer[] actives = selectedItems;
        if( actives == null ) {
            return;
        }
        int i;
        for (i = 0; i < actives.length; i++) {
            if (actives[i].isEditing() && actives[i].isAvailable()) {
                JOptionPane.showMessageDialog((Component) PluginServices.getMainFrame(), PluginServices.getText(this, "no_se_puede_borrar_una_capa_en_edicion"), PluginServices.getText(this, "eliminar_capa"), JOptionPane.WARNING_MESSAGE);
                return;
            }
        }
        if (actives.length <= 0) {
            return;
        }
        int option = application.confirmDialog(
                    i18n.getTranslation("desea_borrar_la_capa"), 
                    i18n.getTranslation("_Question"), 
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE, 
                    "Confirm remove layer from the view"
            );
        if (option != JOptionPane.YES_OPTION) {
            return;
        }
        getMapContext().beginAtomicEvent();
        for (i = actives.length - 1; i >= 0; i--) {
            try {
                actives[i].getParentLayer().removeLayer(actives[i]);

                //Cierra todas las ventanas asociadas a la capa
                IWindow[] wList = PluginServices.getMDIManager().getAllWindows();
                for (int j = 0; j < wList.length; j++) {
                    String name = wList[j].getWindowInfo().getAdditionalInfo();
                    for (int k = 0; k < actives.length; k++) {
                        if ( StringUtils.equals(name, actives[k].getName()) ) {
                            PluginServices.getMDIManager().closeWindow(wList[j]);
                        }
                    }
                }

            } catch (CancelationException e1) {
                e1.printStackTrace();
            }
        }
        // ===============================================
    	/*
         * If the removed layer is in a group, the TOC does not received a
         * notification to refresh, so we force a refreshment unless the TOC is empty
         */
        if (getMapContext().getLayers().getLayersCount() > 0) {
            getMapContext().getLayers().moveTo(0, 0);
        }
        // ===============================================
        getMapContext().endAtomicEvent();
        Project project = ((ProjectExtension) PluginServices.getExtension(ProjectExtension.class)).getProject();
        project.setModified(true);
        PluginServices.getMainFrame().enableControls();
    }
}
