package org.gvsig.app.project.symboltables;

import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.expressionevaluator.ExpressionEvaluatorLocator;
import org.gvsig.expressionevaluator.ExpressionEvaluatorManager;
import org.gvsig.expressionevaluator.MutableSymbolTable;
import org.gvsig.expressionevaluator.SymbolTable;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.expressionevaluator.FeatureSymbolTable;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.tools.util.UnmodifiableBasicList;
import org.gvsig.tools.util.UnmodifiableBasicListToListAdapter;

/**
 *
 * @author jjdelcerro
 */
public class ListOfFeaturesWrapper implements UnmodifiableBasicList<Object> {

    private final List<Feature> features;
    private final FeatureSymbolTable featureSymbolTable;
    private final MutableSymbolTable symbolTable;
    private final Expression expression;
    
    public ListOfFeaturesWrapper(
            List<Feature> features, 
            SymbolTable symbolTable, 
            String expression
        ) {
        DataManager dataManager = DALLocator.getDataManager();
        ExpressionEvaluatorManager expManager = ExpressionEvaluatorLocator.getManager();
        
        this.features = features;
        this.featureSymbolTable = dataManager.createFeatureSymbolTable();
        this.symbolTable = this.featureSymbolTable.createParent();
        this.symbolTable.addSymbolTable(symbolTable);

        this.expression = expManager.createExpression();
        this.expression.setPhrase(expression);
    }

    private Object getValue(Feature feature) {
        this.featureSymbolTable.setFeature(feature);
        Object value = this.expression.execute(this.symbolTable);
        return value;
    }
    
    @Override
    public boolean isEmpty() {
        return this.features.isEmpty();
    }

    @Override
    public List<Object> toList() {
        List<Object> x = new UnmodifiableBasicListToListAdapter<>(this);
        return x;
    }

    @Override
    public Object get(int position) {
        return this.getValue(this.features.get(position));
    }

    @Override
    public int size() {
        return this.features.size();
    }

    @Override
    public Iterator<Object> iterator() {
        return new Itr();
    }
    
    private class Itr implements Iterator<Object> {
        int cursor;       // index of next element to return

        Itr() {}

        @Override
        public boolean hasNext() {
            return cursor != size();
        }

        @Override
        public Object next() {
            int i = cursor;
            if (i >= size())
                throw new NoSuchElementException();
            cursor = i + 1;
            return get(i);
        }

        @Override
        public void remove() {
            throw new UnsupportedOperationException("Not supported yet."); 
        }
    }
}
