package org.gvsig.app.project.symboltables.functionPanels.area;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.function.Predicate;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.gvsig.app.project.symboltables.ProjectSymbolTable;
import org.gvsig.expressionevaluator.swing.Element;
import org.gvsig.expressionevaluator.swing.JExpressionBuilder;
import org.gvsig.fmap.dal.DataTypes;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureAttributeDescriptor;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.fmap.dal.swing.expressionevaluator.FeatureStoreElement;
import org.gvsig.fmap.dal.swing.featuretype.AttributeDescriptorPickerController;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.type.GeometryType;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.tools.swing.api.Component;
import org.gvsig.tools.swing.api.ListElement;

/**
 *
 * @author jjdelcerro
 */
public class AreaAditionalPanel 
        extends AreaAditionalPanelView 
        implements Component
    {

    private final JExpressionBuilder expressionBuilder;
    private AttributeDescriptorPickerController fieldPicker;
    
    public AreaAditionalPanel(JExpressionBuilder expressionBuilder) {
        this.expressionBuilder = expressionBuilder;
        this.initComponents();
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }
    
    private void initComponents() {
        DataSwingManager dataSwingManager = DALSwingLocator.getSwingManager();
        
        this.fieldPicker = dataSwingManager.createAttributeDescriptorPickerController(
            this.cboFields, 
            new Predicate<FeatureAttributeDescriptor>() {
            @Override
            public boolean test(FeatureAttributeDescriptor descriptor) {
                if( descriptor.getDataType().getType()!=DataTypes.GEOMETRY ) {
                    return false;
                }
                GeometryType geomtype = descriptor.getGeomType();
                if( geomtype.getType()==Geometry.TYPES.GEOMETRY ) {
                    return true;
                }
                boolean isSurface = geomtype.isTypeOf(Geometry.TYPES.SURFACE) ||
                        geomtype.isTypeOf(Geometry.TYPES.MULTISURFACE);
                return isSurface;
            }
        });
        final FeatureStore store = this.getFeatureStore();
        try {
            this.fieldPicker.setFeatureType(store.getDefaultFeatureType());
        } catch (DataException ex) {
        }
        
        String[] abbrev = MapContext.getAreaAbbr();
        String[] names = MapContext.getAreaNames();
        List<ListElement<String>> elements = new ArrayList<>();
        for (int j = 0; j < names.length; j++) {
            elements.add(new ListElement<>(names[j]+" ("+abbrev[j]+")", abbrev[j]));
        }
        Collections.sort(elements, new Comparator<ListElement<String>>() {
            @Override
            public int compare(ListElement<String> o1, ListElement<String> o2) {
                return o1.toString().compareTo(o2.toString());
            }
        });
        
        DefaultListModel unitsModel = new DefaultListModel();
        for (ListElement<String> element : elements) {
            unitsModel.addElement(element);
        }
        this.lstUnits.setModel(unitsModel);
        if( lstUnits.getModel().getSize()>0 ) {
            this.lstUnits.setSelectedIndex(0);
        }
        if( this.cboFields.getModel().getSize()>0 ) {
            this.cboFields.setSelectedIndex(0);
        }    
        this.cboFields.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                doUpdatePreview();
            }
        });
        this.lstUnits.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if( e.getValueIsAdjusting() ) {
                    return;
                }
                doUpdatePreview();
            }
        });
        this.btnInsertFunction.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doInsertFunction();
            }
        });
        doUpdatePreview();
    }

    private void doUpdatePreview() {
        FeatureAttributeDescriptor descriptor = this.fieldPicker.get();
        String fieldName;
        if( descriptor == null ) {
            fieldName = "{{geometry}}";
        } else {
            fieldName = descriptor.getName();
        }
        String unitName = (String) ListElement.getSelected(lstUnits);
        String template = ProjectSymbolTable.AREA_NAME + "( "+fieldName+", '"+unitName+"' )";
        this.txtPreview.setText(template);
        this.txtPreview.setCaretPosition(0);
        
    }
    
    private void doInsertFunction() {
        this.expressionBuilder.insertText(this.txtPreview.getText());
    }
    
    private FeatureStore getFeatureStore() {
        List<Element> elements = this.expressionBuilder.getElements();
        for (Element element : elements) {
            if( element instanceof FeatureStoreElement ) {
                FeatureStore store = ((FeatureStoreElement) element).getFeatureStore();
                if( store != null ) {
                    return store;
                }
            }
        }
        return null;
    }
}
