package org.gvsig.fmap.dal.serverexplorer.filesystem.swing;

import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;


public class FilesystemExplorerWizardPanelView extends JPanel
{
   JTable tableFiles = new JTable();
   JButton btnAdd = new JButton();
   JButton btnProperties = new JButton();
   JButton btnRemove = new JButton();
   JButton btnUp = new JButton();
   JButton btnDown = new JButton();
   JButton btnChangeViewProjection = new JButton();

   /**
    * Default constructor
    */
   public FilesystemExplorerWizardPanelView()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE","CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:2DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      tableFiles.setName("tableFiles");
      JScrollPane jscrollpane1 = new JScrollPane();
      jscrollpane1.setViewportView(tableFiles);
      jscrollpane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      jpanel1.add(jscrollpane1,new CellConstraints(2,2,1,1,CellConstraints.DEFAULT,CellConstraints.FILL));

      jpanel1.add(createPanel1(),new CellConstraints(4,2,1,1,CellConstraints.DEFAULT,CellConstraints.TOP));
      addFillComponents(jpanel1,new int[]{ 1,2,3,4,5 },new int[]{ 1,2,3 });
      return jpanel1;
   }

   public JPanel createPanel1()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:GROW(1.0)");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      btnAdd.setActionCommand("JButton");
      btnAdd.setName("btnAdd");
      btnAdd.setText("_add");
      btnAdd.setToolTipText("_select_a_file_to_add_to_the_list");
      jpanel1.add(btnAdd,cc.xy(1,1));

      btnProperties.setActionCommand("JButton");
      btnProperties.setName("btnProperties");
      btnProperties.setText("_properties");
      btnProperties.setToolTipText("_change_default_properties_to_load_the_selected_item");
      jpanel1.add(btnProperties,cc.xy(1,3));

      btnRemove.setActionCommand("JButton");
      btnRemove.setName("btnRemove");
      btnRemove.setText("_remove");
      btnRemove.setToolTipText("_remove_the_selected_item_from_the_list");
      jpanel1.add(btnRemove,cc.xy(1,5));

      btnUp.setActionCommand("JButton");
      btnUp.setName("btnUp");
      btnUp.setText("_up");
      btnUp.setToolTipText("_move_up_the_selected_item_in_the_list");
      jpanel1.add(btnUp,cc.xy(1,7));

      btnDown.setActionCommand("JButton");
      btnDown.setName("btnDown");
      btnDown.setText("_down");
      btnDown.setToolTipText("_move_down_the_selected_item_in_the_list");
      jpanel1.add(btnDown,cc.xy(1,9));

      btnChangeViewProjection.setActionCommand("_Change_view_projection");
      btnChangeViewProjection.setName("btnChangeViewProjection");
      btnChangeViewProjection.setText("_Change_view_projection");
      btnChangeViewProjection.setToolTipText("_Change_view_projection_to_projection_of_the_selected_item_in_the_list\n");
      jpanel1.add(btnChangeViewProjection,cc.xy(1,11));

      addFillComponents(jpanel1,new int[0],new int[]{ 2,4,6,8,10,12 });
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
