package org.gvsig.fmap.dal.serverexplorer.filesystem.swing;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;

import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.DataStoreProviderFactory;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;


/**
 * @author fdiaz
 *
 */
public class FilesystemExplorerWizardTableModel extends AbstractTableModel {

    /**
     *
     */
    private static final long serialVersionUID = -1289509561088757725L;

    private static final int COLUMNS = 2;

    private static final int COLUMN_STORE_PARAMETERS = 0;
    private static final int COLUMN_TILED = 1;

    public static final String COLUMN_STORE_PARAMETERS_NAME = "Fichero";
    public static final String COLUMN_TILED_NAME = "_tiled";

    private List<FilesystemStoreParametersToLoad> data = null;

    /**
     *
     */
    public FilesystemExplorerWizardTableModel() {
        this.data = (List<FilesystemStoreParametersToLoad>) new ArrayList<FilesystemStoreParametersToLoad>();
    }

    @Override
    public int getColumnCount() {
        return COLUMNS;
    }

    @Override
    public int getRowCount() {
        return data.size();
    }


    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
        case COLUMN_STORE_PARAMETERS:
            return FilesystemStoreParametersToLoad.class;
        case COLUMN_TILED:
            return Boolean.class;
        }
        return super.getColumnClass(columnIndex);
    }

    @Override
    public Object getValueAt(int i, int j) {
        if (i < data.size()) {
            FilesystemStoreParametersToLoad filesystemStoreParametersToLoad = (FilesystemStoreParametersToLoad) data.get(i);
            switch (j) {
            case COLUMN_STORE_PARAMETERS:
                return filesystemStoreParametersToLoad.getParameters();
            case COLUMN_TILED:
                return filesystemStoreParametersToLoad.isTiled();
            }
        }
        return null;
    }

    @Override
    public String getColumnName(int col) {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        if(col < COLUMNS){
            switch (col) {
            case COLUMN_STORE_PARAMETERS:
                return i18nManager.getTranslation(COLUMN_STORE_PARAMETERS_NAME);
            case COLUMN_TILED:
                return i18nManager.getTranslation(COLUMN_TILED_NAME);
            default:
                break;
            }
        }
        throw new IndexOutOfBoundsException("Column '"+col+"' > "+COLUMNS);
    }

    public boolean isCellEditable(int row, int col) {
        if (col == COLUMN_TILED && isEditableTiled(row) ) {
            return true;
        }
        return false;
    }

    public void setValueAt(Object value, int row, int col) {
        if (row < data.size()) {
            if (col < COLUMNS) {
                FilesystemStoreParametersToLoad filesystemStoreParametersToLoad = data.get(row);
                switch (col) {
                case COLUMN_STORE_PARAMETERS:
                    if (value == null || !(value instanceof FilesystemStoreParameters)) {
                        throw new IllegalArgumentException("The object '" + value.toString() + "' must be a FilesystemStoreParameters.");
                    }
                    filesystemStoreParametersToLoad.setParameters((FilesystemStoreParameters) value);
                    break;
                case COLUMN_TILED:
                    if (value == null) {
                        filesystemStoreParametersToLoad.setTiled(false);
                        break;
                    }
                    if (!(value instanceof Boolean)) {
                        throw new IllegalArgumentException("The object '" + value.toString() + "' must be a Boolean.");
                    }
                    filesystemStoreParametersToLoad.setTiled((boolean) value);
                    break;

                default:
                    break;
                }

            }
        }
    }

    /**
     * @param fileParams
     * @param projection
     * @param tiled
     */
    public void add(FilesystemStoreParameters fileParams){
        DataManager dataManager = DALLocator.getDataManager();
        DataStoreProviderFactory factory = dataManager.getStoreProviderFactory( ((DataStoreParameters)fileParams).getDataStoreName());
        boolean editable = false;
        boolean tiled = false;
        if( factory != null ) {
            if( factory.getClass().getSimpleName().equalsIgnoreCase("TileProviderFactory")){
                editable = false;
                tiled = true;
            } else {
                switch (factory.isTiledSupported()) {
                case DataStoreProviderFactory.NO:
                    editable = false;
                    tiled = false;
                    break;
                case DataStoreProviderFactory.YES:
                    editable = true;
                    tiled = (factory.worksBestTiled()==DataStoreProviderFactory.YES);
                    break;
                case DataStoreProviderFactory.UNKNOWN:
                    //De momento, si no sabemos si se puede tilear lo marcamos como no
                    editable = false;
                    tiled = false;
                    break;
                default:
                    break;
                }
            }
        }
        FilesystemStoreParametersToLoad filesystemStoreParametersToLoad = new FilesystemStoreParametersToLoad(fileParams, tiled, editable) ;
        data.add(filesystemStoreParametersToLoad);
    }

    /**
     * @param listFileParams
     * @param tiled
     */
    public void add(List<FilesystemStoreParameters> listFileParams){
        for (FilesystemStoreParameters filesystemStoreParameters : listFileParams) {
            this.add(filesystemStoreParameters);
        }
    }

    /**
     * @param row
     */
    public void remove(int row){
        this.data.remove(row);
    }

    /**
     * @param row
     * @return FilesystemStoreParameters
     */
    public FilesystemStoreParameters getParams(int row){
        return this.data.get(row).getParameters();
    }

    /**
     * @param row
     * @return boolean
     */
    public boolean isTiled(int row){
        return this.data.get(row).isTiled();
    }

    /**
     * @param row
     * @return
     */
    public boolean isEditableTiled(int row){
        return this.data.get(row).isEditable();
    }
    /**
     * @param row
     */
    public void up(int row){
        if(row>0 && row<this.data.size()){
            FilesystemStoreParametersToLoad aux = this.data.get(row-1);
            this.data.set(row-1, this.data.get(row));
            this.data.set(row, aux);
        }
    }

    /**
     * @param row
     */
    public void down(int row){
        if(row>=0 && row<(this.data.size()-1)){
            FilesystemStoreParametersToLoad aux = this.data.get(row);
            this.data.set(row, this.data.get(row+1));
            this.data.set(row+1, aux);
        }
    }

    /**
     * @return DataStoreParameters[]
     */
    public DataStoreParameters[] getParameters() {
        List<FilesystemStoreParameters> result = new ArrayList<FilesystemStoreParameters>();
        for (FilesystemStoreParametersToLoad rowData : this.data) {
            result.add(rowData.getParameters());
        }
        return result.toArray(new DataStoreParameters[0]);
    }

    /**
     * @param parameters
     * @return boolean
     */
    public boolean isTiled(DataStoreParameters parameters) {
        for (FilesystemStoreParametersToLoad filesystemStoreParametersToLoad : data) {
            if(filesystemStoreParametersToLoad.getParameters()==parameters){
                return filesystemStoreParametersToLoad.isTiled();
            }
        }
        return false;
    }

    /**
     * @author fdiaz
     *
     */
    public static class FilesystemStoreParametersToLoad{
        private FilesystemStoreParameters params;
        private boolean tiled;
        private boolean editable = false;

        /**
         * @param params
         */
        public FilesystemStoreParametersToLoad(FilesystemStoreParameters params) {
            this.setParameters(params);
        }

        /**
         * @param params
         * @param tiled
         * @param editable
         */
        public FilesystemStoreParametersToLoad(FilesystemStoreParameters params, boolean tiled, boolean editable) {
            this.setParameters(params);
            this.setTiled(tiled);
            this.editable = editable;
        }

        FilesystemStoreParameters getParameters(){
            return this.params;
        }

        /**
         * @param file the file to set
         */
        public void setParameters(FilesystemStoreParameters file) {
            this.params = file;
        }

        /**
         * @return the tiled
         */
        public boolean isTiled() {
            return tiled;
        }

        /**
         * @param tiled the tiled to set
         */
        public void setTiled(boolean tiled) {
                this.tiled = tiled;
        }

        public boolean isEditable() {
            return this.editable;
        }
    }

}
