/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.coreplugin.preferences.general;

import com.lowagie.text.ListItem;
import java.awt.BorderLayout;
import java.util.prefs.Preferences;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.text.DocumentFilter;
import org.apache.commons.lang3.StringUtils;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.preferences.AbstractPreferencePage;
import org.gvsig.andami.preferences.StoreException;
import org.gvsig.i18n.Messages;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.documentfilters.DoubleDocumentFilter;
import org.gvsig.tools.swing.icontheme.IconTheme;
import org.gvsig.tools.swing.icontheme.IconThemeManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class IconThemePage extends AbstractPreferencePage {

    /**
     *
     */
    private static final long serialVersionUID = 4369071892027860769L;

    private static final Logger logger = LoggerFactory.getLogger(IconThemePage.class);

    private final String id;
    private boolean changed = false;
    private IconThemePageLayout panel = null;
    private ImageIcon icon = null;

    public IconThemePage() {
        super();
        initialize();
        id = this.getClass().getName();
        setParentID(GeneralPage.class.getName());
    }

    @Override
    public String getID() {
        return id;
    }

    @Override
    public String getTitle() {
        return Messages.getText("_Icon_theme");
    }

    @Override
    public JPanel getPanel() {
        return this;
    }

    @Override
    public void initializeValues() {
        final IconThemeManager iconManager = ToolsSwingLocator.getIconThemeManager();
        Preferences prefs = Preferences.userRoot().node("gvsig.icontheme");
        String themeId = prefs.get("default-theme",null);
        double scaleFactor;
        try {
            scaleFactor = prefs.getDouble("scaleFactor",1.0);
        } catch(Throwable th) {
            scaleFactor = 1.0;
        }
        ListElement.setSelected(panel.combo_selection, iconManager.get(themeId));
        DoubleDocumentFilter.install(panel.txtScaleFactor);
    }

    @Override
    public void storeValues() throws StoreException {
        Preferences prefs = Preferences.userRoot().node("gvsig.icontheme");
        IconTheme iconTheme = (IconTheme) ListElement.getSelected(panel.combo_selection);
        if (iconTheme != null && iconTheme != iconTheme.getDefault()) {
            prefs.put("default-theme", iconTheme.getID());
        }
        if( DoubleDocumentFilter.isValid(panel.txtScaleFactor) ) {
            double scaleFactor = DoubleDocumentFilter.getValue(panel.txtScaleFactor);
            prefs.putDouble("scaleFactor", scaleFactor);
        }

    }

    @Override
    public void initializeDefaults() {

    }

    @Override
    public ImageIcon getIcon() {
        if (icon == null) {
            icon = IconThemeHelper.getImageIcon("edit-setup-icontheme");
        }
        return icon;
    }

    @Override
    public boolean isValueChanged() {
        if (panel.combo_selection.getSelectedIndex() >= 0) {
            return true;
        }
        if( DoubleDocumentFilter.isValid(panel.txtScaleFactor) ) {
            double scaleFactor = DoubleDocumentFilter.getValue(panel.txtScaleFactor);
            Preferences prefs = Preferences.userRoot().node("gvsig.icontheme");
            try {
                if( scaleFactor != prefs.getDouble("scaleFactor",1.0) ) {
                    return true;
                }
            } catch(Throwable th) {
            }
        }
        return changed;
    }

    @Override
    public void setChangesApplied() {
        changed = false;
    }

    private void initialize() {
        final IconThemeManager iconManager = ToolsSwingLocator.getIconThemeManager();
        this.setLayout(new BorderLayout());

        this.panel = new IconThemePageLayout();

        DefaultComboBoxModel<ListElement<IconTheme>> model = new DefaultComboBoxModel<>();
        for (int i = 0; i < iconManager.getCount(); i++) {
            IconTheme iconTheme = iconManager.get(i);
            model.addElement(new ListElement<>(
                    StringUtils.abbreviate(iconTheme.toString(), 60),
                    iconTheme
                )
            );
        }
        this.panel.combo_selection.setModel(model);

        Preferences prefs = Preferences.userRoot().node("gvsig.icontheme");
        double scaleFactor;
        try {
            scaleFactor = prefs.getDouble("scaleFactor",1.0);
        } catch(Throwable th) {
            scaleFactor = 1.0;
        }
        this.panel.txtScaleFactor.setText(String.format("%f", scaleFactor));

        // Traducir las etiquetas del panel
        panel.label_title.setText(translate(panel.label_title.getText()));
        panel.label_selection.setText(translate(panel.label_selection.getText()));

        this.add(panel, BorderLayout.CENTER);
    }

    private String translate(String s) {
        return Messages.getText(s);
    }

//    private void createDefaultIconTheme() {
//        PluginsManager pluginsManager = PluginsLocator.getManager();
//        IconThemeManager iconManager = ToolsSwingLocator.getIconThemeManager();
//
//        File f = new File(pluginsManager.getApplicationHomeFolder(), "icon-theme");
//        if (!f.exists()) {
//            f.mkdir();
//        }
//        IconTheme theme = iconManager.getDefault();
//        File f2 = new File(f, theme.getID());
//        if (!f2.exists()) {
//            logger.info("Creating default icon theme in " + f.getAbsolutePath());
//            theme.export(f);
//        }
//    }


    @Override
    public boolean isResizeable() {
        return true;
    }
   
}
