/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.coreplugin.preferences.general.appearance;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.Launcher;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.PluginsLocator;
import org.gvsig.andami.PluginsManager;
import org.gvsig.andami.preferences.IPreference;
import org.gvsig.andami.preferences.StoreException;
import org.gvsig.coreplugin.PreferencesExtension;
import org.gvsig.coreplugin.preferences.general.GeneralPage;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.util.ToolsUtilLocator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Appearance page. Where the user can choose Look&Feels and maybe some more
 * stuff.
 */
public class AppearancePage extends AppearancePageView implements IPreference {
    //
    // The class AppearancePageView can extend from AbstractPreferencePage, not JPanel,
    // if rereate it from the GUI builder change te class declaration to:
    //    public abstract class AppearancePageView extends AbstractPreferencePage

    private static final long serialVersionUID = 9004756473020260443L;

    private static final Logger logger = LoggerFactory.getLogger(AppearancePage.class);

    public static String id = AppearancePage.class.getName();
    private ImageIcon icon;
    private boolean enableLookAndFeel = true;
    private final String defaultLookAndFeel;

    public AppearancePage() {
        setParentID(GeneralPage.id);
        installDefaultLookAndFeel();       
        this.defaultLookAndFeel = Launcher.getDefaultLookAndFeel();
        String x = PluginServices.getArgumentByName("enablelaf");
        if( x == null ) {            
            this.enableLookAndFeel = true;
        } else {
            this.enableLookAndFeel = BooleanUtils.toBoolean(x);
        }
        this.initComponents();
    }

    private void installDefaultLookAndFeel() {
        String[][] lafs = {
            { "Plastic XP", "com.jgoodies.looks.plastic.PlasticXPLookAndFeel" },
            { "Quaqua", "ch.randelshofer.quaqua.QuaquaLookAndFeel" },
            { "Web", "com.alee.laf.WebLookAndFeel" },
            { "Sea Glass", "com.seaglasslookandfeel.SeaGlassLookAndFeel" },
//            Substance da errores en la carga de gvSIG y el L&F no funciona 
//            { "Substance", "org.pushingpixels.substance.api.SubstanceLookAndFeel" },
//            { "SubstanceAutumn", "org.pushingpixels.substance.api.skin.SubstanceAutumnLookAndFeel" },
//            { "SubstanceOfficeBlue2007", "org.pushingpixels.substance.api.skin.SubstanceOfficeBlue2007LookAndFeel" },
//            { "SubstanceBusiness", "org.pushingpixels.substance.api.skin.SubstanceBusinessLookAndFeel" },
//            { "SubstanceMistAqua", "org.pushingpixels.substance.api.skin.SubstanceMistAquaLookAndFeel" },
            { "Acryl", "com.jtattoo.plaf.acryl.AcrylLookAndFeel" },
            { "Fast", "com.jtattoo.plaf.fast.FastLookAndFeel" },
            { "Bernstein", "com.jtattoo.plaf.bernstein.BernsteinLookAndFeel" },
            { "Aluminium", "com.jtattoo.plaf.aluminium.AluminiumLookAndFeel" },
            { "Aero", "com.jtattoo.plaf.aero.AeroLookAndFeel"  },
            { "HiFi", "com.jtattoo.plaf.hifi.HiFiLookAndFeel" },
            { "McWin", "com.jtattoo.plaf.mcwin.McWinLookAndFeel" },
            { "Mint", "com.jtattoo.plaf.mint.MintLookAndFeel" },
            { "Noire", "com.jtattoo.plaf.noire.NoireLookAndFeel" },
            { "Smart", "com.jtattoo.plaf.smart.SmartLookAndFeel" },
            { "Luna", "com.jtattoo.plaf.luna.LunaLookAndFeel" },
            { "Texture", "com.jtattoo.plaf.texture.TextureLookAndFeel" }
        };            

        for( String[] laf : lafs ) {
            try {
                UIManager.installLookAndFeel(laf[0], laf[1]);
            } catch (Throwable th) {
                logger.info("Problems installing look and feel '"+laf[1]+"'", th);
            }
        }
    }

    private LookAndFeelInfo[] getSupportedsLookAndFeels() {
        if( !this.enableLookAndFeel ) {
            return null;
        }
        List<LookAndFeelInfo> supporteds = new ArrayList();
        try {
            LookAndFeelInfo[] installedLookAndFeels = UIManager.getInstalledLookAndFeels();
            for( LookAndFeelInfo lookAndfeelInfo : installedLookAndFeels ) {
                try {
                    Class<?> lookAndFeelClass = Class.forName(lookAndfeelInfo.getClassName());
                    LookAndFeel lookAndFeel = (LookAndFeel) lookAndFeelClass.newInstance();
                    if( lookAndFeel.isSupportedLookAndFeel() ) {
                        supporteds.add(lookAndfeelInfo);
                    }
                } catch (Throwable th) {

                }
            }
        } catch (Throwable th) {
            return null;
        }
        if( supporteds.isEmpty() ) {
            return null;
        }
        Collections.sort(supporteds, new Comparator<LookAndFeelInfo>() {
            @Override
            public int compare(LookAndFeelInfo o1, LookAndFeelInfo o2) {
                return StringUtils.compare(o1.getName(),o2.getName());
            }
        });
        return supporteds.toArray(new LookAndFeelInfo[supporteds.size()]);
    }

    private void initComponents() {
        icon = IconThemeHelper.getImageIcon("edit-setup-appearance");
        LookAndFeelInfo[] lafs = this.getSupportedsLookAndFeels();
        if( lafs == null ) {
            this.cboLookAndFeel.setEnabled(false);
        } else {
            DefaultComboBoxModel model = new DefaultComboBoxModel();
            for( LookAndFeelInfo laf : lafs ) {
                model.addElement(new ListElement<>(laf.getName(), laf.getClassName()));
            }
            this.cboLookAndFeel.setModel(model);
            this.cboLookAndFeel.setEnabled(true);
            ListElement.setSelected(cboLookAndFeel, Launcher.getAndamiConfig().getLookAndFeel());

        }
        this.translateAll();
    }

    private void translateAll() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        this.lblNeedRestart.setText("("+i18n.getTranslation("Los_cambios_efectuados_sobre_estos_valores_se_aplicaran_al_reiniciar_la_aplicacion")+")");
        this.lbllookAndFeel.setText(i18n.getTranslation("options.general.select_theme"));
        this.chkUseNativeFileChooser.setText(i18n.getTranslation("_use_native_file_dialog_chooser"));
    }

    @Override
    public boolean isResizeable() {
        return true;
    }

    @Override
    public void initializeValues() {
        this.initializeDefaults();
        this.chkUseNativeFileChooser.setSelected(ToolsUtilLocator.getFileDialogChooserManager().useNativeFileChooser());
    }

    @Override
    public void initializeDefaults() {
        PluginsManager pluginsManager = PluginsLocator.getManager();
        PluginServices plugin = pluginsManager.getPlugin(PreferencesExtension.class);

        DynObject pluginProperties = plugin.getPluginProperties();

        this.chkUseNativeFileChooser.setSelected((Boolean) pluginProperties.getDynValue("useNativeFileChooser"));

        if( this.cboLookAndFeel.getModel().getSize()>0 ) {
            this.cboLookAndFeel.setSelectedIndex(0);
            String currentLaF = getCurrentLookAndFeel();
            if( !StringUtils.isEmpty(currentLaF) ) {
                ListElement.setSelected(cboLookAndFeel, currentLaF);
            } else {
                this.cboLookAndFeel.setSelectedIndex(-1);
            }
            this.cboLookAndFeel.setEnabled(true);
        } else {
            this.cboLookAndFeel.setEnabled(false);
        }
    }
    
    private String getCurrentLookAndFeel() {
        String s = Launcher.getAndamiConfig().getLookAndFeel();
        if( StringUtils.isBlank(s) ) {
            s = defaultLookAndFeel;
        }
        return s;
    }

    @Override
    public String getID() {
        return id;
    }

    @Override
    public String getTitle() {
        I18nManager i18nManager = ToolsLocator.getI18nManager();
        return i18nManager.getTranslation("pref.appearance");
    }

    @Override
    public JPanel getPanel() {
        return this;
    }

    @Override
    public void storeValues() throws StoreException {
        PluginsManager pluginsManager = PluginsLocator.getManager();
        PluginServices plugin = pluginsManager.getPlugin(PreferencesExtension.class);

        DynObject pluginProperties = plugin.getPluginProperties();

        pluginProperties.setDynValue("useNativeFileChooser", this.chkUseNativeFileChooser.isSelected());
        ToolsUtilLocator.getFileDialogChooserManager().setUseNativeFileChooser(this.chkUseNativeFileChooser.isSelected());
        
        if( this.cboLookAndFeel.isEnabled() ) {
           String lookAndFeel = (String) ListElement.getSelected(cboLookAndFeel);
           if( lookAndFeel!=null ) {
               Launcher.getAndamiConfig().setLookAndFeel(lookAndFeel);
           }
        }
    }

    @Override
    public ImageIcon getIcon() {
        return icon;
    }

    @Override
    public boolean isValueChanged() {
        boolean changed = false;
        
        if( !changed ) {
            changed = this.chkUseNativeFileChooser.isSelected() != ToolsUtilLocator.getFileDialogChooserManager().useNativeFileChooser();
        }
        if( !changed ) {
            if( this.cboLookAndFeel.isEnabled() ) {
                String lookAndFeel = (String) ListElement.getSelected(cboLookAndFeel);
                if( lookAndFeel!=null ) {
                    changed = !lookAndFeel.equalsIgnoreCase(getCurrentLookAndFeel());
                }
            }
        }
        return changed;
    }

    @Override
    public void setChangesApplied() {
        // Do nothing
    }

}
