package org.gvsig.export.app.extension;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.PluginsLocator;
import org.gvsig.andami.PluginsManager;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.ExportManager;
import org.gvsig.export.ExportParameters;
import org.gvsig.tools.bookmarksandhistory.Bookmark;
import org.gvsig.tools.bookmarksandhistory.Bookmarks;
import org.gvsig.tools.bookmarksandhistory.History;
import org.gvsig.tools.dynobject.DynObject;

/**
 *
 * @author osc
 */
public class ExportHistoryBookmarksExtension extends Extension {

    @Override
    public void initialize() {
        loadExportHistoryParameters();
        loadExportBookmarksParameters();
    }

    @Override
    public void execute(String actionCommand) {

    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean isVisible() {
        return true;
    }

    private void loadExportHistoryParameters() {
        PluginsManager pluginManager = PluginsLocator.getManager();
        final PluginServices plugin = pluginManager.getPlugin(this);
        final ExportManager exportManager = ExportLocator.getManager();
        History<ExportParameters> history = exportManager.getHistory();
        pluginManager.addShutdownTask("saveExportHistory", new Runnable() {
            @Override
            public void run() {
                DynObject properties = plugin.getPluginProperties();
                ArrayList<ExportParameters> var = new ArrayList<>();
                if (history == null) {
                    return;
                }
                for (ExportParameters historyBookmark : history) {
                    if (historyBookmark != null) {
                        var.add(historyBookmark);
                    } else {
                        logger.warn("Trying to save history but value is null");
                    }
                }
                properties.setDynValue("exportHistory", var);
            }
        }, false, 100);
        DynObject pluginProperties = plugin.getPluginProperties();
        List<ExportParameters> dynBookmarks = (List<ExportParameters>) pluginProperties.getDynValue("exportHistory");
        if (dynBookmarks == null) {
            return;
        }
        for (ExportParameters exportParameters : dynBookmarks) {
            history.add(exportParameters);
        }

    }

    private void loadExportBookmarksParameters() {
        PluginsManager pluginManager = PluginsLocator.getManager();
        final PluginServices plugin = pluginManager.getPlugin(this);
        DynObject pluginProperties = plugin.getPluginProperties();
        final ExportManager exportManager = ExportLocator.getManager();
        Bookmarks<ExportParameters> bookmarks = exportManager.getBookmarks();
        pluginManager.addShutdownTask("saveExportBookmarks", new Runnable() {
            @Override
            public void run() {
                DynObject properties = plugin.getPluginProperties();
                Map<String, ExportParameters> var = new HashMap<>();
                for (Bookmark<ExportParameters> exportBookmark : bookmarks) {
                    ExportParameters value = exportBookmark.getValue();
                    if (value != null) {
                        var.put(exportBookmark.getName(), value);
                    } else {
                        logger.warn("Trying to save bookmark but value is null");
                    }
                }
                properties.setDynValue("exportBookmarks", var);
            }
        }, false, 100);
        Map<String, ExportParameters> dynBookmarks = (Map<String, ExportParameters>) pluginProperties.getDynValue("exportBookmarks");
        if (dynBookmarks == null) {
            return;
        }
        for (String key : dynBookmarks.keySet()) {
            ExportParameters bookmarkParameters = dynBookmarks.get(key);
            if (bookmarkParameters != null) {
                bookmarks.add(key, bookmarkParameters);
            } else {
                logger.warn("Trying to add bookmark but value is null");
            }
            ;
        }

    }

}
