/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.app.extension;

import java.awt.Dimension;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import org.apache.commons.lang3.StringUtils;
import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.ExportManager;
import org.gvsig.export.ExportProcess;
import org.gvsig.export.Filter;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.export.swing.ExportFinishListenerAdapter;
import org.gvsig.export.swing.ExportSwingLocator;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dataTypes.DataTypes;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.util.ArrayUtils;
import org.gvsig.export.swing.ExportSwingManager;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.dal.OpenDataStoreParameters;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.observer.Notification;
import org.gvsig.tools.observer.Observable;
import org.gvsig.tools.observer.Observer;


/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class ExportLayerExtension extends Extension {

    @Override
    public void initialize() {
        IconThemeHelper.registerIcon("action", "layer-export", this);
        IconThemeHelper.registerIcon("picker", "picker-crs", this);
    }

    @Override
    public void postInitialize() {
        super.postInitialize();
        ProjectManager projectManager = ApplicationLocator.getProjectManager();
        ViewManager viewManager = (ViewManager) projectManager.getDocumentManager(ViewManager.TYPENAME);
        viewManager.addTOCContextAction("layer-exportto");
        registerInCatalog();
    }

    @Override
    public void execute(String actionCommand) {
        this.execute(actionCommand, null);
    }

    @Override
    public void execute(String command, Object[] args) {
        if( !"exportto".equalsIgnoreCase(command) ) {
            return;
        }
        Object layer = ArrayUtils.get(args, 0);
        if( layer instanceof FLyrVect ) {
            WindowManager.MODE mode = WindowManager.MODE.fromInteger(
                    (Integer)ArrayUtils.get(args, 1,DataTypes.INT, 0)
            );
            FLyrVect layervect = (FLyrVect) layer;
            MapContext mapContext = layervect.getMapContext();
            this.showExport(layervect, mapContext.getProjection(), mapContext, mode);
            return;
        }
        
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument view = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if ( view != null ) {
            MapContext mapContext = view.getMapContext();
            FLayers layers = mapContext.getLayers();
            FLayer[] actives = layers.getActives();
            try {
                if ( actives.length == 1 ) {
                    if( actives[0] instanceof FLyrVect ) {
                        showExport(
                            (FLyrVect) actives[0], 
                            mapContext.getProjection(),
                            mapContext, 
                            WindowManager.MODE.WINDOW
                        );
                    }
                } else {
                    for (FLayer theLayer : actives) {
                        if (layer instanceof FLyrVect) {
                            showExport(
                                (FLyrVect) theLayer, 
                                mapContext.getProjection(),
                                mapContext, 
                                WindowManager.MODE.DIALOG
                            );
                        }
                    }
                }
            } catch (Exception e) {
                logger.warn("Can't run export wizard.", e);
            }
        }
    }

    public void showExport(
            FLyrVect layer, 
            IProjection contextProjetion,
            final MapContext mapContext, 
            WindowManager.MODE mode
        ) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ExportManager manager = ExportLocator.getManager();
        ExportSwingManager swingManager = ExportSwingLocator.getSwingManager();

        final ExportProcess process = manager.createProcess();
        process.setContext(mapContext);
        process.setSourceFeatureStore(layer.getFeatureStore());
        process.setContextProjection(contextProjetion);
        process.setSourceProjection(layer.getProjection());
        ICoordTrans ct = layer.getCoordTrans();
        if( ct==null ) {
            process.setSourceTransformation(null);
        } else {
            process.setSourceProjection(ct.getPOrig());
            process.setSourceTransformation(ct);
        }
        JExportProcessPanel panel = swingManager.createJExportProcessPanel(
            process,
            new Filter<ExportServiceFactory>() {
                @Override
                public boolean apply(ExportServiceFactory factory) {
                    return factory.hasVectorialSupport();
                }
            }
        );
        panel.addFinishListener(new ExportFinishListenerAdapter() {
            @Override
            public void finished(JExportProcessPanel exportPanel) {
                doAddlayer(process, mapContext);
            }
        });
        JComponent component = panel.asJComponent();
        component.setPreferredSize(new Dimension(component.getPreferredSize().width, 530));
        
        WindowManager winmgr = ToolsSwingLocator.getWindowManager();
        winmgr.showWindow(
                panel.asJComponent(), 
                i18n.getTranslation("_Export_to"), 
                mode
        );
    }

    private void doAddlayer(final ExportProcess process, final MapContext mapContext) {
        if (!SwingUtilities.isEventDispatchThread()) {
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    doAddlayer(process, mapContext);
                }
            });
            return;
        }
        
        ApplicationManager application = ApplicationLocator.getManager();
        I18nManager i18n = ToolsLocator.getI18nManager();
        
        if (process.getTargetOpenStoreParameters()==null) {
            return;
        }

        int n = application.confirmDialog(
                i18n.getTranslation("_Add_new_layer_to_current_view"), 
                i18n.getTranslation("_Export"), 
                JOptionPane.YES_NO_OPTION, 
                JOptionPane.QUESTION_MESSAGE, 
                "AddExportedLayerToView"
        );
        if( n != JOptionPane.YES_OPTION ) {
            return;
        }
        try {
            DataManager dataManager = DALLocator.getDataManager();
            MapContextManager mapContextManager = MapContextLocator.getMapContextManager();
            
            for (OpenDataStoreParameters params : process.getTargetOpenStoreParameters()) {
                DataStore store = dataManager.openStore(params.getDataStoreName(), params);
                FLayer layer = mapContextManager.createLayer(store.getName(), store);
                mapContext.getLayers().addLayer(layer);
            }
            application.messageDialog(
                    i18n.getTranslation("_Layer_added_to_view"), 
                    null, 
                    i18n.getTranslation("_Export"), 
                    JOptionPane.QUESTION_MESSAGE, 
                    "ExportedLayerAddedToView"
            );
        } catch (Exception ex) {
            logger.warn("Can't add exported layer to view",ex);
            application.messageDialog(
                    i18n.getTranslation("_Cant_add_exported_layer_to_view"), 
                    null, 
                    i18n.getTranslation("_Export"), 
                    JOptionPane.WARNING_MESSAGE, 
                    "CantAddExportedLayerToView"
            );
        }
    }

    @Override
    public boolean isEnabled() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument view = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if ( view != null ) {
            return view.getMapContext().hasActiveVectorLayers();
        }
        return false;
    }

    @Override
    public boolean isVisible() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument view = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        return view != null;
    }

    private void registerInCatalog() {
        final ApplicationManager application = ApplicationLocator.getApplicationManager();
        application.addPropertiesObserver(new Observer() {
            @Override
            public void update(Observable observable, Object notification) {
                if( !(notification instanceof Notification) ) {
                    return;
                }
                String propertyName = (String) ((Notification)notification).getValue(0);
                if( !StringUtils.equalsIgnoreCase(propertyName, "Catalog.Manager") ) {
                    return;
                }
                DynObject catalogManager = (DynObject) application.getProperty("Catalog.Manager");
                if (catalogManager == null) {
                    return;
                }
                try {
                    String[] catalogNodes = new String[]{
                        "STORES_REPOSITORY_TABLE",
                        "DATABASE_TABLE",
                        "FOLDERS_FILE",
                        "BOOKMARKS_BOOKMARK"
                    };
                    for (String catalogNode : catalogNodes) {
                        Object action = catalogManager.invokeDynMethod(
                                "getAction",
                                new Object[]{
                                    ExportStoreFromCatalogAction.NAME,
                                    catalogNode
                                }
                        );
                        if (action == null) {
                            catalogManager.invokeDynMethod(
                                    "addAction",
                                    new Object[]{
                                        ExportStoreFromCatalogAction.NAME,
                                        catalogNode,
                                        new ExportStoreFromCatalogAction()
                                    }
                            );
                        }
                    }
                } catch (Exception ex) {
                    logger.warn("Can't add action to catalog", ex);
                }
            }
        });
    }

    
}
