/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.app.extension;


import org.gvsig.andami.plugins.Extension;
import org.gvsig.andami.preferences.IPreference;
import org.gvsig.andami.preferences.IPreferenceExtension;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.app.extension.preferences.ExportPreferencesPage;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.export.spi.ExportServiceManager;

/**
 * Andami extension to register the ExportTo preferences panel.
 * 
 * @author gvSIG Team
 * @version $Id$
 */
public class ExportPreferencesExtension 
        extends Extension 
        implements IPreferenceExtension
    {

    @Override
    public void initialize() {
        ExportServiceManager manager = ExportLocator.getServiceManager();

        ExportPreferences preferences = new ExportPreferences();
        for (ExportServiceFactory factory : manager.getAllServiceFactories()) {
            if (preferences.exits(factory.getName())) {
                factory.setEnabled(preferences.isEnabled(factory.getName()));
            } else {
                factory.setEnabled(true);
                preferences.setEnabled(factory.getName(), true);
            }
        }
    }

    @Override
    public void execute(String actionCommand) {
        // Nothing to do
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean isVisible() {
        return false;
    }

    @Override
    public IPreference[] getPreferencesPages() {
        return new IPreference[] { new ExportPreferencesPage() };
    }

}
