package org.gvsig.export.app.extension;

import java.awt.Dimension;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.table.TableDocument;
import org.gvsig.app.project.documents.table.TableManager;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.ExportManager;
import org.gvsig.export.ExportProcess;
import org.gvsig.export.Filter;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.export.swing.ExportFinishListenerAdapter;
import org.gvsig.export.swing.ExportSwingLocator;
import org.gvsig.export.swing.ExportSwingManager;
import org.gvsig.export.swing.JExportProcessPanel;
import org.gvsig.expressionevaluator.Expression;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.OpenDataStoreParameters;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author jjdelcerro
 */
@SuppressWarnings("UseSpecificCatch")
public class ExportTable {
    
    private static final Logger LOGGER = LoggerFactory.getLogger(ExportTable.class);
    
    public ExportTable() {
        
    }
    
    public void showExport(FeatureStore featureStore, Expression filter) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ExportManager manager = ExportLocator.getManager();
        ExportSwingManager swingManager = ExportSwingLocator.getSwingManager();

        final ExportProcess process = manager.createProcess();
        process.setSourceFeatureStore(featureStore);
        process.setContextProjection(null);
        process.setSourceProjection(null);
        process.setSourceTransformation(null);
        process.setFilterExpression(filter);
        
        JExportProcessPanel panel = swingManager.createJExportProcessPanel(
            process,
            new Filter<ExportServiceFactory>() {
                @Override
                public boolean apply(ExportServiceFactory factory) {
                    return factory.hasTabularSupport();
                }
            }
        );
        panel.addFinishListener(new ExportFinishListenerAdapter() {
            @Override
            public void finished(JExportProcessPanel exportPanel) {
                doAddStore(process);
            }
        });
        JComponent component = panel.asJComponent();
        component.setPreferredSize(new Dimension(component.getPreferredSize().width, 530));
                
        WindowManager winmgr = ToolsSwingLocator.getWindowManager();
        winmgr.showWindow(
                panel.asJComponent(), 
                i18n.getTranslation("_Export_to"), 
                WindowManager.MODE.WINDOW
        );
    }
    

    public void doAddStore(ExportProcess process) {
        ApplicationManager application = ApplicationLocator.getManager();
        I18nManager i18n = ToolsLocator.getI18nManager();
        if (process.getTargetOpenStoreParameters()==null) {
            return;
        }
        int n = application.confirmDialog(
                i18n.getTranslation("_Add_new_table_to_current_project"), 
                i18n.getTranslation("_Export"), 
                JOptionPane.YES_NO_OPTION, 
                JOptionPane.QUESTION_MESSAGE, 
                "AddExportedTableToProject"
        );
        if( n != JOptionPane.YES_OPTION ) {
            return;
        }
        try {
            DataManager dataManager = DALLocator.getDataManager();
            ProjectManager projectManager = application.getProjectManager();
            Project project = application.getCurrentProject();
            for (OpenDataStoreParameters params : process.getTargetOpenStoreParameters()) {
                FeatureStore store = (FeatureStore) dataManager.openStore(
                        params.getDataStoreName(), 
                        params
                );
                TableDocument tableDocument = (TableDocument) projectManager.createDocument(
                        TableManager.TYPENAME, 
                        store.getName()
                );
                tableDocument.setStore(store);
                project.addDocument(tableDocument);
            }
            application.messageDialog(
                    i18n.getTranslation("_Table_added_to_project"), 
                    null, 
                    i18n.getTranslation("_Export"), 
                    JOptionPane.QUESTION_MESSAGE, 
                    "ExportedTableAddedToProject"
            );
        } catch (Exception ex) {
            LOGGER.warn("Can't add exported layer to view",ex);
            application.messageDialog(
                    i18n.getTranslation("_Cant_add_exported_table_to_project"), 
                    null, 
                    i18n.getTranslation("_Export"), 
                    JOptionPane.WARNING_MESSAGE, 
                    "CantAddExportedTableToProject"
            );
        }
    }
    
}
