package org.gvsig.export.app.extension;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.swing.AbstractDALActionFactory;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.DataSwingManager;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.gvsig.fmap.dal.swing.DALActionFactory.DALActionContext;

/**
 *
 * @author jjdelcerro
 */
public class FeatureStoreSearchExportAction 
        extends AbstractAction
    {

    public static class FeatureStoreSearchExportActionFactory extends AbstractDALActionFactory {

        public static final String ACTION_NAME = "Export";
    
        public FeatureStoreSearchExportActionFactory() {
            super(ACTION_NAME);
        }
        
        @Override
        public Action createAction(DALActionContext context) {
            return new FeatureStoreSearchExportAction(context);
        }
        
        public static void selfRegister() {
            DataSwingManager dalSwingManager = DALSwingLocator.getSwingManager();
            dalSwingManager.registerStoreAction(new FeatureStoreSearchExportActionFactory());
        }
        
    }

    private static final Logger LOGGER = LoggerFactory.getLogger(FeatureStoreSearchExportAction.class);
    
    private final DALActionContext context;

    public FeatureStoreSearchExportAction(DALActionContext context) {
        this.context = context;
        I18nManager i18n = ToolsLocator.getI18nManager();
        this.putValue(
                Action.ACTION_COMMAND_KEY, 
                FeatureStoreSearchExportActionFactory.ACTION_NAME
        );
        this.putValue(
                Action.SHORT_DESCRIPTION, 
                i18n.getTranslation("_Export")
        );
        this.putValue(
                Action.SMALL_ICON, 
                ToolsSwingLocator.getIconThemeManager().getCurrent().get("table-export")
        );
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        if( !(this.context.getStore() instanceof FeatureStore) ) {
            return;
        }
        ExportTable exportTable = new ExportTable();
        exportTable.showExport(
            (FeatureStore) this.context.getStore(),
            this.context.getFilter()
        );
    }

    
}
