/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.export.app.extension.preferences;

import java.awt.BorderLayout;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.preferences.AbstractPreferencePage;
import org.gvsig.andami.preferences.StoreException;
import org.gvsig.export.ExportLocator;
import org.gvsig.export.app.extension.ExportPreferences;
import org.gvsig.export.spi.ExportServiceFactory;
import org.gvsig.export.spi.ExportServiceManager;
import org.gvsig.export.swing.ExportSwingLocator;
import org.gvsig.export.swing.ExportSwingManager;
import org.gvsig.export.swing.JExportConfigPanel;
import org.gvsig.i18n.Messages;

/**
 * Exportto related preferences page.
 * 
 * @author gvSIG Team
 * @version $Id$
 */
public class ExportPreferencesPage extends AbstractPreferencePage {

    private static final long serialVersionUID = -5751805016601819817L;

    private final JExportConfigPanel config;

    /**
     * Constructor.
     */
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public ExportPreferencesPage() {
        ExportSwingManager manager = ExportSwingLocator.getSwingManager();
        this.config = manager.createJExportConfigPanel();
        this.config.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                setChanged(true);
            }
        });
        this.setLayout(new BorderLayout());
        this.add(this.config.asJComponent(), BorderLayout.CENTER);
    }
    
    @Override
    public boolean isResizeable() {
            return true;
    }

    @Override
    public String getID() {
        return getClass().getName();
    }

    @Override
    public String getTitle() {
        return Messages.getText("exportto_preferences");
    }

    @Override
    public JPanel getPanel() {
        return this;
    }

    @Override
    public void initializeValues() {
        this.config.reset();
    }

    @Override
    public void initializeDefaults() {
        this.config.reset();
    }

    @Override
    public ImageIcon getIcon() {
        return IconThemeHelper.getImageIcon("export-to-preferences");
    }

    @Override
    public boolean isValueChanged() {
        return this.hasChanged();
    }

    @Override
    public void storeValues() throws StoreException {
        this.config.apply();

        // Persist them
        List<String> selecteds = this.config.getSelectedNames();
        ExportPreferences preferences = new ExportPreferences();
        ExportServiceManager manager = ExportLocator.getServiceManager();

        for (ExportServiceFactory factory : manager.getAllServiceFactories()) {
            if (selecteds.contains(factory.getName())) { 
                preferences.setEnabled(factory.getName(),true);
            } else {
                preferences.setEnabled(factory.getName(), false);
            }
        }
        
    }

    @Override
    public void setChangesApplied() {
        this.setChanged(false);
    }

}
