package org.gvsig.geodb;

import com.jeta.forms.components.border.TitledBorderLabel;
import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;


public class AbstractWizardDBView extends JPanel
{
   TitledBorderLabel lblConnection = new TitledBorderLabel();
   JComboBox cboConnection = new JComboBox();
   JButton btnConnection = new JButton();
   JList lstTables = new JList();
   JList lstColumns = new JList();
   JButton btnDeselectAllColumns = new JButton();
   JButton btnSelectAllColumns = new JButton();
   TitledBorderLabel lblTable = new TitledBorderLabel();
   TitledBorderLabel lblColumns = new TitledBorderLabel();
   JLabel lblName = new JLabel();
   JTextField txtName = new JTextField();
   JLabel lblIdField = new JLabel();
   JComboBox cboIdField = new JComboBox();
   JLabel lblGeometryField = new JLabel();
   JComboBox cboGeometryField = new JComboBox();
   JLabel lblProjection = new JLabel();
   JTextField txtProjection = new JTextField();
   JButton btnProjection = new JButton();
   JLabel lblFilter = new JLabel();
   JTextField txtFilter = new JTextField();
   JButton btnFilter = new JButton();
   JButton btnFilterBookmarks = new JButton();
   JButton btnFilterHistory = new JButton();
   JButton btnAdvancedProperties = new JButton();

   /**
    * Default constructor
    */
   public AbstractWizardDBView()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:8DLU:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblConnection.setName("lblConnection");
      lblConnection.setText("_Connection");
      jpanel1.add(lblConnection,cc.xy(2,1));

      jpanel1.add(createPanel1(),cc.xy(2,3));
      jpanel1.add(createPanel2(),cc.xy(2,6));
      jpanel1.add(createPanel4(),cc.xy(2,9));
      btnAdvancedProperties.setActionCommand("_Advanced_properties");
      btnAdvancedProperties.setName("btnAdvancedProperties");
      btnAdvancedProperties.setText("_Advanced_properties");
      jpanel1.add(btnAdvancedProperties,new CellConstraints(2,11,1,1,CellConstraints.RIGHT,CellConstraints.DEFAULT));

      addFillComponents(jpanel1,new int[]{ 1,3,4 },new int[]{ 1,2,3,4,5,6,7,8,9,10,11,12 });
      return jpanel1;
   }

   public JPanel createPanel1()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      cboConnection.setName("cboConnection");
      jpanel1.add(cboConnection,cc.xy(1,1));

      btnConnection.setActionCommand("...");
      btnConnection.setName("btnConnection");
      btnConnection.setText("...");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnConnection.setBorder(emptyborder1);
      jpanel1.add(btnConnection,cc.xy(3,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel2()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(0.5),FILL:4DLU:NONE,FILL:DEFAULT:GROW(0.5)","CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,FILL:DEFAULT:GROW(1.0),CENTER:2DLU:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lstTables.setName("lstTables");
      JScrollPane jscrollpane1 = new JScrollPane();
      jscrollpane1.setViewportView(lstTables);
      jscrollpane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      jpanel1.add(jscrollpane1,cc.xywh(1,3,1,3));

      lstColumns.setName("lstColumns");
      JScrollPane jscrollpane2 = new JScrollPane();
      jscrollpane2.setViewportView(lstColumns);
      jscrollpane2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane2.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      jpanel1.add(jscrollpane2,cc.xy(3,3));

      jpanel1.add(createPanel3(),cc.xy(3,5));
      lblTable.setName("lblTable");
      lblTable.setText("choose_table");
      jpanel1.add(lblTable,cc.xy(1,1));

      lblColumns.setName("lblColumns");
      lblColumns.setText("table_fields");
      jpanel1.add(lblColumns,cc.xy(3,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[]{ 2,4,5 });
      return jpanel1;
   }

   public JPanel createPanel3()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      btnDeselectAllColumns.setActionCommand("Ninguno");
      btnDeselectAllColumns.setName("btnDeselectAllColumns");
      btnDeselectAllColumns.setText("_None");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnDeselectAllColumns.setBorder(emptyborder1);
      jpanel1.add(btnDeselectAllColumns,cc.xy(4,1));

      btnSelectAllColumns.setActionCommand("Todos");
      btnSelectAllColumns.setName("btnSelectAllColumns");
      btnSelectAllColumns.setText("_All");
      EmptyBorder emptyborder2 = new EmptyBorder(2,2,2,2);
      btnSelectAllColumns.setBorder(emptyborder2);
      jpanel1.add(btnSelectAllColumns,cc.xy(2,1));

      addFillComponents(jpanel1,new int[]{ 1,3 },new int[]{ 1 });
      return jpanel1;
   }

   public JPanel createPanel4()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0)","CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblName.setName("lblName");
      lblName.setText("_Name");
      jpanel1.add(lblName,cc.xy(1,2));

      txtName.setName("txtName");
      jpanel1.add(txtName,cc.xy(3,2));

      lblIdField.setName("lblIdField");
      lblIdField.setText("_Id_field");
      jpanel1.add(lblIdField,cc.xy(1,4));

      cboIdField.setName("cboIdField");
      jpanel1.add(cboIdField,cc.xy(3,4));

      lblGeometryField.setName("lblGeometryField");
      lblGeometryField.setText("_Geo_field");
      jpanel1.add(lblGeometryField,cc.xy(1,6));

      cboGeometryField.setName("cboGeometryField");
      jpanel1.add(cboGeometryField,cc.xy(3,6));

      lblProjection.setName("lblProjection");
      lblProjection.setText("_Projection");
      jpanel1.add(lblProjection,cc.xy(1,8));

      jpanel1.add(createPanel5(),cc.xy(3,8));
      lblFilter.setName("lblFilter");
      lblFilter.setText("_Filter");
      jpanel1.add(lblFilter,cc.xy(1,10));

      jpanel1.add(createPanel6(),cc.xy(3,10));
      addFillComponents(jpanel1,new int[]{ 1,2,3 },new int[]{ 1,3,5,7,9 });
      return jpanel1;
   }

   public JPanel createPanel5()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE","FILL:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtProjection.setName("txtProjection");
      jpanel1.add(txtProjection,cc.xy(1,1));

      btnProjection.setActionCommand("...");
      btnProjection.setName("btnProjection");
      btnProjection.setText("...");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnProjection.setBorder(emptyborder1);
      jpanel1.add(btnProjection,cc.xy(3,1));

      addFillComponents(jpanel1,new int[]{ 2 },new int[0]);
      return jpanel1;
   }

   public JPanel createPanel6()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      txtFilter.setName("txtFilter");
      jpanel1.add(txtFilter,cc.xy(1,1));

      btnFilter.setActionCommand("...");
      btnFilter.setName("btnFilter");
      btnFilter.setText("...");
      EmptyBorder emptyborder1 = new EmptyBorder(2,2,2,2);
      btnFilter.setBorder(emptyborder1);
      jpanel1.add(btnFilter,cc.xy(3,1));

      btnFilterBookmarks.setActionCommand("...");
      btnFilterBookmarks.setName("btnFilterBookmarks");
      btnFilterBookmarks.setText("...");
      EmptyBorder emptyborder2 = new EmptyBorder(2,2,2,2);
      btnFilterBookmarks.setBorder(emptyborder2);
      jpanel1.add(btnFilterBookmarks,cc.xy(7,1));

      btnFilterHistory.setActionCommand("...");
      btnFilterHistory.setName("btnFilterHistory");
      btnFilterHistory.setText("...");
      EmptyBorder emptyborder3 = new EmptyBorder(2,2,2,2);
      btnFilterHistory.setBorder(emptyborder3);
      jpanel1.add(btnFilterHistory,cc.xy(5,1));

      addFillComponents(jpanel1,new int[]{ 2,4,6 },new int[0]);
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
