package org.gvsig.geodb;

import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.store.db.DBStoreParameters;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;
import org.gvsig.fmap.mapcontext.layers.CancelationException;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontrol.MapControl;
import static org.gvsig.geodb.AbstractWizardDB.LOGGER;

/**
 *
 * @author jjdelcerro
 */
public class WizardDBVectorial extends AbstractWizardDB {

    public WizardDBVectorial() {
        super();
    }

    @Override
    protected boolean requireGeometry() {
        return true;
    }

    @Override
    public Object executeWizard() {
        DataManager man = DALLocator.getDataManager();
        MapContextManager mapContextManager = MapContextLocator.getMapContextManager();

        MapControl mapControl = this.getMapCtrl();
        MapContext mapContext = mapControl.getMapContext();

        String groupName = null;
        List<FLayer> all_layers = new ArrayList();

        try {
            FLayer layer = null;
            for (TableInfo tableInfo : this.getTablesInformation()) {
                if (!tableInfo.isSelected()) {
                    continue;
                }
                DBStoreParameters parameter = tableInfo.getParameters();
                if (groupName == null) {
                    String dbname = parameter.getDBName();
                    if (StringUtils.isEmpty(dbname)) {
                        dbname = "Group";
                    }
                    String host = parameter.getHost();
                    if (StringUtils.isEmpty(host)) {
                        groupName = dbname;
                    } else {
                        groupName = dbname + " (" + host + ")";
                    }
                }
                FeatureStore store;
                try {
                    store = (FeatureStore) man.openStore(
                            parameter.getDataStoreName(),
                            parameter);
                } catch (Exception e) {
                    LOGGER.warn("While opening DB store.", e);
                    return null;
                }
                try {

                    layer = mapContextManager.createLayer(
                            tableInfo.getDocumentName(),
                            store
                    );
                    all_layers.add(layer);
                } catch (Exception e) {
                    LOGGER.warn("Can't create layer '" + tableInfo.getDocumentName() + "'.", e);
                }
            }

            if (all_layers.size() > 1) {
                FLayers group = new FLayers();
                group.setMapContext(mapContext);
                group.setParentLayer(mapContext.getLayers());
                group.setName(groupName);

                for (FLayer aLayer : all_layers) {
                    group.addLayer(aLayer);
                }
                if ( group.isOk() ) {
                    group.setVisible(true);
                    mapContext.beginAtomicEvent();
                    try {
                        mapControl.addLayer(group);
                        group.dispose();
                    } catch (CancelationException e) {
                        LOGGER.warn("Can't add group to mapcontext layers.", e);
                    } finally {
                        mapContext.endAtomicEvent();
                    }
                } else {
                    fireLayerErrorEvent(layer, mapControl);
                }

            } else if (all_layers.size() == 1) {
                layer = (FLayer) all_layers.get(0);
                if ( layer.isOk() ) {
                    layer.setVisible(true);
                    mapContext.beginAtomicEvent();
                    try {
                        mapControl.addLayer(layer);
                    } catch (CancelationException e) {
                        LOGGER.warn("Can't add group to mapcontext layers.", e);
                    } finally {
                        mapContext.endAtomicEvent();
                    }
                } else {
                    fireLayerErrorEvent(layer, mapControl);
                }
            }

        } finally {
            // Dispose all created layers. If they have been included into
            // a FLayers object, they will have been binded there.
            for (FLayer layer : all_layers) {
                layer.dispose();
            }
        }
        return null;
    }

    private void fireLayerErrorEvent(FLayer lyr, MapControl mapControl) {
        mapControl.getMapContext().callNewErrorEvent(null);
    }

}
