package org.gvsig.geodb.databaseworkspace;

import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.apache.commons.lang.StringUtils;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DatabaseWorkspaceManager;
import static org.gvsig.fmap.dal.DatabaseWorkspaceManager.CONFIG_NAME_STORESREPOSITORYID;
import static org.gvsig.fmap.dal.DatabaseWorkspaceManager.CONFIG_NAME_STORESREPOSITORYLABEL;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.tools.swing.api.Component;
import org.gvsig.tools.swing.api.pickercontroller.PickerController;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;

/**
 *
 * @author jjdelcerro
 */
public class ConnectToDatabaseWorkspacePanel 
        extends ConnectToDatabaseWorkspacePanelView
        implements Component
    {

    private Dialog dialog;
    private PickerController<JDBCServerExplorerParameters> connectionPicker;

    public ConnectToDatabaseWorkspacePanel() {
        this.initComponents();
    }
    
    public boolean connect() {
        JDBCServerExplorerParameters conn = getConnection();
        if( conn == null ) {
            return false;
        }
        DataManager dataManager = DALLocator.getDataManager();
        DatabaseWorkspaceManager workspace = dataManager.createDatabaseWorkspaceManager(conn);
        dataManager.addDatabaseWorkspace(workspace);
        return true;
    }

    private void initComponents() {
        this.connectionPicker = DALSwingLocator.getSwingManager().createJDBCConnectionPickerController(
                this.cboConnection,
                this.btnConnection
        );
        this.connectionPicker.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                doUpdateStoresRepositoryInfo();
            }
        });
        doUpdateStoresRepositoryInfo();
    }

    @Override
    public JComponent asJComponent() {
        return this;
    }

    public JDBCServerExplorerParameters getConnection() {
        JDBCServerExplorerParameters conn = this.connectionPicker.get();
        if (conn == null) {
            return null;
        }
        DataManager dataManager = DALLocator.getDataManager();
        DatabaseWorkspaceManager workspace = dataManager.createDatabaseWorkspaceManager(conn);
        if( !workspace.isValidStoresRepository() ) {
            return null;
        }
        return conn;
    }

    private void doUpdateEnableComponents() {
        boolean theEnabled = this.getConnection() != null;
        if( dialog!=null ) {
            this.dialog.setButtonEnabled(WindowManager_v2.BUTTONS_OK, theEnabled);
        }
    }
    private void doUpdateStoresRepositoryInfo() {
        if( this.getConnection()==null ) {
            return;
        }
        DataManager dataManager = DALLocator.getDataManager();
        DatabaseWorkspaceManager workspace = dataManager.createDatabaseWorkspaceManager(getConnection());
        String id = workspace.get(CONFIG_NAME_STORESREPOSITORYID);
        String label = workspace.get(CONFIG_NAME_STORESREPOSITORYLABEL);
        this.txtID.setText(StringUtils.defaultIfBlank(id, ""));
        this.txtLabel.setText(StringUtils.defaultIfBlank(label, ""));
        
        this.doUpdateEnableComponents();
    }
    
    
    void setDialog(Dialog dialog) {
       this.dialog = dialog;
       this.doUpdateEnableComponents();
    }
    
}
