package org.gvsig.geodb.databaseworkspace;

import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.Project;
import org.gvsig.app.project.documents.Document;
import org.gvsig.app.project.documents.table.TableDocument;
import org.gvsig.app.project.documents.table.TableManager;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.operations.SingleLayer;
import org.gvsig.tools.swing.api.Component;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.util.LabeledValue;
import org.gvsig.tools.util.LabeledValueImpl;

/**
 *
 * @author jjdelcerro
 */
public class RepositoryAddTablePanel 
        extends RepositoryAddTablePanelView 
        implements Component
    {

    public RepositoryAddTablePanel() {
        this.initComponents();
    }
    
    private void initComponents() {
        ApplicationManager application = ApplicationLocator.getManager();
        ToolsSwingManager toolsSwingManager = ToolsSwingLocator.getToolsSwingManager();
        
        TreeModel layersModel = application.createProjectLayersTreeModel();
        this.treeLayers.setModel(layersModel);
        this.treeLayers.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener() {
            @Override
            public void valueChanged(TreeSelectionEvent e) {
                 doUpdateTableName();
            }
        });
        
        Project project = application.getCurrentProject();
        final List<Document> docs = project.getDocuments(TableManager.TYPENAME);
        if( docs!=null ) {
            DefaultListModel<LabeledValue<FeatureStore>> tablesModel = new DefaultListModel<>();
            for (Document doc : docs) {
                TableDocument table = (TableDocument) doc;
                tablesModel.addElement(
                        new LabeledValueImpl<>(table.getName(), table.getFeatureStore())
                );
            }
            this.lstTables.setModel(tablesModel);
            this.lstTables.addListSelectionListener(new ListSelectionListener() {
                @Override
                public void valueChanged(ListSelectionEvent e) {
                    if( e.getValueIsAdjusting() ) {
                        return;
                    }
                    doUpdateTableName();
                }
            });
            
            toolsSwingManager.addClearButton(this.txtTableName);
            toolsSwingManager.setDefaultPopupMenu(this.txtTableName);
        }

    }
    
    @Override
    public JComponent asJComponent() {
        return this;
    }
    
    private void doUpdateTableName() {
        switch(this.tabStores.getSelectedIndex()) {
            case 0: // Layers
                TreePath selected = this.treeLayers.getSelectionPath();
                FLayer layer = (FLayer) selected.getLastPathComponent();
                if( layer instanceof SingleLayer ) {
                    this.txtTableName.setText(layer.getName());
                }
                break;
            case 1: // Table documents
                LabeledValue<FeatureStore> item = (LabeledValue<FeatureStore>) this.lstTables.getSelectedValue();
                if( item != null ) {
                    this.txtTableName.setText(item.getLabel());
                }
                break;
            default:
                break;
        }
    }
    
    public String getTableName() {
        return StringUtils.defaultIfBlank(this.txtTableName.getText(),null);
    }
    
    public DataStoreParameters getDataStoreParameters() {
        switch(this.tabStores.getSelectedIndex()) {
            case 0: // Layers
                TreePath selected = this.treeLayers.getSelectionPath();
                FLayer layer = (FLayer) selected.getLastPathComponent();
                if( layer instanceof SingleLayer ) {
                    return ((SingleLayer) layer).getDataStore().getParameters();
                }
                return null;
            case 1: // Table documents
                LabeledValue<FeatureStore> item = (LabeledValue<FeatureStore>) this.lstTables.getSelectedValue();
                if( item == null ) {
                    return null;
                }
                return item.getValue().getParameters();
            default:
                return null;
        }
    }
}
