/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.geodb.vectorialdb.wizard;

import java.awt.Window;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.StringUtils;

import org.cresques.cts.IProjection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.gvsig.andami.PluginServices;
import org.gvsig.andami.messages.NotificationManager;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.app.prepareAction.PrepareContext;
import org.gvsig.app.prepareAction.PrepareContextView;
import org.gvsig.app.project.documents.view.gui.DefaultViewPanel;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.serverexplorer.db.DBServerExplorer;
import org.gvsig.fmap.dal.store.db.DBStoreParameters;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;
import org.gvsig.fmap.mapcontext.layers.CancelationException;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontrol.MapControl;



/**
 * Driver-independent GeoDB wizard. Queries the drivers to fill GUI controls.
 * Multi-table selection available.
 *
 * @author jldominguez
 *
 */
public class WizardVectorialDB extends WizardDB {
    private static final long serialVersionUID = -5002685263220038989L;

    private static final Logger logger = LoggerFactory.getLogger(WizardVectorialDB.class);

    private static final String GEODB_WIZARD_TAB_NAME = "GeoDB";
	private DefaultViewPanel view = null;

	private PrepareContextView prepareContext;

    /**
     * This method initializes this
     *
     */
    @Override
    protected void initialize() {
		super.initialize();
        setTabName(GEODB_WIZARD_TAB_NAME);
        setLayout(null);
        setSize(512, 478);

        IWindow iw = PluginServices.getMDIManager().getActiveWindow();

        if (iw == null) {
            return;
        }

        if ((iw instanceof DefaultViewPanel)) {
        	 view = (DefaultViewPanel) iw;
             setMapCtrl(view.getMapControl());
        }
    }
    
    @Override
    protected List getTableList(DBServerExplorer explorer) throws DataException {
        return explorer.list(DBServerExplorer.MODE_GEOMETRY);
    }

    @Override
	protected TablesListItem createTabeListItem(DBStoreParameters param) {
		return new TablesListItemVectorial(param, getMapCtrl(),this);
	}

    @Override
	public void setSettingsPanels(TablesListItem actTable) {
		super.setSettingsPanels(actTable);
	}

    @Override
	protected UserTableSettingsPanel createSettingsPanel(
			TablesListItem actTable) {
		if (actTable == null) {
			return new UserTableSettingsVectorialPanel(null, null, "",
					getMapCtrl(), true, this, null, null);
		}
		String abrev = null;
		if (getMapCtrl() != null) {
			abrev = getMapCtrl().getViewPort().getProjection().getAbrev();
		}

		return ((TablesListItemVectorial) actTable)
				.getUserTableSettingsPanel(abrev);
	}

    @Override
	public DataStoreParameters[] getParameters() {
		try {
			TablesListItem[] selected = getSelectedTables();
			int count = selected.length;
			DBStoreParameters[] dbParameters = new DBStoreParameters[count];
			String strEPSG = null;
			if (getMapCtrl() != null) {
				strEPSG = getMapCtrl().getViewPort().getProjection().getAbrev();
			}

			for (int i = 0; i < count; i++) {
				TablesListItemVectorial item = (TablesListItemVectorial) selected[i];

				dbParameters[i] = getParameterForTable(item);
			}

			return dbParameters;
		} catch (Exception e) {
			logger.warn("Error while creating jdbc layer: " + e.getMessage(), e);
		}

		return null;
	}

    @Override
	public void execute() {
        executeWizard();
    }

    @Override
	public Object executeWizard() {
		MapControl mapControl = this.getMapCtrl();
        MapContext mc = mapControl.getMapContext();
        MapContextManager mapContextManager = MapContextLocator.getMapContextManager();
//        Boolean needCalculateEnvelope  = mc.getViewPort().getEnvelope() == null;
		TablesListItem[] tables = getSelectedTables();

		List<FLayer> all_layers = new ArrayList();
		String groupName = null;
		DBStoreParameters parameter;
		TablesListItem table;
		FeatureStore store;

		DataManager man = DALLocator.getDataManager();

        try {

            FLayer layer;
            for (int i = 0; i < tables.length; i++) {
                table = tables[i];
                UserTableSettingsPanel userTableSettingsPanel = table.getUserTableSettingsPanel();
                parameter = getParameterForTable(table);
                if (i == 0) {
                    String dbname = parameter.getDBName();
                    if( StringUtils.isEmpty(dbname) ) {
                        dbname = "Group";
                    }
                    String host = parameter.getHost();
                    if( StringUtils.isEmpty(host) ) {
                        groupName =  dbname;
                    } else {
                        groupName =  dbname + " (" + host + ")";
                    }
                }
                try {
                    store = (FeatureStore) man.openStore(
                    		parameter.getDataStoreName(),
                    		parameter);
                } catch (Exception e) {
                    logger.warn("While opening DB store.", e);
                    return null;
                }
                try {

                    layer = mapContextManager.createLayer(
                        userTableSettingsPanel.getUserLayerName(), 
                        store
                    );
                    all_layers.add(layer);
                } catch (Exception e) {
                    logger.warn("Can't create layer '"+userTableSettingsPanel.getUserLayerName()+"'.",e);
                }
            }

            if (all_layers.size() > 1) {
                FLayers group = new FLayers();
                group.setMapContext(mc);
                group.setParentLayer(mc.getLayers());
                group.setName(groupName);

                for( FLayer aLayer : all_layers) {
                    group.addLayer(aLayer);
                }
                if ( !(group.isOk())) {
                    // if the layer is not okay (it has errors) process them
                    processErrorsOfLayer(group, mapControl);
                }

                group.setVisible(true);
                mc.beginAtomicEvent();
                try {
                    try {
                        mapControl.addLayer(group);
                        group.dispose();
                    } catch (CancelationException e) {
                        logger.warn("Can't add group to mapcontext layers.",e);
                    }

                } finally {
                    mc.endAtomicEvent();
                }
                return null;
            } else if (all_layers.size() == 1) {
                layer = (FLayer) all_layers.get(0);
                if (!(layer.isOk())) {
                    // if the layer is not okay (it has errors) process them
                    processErrorsOfLayer(layer, mapControl);
                }

                layer.setVisible(true);
                mc.beginAtomicEvent();
                try {
                    try {
                        mapControl.addLayer(layer);
                    } catch (CancelationException e) {
                        return null;
                    }
                } finally {
                    mc.endAtomicEvent();
                }
                return null;

            }
//            if ( needCalculateEnvelope ) {
//        		Envelope env = null;
//                for( FLayer theLayer : all_layers) {
//                    try {
//                        if (env == null) {
//                            env = theLayer.getFullEnvelope();
//                        } else {
//                            env.add(theLayer.getFullEnvelope());
//                        }
//                    } catch(Exception ex) {
//                        // Ignore
//                    }
//                    
//                }                
//                mapControl.getViewPort().setEnvelope(env);
//            }            
            
        } finally {
            // Dispose all created layers. If they have been included into
            // a FLayers object, they will have been binded there.
            for (int i = 0; i < all_layers.size(); i++) {
                FLayer layer = (FLayer) all_layers.get(i);
                layer.dispose();
            }
		}
        return null;
	}

    @Override
	protected PrepareContext getPrepareDataStoreContext() {
		if (this.prepareContext == null) {
			this.prepareContext = new PrepareContextView() {
                @Override
				public Window getOwnerWindow() {
					return null;
				}

                @Override
				public MapControl getMapControl() {
					return WizardVectorialDB.this
							.getMapCtrl();
				}
				
                @Override
				public IProjection getViewProjection() {
					MapControl mapControl = getMapControl();
					if (mapControl!=null){
						return mapControl.getProjection();
					}
					return null;
				}

			};
		}
		return this.prepareContext;
	}

    @Override
	protected DBStoreParameters getParameterForTable(TablesListItem table) {
		DBStoreParameters parameters = table.getParameters();

		UserTableSettingsVectorialPanel userTableSettingsPanel = (UserTableSettingsVectorialPanel) table
				.getUserTableSettingsPanel();

		Envelope _wa = userTableSettingsPanel.getWorkingArea();

		String geomField = userTableSettingsPanel.getGeoFieldName();
		String fidField = userTableSettingsPanel.getIdFieldName();
        //IF is a multiple PK, remove the {} symbols
        if (fidField.startsWith("{") && fidField.endsWith("}")) {
            fidField = fidField.substring(1, fidField.length()-1);
        }
        String[] pkFields = fidField.split(",");        
            
        String[] fields = table.getUserSelectedFieldsPanel()
                .getUserSelectedFields(pkFields, geomField);

		if (userTableSettingsPanel.isSqlActive()) {
			String whereClause = userTableSettingsPanel.getWhereClause();
			parameters.setBaseFilter(whereClause);
		} else {
			parameters.setBaseFilter("");
		}

		parameters.setFields(fields);
		parameters.setDefaultGeometryField(geomField);
		if (_wa != null) {
			parameters.setWorkingArea(_wa);
		}
		
        // ======= Setting PK from combo box
        if (pkFields != null) {
            parameters.setPkFields(pkFields);
        }
        // =========================

		IProjection proj_selected_by_user = userTableSettingsPanel.getProjection();
		parameters.setCRS(proj_selected_by_user);

		return parameters;

	}


}