
package org.gvsig.h2spatial;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.BindException;
import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.filedialogchooser.FileDialogChooser;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.serverexplorer.filesystem.FilesystemStoreParameters;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorer;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel;
import org.gvsig.fmap.geom.primitive.Envelope;
import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.MapContextManager;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.folders.FoldersManager;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;
import org.gvsig.webbrowser.WebBrowserFactory;
import org.gvsig.webbrowser.WebBrowserPanel;
import org.h2.tools.Server;


public class H2SpatialExtension extends Extension {

    public static String DAL_EXPLORER_NAME = "H2Spatial";
    
    @Override
    public void initialize() {
    }

    @Override
    public boolean canQueryByAction() {
        return true;
    }
    
    @Override
    public void postInitialize() {
        MapContextManager mapContextMgr = MapContextLocator.getMapContextManager();
        
        IconThemeHelper.registerIcon("layer", "layer-icon-h2", this);
        mapContextMgr.registerIconLayer("H2Spatial", "layer-icon-h2");
    }
    
    @Override
    public void execute(String actionCommand) {
        switch(actionCommand.toLowerCase()) {
        case "tools-h2spatial-show-admintool":
            startserver();
            startbrowser();
            break;
        
        case "tools-h2spatial-linklayer":
            linkActiveLayers();
            break;

        case "tools-h2spatial-import":
            importTables();
            break;

        case "tools-h2spatial-downloadosm":
            downloadOSM();
            break;
        }
    }
    
    private void startserver()  {
        try {
            Server server = Server.createWebServer("-webPort","8082");
            server.start();
        } catch (SQLException ex) {
            if( ex.getCause() instanceof BindException ) {
                logger.warn("Can't start H2 web server,  Already running ?");
            } else {
                logger.warn("Can't start H2 web server.",ex);
            }
        }
    }

    private List<FLyrVect> getSelectedLayers() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument viewdoc = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( viewdoc == null ) {
            return null;
        }
        FLayer[] layers = viewdoc.getMapContext().getLayers().getActives();
        if( ArrayUtils.isEmpty(layers) ) {
            return null;
        }
        List<FLyrVect> r = new ArrayList<>();
        for( FLayer layer : layers ) {
            if( layer instanceof FLyrVect ) {
                FeatureStore store = ((FLyrVect)layer).getFeatureStore();
                if( store != null ) {
                    if( store.getParameters() instanceof FilesystemStoreParameters &&
                        StringUtils.equalsIgnoreCase(store.getProviderName(),"Shape") ) {
                        r.add((FLyrVect) layer);
                    }
                }
            }
        }
        if( r.isEmpty() ) {
            return null;
        }
        return r;
    }
    
    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean isVisible() {
        return true;
    }

    @Override
    public boolean isEnabled(String action) {
        ApplicationManager application = ApplicationLocator.getManager();

        if( "tools-h2spatial-show-admintool".equalsIgnoreCase(action) ) {
            return true;
        } 
        if( "tools-h2spatial-linklayer".equalsIgnoreCase(action) ) {
            return ! CollectionUtils.isEmpty(this.getSelectedLayers());
        }
        if( "tools-h2spatial-import".equalsIgnoreCase(action) ) {
            return true;
        }
        if( "tools-h2spatial-downloadosm".equalsIgnoreCase(action) ) {
            ViewDocument viewDoc = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
            if( viewDoc == null ) {
                return false;
            }
            Envelope env = viewDoc.getMapContext().getViewPort().getEnvelope();
            return !(env != null && env.isEmpty());
        }
        return false;
    }

    @Override
    public boolean isVisible(String action) {
        return true;
    }

    private void startbrowser() {
        WindowManager winmanager = ToolsSwingLocator.getWindowManager();
        WebBrowserPanel webbrowser = WebBrowserFactory.createWebBrowserPanel();
        winmanager.showWindow(
            webbrowser.asJComponent(), 
            ToolsLocator.getI18nManager().getTranslation("_H2_administration_tool"), 
            WindowManager.MODE.WINDOW
        );
        webbrowser.setPage("http://localhost:8082");
        
    }
    
    private void importTables() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        WindowManager_v2 winmanager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
        final ImportTable panel = new ImportTable();
        final Dialog dlg = winmanager.createDialog(
            panel,
            i18n.getTranslation("_Select_files_to_import"),
            i18n.getTranslation("_Select_files_to_import_as_tables_in_H2_database"),
            WindowManager_v2.BUTTONS_OK_CANCEL
        );
        dlg.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if( dlg.getAction()==WindowManager_v2.BUTTONS_OK ) {
                    doImportTables(
                        panel.getExplorerParameters(),
                        panel.getFileType(),
                        panel.getFile(),
                        panel.getTableName(),
                        panel.getDeleteTables()
                    );
                }
            }
        });
        dlg.show(WindowManager.MODE.WINDOW);
    }
    
    private void doImportTables(JDBCServerExplorerParameters explorerParameters, String fileType, File file, String tableName, boolean deleteTables) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ApplicationManager application = ApplicationLocator.getManager();

        StringBuilder sql = new StringBuilder();
        switch(fileType.toLowerCase()) {
        case "gpx":
            sql.append("CALL GPXRead('");
            sql.append(file.getAbsolutePath());
            sql.append("', '");
            sql.append(tableName);
            sql.append("');");
            break;
        case "geojson":
            sql.append("CALL GeoJsonRead('");
            sql.append(file.getAbsolutePath());
            sql.append("', '");
            sql.append(tableName);
            sql.append("');");
            break;
        case "osm":
            sql.append("CALL OSMRead('");
            sql.append(file.getAbsolutePath());
            sql.append("', '");
            sql.append(tableName);
            sql.append("', ");
            sql.append(BooleanUtils.toStringTrueFalse(deleteTables));
            sql.append(");");
            break;
        default:
            return;
        }
        try {
            DataManager dataManager = DALLocator.getDataManager();
            JDBCServerExplorer explorer = (JDBCServerExplorer) dataManager.openServerExplorer(
                explorerParameters.getExplorerName(), 
                explorerParameters
            );
            explorer.execute(sql.toString());
        } catch(Exception ex) {
            logger.warn("Can't import file.",ex);
            application.messageDialog(
                i18n.getTranslation("_Cant_import_file.") + 
                    "\n" + 
                    i18n.getTranslation("_See_the_log_file_to_more_information"), 
                i18n.getTranslation("_Warning"), 
                JOptionPane.WARNING_MESSAGE
            );
        }
    }
    
    private void linkActiveLayers() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        final List<FLyrVect> layers = this.getSelectedLayers();
        if( CollectionUtils.isEmpty(layers) ) {
            return;
        }
        final JDBCConnectionPanel jdbcConnectionPanel = DALSwingLocator.getSwingManager().createJDBCConnectionPanel();
        jdbcConnectionPanel.setConnectorFilter(new JDBCConnectionPanel.ConnectorFilter() {
            @Override
            public boolean accept(JDBCServerExplorerParameters parameters) {
                return StringUtils.equalsIgnoreCase(parameters.getExplorerName(), DAL_EXPLORER_NAME);
            }
        });
        WindowManager_v2 winmanager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
        final Dialog dlg = winmanager.createDialog(
            jdbcConnectionPanel.asJComponent(),
            i18n.getTranslation("_Select_database"),
            null, 
            WindowManager_v2.BUTTONS_OK_CANCEL
        );
        dlg.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if( dlg.getAction()==WindowManager_v2.BUTTONS_OK ) {
                    JDBCServerExplorerParameters params = jdbcConnectionPanel.getServerExplorerParameters();
                    linkLayers(layers, params);
                }
            }
        });
        dlg.show(WindowManager.MODE.WINDOW);
    }
    
    private void linkLayers(List<FLyrVect> layers, JDBCServerExplorerParameters params) {
        try {
            JDBCServerExplorer explorer = (JDBCServerExplorer) DALLocator.getDataManager().openServerExplorer(
                params.getExplorerName(),
                params
            );
            for( FLyrVect layer : layers ) {
                FilesystemStoreParameters layerParameters = (FilesystemStoreParameters) 
                    layer.getFeatureStore().getParameters();
                File f = layerParameters.getFile();
                String sql = MessageFormat.format(
                    "CALL FILE_TABLE(''{0}'', ''{1}'')",
                    FilenameUtils.normalize(f.getAbsolutePath(), true),
                    layer.getName()
                );
                explorer.execute(sql);
            }
        } catch (Exception ex) {
        }
    }

    private void downloadOSM() {
        
        I18nManager i18n = ToolsLocator.getI18nManager();
        FoldersManager foldersManager = ToolsLocator.getFoldersManager();
        ApplicationManager application = ApplicationLocator.getManager();
        
        ViewDocument viewDoc = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( viewDoc == null ) {
            return;
        }
        final Envelope env = viewDoc.getMapContext().getViewPort().getEnvelope();
        final File[] files = application.showChooserDialog(
            i18n.getTranslation("_Select_the_file_in_which_to_save_the_data"), 
            FileDialogChooser.SAVE_DIALOG, 
            FileDialogChooser.FILES_ONLY, 
            false, 
            foldersManager.get("DataFolder"), 
            new FileNameExtensionFilter(
                "OSM File (.osm, .osm.gz, .osm.bz2)",
                ".osm", ".osm.gz", ".osm.bz2"
            ), 
            false
        );
        
        if( !ArrayUtils.isEmpty(files) ) {
            final JDBCConnectionPanel jdbcConnectionPanel = DALSwingLocator.getSwingManager().createJDBCConnectionPanel();
            jdbcConnectionPanel.setConnectorFilter(new JDBCConnectionPanel.ConnectorFilter() {
                @Override
                public boolean accept(JDBCServerExplorerParameters parameters) {
                    return StringUtils.equalsIgnoreCase(parameters.getExplorerName(), DAL_EXPLORER_NAME);
                }
            });
            WindowManager_v2 winmanager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
            final Dialog dlg = winmanager.createDialog(
                jdbcConnectionPanel.asJComponent(),
                i18n.getTranslation("_Select_database"),
                null, 
                WindowManager_v2.BUTTONS_OK_CANCEL
            );
            dlg.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    if( dlg.getAction()==WindowManager_v2.BUTTONS_OK ) {
                        doDownloadOSM( 
                            env, 
                            files[0], 
                            jdbcConnectionPanel.getServerExplorerParameters()
                        );
                    }
                }
            });
            dlg.show(WindowManager.MODE.WINDOW);
        }
    }
    
    private void doDownloadOSM(Envelope env, File f, JDBCServerExplorerParameters explorerParameters) {
        I18nManager i18n = ToolsLocator.getI18nManager();
        ApplicationManager application = ApplicationLocator.getManager();
        
        try {
            String sql = MessageFormat.format(
                "CALL ST_OSMDownloader(''{0}''::geometry, ''{1}'');",
                env.getGeometry().convertToWKT(),
                f.getAbsolutePath()
            );
            DataManager dataManager = DALLocator.getDataManager();
            JDBCServerExplorer explorer = (JDBCServerExplorer) dataManager.openServerExplorer(
                explorerParameters.getExplorerName(), 
                explorerParameters
            );
            explorer.execute(sql);
        } catch(Exception ex) {
            logger.warn("Can't download from OSM.",ex);
            application.messageDialog(
                i18n.getTranslation("_Cant_download_from_OSM.") + 
                    "\n" + 
                    i18n.getTranslation("_See_the_log_file_to_more_information"), 
                i18n.getTranslation("_Warning"), 
                JOptionPane.WARNING_MESSAGE
            );
        }
    }
}
