
package org.gvsig.h2spatial;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.DataServerExplorerPool;
import org.gvsig.fmap.dal.DataServerExplorerPoolEntry;
import org.gvsig.fmap.dal.store.jdbc.JDBCServerExplorerParameters;
import org.gvsig.fmap.dal.swing.DALSwingLocator;
import org.gvsig.fmap.dal.swing.jdbc.JDBCConnectionPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.folders.FoldersManager;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.swing.api.ListElement;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.ToolsSwingManager;
import org.gvsig.tools.swing.api.threadsafedialogs.ThreadSafeDialogsManager;
import org.gvsig.tools.swing.api.windowmanager.Dialog;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager_v2;


public class ImportTable extends ImportTableView {
    
    private static final long serialVersionUID = 3084071737749916812L;
    
    private File file = null;
    
    public ImportTable() {
        this.initComponents();
    }
    
    private void initComponents() {
        ToolsSwingManager toolsSwingManager = ToolsSwingLocator.getToolsSwingManager();
        toolsSwingManager.translate(lblConnection);
        toolsSwingManager.translate(lblFileType);
        toolsSwingManager.translate(lblFile);
        toolsSwingManager.translate(lblTableName);
        toolsSwingManager.translate(lblDeleteTables);
        
        loadConnectionCombo(this.cboConnection, null);
        this.btnConnection.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doConnectionExplore();
            }
        });
        this.cboFileType.removeAllItems();
        this.cboFileType.addItem("GPX");
        this.cboFileType.addItem("GeoJSon");
        this.cboFileType.addItem("OSM");
        this.btnFile.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                doFileExplore();
            }
        });
        
    }
    
    public JDBCServerExplorerParameters getExplorerParameters() {
        ListElement<JDBCServerExplorerParameters> item = (ListElement<JDBCServerExplorerParameters>) this.cboConnection.getSelectedItem();
        if( item == null ) {
            return null;
        }
        return item.getValue();
    }
    
    public String getFileType() {
        return (String) this.cboFileType.getSelectedItem();
    }
    
    public File getFile() {
        return this.file;
    }
    
    public String getTableName() {
        String s = this.txtTableName.getText();
        if( StringUtils.isEmpty(s) ) {
            if( this.file == null ) {
                return null;
            }
            return FilenameUtils.getBaseName(this.file.getName());
        }
        return s.trim();
    }
    
    public boolean getDeleteTables() {
        return this.chkDeleteTables.isSelected();
    }
    
    private void loadConnectionCombo(JComboBox combo, String selected) {
        combo.removeAllItems();
        combo.addItem("");
        DataManager dataManager = DALLocator.getDataManager();
        DataServerExplorerPool explorersPool = dataManager.getDataServerExplorerPool();
        for( DataServerExplorerPoolEntry entry : explorersPool) {
            if( entry.getExplorerParameters() instanceof JDBCServerExplorerParameters ) {
                ListElement<JDBCServerExplorerParameters> item = new ListElement(
                    entry.getName(),
                    entry.getExplorerParameters()
                );
                combo.addItem(item);
                if( StringUtils.equalsIgnoreCase(selected, item.toString()) ) {
                    combo.setSelectedItem(item);
                }
            }
        }
    }

    private void doConnectionExplore() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        WindowManager_v2 winmanager = (WindowManager_v2) ToolsSwingLocator.getWindowManager();
        final JDBCConnectionPanel jdbcServerExplorer = DALSwingLocator.getSwingManager().createJDBCConnectionPanel();
        jdbcServerExplorer.setConnectorFilter(new JDBCConnectionPanel.ConnectorFilter() {
            @Override
            public boolean accept(JDBCServerExplorerParameters parameters) {
                return StringUtils.equalsIgnoreCase(parameters.getExplorerName(), H2SpatialExtension.DAL_EXPLORER_NAME);
            }
        });
        final Dialog dlg = winmanager.createDialog(
            jdbcServerExplorer.asJComponent(),
            i18n.getTranslation("_Import_file_in_H2_database"),
            i18n.getTranslation("_Import_a_Gpx_Geojson_or_OSM_as_tables_in_a_H2_database"), 
            WindowManager_v2.BUTTONS_OK_CANCEL
        );
        dlg.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if( dlg.getAction()==WindowManager_v2.BUTTONS_OK ) {
                    loadConnectionCombo(cboConnection, jdbcServerExplorer.getConnectionName());
                }
            }
        });
        dlg.show(WindowManager.MODE.WINDOW);
    }    
    
    private void doFileExplore() {
        I18nManager i18n = ToolsLocator.getI18nManager();
        FoldersManager foldersManager = ToolsLocator.getFoldersManager();
        ThreadSafeDialogsManager dialogs = ToolsSwingLocator.getThreadSafeDialogsManager();
        File[] files = dialogs.showChooserDialog(
            i18n.getTranslation("_Select_the_file_to_import"),
            JFileChooser.OPEN_DIALOG,
            JFileChooser.FILES_ONLY,
            false,
            foldersManager.get("DataFolder"),
            new FileFilter() {
                @Override
                public boolean accept(File f) {
                    if( f.isDirectory() ) {
                        return true;
                    }
                    return FilenameUtils.isExtension(f.getName().toLowerCase(), new String[] {"gpx", "geojson", "osm"});
                }

                @Override
                public String getDescription() {
                    return "GPX, GeoJSon and OSM files";
                }
            },
            false
        );
        if( !ArrayUtils.isEmpty(files) ) {
            this.file = files[0];
            this.txtFile.setText(this.file.getAbsolutePath());
            if( StringUtils.isEmpty(this.txtTableName.getText()) ) {
                this.txtTableName.setText(FilenameUtils.getBaseName(this.file.getName()));
            }
        }
    }
}
