package org.gvsig.h2spatial;

import com.jeta.open.i18n.I18NUtils;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


public class ImportTableView extends JPanel
{
   JLabel lblConnection = new JLabel();
   JLabel lblFileType = new JLabel();
   JLabel lblFile = new JLabel();
   JLabel lblTableName = new JLabel();
   JLabel lblDeleteTables = new JLabel();
   JComboBox cboConnection = new JComboBox();
   JButton btnConnection = new JButton();
   JComboBox cboFileType = new JComboBox();
   JTextField txtFile = new JTextField();
   JButton btnFile = new JButton();
   JTextField txtTableName = new JTextField();
   JCheckBox chkDeleteTables = new JCheckBox();

   /**
    * Default constructor
    */
   public ImportTableView()
   {
      initializePanel();
   }

   /**
    * Main method for panel
    */
   public static void main(String[] args)
   {
      JFrame frame = new JFrame();
      frame.setSize(600, 400);
      frame.setLocation(100, 100);
      frame.getContentPane().add(new ImportTableView());
      frame.setVisible(true);

      frame.addWindowListener( new WindowAdapter()
      {
         public void windowClosing( WindowEvent evt )
         {
            System.exit(0);
         }
      });
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE,FILL:DEFAULT:GROW(1.0),FILL:4DLU:NONE,FILL:DEFAULT:NONE,FILL:4DLU:NONE","CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE,CENTER:DEFAULT:NONE,CENTER:2DLU:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblConnection.setName("lblConnection");
      lblConnection.setText("_Connection");
      jpanel1.add(lblConnection,cc.xy(2,2));

      lblFileType.setName("lblFileType");
      lblFileType.setText("_File_type");
      jpanel1.add(lblFileType,cc.xy(2,4));

      lblFile.setName("lblFile");
      lblFile.setText("_File");
      jpanel1.add(lblFile,cc.xy(2,6));

      lblTableName.setName("lblTableName");
      lblTableName.setText("_Table_name");
      jpanel1.add(lblTableName,cc.xy(2,8));

      lblDeleteTables.setName("lblDeleteTables");
      lblDeleteTables.setText("_Delete_tables");
      jpanel1.add(lblDeleteTables,cc.xy(2,10));

      cboConnection.setName("cboConnection");
      jpanel1.add(cboConnection,cc.xy(4,2));

      btnConnection.setActionCommand("...");
      btnConnection.setName("btnConnection");
      btnConnection.setText("...");
      jpanel1.add(btnConnection,cc.xy(6,2));

      cboFileType.setName("cboFileType");
      jpanel1.add(cboFileType,cc.xywh(4,4,3,1));

      txtFile.setBackground(new Color(236,233,216));
      txtFile.setEditable(false);
      txtFile.setName("txtFile");
      jpanel1.add(txtFile,cc.xy(4,6));

      btnFile.setActionCommand("...");
      btnFile.setName("btnFile");
      btnFile.setText("...");
      jpanel1.add(btnFile,cc.xy(6,6));

      txtTableName.setName("txtTableName");
      jpanel1.add(txtTableName,cc.xywh(4,8,3,1));

      chkDeleteTables.setName("chkDeleteTables");
      chkDeleteTables.setSelected(true);
      jpanel1.add(chkDeleteTables,cc.xywh(4,10,3,1));

      addFillComponents(jpanel1,new int[]{ 1,2,3,4,5,6,7 },new int[]{ 1,2,3,4,5,6,7,8,9,10,11 });
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
