/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.labeling.placements;

import java.util.ArrayList;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.ILabelClass;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.IPlacementConstraints;
import org.gvsig.symbology.fmap.mapcontext.rendering.legend.styling.LabelLocationMetrics;
import org.gvsig.tools.task.Cancellable;

public class CompoundLabelPlacement implements ILabelPlacement {
	private final ILabelPlacement[] placements;

	public CompoundLabelPlacement(ILabelPlacement[] suitablePlacements) {
		this.placements = suitablePlacements;
	}

    @Override
	public boolean isSuitableFor(IPlacementConstraints placementConstraints,
			int shapeType) {
		return true;
	}


    @Override
	public ArrayList<LabelLocationMetrics> guess(
			ILabelClass lc,
			Geometry geom,
			IPlacementConstraints constraints,
			double cartographicSymbolSize,
			Cancellable cancel,
			ViewPort vp) {
		
		ArrayList<LabelLocationMetrics> guessed = new ArrayList<>();
		for (int i = 0; cancel != null && !cancel.isCanceled() && i < placements.length; i++) {
			guessed.addAll(placements[i].guess(lc, geom, constraints, cartographicSymbolSize, cancel,vp));
		}
		return guessed;
	}
}
