/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.labeling.placements;

import java.util.ArrayList;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.ILabelClass;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.IPlacementConstraints;
import org.gvsig.symbology.fmap.mapcontext.rendering.legend.styling.LabelLocationMetrics;
import org.gvsig.tools.task.Cancellable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LinePlacementAtBest extends CompoundLabelPlacement {
	
	private static Logger logger = LoggerFactory.getLogger(LinePlacementAtBest.class);
	static private ILabelPlacement[] pl = new ILabelPlacement[] {
		new LinePlacementInTheMiddle(),
		new LinePlacementAtExtremities()
	};

	public LinePlacementAtBest() {
		super(pl);
		// a patch just to keep in mind the end of the lines
	}


	public ArrayList<LabelLocationMetrics> guess(ILabelClass lc,
			Geometry geom, IPlacementConstraints constraints,
			double cartographicSymbolSize, Cancellable cancel, ViewPort vp) {
		
		if (cancel.isCanceled()) {
            return CannotPlaceLabel.NO_PLACES;
        }

		ArrayList<LabelLocationMetrics> llc = super.guess(lc, geom, constraints, cartographicSymbolSize, cancel,vp);
		if (constraints instanceof AbstractPlacementConstraints) {
			AbstractPlacementConstraints clone;
			try {
				clone = (AbstractPlacementConstraints) ((AbstractPlacementConstraints) constraints).clone();
				clone.setLocationAlongTheLine(IPlacementConstraints.AT_THE_END_OF_THE_LINE);
				llc.addAll(pl[1].guess(lc, geom, clone, cartographicSymbolSize, cancel,vp));
			} catch (CloneNotSupportedException e) {
				// this should never happen but
				// anyway a warning does not hurt anyone
				logger.info("Couldn't clone "+constraints.getClass().getName(), e);
			}

		}
		return llc;
	}
    @Override
	public boolean isSuitableFor(
			IPlacementConstraints placementConstraints,
			int shapeType) {
		
        GeometryManager geomManager = GeometryLocator.getGeometryManager();
        if( geomManager.isSubtype(TYPES.CURVE, shapeType) ||
            geomManager.isSubtype(TYPES.MULTICURVE, shapeType) ) {
			return placementConstraints != null && placementConstraints.isAtBestOfLine();
		} else {
			return false;
		}
	}


}