/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.labeling.placements;

import java.awt.geom.Point2D;

import org.apache.batik.ext.awt.geom.PathLength;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.ILabelClass;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.IPlacementConstraints;
import org.gvsig.symbology.fmap.mapcontext.rendering.legend.styling.LabelLocationMetrics;
import org.gvsig.tools.task.Cancellable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LinePlacementInTheMiddle extends AbstractLinePlacement {

	private static Logger logger =
			LoggerFactory.getLogger(LinePlacementInTheMiddle.class);
					
    @Override
	public boolean isSuitableFor(
			IPlacementConstraints placementConstraints,
			int shapeType) {
		
        GeometryManager geomManager = GeometryLocator.getGeometryManager();
        if( geomManager.isSubtype(TYPES.CURVE, shapeType) ||
            geomManager.isSubtype(TYPES.MULTICURVE, shapeType) ) {
			return placementConstraints != null &&
			   placementConstraints.isInTheMiddleOfLine()
			  /* && !placementConstraints.isFollowingLine()*/;
		} else {
			return false;
		}
		
	}

    @Override
	public LabelLocationMetrics initialLocation(ILabelClass lc,
			IPlacementConstraints pc, PathLength pathLen, Cancellable cancel) {
		
		if (cancel.isCanceled()) {
            return null;
        }

		float length = pathLen.lengthOfPath();
		float distance = (float) (length * 0.5);


		double theta = 0;
		if (pc.isParallel()) {
			// get the line theta and apply it
			theta = pathLen.angleAtLength(distance);

		} else if (pc.isPerpendicular()) {
			// get the line theta with 90 degrees
			theta = pathLen.angleAtLength(distance) + AbstractLinePlacement.HALF_PI;
		}

		Point2D p = pathLen.pointAtLength(distance);

		/*
		 * Offset the point to a distance of the height of the
		 * label class's height to make the label appear to
		 * be on the line.
		 *
		 */
		double x = p.getX();
		double y = p.getY();
		double halfHeight = lc.getBounds().getHeight()*.5;
		double halfWidth = lc.getBounds().getWidth()*.5;

		double sinTheta = Math.sin(theta);
		double cosTheta = Math.cos(theta);

		double xOffset = halfHeight * sinTheta;
		double yOffset = halfHeight * cosTheta;

		/*
		 * now, offset the anchor point as much as need to
		 * make the center of the label be the middle of the
		 * line
		 */
		xOffset -= halfWidth * cosTheta;
		yOffset += halfWidth * sinTheta;

		p.setLocation(x + xOffset, y - yOffset);

		return new LabelLocationMetrics(
				p,
				theta,
				true);
	}

}

