/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.labeling.placements;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.geom.GeometryLocator;
import org.gvsig.fmap.geom.GeometryManager;
import org.gvsig.fmap.geom.primitive.Point;
import org.gvsig.fmap.mapcontext.ViewPort;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.ILabelClass;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.IPlacementConstraints;
import org.gvsig.symbology.fmap.mapcontext.rendering.legend.styling.LabelLocationMetrics;
import org.gvsig.tools.task.Cancellable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class MarkerPlacementOnPoint implements ILabelPlacement{
	
	private Logger logger = LoggerFactory.getLogger(MarkerPlacementOnPoint.class);

	public ArrayList<LabelLocationMetrics> guess(
			ILabelClass lc,
			Geometry shp,
			IPlacementConstraints placementConstraints,
			double cartographicSymbolSize, Cancellable cancel) {

		if (cancel.isCanceled()) {
            return CannotPlaceLabel.NO_PLACES;
        }

		ArrayList<LabelLocationMetrics> guessed = new ArrayList<LabelLocationMetrics>();
		
		Point cen = null;
		try {
			cen = shp.centroid();
		} catch (Exception ex) {
			logger.error("While getting centroid", ex);
		}
		Point2D p = new Point2D.Double(cen.getX(), cen.getY());
		Rectangle2D bounds = lc.getBounds();
		//FIXME: Con este posicionamiento, es identico al MarkerCenteredAtPoint
		//pero es as como lo hacen otros programas
		//(tal vez, hubo una confusin por el nombre de la constante ON_TOP_OF_THE_POINT).
		//Entonces sobrara el posicionamiento MarkerCenteredAtPoint debiendose utilizar siempre ste.
		p.setLocation(p.getX() - (bounds.getWidth()*0.5), p.getY() - (bounds.getHeight()*0.5));// - 2);
		guessed.add(new LabelLocationMetrics(
				p, 0, true));
		return guessed;
	}


    @Override
	public ArrayList<LabelLocationMetrics> guess(
			ILabelClass lc,
			Geometry geom,
			IPlacementConstraints placementConstraints,
			double cartographicSymbolSize, Cancellable cancel, ViewPort vp) {
		
		if (cancel.isCanceled()) return CannotPlaceLabel.NO_PLACES;

		Geometry shp = geom.cloneGeometry();
		shp.transform(vp.getAffineTransform());

		ArrayList<LabelLocationMetrics> guessed =
				new ArrayList<LabelLocationMetrics>();
		
		Point cen = null;
		try {
			cen = shp.centroid();
		} catch (Exception ex) {
			logger.error("While getting centroid", ex);
		}
		Point2D p = new Point2D.Double(cen.getX(), cen.getY());
		Rectangle2D bounds = lc.getBounds();
		//FIXME: Con este posicionamiento, es identico al MarkerCenteredAtPoint
		//pero es as como lo hacen otros programas
		//(tal vez, hubo una confusin por el nombre de la constante ON_TOP_OF_THE_POINT).
		//Entonces sobrara el posicionamiento MarkerCenteredAtPoint debiendose utilizar siempre ste.
		p.setLocation(p.getX() - (bounds.getWidth()*0.5), p.getY() - bounds.getHeight()*0.5);// - 2);
		guessed.add(new LabelLocationMetrics(
				p, 0, true));
		return guessed;
	}

    @Override
	public boolean isSuitableFor(
			IPlacementConstraints placementConstraints,
			int shapeType) {
        GeometryManager geomManager = GeometryLocator.getGeometryManager();
        if( geomManager.isSubtype(TYPES.POINT, shapeType) ) {
			return placementConstraints != null && placementConstraints.isOnTopOfThePoint();
		} else{
			return false;
		}
		
	}



}
