/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.shp;

import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.operations.SingleLayer;
import org.gvsig.tools.dynobject.exception.DynMethodNotSupportedException;

public class saveEncodingExtension extends Extension {

    @Override
    public void initialize() {
    }

    @Override
    public void execute(String actionCommand) {
        FLayer[] layers = getActiveLayers();
        if( layers == null ) {
            return;
        }
        List<FLayer> problems = new ArrayList<>();
        for( FLayer layer : layers ) {
            if( layer instanceof SingleLayer ) {
                DataStore ds = ((SingleLayer) layer).getDataStore();
                try {
                    ds.invokeDynMethod("saveEncoding", null);
                } catch (DynMethodNotSupportedException ex) {
                    // Do nothing
                } catch (Throwable ex) {
                    logger.warn("Can't save encoding of layer '" + layer.getName() + "'.", ex);
                    problems.add(layer);
                }
            }
        }
        if( !problems.isEmpty() ) {
            StringBuilder builder = new StringBuilder();
            builder.append("\n");
            for( FLayer problem : problems ) {
                builder.append("- ").append(problem.getName()).append("\n");
            }
            ApplicationManager application = ApplicationLocator.getManager();
            application.messageDialog(
                "_There_have_been_problems_saving_the_encoding_of_the_following_layers:",
                new String[]{builder.toString()},
                "_Problems_saving_encoding_of_layer",
                JOptionPane.WARNING_MESSAGE,
                "_Problems_saving_encoding_of_layer"
            );

        }
    }

    @Override
    public boolean isEnabled() {
        FLayer[] layers = getActiveLayers();
        if( layers == null ) {
            return false;
        }
        for( FLayer layer : layers ) {
            try {
                if( layer instanceof SingleLayer ) {
                    DataStore ds = ((SingleLayer) layer).getDataStore();
                    if( ds.hasDynMethod("saveEncoding") ) {
                        return true;
                    }
                }
            } catch (Throwable ex) {
            }
        }
        return false;
    }

    @Override
    public boolean isVisible() {
        FLayer[] layers = getActiveLayers();
        if( layers == null ) {
            return false;
        }
        for( FLayer layer : layers ) {
            try {
                if( layer instanceof SingleLayer ) {
                    return true;
                }
            } catch (Throwable ex) {
            }
        }
        return false;
    }

    private FLayer[] getActiveLayers() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument viewdoc = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( viewdoc == null ) {
            return null;
        }
        MapContext map = viewdoc.getMapContext();
        if( !map.hasActiveLayers() ) {
            return null;
        }
        FLayer[] layers = viewdoc.getMapContext().getLayers().getActives();
        return layers;
    }
}
