package org.gvsig.symbology.app.symbolinstaller.execution;

import java.io.File;
import org.apache.commons.io.FileUtils;
import org.gvsig.andami.PluginsLocator;
import org.gvsig.andami.PluginsManager;
import org.gvsig.installer.lib.api.InstallerLocator;
import org.gvsig.installer.lib.api.InstallerManager;
import org.gvsig.installer.lib.spi.InstallerProviderLocator;
import org.gvsig.tools.service.spi.ProviderManager;

import org.gvsig.tools.util.Invocable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SymbolsInstallerInitializer implements Invocable {
    
    private static final Logger LOG = LoggerFactory.getLogger(SymbolsInstallerInitializer.class);

    @Override
    public Object call(Object... args) {
        PluginsManager pluginManager = PluginsLocator.getManager();
        File symbolsFolder = FileUtils.getFile(
                pluginManager.getApplicationHomeFolder(),
                "plugins",
                "org.gvsig.app.mainplugin",
                "Symbols"
        );
        this.perform((String) args[0], symbolsFolder);
        return true;
    }
    
    public void perform(String pluginName, File symbolsFolder) {
        try {
            InstallerManager installerManager = InstallerLocator.getInstallerManager();
            ProviderManager providerManager = InstallerProviderLocator.getProviderManager();

            SymbolInstallerExecutionProviderFactory factory = new SymbolInstallerExecutionProviderFactory(symbolsFolder);
            providerManager.addProviderFactory(factory);
            installerManager.setDefaultLocalAddonRepository(symbolsFolder, factory.getName());

        } catch (Exception ex) {
            LOG.warn("Problems initializing scripts installer.", ex);
        }
    }
}
