/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2017 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.timesupport.app.extension;

import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.layers.CancelationException;
import org.gvsig.fmap.mapcontext.layers.LayerCollectionEvent;
import org.gvsig.fmap.mapcontext.layers.LayerCollectionListener;
import org.gvsig.fmap.mapcontext.layers.LayerPositionEvent;
import org.gvsig.timesupport.swing.api.panel.TimeSelectorPanel;


/**
 * @author gvSIG Team
 * @version $Id$
 *
 */
public class MapContextListener implements LayerCollectionListener{
    private final MapContext mapContext;
    private final TimeSelectorPanel timeSelectorPanel;

    public MapContextListener(MapContext mapContext, TimeSelectorPanel timeSelectorPanel) {
        super();
        this.mapContext = mapContext;
        this.timeSelectorPanel = timeSelectorPanel;
    }

    @Override
    public void layerAdded(LayerCollectionEvent e) {
        mapContextUpdated();
    }

    @Override
    public void layerMoved(LayerPositionEvent e) {

    }

    @Override
    public void layerRemoved(LayerCollectionEvent e) {
        mapContextUpdated();        
    }

    @Override
    public void layerAdding(LayerCollectionEvent e) throws CancelationException {

    }

    @Override
    public void layerMoving(LayerPositionEvent e) throws CancelationException {

    }

    @Override
    public void layerRemoving(LayerCollectionEvent e) throws CancelationException {

    }

    @Override
    public void visibilityChanged(LayerCollectionEvent e) throws CancelationException {

    }

    private void mapContextUpdated(){
        MapContext.MapTimeContext timeContext = mapContext.getTimeContext();
        if (timeContext.getInterval() == null){
            timeSelectorPanel.asJComponent().setVisible(false);
        }else{
            timeSelectorPanel.asJComponent().setVisible(true);
            timeSelectorPanel.setIntervals(timeContext.getInterval());
            timeSelectorPanel.setInstants(timeContext.getTimes());  
        }
    }
}
