/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2017 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us at info AT
 * gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.timesupport.app.extension;

import java.util.Iterator;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.ApplicationManager;
import org.gvsig.app.project.documents.view.ViewDocument;
import org.gvsig.app.project.documents.view.ViewManager;
import org.gvsig.fmap.dal.DataStore;
import org.gvsig.fmap.mapcontext.MapContext;
import org.gvsig.fmap.mapcontext.MapContext.MapTimeContext;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontext.layers.operations.SingleLayer;
import org.gvsig.timesupport.AbsoluteIntervalTypeNotRegisteredException;
import org.gvsig.timesupport.swing.api.TimeSupportSwingException;
import org.gvsig.timesupport.swing.api.TimeSupportSwingLocator;
import org.gvsig.timesupport.swing.api.TimeSupportSwingManager;
import org.gvsig.timesupport.swing.api.panel.TimeSelectorPanel;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.locator.LocatorException;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;

/**
 * Andami extension to show TimeSupport in the application.
 *
 * @author gvSIG Team
 */
public class TimeSupportExtension extends Extension {

    @Override
    public void initialize() {
        // Do nothing
    }

    @Override
    public void postInitialize() {
        super.postInitialize();
    }

    @Override
    public void execute(String actionCommand) {
        try {
            showTimeSupport();
        } catch (Exception e) {
            logger.warn("Can't show time support dialog", e);
        }
    }

    public void showTimeSupport() throws CloneNotSupportedException, LocatorException, TimeSupportSwingException, AbsoluteIntervalTypeNotRegisteredException {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument viewDocument = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( viewDocument == null ) {
            return;
        }
        I18nManager i18n = ToolsLocator.getI18nManager();
        TimeSupportSwingManager swingManager = TimeSupportSwingLocator.getSwingManager();

        MapContext mapContext = viewDocument.getMapContext();
        MapTimeContext timeContext = mapContext.getTimeContext();
        
        TimeSelectorPanel timeSelectorPanel = swingManager.createTimeSelectorPanel(timeContext.getInterval());
        timeSelectorPanel.setInstants(timeContext.getTimes());
        timeSelectorPanel.addObserver(new TimeSelectorObserver(viewDocument));

        mapContext.getLayers().addLayerCollectionListener(new MapContextListener(mapContext, timeSelectorPanel));

        WindowManager windowManager = ToolsSwingLocator.getWindowManager();
        windowManager.showWindow(
            timeSelectorPanel.asJComponent(), 
            i18n.getTranslation("temporal_filter") + " - " + viewDocument.getName(), 
            WindowManager.MODE.TOOL
        );
    }

    @Override
    public boolean isEnabled() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument viewDocument = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        if( viewDocument == null ) {
            return false;
        }
        FLayers layers = viewDocument.getMapContext().getLayers();
        for( Iterator<FLayer> iterator = layers.deepiterator(); iterator.hasNext(); ) {
            FLayer layer = iterator.next();
            if( layer instanceof SingleLayer ) {
                DataStore dataStore = ((SingleLayer) layer).getDataStore();
                if( dataStore.getInterval() != null ) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public boolean isVisible() {
        ApplicationManager application = ApplicationLocator.getManager();
        ViewDocument viewDocument = (ViewDocument) application.getActiveDocument(ViewManager.TYPENAME);
        return viewDocument!=null;
    }

}
